@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,14 +27,17 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <vector>
-#include "base/base.h"
 #include "base/bitarray.h"
+
+#include <vector>
+
 #include "base/util.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
 
 namespace mozc {
+namespace {
+
 TEST(BitArray, BitArraySizeTest) {
   {
     BitArray array(0);
@@ -87,9 +90,11 @@ TEST(BitArray, BitArrayTest) {
 
     // verify
     for (size_t j = 0; j < size; ++j) {
-      EXPECT_EQ(static_cast<bool>(target[j]), BitArray::GetValue(data, j));
-      EXPECT_EQ(static_cast<bool>(target[j]), array.get(j));
+      EXPECT_EQ((target[j] != 0), BitArray::GetValue(data, j));
+      EXPECT_EQ((target[j] != 0), array.get(j));
     }
   }
 }
+
+}  // namespace
 }  // namespace mozc
