@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,8 @@
 #include <streambuf>
 #include <string>
 
-#include "base/base.h"
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 
 #ifdef OS_ANDROID
 // This is used only for code generation, so shouldn't be used from android
@@ -55,7 +56,7 @@
 #error "base/codegen_bytearray_stream.h shouldn't be used from android platform."
 #endif
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 // Visual C++ does not support string literals longer than 65535 characters
 // so integer arrays (e.g. arrays of uint64) are used to represent byte arrays
 // on Windows.
@@ -76,7 +77,7 @@
 //       "\\x12\\x34\\x56\\x78...";
 //   const size_t kVAR_size = 123;
 #define MOZC_CODEGEN_BYTEARRAY_STREAM_USES_WORD_ARRAY
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #ifdef MOZC_CODEGEN_BYTEARRAY_STREAM_USES_WORD_ARRAY
 #include <iomanip>
@@ -95,24 +96,18 @@ enum StreamOwner {
 };
 }  // namespace codegenstream
 
-template <class charT, class traits = char_traits<charT> >
-class BasicCodeGenByteArrayStreamBuf
-    : public std::basic_streambuf<charT, traits> {
+class BasicCodeGenByteArrayStreamBuf : public std::streambuf {
  public:
-  typedef charT char_type;
-  typedef typename traits::int_type int_type;
-  typedef typename traits::pos_type pos_type;
-  typedef typename traits::off_type off_type;
-  typedef traits traits_type;
+  typedef char_traits<char> traits_type;
 
   // Args:
   //   output_stream: The output stream to which generated code is written.
   //   own_output_stream: The object pointed to by |output_stream| will be
   //       destroyed if |own_output_stream| equals to |OWN_STREAM|.
-  BasicCodeGenByteArrayStreamBuf(std::basic_ostream<char> *output_stream,
+  BasicCodeGenByteArrayStreamBuf(std::ostream *output_stream,
                                  codegenstream::StreamOwner own_output_stream)
       : internal_output_buffer_size_(kDefaultInternalBufferSize),
-        internal_output_buffer_(new char_type[internal_output_buffer_size_]),
+        internal_output_buffer_(new char[internal_output_buffer_size_]),
         output_stream_(output_stream), own_output_stream_(own_output_stream),
         is_open_(false), output_count_(0) {
     this->setp(internal_output_buffer_.get(),
@@ -150,7 +145,7 @@ class BasicCodeGenByteArrayStreamBuf
     *output_stream_ << "const char k" << var_name_base_ << "_data[] =\n";
 #endif
     output_count_ = 0;
-    return is_open_ = *output_stream_;
+    return is_open_ = !output_stream_->fail();
   }
 
   // Writes the end of a variable definition.
@@ -181,20 +176,20 @@ class BasicCodeGenByteArrayStreamBuf
     *output_stream_ << "const size_t k" << var_name_base_ << "_size = "
                     << output_count_ << ";\n";
     is_open_ = false;
-    return *output_stream_;
+    return !output_stream_->fail();
   }
 
  protected:
   // Flushes.
   virtual int sync() {
     return (overflow() != traits_type::eof() &&
-            output_stream_->flush())
+            !output_stream_->flush().fail())
         ? 0 : -1;
   }
 
   // Writes a given character sequence.  The implementation is expected to be
   // more efficient than the one of the base class.
-  virtual std::streamsize xsputn(const char_type *s, std::streamsize n) {
+  virtual std::streamsize xsputn(const char *s, std::streamsize n) {
     if (n <= this->epptr() - this->pptr()) {
       traits_type::copy(this->pptr(), s, n);
       this->pbump(n);
@@ -208,7 +203,7 @@ class BasicCodeGenByteArrayStreamBuf
   }
 
   // Writes the data body of a variable definition.
-  virtual int_type overflow(int_type c = traits_type::eof()) {
+  virtual int overflow(int c = traits_type::eof()) {
     if (!is_open_) {
       return traits_type::eof();
     }
@@ -217,13 +212,13 @@ class BasicCodeGenByteArrayStreamBuf
                  reinterpret_cast<const char *>(this->pptr()));
     }
     if (!traits_type::eq_int_type(c, traits_type::eof())) {
-      char_type buf = static_cast<char_type>(c);
+      char buf = static_cast<char>(c);
       WriteBytes(reinterpret_cast<const char *>(&buf),
                  reinterpret_cast<const char *>(&buf + 1));
     }
     this->setp(internal_output_buffer_.get(),
                internal_output_buffer_.get() + internal_output_buffer_size_);
-    return !*output_stream_
+    return output_stream_->fail()
         ? traits_type::eof()
         : (traits_type::eq_int_type(c, traits_type::eof())
            ? traits_type::not_eof(c) : c);
@@ -287,13 +282,13 @@ class BasicCodeGenByteArrayStreamBuf
 #endif
 
   size_t internal_output_buffer_size_;
-  scoped_array<char_type> internal_output_buffer_;
+  scoped_ptr<char[]> internal_output_buffer_;
 
   std::basic_ostream<char> *output_stream_;
   codegenstream::StreamOwner own_output_stream_;
 #ifdef MOZC_CODEGEN_BYTEARRAY_STREAM_USES_WORD_ARRAY
   ios_base::fmtflags output_stream_format_flags_;
-  char_type output_stream_format_fill_;
+  char output_stream_format_fill_;
 #endif
 
   bool is_open_;
@@ -306,24 +301,16 @@ class BasicCodeGenByteArrayStreamBuf
   DISALLOW_COPY_AND_ASSIGN(BasicCodeGenByteArrayStreamBuf);
 };
 
-template <class charT, class traits = char_traits<charT> >
-class BasicCodeGenByteArrayOutputStream
-    : public std::basic_ostream<charT, traits> {
+class CodeGenByteArrayOutputStream : public std::ostream {
  public:
-  typedef charT char_type;
-  typedef typename traits::int_type int_type;
-  typedef typename traits::pos_type pos_type;
-  typedef typename traits::off_type off_type;
-  typedef traits traits_type;
-
   // Args:
   //   output_stream: The output stream to which generated code is written.
   //   own_output_stream: The object pointed to by |output_stream| will be
   //       destroyed if |own_output_stream| equals to |OWN_STREAM|.
-  BasicCodeGenByteArrayOutputStream(
-      std::basic_ostream<char> *output_stream,
+  CodeGenByteArrayOutputStream(
+      std::ostream *output_stream,
       codegenstream::StreamOwner own_output_stream)
-      : std::basic_ostream<char_type, traits_type>(NULL),
+      : std::ostream(NULL),
         streambuf_(output_stream, own_output_stream) {
     this->rdbuf(&streambuf_);
   }
@@ -346,13 +333,11 @@ class BasicCodeGenByteArrayOutputStream
   }
 
  private:
-  BasicCodeGenByteArrayStreamBuf<char_type, traits_type> streambuf_;
+  BasicCodeGenByteArrayStreamBuf streambuf_;
 
-  DISALLOW_COPY_AND_ASSIGN(BasicCodeGenByteArrayOutputStream);
+  DISALLOW_COPY_AND_ASSIGN(CodeGenByteArrayOutputStream);
 };
 
-typedef BasicCodeGenByteArrayOutputStream<char> CodeGenByteArrayOutputStream;
-
 }  // namespace mozc
 
 #endif  // MOZC_BASE_CODEGEN_BYTEARRAY_STREAM_H_
