@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -49,63 +49,63 @@ namespace {
 
 // Table of number character of Kansuji
 const char *const kNumKanjiDigits[] = {
-  //   "〇", "一", "二", "三", "四", "五", "六", "七", "八", "九", NULL
+  // "〇", "一", "二", "三", "四", "五", "六", "七", "八", "九", nullptr
   "\xe3\x80\x87", "\xe4\xb8\x80", "\xe4\xba\x8c", "\xe4\xb8\x89",
   "\xe5\x9b\x9b", "\xe4\xba\x94", "\xe5\x85\xad", "\xe4\xb8\x83",
-  "\xe5\x85\xab", "\xe4\xb9\x9d", NULL
+  "\xe5\x85\xab", "\xe4\xb9\x9d", nullptr
 };
 const char *const kNumKanjiOldDigits[] = {
-  //   NULL, "壱", "弐", "参", "四", "五", "六", "七", "八", "九"
-  NULL, "\xe5\xa3\xb1", "\xe5\xbc\x90", "\xe5\x8f\x82", "\xe5\x9b\x9b",
+  // nullptr, "壱", "弐", "参", "四", "五", "六", "七", "八", "九"
+  nullptr, "\xe5\xa3\xb1", "\xe5\xbc\x90", "\xe5\x8f\x82", "\xe5\x9b\x9b",
   "\xe4\xba\x94", "\xe5\x85\xad", "\xe4\xb8\x83", "\xe5\x85\xab",
   "\xe4\xb9\x9d"
 };
 const char *const kNumFullWidthDigits[] = {
-  //   "０", "１", "２", "３", "４", "５", "６", "７", "８", "９", NULL
+  // "０", "１", "２", "３", "４", "５", "６", "７", "８", "９", nullptr
   "\xef\xbc\x90", "\xef\xbc\x91", "\xef\xbc\x92", "\xef\xbc\x93",
   "\xef\xbc\x94", "\xef\xbc\x95", "\xef\xbc\x96", "\xef\xbc\x97",
-  "\xef\xbc\x98", "\xef\xbc\x99", NULL
+  "\xef\xbc\x98", "\xef\xbc\x99", nullptr
 };
 const char *const kNumHalfWidthDigits[] = {
-  "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", NULL
+  "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", nullptr
 };
 
 // Table of Kanji number ranks
 const char *const kNumKanjiRanks[] = {
-  //   NULL, "", "十", "百", "千"
-  NULL, "", "\xe5\x8d\x81", "\xe7\x99\xbe", "\xe5\x8d\x83"
+  // nullptr, "", "十", "百", "千"
+  nullptr, "", "\xe5\x8d\x81", "\xe7\x99\xbe", "\xe5\x8d\x83"
 };
 const char *const kNumKanjiBiggerRanks[] = {
-  //   "", "万", "億", "兆", "京"
+  // "", "万", "億", "兆", "京"
   "", "\xe4\xb8\x87", "\xe5\x84\x84", "\xe5\x85\x86", "\xe4\xba\xac"
 };
 const char *const kNumKanjiOldRanks[] = {
-  //   NULL, "", "拾", "百", "阡"
-  NULL, "", "\xe6\x8b\xbe", "\xe7\x99\xbe", "\xe9\x98\xa1"
+  // nullptr, "", "拾", "百", "阡"
+  nullptr, "", "\xe6\x8b\xbe", "\xe7\x99\xbe", "\xe9\x98\xa1"
 };
 const char *const kNumKanjiBiggerOldRanks[] = {
-  //   "", "萬", "億", "兆", "京"
+  // "", "萬", "億", "兆", "京"
   "", "\xe8\x90\xac", "\xe5\x84\x84", "\xe5\x85\x86", "\xe4\xba\xac"
 };
 
 const char *const kRomanNumbersCapital[] = {
-  // NULL, "Ⅰ", "Ⅱ", "Ⅲ", "Ⅳ", "Ⅴ", "Ⅵ", "Ⅶ", "Ⅷ", "Ⅸ",
-  // "Ⅹ", "Ⅺ", "Ⅻ", NULL
-  NULL, "\xe2\x85\xa0", "\xe2\x85\xa1", "\xe2\x85\xa2", "\xe2\x85\xa3",
+  // nullptr, "Ⅰ", "Ⅱ", "Ⅲ", "Ⅳ", "Ⅴ", "Ⅵ", "Ⅶ", "Ⅷ", "Ⅸ",
+  // "Ⅹ", "Ⅺ", "Ⅻ", nullptr
+  nullptr, "\xe2\x85\xa0", "\xe2\x85\xa1", "\xe2\x85\xa2", "\xe2\x85\xa3",
   "\xe2\x85\xa4", "\xe2\x85\xa5", "\xe2\x85\xa6", "\xe2\x85\xa7",
-  "\xe2\x85\xa8", "\xe2\x85\xa9", "\xe2\x85\xaa", "\xe2\x85\xab", NULL
+  "\xe2\x85\xa8", "\xe2\x85\xa9", "\xe2\x85\xaa", "\xe2\x85\xab", nullptr
 };
 
 const char *const kRomanNumbersSmall[] = {
-  // NULL, "ⅰ", "ⅱ", "ⅲ", "ⅳ", "ⅴ", "ⅵ", "ⅶ", "ⅷ", "ⅸ",
-  // "ⅹ", "ⅺ", "ⅻ", NULL
-  NULL, "\xe2\x85\xb0", "\xe2\x85\xb1", "\xe2\x85\xb2", "\xe2\x85\xb3",
+  // nullptr, "ⅰ", "ⅱ", "ⅲ", "ⅳ", "ⅴ", "ⅵ", "ⅶ", "ⅷ", "ⅸ",
+  // "ⅹ", "ⅺ", "ⅻ", nullptr
+  nullptr, "\xe2\x85\xb0", "\xe2\x85\xb1", "\xe2\x85\xb2", "\xe2\x85\xb3",
   "\xe2\x85\xb4", "\xe2\x85\xb5", "\xe2\x85\xb6", "\xe2\x85\xb7",
-  "\xe2\x85\xb8", "\xe2\x85\xb9", "\xe2\x85\xba", "\xe2\x85\xbb", NULL
+  "\xe2\x85\xb8", "\xe2\x85\xb9", "\xe2\x85\xba", "\xe2\x85\xbb", nullptr
 };
 
 const char *const kCircledNumbers[] = {
-  NULL,
+  nullptr,
   // "①", "②", "③", "④", "⑤", "⑥", "⑦", "⑧", "⑨", "⑩"
   "\xe2\x91\xa0", "\xe2\x91\xa1", "\xe2\x91\xa2", "\xe2\x91\xa3",
   "\xe2\x91\xa4", "\xe2\x91\xa5", "\xe2\x91\xa6", "\xe2\x91\xa7",
@@ -124,7 +124,7 @@ const char *const kCircledNumbers[] = {
   "\xE3\x8A\xB5", "\xE3\x8A\xB6", "\xE3\x8A\xB7", "\xE3\x8A\xB8",
   "\xE3\x8A\xB9", "\xE3\x8A\xBA", "\xE3\x8A\xBB", "\xE3\x8A\xBC",
   "\xE3\x8A\xBD", "\xE3\x8A\xBE", "\xE3\x8A\xBF",
-  NULL
+  nullptr
 };
 
 // Structure to store character set variations.
@@ -139,7 +139,7 @@ struct NumberStringVariation {
 
 // Judges given string is a decimal number (including integer) or not.
 // It accepts strings whose last point is a decimal point like "123456."
-bool IsDecimalNumber(const string &str) {
+bool IsDecimalNumber(StringPiece str) {
   int num_point = 0;
   for (size_t i = 0; i < str.size(); ++i) {
     if (str[i] == '.') {
@@ -156,17 +156,36 @@ bool IsDecimalNumber(const string &str) {
   return true;
 }
 
-const int kInt32BufferSize = 12;  // "-2147483648\0"
+const char kAsciiZero = '0';
+const char kAsciiOne = '1';
+const char kAsciiNine = '9';
+
+template <typename T>
+string SimpleItoaImpl(T number) {
+  stringstream ss;
+  ss << number;
+  return ss.str();
+}
 
 }  // namespace
 
 string NumberUtil::SimpleItoa(int32 number) {
-  char buffer[kInt32BufferSize];
-  const int length = snprintf(buffer, kInt32BufferSize, "%d", number);
-  return string(buffer, length);
+  return SimpleItoaImpl(number);
 }
 
-int NumberUtil::SimpleAtoi(const string &str) {
+string NumberUtil::SimpleItoa(uint32 number) {
+  return SimpleItoaImpl(number);
+}
+
+string NumberUtil::SimpleItoa(int64 number) {
+  return SimpleItoaImpl(number);
+}
+
+string NumberUtil::SimpleItoa(uint64 number) {
+  return SimpleItoaImpl(number);
+}
+
+int NumberUtil::SimpleAtoi(StringPiece str) {
   stringstream ss;
   ss << str;
   int i = 0;
@@ -179,7 +198,7 @@ namespace {
 // TODO(hidehiko): Refactoring with GetScriptType in Util class.
 inline bool IsArabicDecimalChar32(char32 ucs4) {
   // Halfwidth digit.
-  if ('0' <= ucs4 && ucs4 <= '9') {
+  if (kAsciiZero <= ucs4 && ucs4 <= kAsciiNine) {
     return true;
   }
 
@@ -193,7 +212,10 @@ inline bool IsArabicDecimalChar32(char32 ucs4) {
 
 }  // namespace
 
-bool NumberUtil::IsArabicNumber(const string &input_string) {
+bool NumberUtil::IsArabicNumber(StringPiece input_string) {
+  if (input_string.empty()) {
+    return false;
+  }
   for (ConstChar32Iterator iter(input_string); !iter.Done(); iter.Next()) {
     if (!IsArabicDecimalChar32(iter.Get())) {
       // Found non-Arabic decimal character.
@@ -205,7 +227,10 @@ bool NumberUtil::IsArabicNumber(const string &input_string) {
   return true;
 }
 
-bool NumberUtil::IsDecimalInteger(const string &str) {
+bool NumberUtil::IsDecimalInteger(StringPiece str) {
+  if (str.empty()) {
+    return false;
+  }
   for (size_t i = 0; i < str.size(); ++i) {
     if (!isdigit(str[i])) {
       return false;
@@ -220,16 +245,16 @@ namespace {
 // http://ja.wikipedia.org/wiki/%E5%A4%A7%E5%AD%97_(%E6%95%B0%E5%AD%97)
 const NumberStringVariation kKanjiVariations[] = {
   // "数字"
-  {kNumHalfWidthDigits, 10, "\xE6\x95\xB0\xE5\xAD\x97", NULL, NULL,
+  {kNumHalfWidthDigits, 10, "\xE6\x95\xB0\xE5\xAD\x97", nullptr, nullptr,
    NumberUtil::NumberString::NUMBER_ARABIC_AND_KANJI_HALFWIDTH},
   // "数字"
-  {kNumFullWidthDigits, 10, "\xE6\x95\xB0\xE5\xAD\x97", NULL, NULL,
+  {kNumFullWidthDigits, 10, "\xE6\x95\xB0\xE5\xAD\x97", nullptr, nullptr,
    NumberUtil::NumberString::NUMBER_ARABIC_AND_KANJI_FULLWIDTH},
   // "漢数字"
-  {kNumKanjiDigits, 10, "\xE6\xBC\xA2\xE6\x95\xB0\xE5\xAD\x97", NULL, NULL,
-   NumberUtil::NumberString::NUMBER_KANJI},
+  {kNumKanjiDigits, 10, "\xE6\xBC\xA2\xE6\x95\xB0\xE5\xAD\x97",
+   nullptr, nullptr, NumberUtil::NumberString::NUMBER_KANJI},
   // "大字"
-  {kNumKanjiOldDigits, 10, "\xE5\xA4\xA7\xE5\xAD\x97", NULL, NULL,
+  {kNumKanjiOldDigits, 10, "\xE5\xA4\xA7\xE5\xAD\x97", nullptr, nullptr,
    NumberUtil::NumberString::NUMBER_OLD_KANJI},
 };
 
@@ -241,7 +266,7 @@ const char kOldTwenty[] = "\xE5\xBB\xBF";
 
 }  // namespace
 
-bool NumberUtil::ArabicToKanji(const string &input_num,
+bool NumberUtil::ArabicToKanji(StringPiece input_num,
                                vector<NumberString> *output) {
   DCHECK(output);
   // "零"
@@ -252,13 +277,11 @@ bool NumberUtil::ArabicToKanji(const string &input_num,
     return false;
   }
 
-  // We don't convert a number starting with '0', other than 0 itself.
-  if (input_num[0] == '0') {
-    const char *p = input_num.c_str();
-    while (*p == '0') {
-      ++p;
-    }
-    if (*p == '\0') {
+  {
+    // We don't convert a number starting with '0', other than 0 itself.
+    StringPiece::size_type i;
+    for (i = 0; i < input_num.size() && input_num[i] == kAsciiZero; ++i) {}
+    if (i == input_num.size()) {
       output->push_back(
           // "大字"
           NumberString(kNumZero, "\xE5\xA4\xA7\xE5\xAD\x97",
@@ -277,8 +300,8 @@ bool NumberUtil::ArabicToKanji(const string &input_num,
   // (N * kDigitsInBigRank).
   const int filled_zero_num = (kDigitsInBigRank -
       (input_num.size() % kDigitsInBigRank)) % kDigitsInBigRank;
-  string input(filled_zero_num, '0');
-  input.append(input_num);
+  string input(filled_zero_num, kAsciiZero);
+  input_num.AppendToString(&input);
 
   // Segment into kDigitsInBigRank-digits pieces
   vector<string> ranked_numbers;
@@ -319,22 +342,22 @@ bool NumberUtil::ArabicToKanji(const string &input_num,
       string segment_result;
       bool leading = true;
       for (size_t i = 0; i < segment.size(); ++i) {
-        if (leading && segment[i] == '0') {
+        if (leading && segment[i] == kAsciiZero) {
           continue;
         }
 
         leading = false;
         if (style == NumberString::NUMBER_ARABIC_AND_KANJI_HALFWIDTH ||
             style == NumberString::NUMBER_ARABIC_AND_KANJI_FULLWIDTH) {
-          segment_result += digits[segment[i] - '0'];
+          segment_result += digits[segment[i] - kAsciiZero];
         } else {
-          if (segment[i] == '0') {
+          if (segment[i] == kAsciiZero) {
             continue;
           }
           // In "大字" style, "壱" is also required on every rank.
           if (style == NumberString::NUMBER_OLD_KANJI ||
-              i == kDigitsInBigRank - 1 || segment[i] != '1') {
-            segment_result += digits[segment[i] - '0'];
+              i == kDigitsInBigRank - 1 || segment[i] != kAsciiOne) {
+            segment_result += digits[segment[i] - kAsciiZero];
           }
           segment_result += ranks[kDigitsInBigRank - i];
         }
@@ -389,7 +412,7 @@ const NumberStringVariation kNumDigitsVariations[] = {
 }  // namespace
 
 bool NumberUtil::ArabicToSeparatedArabic(
-    const string &input_num, vector<NumberString> *output) {
+    StringPiece input_num, vector<NumberString> *output) {
   DCHECK(output);
 
   if (!IsDecimalNumber(input_num)) {
@@ -397,16 +420,17 @@ bool NumberUtil::ArabicToSeparatedArabic(
   }
 
   // Separate a number into an integral part and a fractional part.
-  size_t point_pos = input_num.find('.');
-  string integer(input_num, 0, point_pos);
-  // |fraction| has the decimal point with digits in fractional part.
-  string fraction;
-  if (point_pos != string::npos) {
-    fraction.assign(input_num, point_pos, input_num.size() - point_pos);
+  StringPiece::size_type point_pos = input_num.find('.');
+  if (point_pos == StringPiece::npos) {
+    point_pos = input_num.size();
   }
+  const StringPiece integer(input_num, 0, point_pos);
+  // |fraction| has the decimal point with digits in fractional part.
+  const StringPiece fraction(input_num, point_pos,
+                             input_num.size() - point_pos);
 
   // We don't add separator to number whose integral part starts with '0'
-  if (integer[0] == '0') {
+  if (integer[0] == kAsciiZero) {
     return false;
   }
 
@@ -417,12 +441,12 @@ bool NumberUtil::ArabicToSeparatedArabic(
     string result;
 
     // integral part
-    for (size_t j = 0; j < integer.size(); ++j) {
+    for (StringPiece::size_type j = 0; j < integer.size(); ++j) {
       // We don't add separater first
       if (j != 0 && (integer.size() - j) % 3 == 0) {
         result.append(variation.separator);
       }
-      const uint32 d = integer[j] - '0';
+      const uint32 d = static_cast<uint32>(integer[j] - kAsciiZero);
       if (d <= 9 && digits[d]) {
         result.append(digits[d]);
       }
@@ -432,8 +456,8 @@ bool NumberUtil::ArabicToSeparatedArabic(
     if (!fraction.empty()) {
       DCHECK_EQ(fraction[0], '.');
       result.append(variation.point);
-      for (size_t j = 1; j < fraction.size(); ++j) {
-        result.append(digits[static_cast<int>(fraction[j] - '0')]);
+      for (StringPiece::size_type j = 1; j < fraction.size(); ++j) {
+        result.append(digits[static_cast<int>(fraction[j] - kAsciiZero)]);
       }
     }
 
@@ -450,16 +474,16 @@ namespace {
 const NumberStringVariation kSingleDigitsVariations[] = {
   // "漢数字"
   {kNumKanjiDigits, 10, "\xE6\xBC\xA2\xE6\x95\xB0\xE5\xAD\x97",
-   NULL, NULL, NumberUtil::NumberString::NUMBER_KANJI_ARABIC},
+   nullptr, nullptr, NumberUtil::NumberString::NUMBER_KANJI_ARABIC},
   // "数字"
   {kNumFullWidthDigits, 10, "\xE6\x95\xB0\xE5\xAD\x97",
-   NULL, NULL, NumberUtil::NumberString::DEFAULT_STYLE},
+   nullptr, nullptr, NumberUtil::NumberString::DEFAULT_STYLE},
 };
 
 }  // namespace
 
 bool NumberUtil::ArabicToWideArabic(
-    const string &input_num, vector<NumberString> *output) {
+    StringPiece input_num, vector<NumberString> *output) {
   DCHECK(output);
 
   if (!IsDecimalInteger(input_num)) {
@@ -470,8 +494,9 @@ bool NumberUtil::ArabicToWideArabic(
     const NumberStringVariation &variation = kSingleDigitsVariations[i];
     // TODO(peria): Bring |result| out if it improves the performance.
     string result;
-    for (size_t j = 0; j < input_num.size(); ++j) {
-      result.append(variation.digits[static_cast<int>(input_num[j] - '0')]);
+    for (StringPiece::size_type j = 0; j < input_num.size(); ++j) {
+      result.append(
+          variation.digits[static_cast<int>(input_num[j] - kAsciiZero)]);
     }
     if (!result.empty()) {
       output->push_back(
@@ -488,22 +513,22 @@ const NumberStringVariation kSpecialNumericVariations[] = {
    // "ローマ数字(大文字)",
    "\xE3\x83\xAD\xE3\x83\xBC\xE3\x83\x9E\xE6\x95\xB0"
    "\xE5\xAD\x97(\xE5\xA4\xA7\xE6\x96\x87\xE5\xAD\x97)",
-   NULL, NULL, NumberUtil::NumberString::NUMBER_ROMAN_CAPITAL},
+   nullptr, nullptr, NumberUtil::NumberString::NUMBER_ROMAN_CAPITAL},
   {kRomanNumbersSmall, arraysize(kRomanNumbersSmall),
    // "ローマ数字(小文字)",
    "\xE3\x83\xAD\xE3\x83\xBC\xE3\x83\x9E\xE6\x95\xB0"
    "\xE5\xAD\x97(\xE5\xB0\x8F\xE6\x96\x87\xE5\xAD\x97)",
-   NULL, NULL, NumberUtil::NumberString::NUMBER_ROMAN_SMALL},
+   nullptr, nullptr, NumberUtil::NumberString::NUMBER_ROMAN_SMALL},
   {kCircledNumbers, arraysize(kCircledNumbers),
    // "丸数字"
    "\xE4\xB8\xB8\xE6\x95\xB0\xE5\xAD\x97",
-   NULL, NULL, NumberUtil::NumberString::NUMBER_CIRCLED},
+   nullptr, nullptr, NumberUtil::NumberString::NUMBER_CIRCLED},
 };
 
 }  // namespace
 
 bool NumberUtil::ArabicToOtherForms(
-    const string &input_num, vector<NumberString> *output) {
+    StringPiece input_num, vector<NumberString> *output) {
   DCHECK(output);
 
   if (!IsDecimalInteger(input_num)) {
@@ -555,7 +580,7 @@ const int kMaxInt64Size = 24;
 }  // namespace
 
 bool NumberUtil::ArabicToOtherRadixes(
-    const string &input_num, vector<NumberString> *output) {
+    StringPiece input_num, vector<NumberString> *output) {
   DCHECK(output);
 
   if (!IsDecimalInteger(input_num)) {
@@ -590,7 +615,7 @@ bool NumberUtil::ArabicToOtherRadixes(
   if (n > 1) {
     string binary;
     for (uint64 num = n; num; num >>= 1) {
-      binary.push_back('0' + static_cast<char>(num & 0x1));
+      binary.push_back(kAsciiZero + static_cast<char>(num & 0x1));
     }
     // "b0" will be "0b" in head of |binary|
     binary.append("b0");
@@ -605,16 +630,13 @@ bool NumberUtil::ArabicToOtherRadixes(
 
 namespace {
 
-const char *SkipWhiteSpace(const char *ptr) {
-  while (isspace(*ptr)) {
-    ++ptr;
-  }
-  return ptr;
+const StringPiece SkipWhiteSpace(StringPiece str) {
+  StringPiece::size_type i;
+  for (i = 0; i < str.size() && isspace(str[i]); ++i) {}
+  DCHECK(i == str.size() || !isspace(str[i]));
+  return StringPiece(str, i);
 }
 
-// TODO(yukawa): this should be moved into ports.h
-const uint64 kUInt64Max = 0xFFFFFFFFFFFFFFFFull;
-
 // There is an informative discussion about the overflow detection in
 // "Hacker's Delight" (http://www.hackersdelight.org/basics.pdf)
 //   2-12 'Overflow Detection'
@@ -623,7 +645,7 @@ const uint64 kUInt64Max = 0xFFFFFFFFFFFFFFFFull;
 // return false when an integer overflow happens.
 bool AddAndCheckOverflow(uint64 arg1, uint64 arg2, uint64 *output) {
   *output = arg1 + arg2;
-  if (arg2 > (kUInt64Max - arg1)) {
+  if (arg2 > (kuint64max - arg1)) {
     // overflow happens
     return false;
   }
@@ -634,98 +656,211 @@ bool AddAndCheckOverflow(uint64 arg1, uint64 arg2, uint64 *output) {
 // return false when an integer overflow happens.
 bool MultiplyAndCheckOverflow(uint64 arg1, uint64 arg2, uint64 *output) {
   *output = arg1 * arg2;
-  if (arg1 != 0 && arg2 > (kUInt64Max / arg1)) {
+  if (arg1 != 0 && arg2 > (kuint64max / arg1)) {
     // overflow happens
     return false;
   }
   return true;
 }
 
+// A simple wrapper of strtoull function. |c_str| must be terminated by '\0'.
+inline uint64 StrToUint64(const char* c_str, char** end_ptr, int base) {
+#ifdef OS_WIN
+  return _strtoui64(c_str, end_ptr, base);
+#else  // OS_WIN
+  return strtoull(c_str, end_ptr, base);
+#endif  // OS_WIN
+}
+
 // Converts a string which describes a number into an uint64 value in |base|
 // radix.  Does not convert octal or hexadecimal strings with "0" or "0x"
 // suffixes.
-bool SafeStrToUInt64WithBase(const string &str, int base, uint64 *value) {
+bool SafeStrToUInt64WithBase(StringPiece str, int base, uint64 *value) {
   DCHECK(value);
 
-  const char *ptr = SkipWhiteSpace(str.c_str());
-  // strtoull() does not check if the input is negative.
-  if (*ptr == '-') {
+  // Maximum possible length of number string, including terminating '\0'. Note
+  // that the maximum possible length is achieved when str="111...11" (64
+  // unities) and base=2.
+  const size_t kMaxPossibleLength = 65;
+
+  // Leading white spaces are allowed.
+  const StringPiece stripped_str = SkipWhiteSpace(str);
+  if (stripped_str.empty() || stripped_str.size() >= kMaxPossibleLength) {
+    return false;
+  }
+  // StrToUint64() does not check if the input is negative.  However, a leading
+  // '+' is OK.
+  if (stripped_str[0] == '-') {
     return false;
   }
 
-  char *end_ptr;
+  // Since StringPiece doesn't end with '\0', we make a c-string on stack here.
+  char buf[kMaxPossibleLength];
+  memcpy(buf, str.data(), str.size());
+  buf[str.size()] = '\0';
+
+  char *end_ptr = nullptr;
   errno = 0;
-#ifdef OS_WINDOWS
-  *value = _strtoui64(ptr, &end_ptr, base);
-#else  // OS_WINDOWS
-  *value = strtoull(ptr, &end_ptr, base);
-#endif  // OS_WINDOWS
-  if (errno) {
+  *value = StrToUint64(buf, &end_ptr, base);
+  if (errno != 0 || end_ptr == buf) {  // Failed to parse uint64.
     return false;
   }
-  if (ptr == end_ptr) {
+  // Trailing white spaces are allowed.
+  const StringPiece trailing_str(end_ptr, buf + str.size() - end_ptr);
+  return SkipWhiteSpace(trailing_str).empty();
+}
+
+template <typename T1, typename T2>
+struct GenericFalseTypeArity2 {
+  // TODO(yukawa): Use std::false_type once C++11 is enabled everywhere.
+  static const bool value = false;
+};
+
+template <typename SrcType, typename DestType>
+bool SafeCast(SrcType src, DestType *dest) {
+  static_assert(GenericFalseTypeArity2<SrcType, DestType>::value,
+                "Shouldn't be used with implicit type conversion.");
+  return false;
+}
+
+template <>
+bool SafeCast(int64 src, int32 *dest) {
+  if (src < static_cast<int64>(kint32min) ||
+      static_cast<int64>(kint32max) < src) {
     return false;
   }
-  return *SkipWhiteSpace(end_ptr) == '\0';
+  *dest = static_cast<int32>(src);
+  return true;
+}
+
+template <>
+bool SafeCast(uint64 src, int64 *dest) {
+  if (src > static_cast<uint64>(kint64max)) {
+    return false;
+  }
+  *dest = static_cast<int64>(src);
+  return true;
+}
+
+template <>
+bool SafeCast(uint64 src, uint32 *dest) {
+  if (src > static_cast<uint64>(kuint32max)) {
+    return false;
+  }
+  *dest = static_cast<uint32>(src);
+  return true;
+}
+
+template <typename SrcType, typename DestType>
+bool SafeUnaryNegation(SrcType src, DestType *dest) {
+  static_assert(GenericFalseTypeArity2<SrcType, DestType>::value,
+                "Shouldn't be used with implicit type conversion.");
+  return false;
+}
+
+template <>
+bool SafeUnaryNegation(uint64 src, int64 *dest) {
+  int64 tmp = 0;
+  if (!SafeCast(src, &tmp)) {
+    if (src == 0x8000000000000000ul) {
+      // This is an exceptional case. |src| isn't in the range of int64,
+      // but |-src| is in the range.
+      *dest = kint64min;
+      return true;
+    }
+    return false;
+  }
+  *dest = -tmp;
+  return true;
 }
 
 }  // namespace
 
-bool NumberUtil::SafeStrToUInt32(const string &str, uint32 *value) {
+bool NumberUtil::SafeStrToInt32(StringPiece str, int32 *value) {
+  int64 tmp;
+  if (!SafeStrToInt64(str, &tmp)) {
+    return false;
+  }
+  return SafeCast(tmp, value);
+}
+
+bool NumberUtil::SafeStrToInt64(StringPiece str, int64 *value) {
+  const StringPiece stripped_str = SkipWhiteSpace(str);
+  if (stripped_str.empty()) {
+    return false;
+  }
+  uint64 tmp;
+  if (stripped_str[0] == '-') {
+    StringPiece opposite_str = StringPiece(stripped_str,
+                                           1,
+                                           stripped_str.size() - 1);
+    if (!SafeStrToUInt64WithBase(opposite_str, 10, &tmp)) {
+      return false;
+    }
+    return SafeUnaryNegation(tmp, value);
+  }
+  if (!SafeStrToUInt64WithBase(str, 10, &tmp)) {
+    return false;
+  }
+  return SafeCast(tmp, value);
+}
+
+bool NumberUtil::SafeStrToUInt32(StringPiece str, uint32 *value) {
   uint64 tmp;
   if (!SafeStrToUInt64WithBase(str, 10, &tmp)) {
     return false;
   }
-  *value = tmp;
-  return static_cast<uint64>(*value) == tmp;
+  return SafeCast(tmp, value);
 }
 
-bool NumberUtil::SafeHexStrToUInt32(const string &str, uint32 *value) {
+bool NumberUtil::SafeHexStrToUInt32(StringPiece str, uint32 *value) {
   uint64 tmp;
   if (!SafeStrToUInt64WithBase(str, 16, &tmp)) {
     return false;
   }
-  *value = tmp;
-  return static_cast<uint64>(*value) == tmp;
+  return SafeCast(tmp, value);
 }
 
-bool NumberUtil::SafeOctStrToUInt32(const string &str, uint32 *value) {
+bool NumberUtil::SafeOctStrToUInt32(StringPiece str, uint32 *value) {
   uint64 tmp;
   if (!SafeStrToUInt64WithBase(str, 8, &tmp)) {
     return false;
   }
-  *value = tmp;
-  return static_cast<uint64>(*value) == tmp;
+  return SafeCast(tmp, value);
 }
 
-bool NumberUtil::SafeStrToUInt64(const string &str, uint64 *value) {
+bool NumberUtil::SafeStrToUInt64(StringPiece str, uint64 *value) {
   return SafeStrToUInt64WithBase(str, 10, value);
 }
 
-bool NumberUtil::SafeStrToDouble(const string &str, double *value) {
+bool NumberUtil::SafeStrToDouble(StringPiece str, double *value) {
   DCHECK(value);
-
-  const char *ptr = SkipWhiteSpace(str.c_str());
+  // Note that StringPiece isn't terminated by '\0'.  However, since strtod
+  // requires null-terminated string, we make a string here. If we have a good
+  // estimate of the maximum possible length of the input string, we may be able
+  // to use char buffer instead.  Note: const reference ensures the life of this
+  // temporary string until the end!
+  const string &s = str.as_string();
+  const char* ptr = s.c_str();
 
   char *end_ptr;
   errno = 0;  // errno only gets set on errors
   // strtod of GCC accepts hexadecimal number like "0x1234", but that of
   // VisualC++ does not.
+  // Note that strtod accepts white spaces at the beginning of the parameter.
   *value = strtod(ptr, &end_ptr);
-  if (errno) {
+  if (errno != 0 ||
+      ptr == end_ptr ||
+      *value ==  numeric_limits<double>::infinity() ||
+      *value == -numeric_limits<double>::infinity()) {
     return false;
   }
-  if ((*value ==  numeric_limits<double>::infinity()) ||
-      (*value == -numeric_limits<double>::infinity())) {
-    return false;
-  }
-  if (ptr == end_ptr) {
-    return false;
-  }
-  return *SkipWhiteSpace(end_ptr) == '\0';
+  // Trailing white spaces are allowed.
+  const StringPiece trailing_str(end_ptr, ptr + s.size() - end_ptr);
+  return SkipWhiteSpace(trailing_str).empty();
 }
 
-bool NumberUtil::SafeStrToFloat(const string &str, float *value) {
+bool NumberUtil::SafeStrToFloat(StringPiece str, float *value) {
   double double_value;
   if (!SafeStrToDouble(str, &double_value)) {
     return false;
@@ -768,7 +903,7 @@ bool ReduceLeadingNumbersAsBase10System(
 // Returns false if a number greater than 10 was found or overflow happened.
 bool InterpretNumbersAsBase10System(const vector<uint64> &numbers,
                                     uint64 *output) {
-  vector<uint64>::const_iterator begin = numbers.begin();
+  auto begin = numbers.begin();
   const bool success =
       ReduceLeadingNumbersAsBase10System(&begin, numbers.end(), output);
   // Check if the whole numbers were reduced.
@@ -920,8 +1055,8 @@ bool ReduceNumberLessThan10000(vector<uint64>::const_iterator *begin,
 //   "一十二百" = [1, 10, 2, 100] => error
 bool InterpretNumbersInJapaneseWay(const vector<uint64> &numbers,
                                    uint64 *output) {
-  uint64 last_base = kUInt64Max;
-  vector<uint64>::const_iterator begin = numbers.begin();
+  uint64 last_base = kuint64max;
+  auto begin = numbers.begin();
   *output = 0;
   do {
     uint64 coef = 0;
@@ -950,8 +1085,7 @@ bool InterpretNumbersInJapaneseWay(const vector<uint64> &numbers,
 // depending on the maximum number in the sequence.
 bool NormalizeNumbersHelper(const vector<uint64> &numbers,
                             uint64 *number_output) {
-  const vector<uint64>::const_iterator itr_max = max_element(numbers.begin(),
-                                                             numbers.end());
+  const auto itr_max = max_element(numbers.begin(), numbers.end());
   if (itr_max == numbers.end()) {
     return false;  // numbers is empty
   }
@@ -965,7 +1099,7 @@ bool NormalizeNumbersHelper(const vector<uint64> &numbers,
 }
 
 // TODO(peria): Do refactoring this method.
-bool NormalizeNumbersInternal(const string &input,
+bool NormalizeNumbersInternal(StringPiece input,
                               bool trim_leading_zeros,
                               bool allow_suffix,
                               string *kanji_output,
@@ -1036,7 +1170,7 @@ bool NormalizeNumbersInternal(const string &input,
     if (num_zeros == numbers.size()) {
       --num_zeros;
     }
-    arabic_output->append(num_zeros, '0');
+    arabic_output->append(num_zeros, kAsciiZero);
   }
 
   char buf[kMaxInt64Size];
@@ -1049,7 +1183,7 @@ bool NormalizeNumbersInternal(const string &input,
 
 // Convert Kanji numbers into Arabic numbers:
 // e.g. "百二十万" -> 1200000
-bool NumberUtil::NormalizeNumbers(const string &input,
+bool NumberUtil::NormalizeNumbers(StringPiece input,
                                   bool trim_leading_zeros,
                                   string *kanji_output,
                                   string *arabic_output) {
@@ -1058,10 +1192,10 @@ bool NumberUtil::NormalizeNumbers(const string &input,
                                   false,  // allow_suffix
                                   kanji_output,
                                   arabic_output,
-                                  NULL);
+                                  nullptr);
 }
 
-bool NumberUtil::NormalizeNumbersWithSuffix(const string &input,
+bool NumberUtil::NormalizeNumbersWithSuffix(StringPiece input,
                                             bool trim_leading_zeros,
                                             string *kanji_output,
                                             string *arabic_output,
@@ -1083,7 +1217,7 @@ namespace {
 
 }  // namespace
 
-void NumberUtil::KanjiNumberToArabicNumber(const string &input,
+void NumberUtil::KanjiNumberToArabicNumber(StringPiece input,
                                            string *output) {
   TextConverter::Convert(kanjinumber_to_arabicnumber_da,
                          kanjinumber_to_arabicnumber_table,
