@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,8 @@
 
 #include <functional>
 
+#include "base/port.h"
+
 namespace mozc {
 
 template <class ForwardIterator>
@@ -53,6 +55,17 @@ void STLDeleteElements(T *container) {
   container->clear();
 }
 
+template <typename C>
+class ScopedElementsDeleter {
+ public:
+  explicit ScopedElementsDeleter(C *container) : container_(container) {}
+  ~ScopedElementsDeleter() { STLDeleteElements(container_); }
+
+ private:
+  C *container_;
+  DISALLOW_COPY_AND_ASSIGN(ScopedElementsDeleter);
+};
+
 // Comparator functor based of a member.
 // Key: a function object to return key from the given value.
 // Comparator: a base comparator function object.
