@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,26 +31,26 @@
 
 #include "client/client.h"
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <Windows.h>
 #include <ShellAPI.h>
 #else
-#include <sys/types.h>
 #include <unistd.h>
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #include <cstddef>
 
-#include "base/base.h"
 #include "base/const.h"
-#include "base/crash_report_util.h"
 #include "base/file_stream.h"
+#include "base/file_util.h"
 #include "base/logging.h"
 #include "base/process.h"
 #include "base/run_level.h"
 #include "base/singleton.h"
+#include "base/system_util.h"
 #include "base/util.h"
 #include "base/version.h"
+#include "config/config.pb.h"
 #include "ipc/ipc.h"
 #include "session/commands.pb.h"
 
@@ -85,7 +85,6 @@ Client::Client()
     : id_(0),
       server_launcher_(new ServerLauncher),
       result_(new char[kResultBufferSize]),
-      preferences_(NULL),
       timeout_(kDefaultTimeout),
       server_status_(SERVER_UNKNOWN),
       server_protocol_version_(0),
@@ -192,8 +191,8 @@ void Client::DumpQueryOfDeath() {
 
 void Client::DumpHistorySnapshot(const string &filename,
                                   const string &label) const {
-  const string snapshot_file = Util::JoinPath(Util::GetUserProfileDirectory(),
-                                              filename);
+  const string snapshot_file =
+      FileUtil::JoinPath(SystemUtil::GetUserProfileDirectory(), filename);
   // open with append mode
   OutputFileStream output(snapshot_file.c_str(), ios::app);
 
@@ -254,11 +253,13 @@ void Client::PushHistory(const commands::Input &input,
 // Clear the history and push IMEOn command for initialize session.
 void Client::ResetHistory() {
   history_inputs_.clear();
-#if defined(OS_WINDOWS) || defined(OS_MACOSX)
-  // On Windows/Mac, we should send ON key at the first of each input session
+#if defined(OS_MACOSX)
+  // On Mac, we should send ON key at the first of each input session
   // excepting the very first session, because when the session is restored,
   // its state is direct. On the first session, users should send ON key
   // by themselves.
+  // On Windows, this is not required because now we can send IME On/Off
+  // state with the key event. See b/8601275
   // Note that we are assuming that ResetHistory is called only when the
   // client is ON.
   // TODO(toshiyuki): Make sure that this assuming is reasonable or not.
@@ -279,37 +280,42 @@ void Client::GetHistoryInputs(vector<commands::Input> *output) const {
   }
 }
 
-bool Client::SendKey(const commands::KeyEvent &key,
-                      commands::Output *output) {
-#ifdef DEBUG
-  if (IsAbortKey(key)) {
-    DCHECK(CrashReportUtil::Abort()) << "Not aborted by CrashReportUtil::Abort";
-  }
-#endif  // DEBUG
+bool Client::SendKeyWithContext(const commands::KeyEvent &key,
+                                const commands::Context &context,
+                                commands::Output *output) {
   commands::Input input;
   input.set_type(commands::Input::SEND_KEY);
   input.mutable_key()->CopyFrom(key);
+  // If the pointer of |context| is not the default_instance, update the data.
+  if (&context != &commands::Context::default_instance()) {
+    input.mutable_context()->CopyFrom(context);
+  }
   return EnsureCallCommand(&input, output);
 }
 
-bool Client::TestSendKey(const commands::KeyEvent &key,
-                          commands::Output *output) {
-#ifdef DEBUG
-  if (IsAbortKey(key)) {
-    DCHECK(CrashReportUtil::Abort()) << "Not aborted by CrashReportUtil::Abort";
-  }
-#endif  // DEBUG
+bool Client::TestSendKeyWithContext(const commands::KeyEvent &key,
+                                    const commands::Context &context,
+                                    commands::Output *output) {
   commands::Input input;
   input.set_type(commands::Input::TEST_SEND_KEY);
+  // If the pointer of |context| is not the default_instance, update the data.
+  if (&context != &commands::Context::default_instance()) {
+    input.mutable_context()->CopyFrom(context);
+  }
   input.mutable_key()->CopyFrom(key);
   return EnsureCallCommand(&input, output);
 }
 
-bool Client::SendCommand(const commands::SessionCommand &command,
-                          commands::Output *output) {
+bool Client::SendCommandWithContext(const commands::SessionCommand &command,
+                                    const commands::Context &context,
+                                    commands::Output *output) {
   commands::Input input;
   input.set_type(commands::Input::SEND_COMMAND);
   input.mutable_command()->CopyFrom(command);
+  // If the pointer of |context| is not the default_instance, update the data.
+  if (&context != &commands::Context::default_instance()) {
+    input.mutable_context()->CopyFrom(context);
+  }
   return EnsureCallCommand(&input, output);
 }
 
@@ -421,7 +427,7 @@ bool Client::CreateSession() {
   commands::ApplicationInfo *info = input.mutable_application_info();
   DCHECK(info);
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   info->set_process_id(static_cast<uint32>(::GetCurrentProcessId()));
   info->set_thread_id(static_cast<uint32>(::GetCurrentThreadId()));
 #else
@@ -575,48 +581,6 @@ bool Client::PingServer() const {
   return true;
 }
 
-bool Client::StartCloudSync() {
-  return CallCommand(commands::Input::START_CLOUD_SYNC);
-}
-
-bool Client::ClearCloudSync() {
-  return CallCommand(commands::Input::CLEAR_CLOUD_SYNC);
-}
-
-bool Client::GetCloudSyncStatus(commands::CloudSyncStatus *cloud_sync_status) {
-  DCHECK(cloud_sync_status);
-  commands::Input input;
-  InitInput(&input);
-  input.set_type(commands::Input::GET_CLOUD_SYNC_STATUS);
-  commands::Output output;
-  if (!Call(input, &output)) {
-    LOG(ERROR) << "Call failed.  Server is not started.";
-    return false;
-  }
-
-  if (!output.has_cloud_sync_status()) {
-    LOG(ERROR) << "No CloudSyncStatus.  Server may be stale.";
-    return false;
-  }
-
-  cloud_sync_status->CopyFrom(output.cloud_sync_status());
-  return true;
-}
-
-bool Client::AddAuthCode(const commands::Input::AuthorizationInfo &auth_info) {
-  commands::Input input;
-  InitInput(&input);
-  input.set_type(commands::Input::ADD_AUTH_CODE);
-  input.mutable_auth_code()->CopyFrom(auth_info);
-  commands::Output output;
-  if (!Call(input, &output)) {
-    LOG(ERROR) << "Call failed.  Server is not started.";
-    return false;
-  }
-
-  return true;
-}
-
 bool Client::CallCommand(commands::Input::CommandType type) {
   commands::Input input;
   InitInput(&input);
@@ -841,16 +805,6 @@ void Client::Reset() {
   server_process_id_ = 0;
 }
 
-bool Client::IsAbortKey(const commands::KeyEvent &key) {
-  return key.has_special_key() &&
-      key.special_key() == commands::KeyEvent::F12 &&
-      key.modifier_keys().size() == 2 &&
-      ((key.modifier_keys(0) == commands::KeyEvent::CTRL &&
-        key.modifier_keys(1) == commands::KeyEvent::ALT) ||
-       (key.modifier_keys(0) == commands::KeyEvent::ALT &&
-        key.modifier_keys(1) == commands::KeyEvent::CTRL));
-}
-
 bool Client::TranslateProtoBufToMozcToolArg(const commands::Output &output,
                                              string *mode) {
   if (!output.has_launch_tool_mode() || mode == NULL) {
@@ -903,8 +857,8 @@ bool Client::LaunchTool(const string &mode, const string &extra_arg) {
   }
 
   if (mode == "administration_dialog") {
-#ifdef OS_WINDOWS
-    const string &path = mozc::Util::GetToolPath();
+#ifdef OS_WIN
+    const string &path = mozc::SystemUtil::GetToolPath();
     wstring wpath;
     Util::UTF8ToWide(path.c_str(), &wpath);
     wpath = L"\"" + wpath + L"\"";
@@ -919,22 +873,22 @@ bool Client::LaunchTool(const string &mode, const string &extra_arg) {
     // http://b/2415191
     const int result =
         reinterpret_cast<int>(::ShellExecute(0,
-                                             mozc::Util::IsVistaOrLater() ?
+                                             SystemUtil::IsVistaOrLater() ?
                                              L"runas" : L"open",
                                              wpath.c_str(),
                                              L"--mode=administration_dialog",
-                                             mozc::Util::GetSystemDir(),
+                                             SystemUtil::GetSystemDir(),
                                              SW_SHOW));
     if (result <= 32) {
       LOG(ERROR) << "::ShellExecute failed: " << result;
       return false;
     }
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
     return false;
   }
 
-#if defined(OS_WINDOWS) || defined(OS_LINUX)
+#if defined(OS_WIN) || defined(OS_LINUX)
   string arg = "--mode=" + mode;
   if (!extra_arg.empty()) {
     arg += " ";
@@ -944,7 +898,7 @@ bool Client::LaunchTool(const string &mode, const string &extra_arg) {
     LOG(ERROR) << "Cannot execute: " << kMozcTool << " " << arg;
     return false;
   }
-#endif  // OS_WINDOWS || OS_LINUX
+#endif  // OS_WIN || OS_LINUX
 
   // TODO(taku): move MacProcess inside SpawnMozcProcess.
   // TODO(taku): support extra_arg.
