@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,8 @@
 
 #include <string>
 #include <vector>
-#include "base/base.h"
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "client/client_interface.h"
 #include "session/commands.pb.h"
 #include "testing/base/public/gunit_prod.h"
@@ -110,12 +111,15 @@ class Client : public ClientInterface {
 
   bool CheckVersionOrRestartServer();
 
-  bool SendKey(const commands::KeyEvent &key,
-               commands::Output *output);
-  bool TestSendKey(const commands::KeyEvent &key,
-                   commands::Output *output);
-  bool SendCommand(const commands::SessionCommand &command,
-                   commands::Output *output);
+  bool SendKeyWithContext(const commands::KeyEvent &key,
+                          const commands::Context &context,
+                          commands::Output *output);
+  bool TestSendKeyWithContext(const commands::KeyEvent &key,
+                              const commands::Context &context,
+                              commands::Output *output);
+  bool SendCommandWithContext(const commands::SessionCommand &command,
+                              const commands::Context &context,
+                              commands::Output *output);
 
   bool GetConfig(config::Config *config);
   bool SetConfig(const config::Config &config);
@@ -131,13 +135,6 @@ class Client : public ClientInterface {
   bool NoOperation();
   bool PingServer() const;
 
-  // these methods are expected to be called from
-  // the converter process.
-  bool StartCloudSync();
-  bool ClearCloudSync();
-  bool GetCloudSyncStatus(commands::CloudSyncStatus *cloud_sync_status);
-  bool AddAuthCode(const commands::Input::AuthorizationInfo &auth_info);
-
   void Reset();
 
   void EnableCascadingWindow(bool enable);
@@ -225,9 +222,6 @@ class Client : public ClientInterface {
   void PushHistory(const commands::Input &input,
                    const commands::Output &output);
   void ResetHistory();
-  // Returns true if the |key| corresponds to the key combination for aborting
-  // the application.
-  bool IsAbortKey(const commands::KeyEvent &key);
 
   // The alias of
   // DumpHistorySnapshot("query_of_death.log", "QUERY OF DEATH");
@@ -249,9 +243,8 @@ class Client : public ClientInterface {
   uint64 id_;
   IPCClientFactoryInterface *client_factory_;
   scoped_ptr<ServerLauncherInterface> server_launcher_;
-  scoped_array<char> result_;
+  scoped_ptr<char[]> result_;
   scoped_ptr<config::Config> preferences_;
-  bool use_cascading_window_;
   int timeout_;
   ServerStatus server_status_;
   uint32 server_protocol_version_;
