@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -36,24 +36,27 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
-#include "transliteration/transliteration.h"
+#include "base/port.h"
+#include "base/protobuf/repeated_field.h"
+#include "base/scoped_ptr.h"
+#include "composer/internal/typing_corrector.h"
+#include "composer/internal/transliterators.h"
+#include "composer/type_corrected_query.h"
 #include "session/commands.pb.h"
+#include "transliteration/transliteration.h"
 // for FRIEND_TEST()
 #include "testing/base/public/gunit_prod.h"
 
-namespace mozc {
-namespace commands {
-class Context;
-class KeyEvent;
-class Request;
-}  // namespace commands
 
+namespace mozc {
 namespace composer {
 
 class CompositionInterface;
 class Table;
 
+typedef commands::KeyEvent::ProbableKeyEvent ProbableKeyEvent;
+typedef mozc::protobuf::RepeatedPtrField<ProbableKeyEvent> ProbableKeyEvents;
+
 class Composer {
  public:
   // Pseudo commands in composer.
@@ -61,7 +64,7 @@ class Composer {
     REWIND,
   };
 
-  Composer(const Table *table, const commands::Request &request);
+  Composer(const Table *table, const commands::Request *request);
   virtual ~Composer();
 
   // Reset all composing data except table.
@@ -81,12 +84,12 @@ class Composer {
 
   void SetTable(const Table *table);
 
-  void SetRequest(const commands::Request &request);
+  void SetRequest(const commands::Request *request);
 
   void SetInputMode(transliteration::TransliterationType mode);
   void SetTemporaryInputMode(transliteration::TransliterationType mode);
-  void SetInputFieldType(commands::SessionCommand::InputFieldType type);
-  commands::SessionCommand::InputFieldType GetInputFieldType() const;
+  void SetInputFieldType(commands::Context::InputFieldType type);
+  commands::Context::InputFieldType GetInputFieldType() const;
 
   // Update the input mode considering the input modes of the
   // surrounding characters.
@@ -101,25 +104,29 @@ class Composer {
   transliteration::TransliterationType GetOutputMode() const;
   void SetOutputMode(transliteration::TransliterationType mode);
 
-  // Return preedit strings
+  // Returns preedit strings
   void GetPreedit(string *left, string *focused, string *right) const;
 
-  // Return a preedit string with user's preferences.
+  // Returns a preedit string with user's preferences.
   void GetStringForPreedit(string *output) const;
 
-  // Return a submit string with user's preferences.  The difference
+  // Returns a submit string with user's preferences.  The difference
   // from the preedit string is the handling of the last 'n'.
   void GetStringForSubmission(string *output) const;
 
-  // Return a conversion query normalized ascii characters in half width
+  // Returns a conversion query normalized ascii characters in half width
   void GetQueryForConversion(string *output) const;
 
-  // Return a prediction query trimmed the tail alphabet characters.
+  // Returns a prediction query trimmed the tail alphabet characters.
   void GetQueryForPrediction(string *output) const;
 
-  // Return a expanded prediction query.
+  // Returns a expanded prediction query.
   void GetQueriesForPrediction(string *base, set<string> *expanded) const;
 
+  // Returns a type-corrected prediction queries.
+  void GetTypeCorrectedQueriesForPrediction(
+      vector<TypeCorrectedQuery> *queries) const;
+
   size_t GetLength() const;
   size_t GetCursor() const;
   void EditErase();
@@ -131,11 +138,17 @@ class Composer {
 
   void InsertCharacter(const string &input);
   void InsertCharacterPreedit(const string &input);
-  void InsertCharacterKeyAndPreedit(const string &key, const string &preedit);
-  void InsertCharacterPreeditAt(size_t pos, const string &input);
-  void InsertCharacterKeyAndPreeditAt(size_t pos,
-                                      const string &key,
-                                      const string &preedit);
+  bool InsertCharacterKeyAndPreedit(const string &key, const string &preedit);
+  void InsertCharacterForProbableKeyEvents(
+      const string &input,
+      const ProbableKeyEvents &probable_key_events);
+  void InsertCharacterPreeditForProbableKeyEvents(
+      const string &input,
+      const ProbableKeyEvents &probable_key_events);
+  void InsertCharacterKeyAndPreeditForProbableKeyEvents(
+      const string &key,
+      const string &preedit,
+      const ProbableKeyEvents &probable_key_events);
   bool InsertCharacterKeyEvent(const commands::KeyEvent &key);
   void InsertCommandCharacter(const InternalCommand internal_command);
   void Delete();
@@ -148,6 +161,17 @@ class Composer {
   void MoveCursorToEnd();
   void MoveCursorTo(uint32 new_position);
 
+  // Returns raw input from a user.
+  // The main purpose is Transliteration.
+  void GetRawString(string *raw_string) const;
+
+  // Returns substring of raw input.  The position and size is based on the
+  // composed string.  For example, when [さ|sa][し|shi][み|mi] is the
+  // composition, GetRawSubString(0, 2) returns "sashi".
+  void GetRawSubString(const size_t position,
+                       const size_t size,
+                       string *raw_sub_string) const;
+
   // Generate transliterations.
   void GetTransliterations(transliteration::Transliterations *t13ns) const;
 
@@ -169,12 +193,6 @@ class Composer {
   // status and user's settings.
   void AutoSwitchMode();
 
-  // Insert preceding text if |context| meets some condition.  It returns the
-  // number of characters at the end of context.preceding_text() which are
-  // involved to composition and must be deleted from preceding text.
-  // NOTE: Do not call this function if client cannot delete preceding text.
-  size_t InsertPrecedingText(const commands::Context &context);
-
   // Return true if the composition is adviced to be committed immediately.
   bool ShouldCommit() const;
 
@@ -210,6 +228,10 @@ class Composer {
  private:
   FRIEND_TEST(ComposerTest, ApplyTemporaryInputMode);
 
+  bool InsertCharacterInternal(const string &input);
+  bool InsertCharacterKeyAndPreeditInternal(const string &key,
+                                            const string &preedit);
+
   // Change input mode temporarily accoding to the current context and
   // the given input character.
   // This function have a bug when key has characters input with Preedit.
@@ -217,6 +239,12 @@ class Composer {
   // Actual behavior:   InsertPreedit("A") + InsertKey("a") -> "Aa"
   void ApplyTemporaryInputMode(const string &key, bool caps_locked);
 
+  // Generate transliterated substrings.
+  void GetTransliteratedText(Transliterators::Transliterator transliterator,
+                             const size_t position,
+                             const size_t size,
+                             string *output) const;
+
   size_t position_;
   // Whether the next insertion is the beginning of typing after an
   // editing command like SetInputMode or not.  Some conversion rules
@@ -229,18 +257,20 @@ class Composer {
   // On reset, comeback_input_mode_ is used as the input mode.
   transliteration::TransliterationType comeback_input_mode_;
   // Type of the input field to input texts.
-  commands::SessionCommand::InputFieldType input_field_type_;
+  commands::Context::InputFieldType input_field_type_;
 
   size_t shifted_sequence_count_;
   scoped_ptr<CompositionInterface> composition_;
 
+  TypingCorrector typing_corrector_;
+
   // The original text for the composition.  The value is usually
   // empty, and used for reverse conversion.
   string source_text_;
 
   size_t max_length_;
 
-  commands::Request request_;
+  const commands::Request *request_;
 
   DISALLOW_COPY_AND_ASSIGN(Composer);
 };
