@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,8 @@
 
 #include "composer/internal/converter.h"
 
-#include "base/base.h"
+#include <string>
+
 #include "composer/table.h"
 #include "testing/base/public/gunit.h"
 
@@ -62,8 +63,8 @@ static void InitTable(mozc::composer::Table* table) {
 
 TEST(ConverterTest, Converter) {
   static const struct TestCase {
-    const char* input;
-    const char* expected_output;
+    const char *input;
+    const char *expected_output;
   } test_cases[] = {
     // "あ"
     { "a", "\xE3\x81\x82" },
@@ -89,14 +90,14 @@ TEST(ConverterTest, Converter) {
     { "nannkanain", "\xE3\x81\xAA\xE3\x82\x93\xE3\x81\x8B\xE3\x81\xAA\xE3\x81"
                     "\x84\xE3\x82\x93" },
   };
-  static const int size = ARRAYSIZE_UNSAFE(test_cases);
+  static const int size = arraysize(test_cases);
 
   mozc::composer::Table table;
   InitTable(&table);
   mozc::composer::Converter converter(table);
 
   for (int i = 0; i < size; ++i) {
-    const TestCase& test = test_cases[i];
+    const TestCase &test = test_cases[i];
     string output;
     converter.Convert(test.input, &output);
     EXPECT_EQ(test.expected_output, output);
