@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 #ifndef MOZC_CONVERTER_BOUNDARY_STRUCT_H_
 #define MOZC_CONVERTER_BOUNDARY_STRUCT_H_
 
-#include "base/base.h"
+#include "base/port.h"
 
 namespace mozc {
 struct BoundaryData {
