@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,19 +34,17 @@
 
 #include <algorithm>
 #include <climits>
-#include <map>
-#include <set>
-#include <utility>
+#include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/logging.h"
-#include "base/singleton.h"
+#include "base/port.h"
 #include "base/util.h"
 #include "converter/node.h"
 #include "converter/segments.h"
 #include "dictionary/pos_matcher.h"
 #include "dictionary/suppression_dictionary.h"
+#include "prediction/suggestion_filter.h"
 
 namespace mozc {
 namespace converter {
@@ -77,20 +75,21 @@ const int   kMinCost                 = 100;
 const int   kCostOffset              = 6907;
 const int   kStructureCostOffset     = 3453;
 const int   kMinStructureCostOffset  = 1151;
-const int32 kNoFilterRank            = 3;
-const int32 kNoFilterIfSameIdRank    = 10;
 const int32 kStopEnmerationCacheSize = 15;
 
 }  // anonymous namespace
 
 CandidateFilter::CandidateFilter(
     const SuppressionDictionary *suppression_dictionary,
-    const POSMatcher *pos_matcher)
+    const POSMatcher *pos_matcher,
+    const SuggestionFilter *suggestion_filter)
     : suppression_dictionary_(suppression_dictionary),
       pos_matcher_(pos_matcher),
+      suggestion_filter_(suggestion_filter),
       top_candidate_(NULL) {
   CHECK(suppression_dictionary_);
   CHECK(pos_matcher_);
+  CHECK(suggestion_filter_);
 }
 
 CandidateFilter::~CandidateFilter() {}
@@ -101,10 +100,50 @@ void CandidateFilter::Reset() {
 }
 
 CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
+    const string &original_key,
     const Segment::Candidate *candidate,
-    const vector<const Node *> &nodes) {
+    const vector<const Node *> &nodes,
+    Segments::RequestType request_type) {
   DCHECK(candidate);
 
+  // Filtering by the suggestion filter, which is applied only for the
+  // PREDICTION and SUGGESTION modes.
+  switch (request_type) {
+    case Segments::PREDICTION:
+      // In the PREDICTION mode, the suggestion filter is not applied and the
+      // same filtering rule as the CONVERSION mode is used because the
+      // PREDICTION is triggered by user action (hitting tab keys), i.e.,
+      // prediction candidates are not automatically shown to users. On the
+      // contrary, since a user hit tab keys to run prediction, even unfavorable
+      // words might be what the user wants to type.  Therefore, filtering rule
+      // is relaxed for the PREDICTION mode: we don't apply the suggestion
+      // filter if the user input key is exactly the same as candidate's.
+      if (original_key == candidate->key) {
+        break;
+      }
+      FALLTHROUGH_INTENDED;
+    case Segments::SUGGESTION:
+      // In contrast to the PREDICTION mode, the SUGGESTION is triggered without
+      // any user actions, i.e., suggestion candidates are automatically
+      // displayed to users.  Therefore, it's better to filter unfavorable words
+      // in this mode.
+      CHECK(suggestion_filter_);
+      if (suggestion_filter_->IsBadSuggestion(candidate->value)) {
+        return BAD_CANDIDATE;
+      }
+      // TODO(noriyukit): In the implementation below, the possibility remains
+      // that multiple nodes constitute bad candidates. For stronger filtering,
+      // we may want to check all the possibilities.
+      for (size_t i = 0; i < nodes.size(); ++i) {
+        if (suggestion_filter_->IsBadSuggestion(nodes[i]->value)) {
+          return BAD_CANDIDATE;
+        }
+      }
+      break;
+    default:
+      break;
+  }
+
   // In general, the cost of constrained node tends to be overestimated.
   // If the top candidate has constrained node, we skip the main body
   // of CandidateFilter, meaning that the node is not treated as the top
@@ -159,6 +198,23 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     return CandidateFilter::BAD_CANDIDATE;
   }
 
+  // Suppress "書います", "買いて"
+  if (Util::GetScriptType(nodes[0]->value) != Util::HIRAGANA &&
+      ((nodes.size() >= 2 &&
+        pos_matcher_->IsKagyoTaConnectionVerb(nodes[0]->rid) &&
+        pos_matcher_->IsMasuSuffix(nodes[1]->lid)) ||
+       (nodes[0]->lid != nodes[0]->rid &&
+        pos_matcher_->IsKagyoTaConnectionVerb(nodes[0]->lid) &&
+        pos_matcher_->IsMasuSuffix(nodes[0]->rid)) ||
+       (nodes.size() >= 2 &&
+        pos_matcher_->IsWagyoRenyoConnectionVerb(nodes[0]->rid) &&
+        pos_matcher_->IsTeSuffix(nodes[1]->lid)) ||
+       (nodes[0]->lid != nodes[0]->rid &&
+        pos_matcher_->IsWagyoRenyoConnectionVerb(nodes[0]->lid) &&
+        pos_matcher_->IsTeSuffix(nodes[0]->rid)))) {
+    return CandidateFilter::BAD_CANDIDATE;
+  }
+
   // The candidate consists of only one token
   if (nodes.size() == 1) {
     VLOG(1) << "don't filter single segment";
@@ -181,6 +237,20 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
+  // "好かっ|たり" vs  "良かっ|たり" have same non_content_value.
+  // "良かっ|たり" is also a good candidate but it is not the top candidate.
+  if (top_candidate_ != candidate &&
+      top_candidate_->content_value != top_candidate_->value &&
+      (top_candidate_->value.compare(
+          top_candidate_->content_value.size(),
+          top_candidate_->value.size() - top_candidate_->content_value.size(),
+          candidate->value,
+          candidate->content_value.size(),
+          candidate->value.size() - candidate->content_value.size()) == 0)) {
+    VLOG(1) << "don't filter if non-content value are the same";
+    return CandidateFilter::GOOD_CANDIDATE;
+  }
+
   // Check Katakana transliterations
   // Skip this check when the conversion mode is real-time;
   // otherwise this ruins the whole sentence
@@ -223,18 +293,18 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   if (candidate_size >= 1 && nodes.size() > 1 &&
       nodes[0]->lid == nodes[0]->rid &&
       pos_matcher_->IsWeakCompoundPrefix(nodes[0]->lid)) {
-    VLOG(1) << "removing noisy prefix pattern";
+    VLOG(2) << "removing noisy prefix pattern";
     return CandidateFilter::BAD_CANDIDATE;
   }
 
-  // don't drop personal names aggressivly.
-  // We have to show personal names even if they are minor enough.
+  // Don't drop personal names aggressivly.
+  // We have to show personal names even if they are too minor.
   // We basically ignores the cost threadshould. Filter candidate
-  // only with StructureCost
+  // only by structure cost.
   int cost_offset = kCostOffset;
   if (candidate->lid == pos_matcher_->GetLastNameId() ||
       candidate->lid == pos_matcher_->GetFirstNameId()) {
-    cost_offset = INT_MAX;
+    cost_offset = INT_MAX - top_cost;
   }
 
   // Filters out candidates with higher cost.
@@ -265,11 +335,17 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   }
 
   // Filters out candidates with higher cost structure.
-  if (top_structure_cost + kStructureCostOffset < candidate->structure_cost) {
+  if (max(top_structure_cost, kMinStructureCostOffset) + kStructureCostOffset <
+      candidate->structure_cost) {
     // We don't stop enumeration here. Just drops high cost structure
     // looks enough.
+    // |top_structure_cost| can be so small especially for compound or
+    // web dictionary entries.
+    // For avoiding over filtering, we use kMinStructureCostOffset if
+    // |top_structure_cost| is small.
     VLOG(2) << "structure cost is invalid:  "
-            << candidate->value << " " << candidate->structure_cost
+              << candidate->value << " " << candidate->content_value << " "
+              << candidate->structure_cost
             << " " << candidate->cost;
     return CandidateFilter::BAD_CANDIDATE;
   }
@@ -278,15 +354,25 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
 }
 
 CandidateFilter::ResultType CandidateFilter::FilterCandidate(
+    const string &original_key,
     const Segment::Candidate *candidate,
-    const vector<const Node *> &nodes) {
-  const CandidateFilter::ResultType result =
-      FilterCandidateInternal(candidate, nodes);
-  if (result != CandidateFilter::GOOD_CANDIDATE) {
+    const vector<const Node *> &nodes,
+    Segments::RequestType request_type) {
+  if (request_type == Segments::REVERSE_CONVERSION) {
+    // In reverse conversion, only remove duplicates because the filtering
+    // criteria of FilterCandidateInternal() are completely designed for
+    // (forward) conversion.
+    const bool inserted = seen_.insert(candidate->value).second;
+    return inserted ? GOOD_CANDIDATE : BAD_CANDIDATE;
+  } else {
+    const ResultType result = FilterCandidateInternal(original_key, candidate,
+                                                      nodes, request_type);
+    if (result != GOOD_CANDIDATE) {
+      return result;
+    }
+    seen_.insert(candidate->value);
     return result;
   }
-  seen_.insert(candidate->value);
-  return result;
 }
 
 }  // namespace converter
