@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,50 +32,32 @@
 #include <algorithm>
 #include <climits>
 #include <string>
-#include <utility>
 #include <vector>
 
-#include "base/base.h"
 #include "base/logging.h"
+#include "base/number_util.h"
+#include "base/port.h"
 #include "base/util.h"
 #include "composer/composer.h"
 #include "converter/connector_interface.h"
 #include "converter/conversion_request.h"
 #include "converter/immutable_converter_interface.h"
 #include "converter/segments.h"
-#include "converter/user_data_manager_interface.h"
 #include "dictionary/dictionary_interface.h"
-#include "dictionary/pos_group.h"
 #include "dictionary/pos_matcher.h"
 #include "dictionary/suppression_dictionary.h"
 #include "prediction/predictor_interface.h"
 #include "rewriter/rewriter_interface.h"
-#include "session/request_handler.h"
 #include "transliteration/transliteration.h"
+#include "usage_stats/usage_stats.h"
+
+using mozc::usage_stats::UsageStats;
 
 namespace mozc {
 namespace {
 
 const size_t kErrorIndex = static_cast<size_t>(-1);
 
-class UserDataManagerImpl : public UserDataManagerInterface {
- public:
-  explicit UserDataManagerImpl(PredictorInterface *predictor,
-                               RewriterInterface *rewriter)
-      : predictor_(predictor), rewriter_(rewriter) {}
-  ~UserDataManagerImpl();
-
-  virtual bool Sync();
-  virtual bool Reload();
-  virtual bool ClearUserHistory();
-  virtual bool ClearUserPrediction();
-  virtual bool ClearUnusedUserPrediction();
-
- private:
-  PredictorInterface *predictor_;
-  RewriterInterface *rewriter_;
-};
-
 size_t GetSegmentIndex(const Segments *segments,
                        size_t segment_index) {
   const size_t history_segments_size = segments->history_segments_size();
@@ -89,7 +71,6 @@ size_t GetSegmentIndex(const Segments *segments,
 void SetKey(Segments *segments, const string &key) {
   segments->set_max_history_segments_size(4);
   segments->clear_conversion_segments();
-  segments->clear_revert_entries();
 
   mozc::Segment *seg = segments->add_segment();
   DCHECK(seg);
@@ -101,12 +82,13 @@ void SetKey(Segments *segments, const string &key) {
   VLOG(2) << segments->DebugString();
 }
 
-bool IsMobile() {
-  return GET_REQUEST(zero_query_suggestion) &&
-      GET_REQUEST(mixed_conversion);
+bool IsMobile(const ConversionRequest &request) {
+  return request.request().zero_query_suggestion() &&
+      request.request().mixed_conversion();
 }
 
-bool IsValidSegments(const Segments &segments) {
+bool IsValidSegments(const ConversionRequest &request,
+                     const Segments &segments) {
   // All segments should have candidate
   for (size_t i = 0; i < segments.segments_size(); ++i) {
     if (segments.segment(i).candidates_size() != 0) {
@@ -115,20 +97,117 @@ bool IsValidSegments(const Segments &segments) {
     // On mobile, we don't distinguish candidates and meta candidates
     // So it's ok if we have meta candidates even if we don't have candidates
     // TODO(team): we may remove mobile check if other platforms accept
-    // meta candidate only segemnt
-    if (IsMobile() && segments.segment(i).meta_candidates_size() != 0) {
+    // meta candidate only segment
+    if (IsMobile(request) && segments.segment(i).meta_candidates_size() != 0) {
       continue;
     }
     return false;
   }
   return true;
 }
+
+// Extracts the last substring that consists of the same script type.
+// Returns true if the last substring is successfully extracted.
+//   Examples:
+//   - "" -> false
+//   - "x " -> "x" / ALPHABET
+//   - "x  " -> false
+//   - "C60" -> "60" / NUMBER
+//   - "200x" -> "x" / ALPHABET
+//   (currently only NUMBER and ALPHABET are supported)
+bool ExtractLastTokenWithScriptType(const string &text,
+                                    string *last_token,
+                                    Util::ScriptType *last_script_type) {
+  last_token->clear();
+  *last_script_type = Util::SCRIPT_TYPE_SIZE;
+
+  ConstChar32ReverseIterator iter(text);
+  if (iter.Done()) {
+    return false;
+  }
+
+  // Allow one whitespace at the end.
+  if (iter.Get() == ' ') {
+    iter.Next();
+    if (iter.Done()) {
+      return false;
+    }
+    if (iter.Get() == ' ') {
+      return false;
+    }
+  }
+
+  vector<char32> reverse_last_token;
+  Util::ScriptType last_script_type_found = Util::GetScriptType(iter.Get());
+  for (; !iter.Done(); iter.Next()) {
+    const char32 w = iter.Get();
+    if ((w == ' ') || (Util::GetScriptType(w) != last_script_type_found)) {
+      break;
+    }
+    reverse_last_token.push_back(w);
+  }
+
+  *last_script_type = last_script_type_found;
+  // TODO(yukawa): Replace reverse_iterator with const_reverse_iterator when
+  //     build failure on Android is fixed.
+  for (vector<char32>::reverse_iterator it = reverse_last_token.rbegin();
+       it != reverse_last_token.rend(); ++it) {
+    Util::UCS4ToUTF8Append(*it, last_token);
+  }
+  return true;
+}
+
+// Tries normalizing input text as a math expression, where full-width numbers
+// and math symbols are converted to their half-width equivalents except for
+// some special symbols, e.g., "×", "÷", and "・". Returns false if the input
+// string contains non-math characters.
+bool TryNormalizingKeyAsMathExpression(StringPiece s, string *key) {
+  key->reserve(s.size());
+  for (ConstChar32Iterator iter(s); !iter.Done(); iter.Next()) {
+    // Half-width arabic numbers.
+    if ('0' <= iter.Get() && iter.Get() <= '9') {
+      key->append(1, static_cast<char>(iter.Get()));
+      continue;
+    }
+    // Full-width arabic numbers ("０" -- "９")
+    if (0xFF10 <= iter.Get() && iter.Get() <= 0xFF19) {
+      const char c = iter.Get() - 0xFF10 + '0';
+      key->append(1, c);
+      continue;
+    }
+    switch (iter.Get()) {
+      case 0x002B: case 0xFF0B:  // "+", "＋"
+        key->append(1, '+');
+        break;
+      case 0x002D: case 0x30FC:  // "-", "ー"
+        key->append(1, '-');
+        break;
+      case 0x002A: case 0xFF0A: case 0x00D7:  // "*", "＊", "×"
+        key->append(1, '*');
+        break;
+      case 0x002F: case 0xFF0F: case 0x30FB: case 0x00F7:
+        // "/",  "／", "・", "÷"
+        key->append(1, '/');
+        break;
+      case 0x0028: case 0xFF08:  // "(", "（"
+        key->append(1, '(');
+        break;
+      case 0x0029: case 0xFF09:  // ")", "）"
+        key->append(1, ')');
+        break;
+      case 0x003D: case 0xFF1D:  // "=", "＝"
+        key->append(1, '=');
+        break;
+      default:
+        return false;
+    }
+  }
+  return true;
+}
+
 }  // namespace
 
 ConverterImpl::ConverterImpl() : pos_matcher_(NULL),
-                                 pos_group_(NULL),
-                                 rewriter_(NULL),
-                                 user_data_manager_(NULL),
                                  immutable_converter_(NULL),
                                  general_noun_id_(kuint16max) {
 }
@@ -136,53 +215,19 @@ ConverterImpl::ConverterImpl() : pos_matcher_(NULL),
 ConverterImpl::~ConverterImpl() {}
 
 void ConverterImpl::Init(const POSMatcher *pos_matcher,
-                         const PosGroup *pos_group,
+                         const SuppressionDictionary *suppression_dictionary,
                          PredictorInterface *predictor,
                          RewriterInterface *rewriter,
                          ImmutableConverterInterface *immutable_converter) {
   // Initializes in order of declaration.
   pos_matcher_ = pos_matcher;
-  pos_group_ = pos_group;
+  suppression_dictionary_ = suppression_dictionary;
   predictor_.reset(predictor);
   rewriter_.reset(rewriter);
-  user_data_manager_.reset(new UserDataManagerImpl(predictor, rewriter));
   immutable_converter_ = immutable_converter;
   general_noun_id_ = pos_matcher_->GetGeneralNounId();
 }
 
-bool ConverterImpl::SetupHistorySegmentsFromPrecedingText(
-    const string &preceding_text, Segments *segments) const {
-  if (!StartReverseConversion(segments, preceding_text)) {
-    LOG(WARNING) << "Reverse conversion failed for " << preceding_text;
-    return false;
-  }
-  // Below we change the resulting segments to history segments. We should
-  // guarantee that the total segments size doesn't exceed the maximum size of
-  // history segments, by erasing leading segments as necessary.
-  if (segments->segments_size() > segments->max_history_segments_size()) {
-    segments->erase_segments(
-        0, segments->segments_size() - segments->max_history_segments_size());
-  }
-  for (int i = 0; i < segments->segments_size(); ++i) {
-    Segment *segment = segments->mutable_segment(i);
-    DCHECK(segment);
-    if (segment->candidates_size() == 0) {
-      LOG(WARNING) << "Empty segment was returned by reverse conversion";
-      return false;
-    }
-    // Need to swap key and value as the reverse conversion stores reading
-    // (usual key for conversion) to value.
-    for (int j = 0; j < segment->candidates_size(); ++j) {
-      Segment::Candidate *candidate = segment->mutable_candidate(j);
-      swap(candidate->key, candidate->value);
-      swap(candidate->content_key, candidate->content_value);
-    }
-    segment->set_segment_type(Segment::HISTORY);
-    segment->set_key(segment->candidate(0).key);
-  }
-  return true;
-}
-
 bool ConverterImpl::StartConversionForRequest(const ConversionRequest &request,
                                               Segments *segments) const {
   if (!request.has_composer()) {
@@ -190,42 +235,57 @@ bool ConverterImpl::StartConversionForRequest(const ConversionRequest &request,
     return false;
   }
 
-  // Existing history segments are used prior to preceding text.
-  if (segments->history_segments_size() == 0 &&
-      !request.preceding_text().empty()) {
-    if (!SetupHistorySegmentsFromPrecedingText(request.preceding_text(),
-                                               segments)) {
-      LOG(WARNING) << "Failed to utilize preceding text: "
-                   << request.preceding_text();
-      segments->Clear();  // fall back to a normal conversion
-    }
-  }
   string conversion_key;
-  request.composer().GetQueryForConversion(&conversion_key);
+  switch (request.composer_key_selection()) {
+    case ConversionRequest::CONVERSION_KEY:
+      request.composer().GetQueryForConversion(&conversion_key);
+      break;
+    case ConversionRequest::PREDICTION_KEY:
+      request.composer().GetQueryForPrediction(&conversion_key);
+      break;
+    default:
+      LOG(FATAL) << "Should never reach here";
+  }
   SetKey(segments, conversion_key);
   segments->set_request_type(Segments::CONVERSION);
-  if (!immutable_converter_->Convert(segments)) {
-    return false;
-  }
-  rewriter_->Rewrite(request, segments);
-  return IsValidSegments(*segments);
+  immutable_converter_->ConvertForRequest(request, segments);
+  RewriteAndSuppressCandidates(request, segments);
+  return IsValidSegments(request, *segments);
 }
 
 bool ConverterImpl::StartConversion(Segments *segments,
                                     const string &key) const {
   SetKey(segments, key);
   segments->set_request_type(Segments::CONVERSION);
-  if (!immutable_converter_->Convert(segments)) {
-    return false;
-  }
-  rewriter_->Rewrite(ConversionRequest(), segments);
-  return IsValidSegments(*segments);
+  const ConversionRequest default_request;
+  immutable_converter_->ConvertForRequest(default_request, segments);
+  RewriteAndSuppressCandidates(default_request, segments);
+  return IsValidSegments(default_request, *segments);
 }
 
 bool ConverterImpl::StartReverseConversion(Segments *segments,
                                            const string &key) const {
   segments->Clear();
+  if (key.empty()) {
+    return false;
+  }
   SetKey(segments, key);
+
+  // Check if |key| looks like a math expression.  In such case, there's no
+  // chance to get the correct reading by the immutable converter.  Rather,
+  // simply returns normalized value.
+  {
+    string value;
+    if (TryNormalizingKeyAsMathExpression(key, &value)) {
+      Segment::Candidate *cand =
+          segments->mutable_segment(0)->push_back_candidate();
+      cand->Init();
+      cand->key = key;
+      cand->value.swap(value);
+      return true;
+    }
+  }
+
   segments->set_request_type(Segments::REVERSE_CONVERSION);
   if (!immutable_converter_->Convert(segments)) {
     return false;
@@ -246,6 +306,31 @@ bool ConverterImpl::StartReverseConversion(Segments *segments,
   return true;
 }
 
+// static
+void ConverterImpl::MaybeSetConsumedKeySizeToCandidate(
+    size_t consumed_key_size, Segment::Candidate* candidate) {
+  if (candidate->attributes & Segment::Candidate::PARTIALLY_KEY_CONSUMED) {
+    // If PARTIALLY_KEY_CONSUMED is set already,
+    // the candidate has set appropriate attribute and size by predictor.
+    return;
+  }
+  candidate->attributes |= Segment::Candidate::PARTIALLY_KEY_CONSUMED;
+  candidate->consumed_key_size = consumed_key_size;
+}
+
+// static
+void ConverterImpl::MaybeSetConsumedKeySizeToSegment(size_t consumed_key_size,
+                                                     Segment* segment) {
+  for (size_t i = 0; i < segment->candidates_size(); ++i) {
+    MaybeSetConsumedKeySizeToCandidate(consumed_key_size,
+                                       segment->mutable_candidate(i));
+  }
+  for (size_t i = 0; i < segment->meta_candidates_size(); ++i) {
+    MaybeSetConsumedKeySizeToCandidate(consumed_key_size,
+                                       segment->mutable_meta_candidate(i));
+  }
+}
+
 // TODO(noriyukit): |key| can be a member of ConversionRequest.
 bool ConverterImpl::Predict(const ConversionRequest &request,
                             const string &key,
@@ -272,7 +357,7 @@ bool ConverterImpl::Predict(const ConversionRequest &request,
     // However, we don't have to do so for suggestion. Here, we are deciding
     // whether the input key is changed or not by using segment key. This is not
     // perfect because for roman input, conversion key is not updated by
-    // imcomplete input, for example, conversion key is "あ" for the input "a",
+    // incomplete input, for example, conversion key is "あ" for the input "a",
     // and will still be "あ" for the input "ak". For avoiding mis-reset of
     // the results, we will reset always for suggestion request type.
     SetKey(segments, key);
@@ -281,11 +366,24 @@ bool ConverterImpl::Predict(const ConversionRequest &request,
   DCHECK_EQ(key, segments->conversion_segment(0).key());
 
   segments->set_request_type(request_type);
-  if (!predictor_->PredictForRequest(request, segments)) {
-    return false;
+  predictor_->PredictForRequest(request, segments);
+  RewriteAndSuppressCandidates(request, segments);
+  if (request_type == Segments::PARTIAL_SUGGESTION ||
+      request_type == Segments::PARTIAL_PREDICTION) {
+    // Here 1st segment's key is the query string of
+    // the partial prediction/suggestion.
+    // e.g. If the composition is "わた|しは", the key is "わた".
+    // If partial prediction/suggestion candidate is submitted,
+    // all the characters which are located from the head to the cursor
+    // should be submitted (in above case "わた" should be submitted).
+    // To do this, PARTIALLY_KEY_CONSUMED and consumed_key_sizconsumed_key_size
+    // should be set.
+    // Note that this process should be done in a predictor because
+    // we have to do this on the candidates created by rewriters.
+    MaybeSetConsumedKeySizeToSegment(
+        Util::CharsLen(key), segments->mutable_conversion_segment(0));
   }
-  rewriter_->Rewrite(request, segments);
-  return IsValidSegments(*segments);
+  return IsValidSegments(request, *segments);
 }
 
 bool ConverterImpl::StartPredictionForRequest(const ConversionRequest &request,
@@ -295,16 +393,6 @@ bool ConverterImpl::StartPredictionForRequest(const ConversionRequest &request,
     return false;
   }
 
-  // Existing history segments are used prior to preceding text.
-  if (segments->history_segments_size() == 0 &&
-      !request.preceding_text().empty()) {
-    if (!SetupHistorySegmentsFromPrecedingText(request.preceding_text(),
-                                               segments)) {
-      LOG(WARNING) << "Failed to utilize preceding text: "
-                   << request.preceding_text();
-      segments->Clear();  // fall back to a normal prediction
-    }
-  }
   string prediction_key;
   request.composer().GetQueryForPrediction(&prediction_key);
   return Predict(request, prediction_key, Segments::PREDICTION, segments);
@@ -312,13 +400,13 @@ bool ConverterImpl::StartPredictionForRequest(const ConversionRequest &request,
 
 bool ConverterImpl::StartPrediction(Segments *segments,
                                     const string &key) const {
-  ConversionRequest default_request;
+  const ConversionRequest default_request;
   return Predict(default_request, key, Segments::PREDICTION, segments);
 }
 
 bool ConverterImpl::StartSuggestion(Segments *segments,
                                     const string &key) const {
-  ConversionRequest default_request;
+  const ConversionRequest default_request;
   return Predict(default_request, key, Segments::SUGGESTION, segments);
 }
 
@@ -332,7 +420,7 @@ bool ConverterImpl::StartSuggestionForRequest(const ConversionRequest &request,
 
 bool ConverterImpl::StartPartialSuggestion(Segments *segments,
                                            const string &key) const {
-  ConversionRequest default_request;
+  const ConversionRequest default_request;
   return Predict(default_request, key, Segments::PARTIAL_SUGGESTION, segments);
 }
 
@@ -353,7 +441,7 @@ bool ConverterImpl::StartPartialSuggestionForRequest(
 
 bool ConverterImpl::StartPartialPrediction(Segments *segments,
                                            const string &key) const {
-  ConversionRequest default_request;
+  const ConversionRequest default_request;
   return Predict(default_request, key, Segments::PARTIAL_PREDICTION, segments);
 }
 
@@ -373,8 +461,12 @@ bool ConverterImpl::StartPartialPredictionForRequest(
                  Segments::PARTIAL_PREDICTION, segments);
 }
 
-bool ConverterImpl::FinishConversion(Segments *segments) const {
-  for (int i = 0; i < segments->segments_size(); ++i) {
+bool ConverterImpl::FinishConversion(const ConversionRequest &request,
+                                     Segments *segments) const {
+  CommitUsageStats(segments, segments->history_segments_size(),
+                   segments->conversion_segments_size());
+
+  for (size_t i = 0; i < segments->segments_size(); ++i) {
     Segment *seg = segments->mutable_segment(i);
     DCHECK(seg);
     // revert SUBMITTED segments to FIXED_VALUE
@@ -391,9 +483,7 @@ bool ConverterImpl::FinishConversion(Segments *segments) const {
   }
 
   segments->clear_revert_entries();
-  if (segments->request_type() == Segments::CONVERSION) {
-    rewriter_->Finish(segments);
-  }
+  rewriter_->Finish(request, segments);
   predictor_->Finish(segments);
 
   // Remove the front segments except for some segments which will be
@@ -435,6 +525,31 @@ bool ConverterImpl::RevertConversion(Segments *segments) const {
   return true;
 }
 
+bool ConverterImpl::ReconstructHistory(Segments *segments,
+                                       const string &preceding_text) const {
+  segments->Clear();
+
+  string key;
+  string value;
+  uint16 id;
+  if (!GetLastConnectivePart(preceding_text, &key, &value, &id)) {
+    return false;
+  }
+
+  Segment *segment = segments->add_segment();
+  segment->set_key(key);
+  segment->set_segment_type(Segment::HISTORY);
+  Segment::Candidate *candidate = segment->push_back_candidate();
+  candidate->rid = id;
+  candidate->lid = id;
+  candidate->content_key = key;
+  candidate->key = key;
+  candidate->content_value = value;
+  candidate->value = value;
+  candidate->attributes = Segment::Candidate::NO_LEARNING;
+  return true;
+}
+
 bool ConverterImpl::CommitSegmentValueInternal(
     Segments *segments, size_t segment_index, int candidate_index,
     Segment::SegmentType segment_type) const {
@@ -471,17 +586,30 @@ bool ConverterImpl::CommitPartialSuggestionSegmentValue(
     const string &current_segment_key, const string &new_segment_key) const {
   DCHECK_GT(segments->conversion_segments_size(), 0);
 
-  size_t raw_segment_index = GetSegmentIndex(segments, segment_index);
+  const size_t raw_segment_index = GetSegmentIndex(segments, segment_index);
   if (!CommitSegmentValueInternal(segments, segment_index, candidate_index,
                                   Segment::SUBMITTED)) {
     return false;
   }
+  CommitUsageStats(segments, raw_segment_index, 1);
+
   Segment *segment = segments->mutable_segment(raw_segment_index);
+  DCHECK_LT(0, segment->candidates_size());
+  const Segment::Candidate &submitted_candidate = segment->candidate(0);
+  const bool auto_partial_suggestion =
+      Util::CharsLen(submitted_candidate.key) != Util::CharsLen(segment->key());
   segment->set_key(current_segment_key);
+
   Segment *new_segment = segments->insert_segment(raw_segment_index + 1);
   new_segment->set_key(new_segment_key);
   DCHECK_GT(segments->conversion_segments_size(), 0);
 
+  if (auto_partial_suggestion) {
+    UsageStats::IncrementCount("CommitAutoPartialSuggestion");
+  } else {
+    UsageStats::IncrementCount("CommitPartialSuggestion");
+  }
+
   return true;
 }
 
@@ -513,10 +641,21 @@ bool ConverterImpl::FreeSegmentValue(Segments *segments,
   return immutable_converter_->Convert(segments);
 }
 
-bool ConverterImpl::SubmitFirstSegment(Segments *segments,
-                                       size_t candidate_index) const {
-  return CommitSegmentValueInternal(segments, 0, candidate_index,
-                                    Segment::SUBMITTED);
+bool ConverterImpl::CommitSegments(
+    Segments *segments,
+    const vector<size_t> &candidate_index) const {
+  const size_t conversion_segment_index = segments->history_segments_size();
+  for (size_t i = 0; i < candidate_index.size(); ++i) {
+    // 2nd argument must always be 0 because on each iteration
+    // 1st segment is submitted.
+    // Using 0 means submitting 1st segment iteratively.
+    if (!CommitSegmentValueInternal(segments, 0, candidate_index[i],
+                                    Segment::SUBMITTED)) {
+      return false;
+    }
+  }
+  CommitUsageStats(segments, conversion_segment_index, candidate_index.size());
+  return true;
 }
 
 bool ConverterImpl::ResizeSegment(Segments *segments,
@@ -545,7 +684,7 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
   const Segment &cur_segment = segments->segment(segment_index);
   const size_t cur_length = Util::CharsLen(cur_segment.key());
 
-  // length canont become 0
+  // length cannot become 0
   if (cur_length + offset_length == 0) {
     return false;
   }
@@ -578,7 +717,7 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
       segment->set_key(new_key);
     }  // scope out |segment|, |new_key|
 
-    if (length < 0) {  // remaning part
+    if (length < 0) {  // remaining part
       Segment *segment = segments->insert_segment(segment_index + 1);
       segment->set_segment_type(Segment::FREE);
       string new_key;
@@ -624,12 +763,8 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
 
   segments->set_resized(true);
 
-  if (!immutable_converter_->Convert(segments)) {
-    return false;
-  }
-
-  rewriter_->Rewrite(request, segments);
-
+  immutable_converter_->ConvertForRequest(request, segments);
+  RewriteAndSuppressCandidates(request, segments);
   return true;
 }
 
@@ -687,12 +822,8 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
 
   segments->set_resized(true);
 
-  if (!immutable_converter_->Convert(segments)) {
-    return false;
-  }
-
-  rewriter_->Rewrite(request, segments);
-
+  immutable_converter_->ConvertForRequest(request, segments);
+  RewriteAndSuppressCandidates(request, segments);
   return true;
 }
 
@@ -748,60 +879,103 @@ void ConverterImpl::CompletePOSIds(Segment::Candidate *candidate) const {
       }
     }
   }
-  LOG(WARNING) << "Cannot set lid/rid. use default value. "
-               << "key: " << candidate->key << ", "
-               << "value: " << candidate->value << ", "
-               << "lid: " << candidate->lid << ", "
-               << "rid: " << candidate->rid;
+  DVLOG(2) << "Cannot set lid/rid. use default value. "
+           << "key: " << candidate->key << ", "
+           << "value: " << candidate->value << ", "
+           << "lid: " << candidate->lid << ", "
+           << "rid: " << candidate->rid;
 }
 
-UserDataManagerInterface *ConverterImpl::GetUserDataManager() {
-  return user_data_manager_.get();
+void ConverterImpl::RewriteAndSuppressCandidates(
+    const ConversionRequest &request, Segments *segments) const {
+  if (!rewriter_->Rewrite(request, segments)) {
+    return;
+  }
+  // Optimization for common use case: Since most of users don't use suppression
+  // dictionary and we can skip the subsequent check.
+  if (suppression_dictionary_->IsEmpty()) {
+    return;
+  }
+  // Although the suppression dictionary is applied at node-level in dictionary
+  // layer, there's possibility that bad words are generated from multiple nodes
+  // and by rewriters. Hence, we need to apply it again at the last stage of
+  // converter.
+  for (size_t i = 0; i < segments->conversion_segments_size(); ++i) {
+    Segment *seg = segments->mutable_conversion_segment(i);
+    for (size_t j = 0; j < seg->candidates_size(); ) {
+      const Segment::Candidate &cand = seg->candidate(j);
+      if (suppression_dictionary_->SuppressEntry(cand.key, cand.value)) {
+        seg->erase_candidate(j);
+      } else {
+        ++j;
+      }
+    }
+  }
 }
 
-UserDataManagerImpl::~UserDataManagerImpl() {}
+void ConverterImpl::CommitUsageStats(const Segments *segments,
+                                     size_t begin_segment_index,
+                                     size_t segment_length) const {
+  if (segment_length == 0) {
+    return;
+  }
+  if (begin_segment_index + segment_length > segments->segments_size()) {
+    LOG(ERROR) << "Invalid state. segments size: " << segments->segments_size()
+               << " required size: " << begin_segment_index + segment_length;
+    return;
+  }
 
-bool UserDataManagerImpl::Sync() {
-  return (rewriter_->Sync() && predictor_->Sync());
+  // Timing stats are scaled by 1,000 to improve the accuracy of average values.
+
+  uint64 submitted_total_length = 0;
+  for (size_t i = 0; i < segment_length; ++i) {
+    const Segment &segment = segments->segment(begin_segment_index + i);
+    const uint32 submitted_length = Util::CharsLen(segment.candidate(0).value);
+    UsageStats::UpdateTiming("SubmittedSegmentLengthx1000",
+                             submitted_length * 1000);
+    submitted_total_length += submitted_length;
+  }
+
+  UsageStats::UpdateTiming("SubmittedLengthx1000",
+                           submitted_total_length * 1000);
+  UsageStats::UpdateTiming("SubmittedSegmentNumberx1000",
+                           segment_length * 1000);
+  UsageStats::IncrementCountBy("SubmittedTotalLength", submitted_total_length);
 }
 
-bool UserDataManagerImpl::Reload() {
-  return (rewriter_->Reload() && predictor_->Reload());
-}
+bool ConverterImpl::GetLastConnectivePart(const string &preceding_text,
+                                          string *key,
+                                          string *value,
+                                          uint16 *id) const {
+  key->clear();
+  value->clear();
+  *id = general_noun_id_;
 
-bool UserDataManagerImpl::ClearUserHistory() {
-  rewriter_->Clear();
-  return true;
-}
+  Util::ScriptType last_script_type = Util::SCRIPT_TYPE_SIZE;
+  string last_token;
+  if (!ExtractLastTokenWithScriptType(preceding_text,
+                                      &last_token,
+                                      &last_script_type)) {
+    return false;
+  }
 
-bool UserDataManagerImpl::ClearUserPrediction() {
-  predictor_->ClearAllHistory();
-  return true;
-}
-
-bool UserDataManagerImpl::ClearUnusedUserPrediction() {
-  predictor_->ClearUnusedHistory();
-  return true;
-}
-
-void ConverterUtil::InitSegmentsFromString(const string &key,
-                                           const string &preedit,
-                                           Segments *segments) {
-  segments->clear_conversion_segments();
-  // the request mode is CONVERSION, as the user experience
-  // is similar to conversion. UserHistryPredictor distinguishes
-  // CONVERSION from SUGGESTION now.
-  segments->set_request_type(Segments::CONVERSION);
-  Segment *segment = segments->add_segment();
-  segment->Clear();
-  segment->set_key(key);
-  segment->set_segment_type(Segment::FIXED_VALUE);
-  Segment::Candidate *c = segment->add_candidate();
-  c->Init();
-  c->value = preedit;
-  c->content_value = preedit;
-  c->key = key;
-  c->content_key = key;
+  // Currently only NUMBER and ALPHABET are supported.
+  switch (last_script_type) {
+    case Util::NUMBER: {
+      Util::FullWidthAsciiToHalfWidthAscii(last_token, key);
+      swap(*value, last_token);
+      *id = pos_matcher_->GetNumberId();
+      return true;
+    }
+    case Util::ALPHABET: {
+      Util::FullWidthAsciiToHalfWidthAscii(last_token, key);
+      swap(*value, last_token);
+      *id = pos_matcher_->GetUniqueNounId();
+      return true;
+    }
+    default:
+      return false;
+  }
 }
 
 }  // namespace mozc
