@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,15 +31,14 @@
 #define MOZC_CONVERTER_CONVERTER_INTERFACE_H_
 
 #include <string>
-#include "base/base.h"
+#include "base/port.h"
 #include "converter/conversion_request.h"
 #include "converter/segments.h"
 
 namespace mozc {
-class UserDataManagerInterface;
 
 namespace composer {
-  class Composer;
+class Composer;
 }  // namespace composer
 
 class ConverterInterface {
@@ -52,7 +51,7 @@ class ConverterInterface {
                                          Segments *segments) const = 0;
 
   // Start conversion with key.
-  // key is a request writtein in Hiragana sequence
+  // key is a request written in Hiragana sequence
   virtual bool StartConversion(Segments *segments,
                                const string &key) const = 0;
 
@@ -93,8 +92,9 @@ class ConverterInterface {
                                       const string &key) const = 0;
 
   // Finish conversion.
-  // Segments are cleared. Context is not clreared
-  virtual bool FinishConversion(Segments *segments) const = 0;
+  // Segments are cleared. Context is not cleared
+  virtual bool FinishConversion(const ConversionRequest &request,
+                                Segments *segments) const = 0;
 
   // Clear segments and keep the context
   virtual bool CancelConversion(Segments *segments) const = 0;
@@ -105,6 +105,10 @@ class ConverterInterface {
   // Revert last Finish operation
   virtual bool RevertConversion(Segments *segments) const = 0;
 
+  // Reconstruct history segments from given preceding text.
+  virtual bool ReconstructHistory(Segments *segments,
+                                  const string &preceding_text) const = 0;
+
   // Expand the bunsetsu-segment at "segment_index" by candidate_size
   // DEPRECATED: This method doesn't take any effect.
   // TODO(taku): remove this method.
@@ -148,11 +152,14 @@ class ConverterInterface {
   virtual bool FreeSegmentValue(Segments *segments,
                                 size_t segment_index) const = 0;
 
-  // Commit the first segment and move the candidate
-  // into history segment temporally.
-  // Session can use this method for PartialSubmit.
-  virtual bool SubmitFirstSegment(Segments *segments,
-                                  size_t candidate_index) const = 0;
+  // Commit segments of which the range is [0, candidate_index.size()]
+  // and move the candidates into history segment temporally.
+  // Session can use this method for PartialCommit.
+  // |candidate_index| is a vector containing candidate index.
+  // candidate_index[0] corresponds to the index of the candidate of
+  // 1st segment.
+  virtual bool CommitSegments(Segments *segments,
+                              const vector<size_t> &candidate_index) const = 0;
 
   // Resize segment_index-th segment by offset_length.
   // offset_lenth can be negative.
@@ -171,8 +178,6 @@ class ConverterInterface {
                              const uint8 *new_size_array,
                              size_t array_size) const = 0;
 
-  virtual UserDataManagerInterface *GetUserDataManager() = 0;
-
  protected:
   ConverterInterface() {}
 
@@ -180,20 +185,6 @@ class ConverterInterface {
   DISALLOW_COPY_AND_ASSIGN(ConverterInterface);
 };
 
-// static converter utilities
-class ConverterUtil {
- public:
-  // Make a segment having one candidate. The value of candidate is the
-  // same as the preedit.  This function can be used for error handling.
-  // When the converter fails, we can call this function to make a
-  // virtual segment.
-  static void InitSegmentsFromString(const string &key,
-                                     const string &preedit,
-                                     Segments *segments);
-
- private:
-  ConverterUtil() {}
-  ~ConverterUtil() {}
-};
 }  // namespace mozc
+
 #endif  // MOZC_CONVERTER_CONVERTER_INTERFACE_H_
