@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@ void SetSegments(Segments *segments, const string &cand_value) {
   meta_cand->Init();
   meta_cand->value = "TestT13N";
 }
-}
+}  // namespace
 
 class ConverterMockTest : public testing::Test {
  protected:
@@ -149,7 +149,8 @@ TEST_F(ConverterMockTest, SetFinishConversion) {
   Segments output, expect;
   SetSegments(&expect, "FinishConversion");
   GetMock()->SetFinishConversion(&expect, true);
-  EXPECT_TRUE(converter->FinishConversion(&output));
+  const ConversionRequest default_request;
+  EXPECT_TRUE(converter->FinishConversion(default_request, &output));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
@@ -203,13 +204,15 @@ TEST_F(ConverterMockTest, SetFreeSegmentValue) {
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
-TEST_F(ConverterMockTest, SetSubmitFirstSegment) {
+TEST_F(ConverterMockTest, SetCommitSegments) {
   ConverterInterface *converter = GetMock();
 
   Segments output, expect;
-  SetSegments(&expect, "SubmitFirstSegment");
-  GetMock()->SetSubmitFirstSegment(&expect, true);
-  EXPECT_TRUE(converter->SubmitFirstSegment(&output, 1));
+  SetSegments(&expect, "CommitSegments");
+  GetMock()->SetCommitSegments(&expect, true);
+  vector<size_t> singleton_vector;
+  singleton_vector.push_back(1);
+  EXPECT_TRUE(converter->CommitSegments(&output, singleton_vector));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
@@ -219,7 +222,8 @@ TEST_F(ConverterMockTest, SetResizeSegment1) {
   Segments output, expect;
   SetSegments(&expect, "ResizeSegment1");
   GetMock()->SetResizeSegment1(&expect, true);
-  EXPECT_TRUE(converter->ResizeSegment(&output, ConversionRequest(), 1, 5));
+  const ConversionRequest default_request;
+  EXPECT_TRUE(converter->ResizeSegment(&output, default_request, 1, 5));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
 
@@ -230,7 +234,8 @@ TEST_F(ConverterMockTest, SetResizeSegment2) {
   SetSegments(&expect, "ResizeSegment2");
   GetMock()->SetResizeSegment2(&expect, true);
   uint8 size_array[] = {1, 2, 3};
-  EXPECT_TRUE(converter->ResizeSegment(&output, ConversionRequest(), 1, 5,
+  const ConversionRequest default_request;
+  EXPECT_TRUE(converter->ResizeSegment(&output, default_request, 1, 5,
                                        size_array, arraysize(size_array)));
   EXPECT_EQ(expect.DebugString(), output.DebugString());
 }
@@ -348,7 +353,8 @@ TEST_F(ConverterMockTest, GetFinishConversion) {
   Segments input;
   SetSegments(&input, "FinishConversion");
   const string input_str = input.DebugString();
-  converter->FinishConversion(&input);
+  ConversionRequest default_request;
+  converter->FinishConversion(default_request, &input);
 
   Segments last_segment;
   GetMock()->GetFinishConversion(&last_segment);
@@ -447,22 +453,27 @@ TEST_F(ConverterMockTest, GetFreeSegmentValue) {
   EXPECT_EQ(input_idx, last_idx);
 }
 
-TEST_F(ConverterMockTest, GetSubmitFirstSegment) {
+TEST_F(ConverterMockTest, GetCommitSegments) {
   ConverterInterface *converter = GetMock();
 
   Segments input;
-  size_t input_idx = 1;
-  SetSegments(&input, "SubmitFirstSegment");
+  size_t input_idx1 = 1;
+  size_t input_idx2 = 2;
+  SetSegments(&input, "CommitSegments");
   const string input_str = input.DebugString();
-  converter->SubmitFirstSegment(&input, input_idx);
+  vector<size_t> index_list;
+  index_list.push_back(input_idx1);
+  index_list.push_back(input_idx2);
+  converter->CommitSegments(&input, index_list);
 
   Segments last_segment;
-  size_t last_idx;
-  GetMock()->GetSubmitFirstSegment(&last_segment, &last_idx);
+  vector<size_t> last_idx;
+  GetMock()->GetCommitSegments(&last_segment, &last_idx);
   const string last_segment_str = last_segment.DebugString();
 
   EXPECT_EQ(input_str, last_segment_str);
-  EXPECT_EQ(input_idx, last_idx);
+  EXPECT_EQ(input_idx1, last_idx[0]);
+  EXPECT_EQ(input_idx2, last_idx[1]);
 }
 
 TEST_F(ConverterMockTest, GetResizeSegment1) {
@@ -473,8 +484,9 @@ TEST_F(ConverterMockTest, GetResizeSegment1) {
   int input_offset = 3;
   SetSegments(&input, "ResizeSegment1");
   const string input_str = input.DebugString();
+  const ConversionRequest default_request;
   converter->ResizeSegment(
-      &input, ConversionRequest(), input_idx, input_offset);
+      &input, default_request, input_idx, input_offset);
 
   Segments last_segment;
   size_t last_idx;
@@ -495,7 +507,8 @@ TEST_F(ConverterMockTest, GetResizeSegment2) {
   uint8 input_array[] = {1, 2, 3};
   SetSegments(&input, "ResizeSegment2");
   const string input_str = input.DebugString();
-  converter->ResizeSegment(&input, ConversionRequest(), input_idx, input_size,
+  const ConversionRequest default_request;
+  converter->ResizeSegment(&input, default_request, input_idx, input_size,
                            input_array, arraysize(input_array));
 
   Segments last_segment;
