@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,8 @@
 #define MOZC_CONVERTER_GEN_SEGMENTER_BITARRAY_H_
 
 #include <string>
-#include "base/base.h"
+
+#include "base/port.h"
 
 namespace mozc {
 
