@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,23 +29,22 @@
 
 #include "converter/lattice.h"
 
+#include <sstream>
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/logging.h"
+#include "base/port.h"
 #include "base/singleton.h"
 #include "base/util.h"
 #include "converter/node.h"
 #include "converter/node_allocator.h"
-#include "dictionary/pos_matcher.h"
 
 DEFINE_bool(disable_lattice_cache,
             false,
             "do not use cache feature for lattice");
 
 namespace mozc {
-
 namespace {
 
 Node *InitBOSNode(Lattice *lattice, uint16 length) {
@@ -137,6 +136,7 @@ string GetCommonPrefix(const string &str1, const string &str2) {
   }
   return common_prefix;
 }
+
 }  // namespace
 
 struct LatticeDisplayNodeInfo {
@@ -145,7 +145,7 @@ struct LatticeDisplayNodeInfo {
   string display_node_str_;
 };
 
-Lattice::Lattice() : node_allocator_(new NodeAllocator) {}
+Lattice::Lattice() : history_end_pos_(0), node_allocator_(new NodeAllocator) {}
 
 Lattice::~Lattice() {}
 
@@ -165,9 +165,9 @@ Node *Lattice::end_nodes(size_t pos) const {
   return end_nodes_[pos];
 }
 
-void Lattice::SetKey(const string &key) {
+void Lattice::SetKey(StringPiece key) {
   Clear();
-  key_ = key;
+  key.CopyToString(&key_);
   const size_t size = key.size();
   begin_nodes_.resize(size + 4);
   end_nodes_.resize(size + 4);
@@ -232,10 +232,11 @@ void Lattice::Clear() {
   end_nodes_.clear();
   node_allocator_->Free();
   cache_info_.clear();
+  history_end_pos_ = 0;
 }
 
 void Lattice::SetDebugDisplayNode(size_t begin_pos, size_t end_pos,
-                                         const string &str) {
+                                  const string &str) {
   LatticeDisplayNodeInfo *info = Singleton<LatticeDisplayNodeInfo>::get();
   info->display_node_begin_pos_ = begin_pos;
   info->display_node_end_pos_ = end_pos;
@@ -247,6 +248,14 @@ void Lattice::ResetDebugDisplayNode() {
   info->display_node_str_.clear();
 }
 
+void Lattice::set_history_end_pos(size_t pos) {
+  history_end_pos_ = pos;
+}
+
+size_t Lattice::history_end_pos() const {
+  return history_end_pos_;
+}
+
 void Lattice::UpdateKey(const string &new_key) {
   if (FLAGS_disable_lattice_cache) {
     SetKey(new_key);
@@ -349,7 +358,7 @@ void Lattice::ShrinkKey(const size_t new_len) {
   fill(cache_info_.begin() + new_len, cache_info_.end(), 0);
 
   // update key
-  key_ = key_.substr(0, new_len);
+  key_.erase(new_len);
 }
 
 size_t Lattice::cache_info(const size_t pos) const {
@@ -474,4 +483,5 @@ string Lattice::DebugString() const {
 
   return os.str();
 }
+
 }  // namespace mozc
