@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,28 +27,42 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#ifndef MOZC_DICTIONARY_TEXT_DICTIONARY_TOKEN_H_
-#define MOZC_DICTIONARY_TEXT_DICTIONARY_TOKEN_H_
+#ifndef MOZC_DICTIONARY_DICTIONARY_TOKEN_H_
+#define MOZC_DICTIONARY_DICTIONARY_TOKEN_H_
 
 #include <string>
 
+#include "base/port.h"
+
 namespace mozc {
 
 struct Token {
-  // bit field
+  typedef uint8 AttributesBitfield;
+
   enum Attribute {
     NONE = 0,
     SPELLING_CORRECTION = 1,
     LABEL_SIZE = 2,
+    // * CAUTION *
+    // If you are going to add new attributes, make sure that they have larger
+    // values than LABEL_SIZE!! The attributes having less values than it are
+    // tightly integrated with the system dictionary codec.
+
+    // The following attribute is not stored in the system dictionary but is
+    // added by dictionary modules when looking up from user dictionary.
+    USER_DICTIONARY = 1 << 7,
   };
+
+  Token() : cost(0), lid(0), rid(0), attributes(NONE) {}
+
   string key;
   string value;
   int    cost;
   int    lid;
   int    rid;
-  Attribute attributes;
-  Token() : cost(0), lid(0), rid(0), attributes(NONE) {}
+  AttributesBitfield attributes;
 };
-}  // mozc
 
-#endif  // MOZC_DICTIONARY_TEXT_DICTIONARY_TOKEN_H_
+}  // namespace mozc
+
+#endif  // MOZC_DICTIONARY_DICTIONARY_TOKEN_H_
