@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -37,20 +37,22 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/codegen_bytearray_stream.h"
 #include "base/file_stream.h"
+#include "base/flags.h"
 #include "base/util.h"
+#include "data_manager/testing/mock_user_pos_manager.h"
 #include "data_manager/user_pos_manager.h"
-#include "dictionary/system/system_dictionary_builder.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/pos_matcher.h"
+#include "dictionary/system/system_dictionary_builder.h"
 #include "dictionary/text_dictionary_loader.h"
 
 DEFINE_string(input, "", "space separated input text files");
 DEFINE_string(output, "", "output binary file");
 DEFINE_bool(make_header, false, "make header mode");
-
+DEFINE_bool(gen_test_dictionary, false,
+            "generate test dictionary (use mock POSManager)");
 
 namespace mozc {
 namespace {
@@ -70,20 +72,22 @@ void GetInputFileName(const string &input_file,
                       string *reading_correction_input) {
   CHECK(system_dictionary_input);
   CHECK(reading_correction_input);
-  vector<string> input_files;
-  Util::SplitStringUsing(input_file, " ", &input_files);
   system_dictionary_input->clear();
   reading_correction_input->clear();
-  for (size_t i = 0; i < input_files.size(); ++i) {
-    if (Util::EndsWith(input_files[i], kReadingCorrectionFile)) {
-      Util::AppendStringWithDelimiter(",", input_files[i],
+  const StringPiece kDelimiter(", ", 1);
+  for (SplitIterator<SingleDelimiter> iter(input_file, " ");
+       !iter.Done(); iter.Next()) {
+    const StringPiece &input_file = iter.Get();
+    if (Util::EndsWith(input_file, kReadingCorrectionFile)) {
+      Util::AppendStringWithDelimiter(kDelimiter, input_file,
                                       reading_correction_input);
     } else {
-      Util::AppendStringWithDelimiter(",", input_files[i],
+      Util::AppendStringWithDelimiter(kDelimiter, input_file,
                                       system_dictionary_input);
     }
   }
 }
+
 }  // namespace
 }  // namespace mozc
 
@@ -96,21 +100,16 @@ int main(int argc, char **argv) {
                          &reading_correction_input);
 
   const mozc::POSMatcher *pos_matcher =
+      FLAGS_gen_test_dictionary ?
+      mozc::testing::MockUserPosManager::GetUserPosManager()->GetPOSMatcher() :
       mozc::UserPosManager::GetUserPosManager()->GetPOSMatcher();
   CHECK(pos_matcher);
 
   mozc::TextDictionaryLoader loader(*pos_matcher);
-  loader.Open(system_dictionary_input);
-
-  if (!reading_correction_input.empty()) {
-    loader.OpenReadingCorrection(reading_correction_input);
-  }
-
-  vector<mozc::Token *> tokens;
-  loader.CollectTokens(&tokens);
+  loader.Load(system_dictionary_input, reading_correction_input);
 
   mozc::dictionary::SystemDictionaryBuilder builder;
-  builder.BuildFromTokens(tokens);
+  builder.BuildFromTokens(loader.tokens());
 
   scoped_ptr<ostream> output_stream(
       new mozc::OutputFileStream(FLAGS_output.c_str(),
