@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,8 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/logging.h"
+#include "base/scoped_ptr.h"
 #include "base/util.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/system/codec_interface.h"
@@ -317,22 +317,24 @@ class SystemDictionaryCodecMock : public SystemDictionaryCodecInterface {
   const string GetSectionNameForValue() const { return "Mock"; }
   const string GetSectionNameForTokens() const { return "Mock"; }
   const string GetSectionNameForPos() const { return "Mock"; }
-  void EncodeKey(const string &src, string *dst) const {}
-  void DecodeKey(const string &src, string *dst) const {}
-  void EncodeValue(const string &src, string *dst) const {}
-  void DecodeValue(const string &src, string *dst) const {}
-  void EncodeTokens(
+  virtual void EncodeKey(const StringPiece src, string *dst) const {}
+  virtual void DecodeKey(const StringPiece src, string *dst) const {}
+  virtual size_t GetEncodedKeyLength(const StringPiece src) const { return 0; }
+  virtual size_t GetDecodedKeyLength(const StringPiece src) const { return 0; }
+  virtual void EncodeValue(const StringPiece src, string *dst) const {}
+  virtual void DecodeValue(const StringPiece src, string *dst) const {}
+  virtual void EncodeTokens(
       const vector<TokenInfo> &tokens, string *output) const {}
-  void DecodeTokens(
+  virtual void DecodeTokens(
       const uint8 *ptr, vector<TokenInfo> *tokens) const {}
-  bool DecodeToken(
+  virtual bool DecodeToken(
       const uint8 *ptr, TokenInfo *token_info, int *read_bytes) const {
     *read_bytes = 0;
     return false;
   }
-  bool ReadTokenForReverseLookup(
+  virtual bool ReadTokenForReverseLookup(
       const uint8 *ptr, int *value_id, int *read_bytes) const { return false; }
-  uint8 GetTokensTerminationFlag() const { return 0xff; }
+  virtual uint8 GetTokensTerminationFlag() const { return 0xff; }
 };
 
 TEST_F(SystemDictionaryCodecTest, FactoryTest) {
@@ -351,9 +353,11 @@ TEST_F(SystemDictionaryCodecTest, KeyCodecKanaTest) {
   codec->EncodeKey(original, &encoded);
   // hiragana should be encoded in 1 byte
   EXPECT_EQ(2, encoded.size());
+  EXPECT_EQ(encoded.size(), codec->GetEncodedKeyLength(original));
   string decoded;
   codec->DecodeKey(encoded, &decoded);
   EXPECT_EQ(original, decoded);
+  EXPECT_EQ(decoded.size(), codec->GetDecodedKeyLength(encoded));
 }
 
 
@@ -365,9 +369,11 @@ TEST_F(SystemDictionaryCodecTest, KeyCodecSymbolTest) {
   codec->EncodeKey(original, &encoded);
   // middle dot and prolonged sound should be encoded in 1 byte
   EXPECT_EQ(2, encoded.size());
+  EXPECT_EQ(encoded.size(), codec->GetEncodedKeyLength(original));
   string decoded;
   codec->DecodeKey(encoded, &decoded);
   EXPECT_EQ(original, decoded);
+  EXPECT_EQ(decoded.size(), codec->GetDecodedKeyLength(encoded));
 }
 
 TEST_F(SystemDictionaryCodecTest, ValueCodecTest) {
@@ -776,12 +782,12 @@ TEST_F(SystemDictionaryCodecTest, CodecTest) {
   {  // Value
     string original;
     {
-      uint16 a_ucs2 = '!';
+      char32 a_ucs4 = '!';
       Util::SetRandomSeed(0);
       for (size_t i = 0; i < 10000; ++i) {
         // U+4E00-9FFF CJK Unified Ideographs
-        uint16 c = a_ucs2 + static_cast<uint16>(Util::Random(0x9f00));
-        Util::UCS2ToUTF8Append(c, &original);
+        const char32 c = a_ucs4 + static_cast<uint16>(Util::Random(0x9f00));
+        Util::UCS4ToUTF8Append(c, &original);
       }
     }
     string encoded;
@@ -793,18 +799,20 @@ TEST_F(SystemDictionaryCodecTest, CodecTest) {
   {  // Key
     string original;
     {
-      uint16 a_ucs2 = 0x3041;  // "ぁ"
+      char32 a_ucs4 = 0x3041;  // "ぁ"
       Util::SetRandomSeed(0);
       for (size_t i = 0; i < 1000; ++i) {
-        uint16 c = a_ucs2 + static_cast<uint16>(Util::Random(1000));
-        Util::UCS2ToUTF8Append(c, &original);
+        const char32 c = a_ucs4 + static_cast<uint16>(Util::Random(1000));
+        Util::UCS4ToUTF8Append(c, &original);
       }
     }
     string encoded;
     codec->EncodeKey(original, &encoded);
+    EXPECT_EQ(encoded.size(), codec->GetEncodedKeyLength(original));
     string decoded;
     codec->DecodeKey(encoded, &decoded);
     EXPECT_EQ(original, decoded);
+    EXPECT_EQ(decoded.size(), codec->GetDecodedKeyLength(encoded));
   }
 }
 
