@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,20 +30,20 @@
 #include "dictionary/user_dictionary.h"
 
 #include <algorithm>
+#include <limits>
 #include <set>
 #include <string>
 
-#include "base/base.h"
+#include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "base/mutex.h"
 #include "base/singleton.h"
 #include "base/stl_util.h"
 #include "base/thread.h"
-#include "base/trie.h"
+#include "base/util.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
-#include "converter/node.h"
-#include "converter/node_allocator.h"
+#include "dictionary/dictionary_token.h"
 #include "dictionary/pos_matcher.h"
 #include "dictionary/suppression_dictionary.h"
 #include "dictionary/user_dictionary_storage.h"
@@ -55,14 +55,21 @@ namespace mozc {
 
 namespace {
 
-class POSTokenLess {
- public:
+struct OrderByKey {
   bool operator()(const UserPOS::Token *lhs,
                   const UserPOS::Token *rhs) const {
     return lhs->key < rhs->key;
   }
 };
 
+struct OrderByKeyThenById {
+  bool operator()(const UserPOS::Token *lhs,
+                  const UserPOS::Token *rhs) const {
+    const int comp = lhs->key.compare(rhs->key);
+    return comp == 0 ? (lhs->id < rhs->id) : (comp < 0);
+  }
+};
+
 class UserDictionaryFileManager {
  public:
   UserDictionaryFileManager() {}
@@ -86,6 +93,17 @@ class UserDictionaryFileManager {
   Mutex mutex_;
   DISALLOW_COPY_AND_ASSIGN(UserDictionaryFileManager);
 };
+
+void FillTokenFromUserPOSToken(const UserPOS::Token &user_pos_token,
+                               Token *token) {
+  token->key = user_pos_token.key;
+  token->value = user_pos_token.value;
+  token->cost = user_pos_token.cost;
+  token->lid = user_pos_token.id;
+  token->rid = user_pos_token.id;
+  token->attributes = Token::USER_DICTIONARY;
+}
+
 }  // namespace
 
 class TokensIndex : public vector<UserPOS::Token *> {
@@ -103,11 +121,10 @@ class TokensIndex : public vector<UserPOS::Token *> {
     clear();
   }
 
-  void Load(const UserDictionaryStorage &storage) {
+  void Load(const user_dictionary::UserDictionaryStorage &storage) {
     Clear();
     set<uint64> seen;
     vector<UserPOS::Token> tokens;
-    int sync_words_count = 0;
 
     if (!suppression_dictionary_->IsLocked()) {
       LOG(ERROR) << "SuppressionDictionary must be locked first";
@@ -121,10 +138,6 @@ class TokensIndex : public vector<UserPOS::Token *> {
         continue;
       }
 
-      if (dic.syncable()) {
-        sync_words_count += dic.entries_size();
-      }
-
       for (size_t j = 0; j < dic.entries_size(); ++j) {
         const UserDictionaryStorage::UserDictionaryEntry &entry =
             dic.entries(j);
@@ -142,7 +155,12 @@ class TokensIndex : public vector<UserPOS::Token *> {
         Util::NormalizeVoicedSoundMark(tmp, &reading);
 
         DCHECK_LE(0, entry.pos());
+MOZC_CLANG_PUSH_WARNING();
+#if MOZC_CLANG_HAS_WARNING(tautological-constant-out-of-range-compare)
+MOZC_CLANG_DISABLE_WARNING(tautological-constant-out-of-range-compare);
+#endif  // MOZC_CLANG_HAS_WARNING(tautological-constant-out-of-range-compare)
         DCHECK_LE(entry.pos(), 255);
+MOZC_CLANG_POP_WARNING();
         const uint64 fp = Util::Fingerprint(reading +
                                             "\t" +
                                             entry.value() +
@@ -163,12 +181,14 @@ class TokensIndex : public vector<UserPOS::Token *> {
               UserDictionaryUtil::GetStringPosType(entry.pos()), &tokens);
           for (size_t k = 0; k < tokens.size(); ++k) {
             this->push_back(new UserPOS::Token(tokens[k]));
+            Util::StripWhiteSpaces(entry.comment(), &this->back()->comment);
           }
         }
       }
     }
 
-    sort(this->begin(), this->end(), POSTokenLess());
+    // Sort first by key and then by POS ID.
+    sort(this->begin(), this->end(), OrderByKeyThenById());
 
     suppression_dictionary_->UnLock();
 
@@ -176,8 +196,6 @@ class TokensIndex : public vector<UserPOS::Token *> {
 
     usage_stats::UsageStats::SetInteger("UserRegisteredWord",
                                         static_cast<int>(this->size()));
-    usage_stats::UsageStats::SetInteger("UserRegisteredSyncWord",
-                                        sync_words_count);
   }
 
  private:
@@ -214,15 +232,22 @@ class UserDictionaryReloader : public Thread {
   }
 
   virtual void Run() {
-    scoped_ptr<UserDictionaryStorage>
-        storage(
-            new UserDictionaryStorage
-            (Singleton<UserDictionaryFileManager>::get()->GetFileName()));
+    scoped_ptr<UserDictionaryStorage> storage(new UserDictionaryStorage(
+        Singleton<UserDictionaryFileManager>::get()->GetFileName()));
+
     // Load from file
     if (!storage->Load()) {
       return;
     }
 
+    if (storage->ConvertSyncDictionariesToNormalDictionaries()) {
+      LOG(INFO) << "Syncable dictionaries are converted to normal dictionaries";
+      if (storage->Lock()) {
+        storage->Save();
+        storage->UnLock();
+      }
+    }
+
     if (auto_register_mode_ &&
         !storage->AddToAutoRegisteredDictionary(key_, value_, pos_)) {
       LOG(ERROR) << "failed to execute AddToAutoRegisteredDictionary";
@@ -241,19 +266,21 @@ class UserDictionaryReloader : public Thread {
   string key_;
   string value_;
   user_dictionary::UserDictionary::PosType pos_;
+
+  DISALLOW_COPY_AND_ASSIGN(UserDictionaryReloader);
 };
 
 UserDictionary::UserDictionary(const UserPOSInterface *user_pos,
                                const POSMatcher *pos_matcher,
                                SuppressionDictionary *suppression_dictionary)
-    : reloader_(new UserDictionaryReloader(this)),
+    : ALLOW_THIS_IN_INITIALIZER_LIST(
+          reloader_(new UserDictionaryReloader(this))),
       user_pos_(user_pos),
       pos_matcher_(pos_matcher),
       suppression_dictionary_(suppression_dictionary),
-      empty_limit_(Limit()),
-      tokens_(new TokensIndex(user_pos_, suppression_dictionary)),
+      tokens_(new TokensIndex(user_pos_.get(), suppression_dictionary)),
       mutex_(new ReaderWriterMutex) {
-  DCHECK(user_pos_);
+  DCHECK(user_pos_.get());
   DCHECK(pos_matcher_);
   DCHECK(suppression_dictionary_);
   Reload();
@@ -264,188 +291,185 @@ UserDictionary::~UserDictionary() {
   delete tokens_;
 }
 
-Node *UserDictionary::LookupPredictiveWithLimit(
-    const char *str, int size, const Limit &limit,
-    NodeAllocatorInterface *allocator) const {
+bool UserDictionary::HasValue(StringPiece value) const {
+  // TODO(noriyukit): Currently, we don't support HasValue() for user dictionary
+  // because we need to search tokens linearly, which might be slow in extreme
+  // cases where 100K entries exist.  Note: HasValue() method is used only in
+  // UserHistoryPredictor for privacy sensitivity check.
+  return false;
+}
+
+void UserDictionary::LookupPredictive(
+    StringPiece key,
+    bool,  // use_kana_modifier_insensitive_lookup
+    Callback *callback) const {
   scoped_reader_lock l(mutex_.get());
 
-  if (size == 0) {
-    LOG(WARNING) << "string of length zero is passed.";
-    return NULL;
+  if (key.empty()) {
+    VLOG(2) << "string of length zero is passed.";
+    return;
   }
-
   if (tokens_->empty()) {
-    return NULL;
+    return;
   }
-
   if (GET_CONFIG(incognito_mode)) {
-    return NULL;
+    return;
   }
 
-  DCHECK(allocator != NULL);
-  Node *result_node = NULL;
-  string key(str, size);
-
-  // Look for a starting point of iteration over dictionary contents.
+  // Find the starting point of iteration over dictionary contents.
   UserPOS::Token key_token;
-  key_token.key = key;
+  key.CopyToString(&key_token.key);
   vector<UserPOS::Token *>::const_iterator it =
-      lower_bound(tokens_->begin(), tokens_->end(), &key_token, POSTokenLess());
+      lower_bound(tokens_->begin(), tokens_->end(), &key_token, OrderByKey());
 
+  Token token;
   for (; it != tokens_->end(); ++it) {
     if (!Util::StartsWith((*it)->key, key)) {
       break;
     }
-    // check begin with
-    if (limit.begin_with_trie != NULL) {
-      string value;
-      size_t key_length = 0;
-      bool has_subtrie = false;
-      if (!limit.begin_with_trie->LookUpPrefix((*it)->key.data() + size, &value,
-                                               &key_length, &has_subtrie)) {
+    switch (callback->OnKey((*it)->key)) {
+      case Callback::TRAVERSE_DONE:
+        return;
+      case Callback::TRAVERSE_NEXT_KEY:
+      case Callback::TRAVERSE_CULL:
         continue;
-      }
+      default:
+        break;
     }
-
-    Node *new_node = allocator->NewNode();
-    DCHECK(new_node);
+    FillTokenFromUserPOSToken(**it, &token);
+    // Override POS IDs for suggest only words.
     if (pos_matcher_->IsSuggestOnlyWord((*it)->id)) {
-      new_node->lid = pos_matcher_->GetUnknownId();
-      new_node->rid = pos_matcher_->GetUnknownId();
-    } else {
-      new_node->lid = (*it)->id;
-      new_node->rid = (*it)->id;
+      token.lid = token.rid = pos_matcher_->GetUnknownId();
+    }
+    if (callback->OnToken((*it)->key, (*it)->key, token) ==
+        Callback::TRAVERSE_DONE) {
+      return;
     }
-    new_node->wcost = (*it)->cost;
-    new_node->key = (*it)->key;
-    new_node->value = (*it)->value;
-    new_node->node_type = Node::NOR_NODE;
-    new_node->attributes |= Node::NO_VARIANTS_EXPANSION;
-    new_node->attributes |= Node::USER_DICTIONARY;
-    new_node->bnext = result_node;
-    result_node = new_node;
   }
-  return result_node;
 }
 
-Node *UserDictionary::LookupPredictive(
-    const char *str, int size, NodeAllocatorInterface *allocator) const {
-  return LookupPredictiveWithLimit(str, size, empty_limit_, allocator);
-}
-
-Node *UserDictionary::LookupPrefixWithLimit(
-    const char *str,
-    int size,
-    const Limit &limit,
-    NodeAllocatorInterface *allocator) const {
+// UserDictionary doesn't support kana modifier insensitive lookup.
+void UserDictionary::LookupPrefix(
+    StringPiece key, bool /*use_kana_modifier_insensitive_lookup*/,
+    Callback *callback) const {
   scoped_reader_lock l(mutex_.get());
 
-  if (size == 0) {
+  if (key.empty()) {
     LOG(WARNING) << "string of length zero is passed.";
-    return NULL;
+    return;
   }
-
   if (tokens_->empty()) {
-    return NULL;
+    return;
   }
-
   if (GET_CONFIG(incognito_mode)) {
-    return NULL;
+    return;
   }
 
-
-  DCHECK(allocator != NULL);
-  Node *result_node = NULL;
-  string key(str, size);
-
-  // Look for a starting point of iteration over dictionary contents.
+  // Find the starting point for iteration over dictionary contents.
   UserPOS::Token key_token;
-  key_token.key = key.substr(0, Util::OneCharLen(key.c_str()));
+  key_token.key.assign(key.data(), Util::OneCharLen(key.data()));
   vector<UserPOS::Token *>::const_iterator it =
-      lower_bound(tokens_->begin(), tokens_->end(), &key_token, POSTokenLess());
+      lower_bound(tokens_->begin(), tokens_->end(), &key_token, OrderByKey());
 
+  Token token;
   for (; it != tokens_->end(); ++it) {
     if ((*it)->key > key) {
       break;
     }
-
     if (pos_matcher_->IsSuggestOnlyWord((*it)->id)) {
       continue;
     }
-
     if (!Util::StartsWith(key, (*it)->key)) {
       continue;
     }
-
-    // check the lower limit of key length
-    if ((*it)->key.size() < limit.key_len_lower_limit) {
-      continue;
+    switch (callback->OnKey((*it)->key)) {
+      case Callback::TRAVERSE_DONE:
+        return;
+      case Callback::TRAVERSE_NEXT_KEY:
+        continue;
+      case Callback::TRAVERSE_CULL:
+        LOG(FATAL) << "UserDictionary doesn't support culling.";
+        break;
+      default:
+        break;
+    }
+    FillTokenFromUserPOSToken(**it, &token);
+    switch (callback->OnToken((*it)->key, (*it)->key, token)) {
+      case Callback::TRAVERSE_DONE:
+        return;
+      case Callback::TRAVERSE_CULL:
+        LOG(FATAL) << "UserDictionary doesn't support culling.";
+        break;
+      default:
+        break;
     }
-
-    Node *new_node = allocator->NewNode();
-    DCHECK(new_node);
-    new_node->lid = (*it)->id;
-    new_node->rid = (*it)->id;
-    new_node->wcost = (*it)->cost;
-    new_node->key = (*it)->key;
-    new_node->value = (*it)->value;
-    new_node->node_type = Node::NOR_NODE;
-    new_node->attributes |= Node::NO_VARIANTS_EXPANSION;
-    new_node->attributes |= Node::USER_DICTIONARY;
-    new_node->bnext = result_node;
-    result_node = new_node;
   }
-
-  return result_node;
 }
 
-Node *UserDictionary::LookupPrefix(const char *str, int size,
-                                   NodeAllocatorInterface *allocator) const {
-  return LookupPrefixWithLimit(str, size, empty_limit_, allocator);
-}
-
-Node *UserDictionary::LookupExact(const char *str, int size,
-                                  NodeAllocatorInterface *allocator) const {
+void UserDictionary::LookupExact(StringPiece key, Callback *callback) const {
   scoped_reader_lock l(mutex_.get());
-  if (size == 0 || tokens_->empty() || GET_CONFIG(incognito_mode)) {
-    return NULL;
+  if (key.empty() || tokens_->empty() || GET_CONFIG(incognito_mode)) {
+    return;
   }
-  DCHECK(allocator);
   UserPOS::Token key_token;
-  key_token.key.assign(str, size);
+  key.CopyToString(&key_token.key);
   typedef vector<UserPOS::Token *>::const_iterator TokenIterator;
   pair<TokenIterator, TokenIterator> range =
-      equal_range(tokens_->begin(), tokens_->end(), &key_token, POSTokenLess());
+      equal_range(tokens_->begin(), tokens_->end(), &key_token, OrderByKey());
+  if (range.first == range.second) {
+    return;
+  }
+  if (callback->OnKey(key) != Callback::TRAVERSE_CONTINUE) {
+    return;
+  }
 
-  Node *head = NULL;
+  Token token;
   for (; range.first != range.second; ++range.first) {
-    const UserPOS::Token *token = *range.first;
-    if (pos_matcher_->IsSuggestOnlyWord(token->id)) {
+    const UserPOS::Token &user_pos_token = **range.first;
+    if (pos_matcher_->IsSuggestOnlyWord(user_pos_token.id)) {
       continue;
     }
-    Node *node = allocator->NewNode();
-    DCHECK(node);
-    node->lid = token->id;
-    node->rid = token->id;
-    node->wcost = token->cost;
-    node->key = token->key;
-    node->value = token->value;
-    node->node_type = Node::NOR_NODE;
-    node->attributes |= Node::NO_VARIANTS_EXPANSION;
-    node->attributes |= Node::USER_DICTIONARY;
-    node->bnext = head;
-    head = node;
+    FillTokenFromUserPOSToken(user_pos_token, &token);
+    if (callback->OnToken(key, key, token) != Callback::TRAVERSE_CONTINUE) {
+      return;
+    }
   }
-  return head;
 }
 
-Node *UserDictionary::LookupReverse(const char *str, int size,
-                                    NodeAllocatorInterface *allocator) const {
+void UserDictionary::LookupReverse(StringPiece str,
+                                   NodeAllocatorInterface *allocator,
+                                   Callback *callback) const {
   if (GET_CONFIG(incognito_mode)) {
-    return NULL;
+    return;
+  }
+}
+
+bool UserDictionary::LookupComment(StringPiece key, StringPiece value,
+                                   string *comment) const {
+  if (key.empty() || GET_CONFIG(incognito_mode)) {
+    return false;
   }
 
-  return NULL;
+  scoped_reader_lock l(mutex_.get());
+  if (tokens_->empty()) {
+    return false;
+  }
+
+  UserPOS::Token key_token;
+  key.CopyToString(&key_token.key);
+  typedef vector<UserPOS::Token *>::const_iterator TokenIterator;
+  pair<TokenIterator, TokenIterator> range =
+      equal_range(tokens_->begin(), tokens_->end(), &key_token, OrderByKey());
+
+  // Set the comment that was found first.
+  for (; range.first != range.second; ++range.first) {
+    const UserPOS::Token *token = *range.first;
+    if (token->value == value && !token->comment.empty()) {
+      comment->assign(token->comment);
+      return true;
+    }
+  }
+  return false;
 }
 
 bool UserDictionary::Reload() {
@@ -460,6 +484,32 @@ bool UserDictionary::Reload() {
   return true;
 }
 
+namespace {
+
+class FindValueCallback : public DictionaryInterface::Callback {
+ public:
+  explicit FindValueCallback(StringPiece value)
+      : value_(value), found_(false) {}
+
+  virtual ResultType OnToken(StringPiece,  // key
+                             StringPiece,  // actual_key
+                             const Token &token) {
+    if (token.value == value_) {
+      found_ = true;
+      return TRAVERSE_DONE;
+    }
+    return TRAVERSE_CONTINUE;
+  }
+
+  bool found() const { return found_; }
+
+ private:
+  const StringPiece value_;
+  bool found_;
+};
+
+}  // namespace
+
 bool UserDictionary::AddToAutoRegisteredDictionary(
     const string &key, const string &value,
     user_dictionary::UserDictionary::PosType pos) {
@@ -467,13 +517,11 @@ bool UserDictionary::AddToAutoRegisteredDictionary(
     return false;
   }
 
-  scoped_ptr<NodeAllocator> allocator(new NodeAllocator);
-  Node *result = LookupPrefix(key.data(), key.size(), allocator.get());
-  for (Node *node = result; node != NULL; node = node->bnext) {
-    if (node->key == key && node->value == value) {
-      // Already registered
-      return false;
-    }
+  FindValueCallback callback(value);
+  LookupExact(key, &callback);
+  if (callback.found()) {
+    // Already registered.
+    return false;
   }
 
   suppression_dictionary_->Lock();
@@ -497,7 +545,8 @@ void UserDictionary::Swap(TokensIndex *new_tokens) {
   delete old_tokens;
 }
 
-bool UserDictionary::Load(const UserDictionaryStorage &storage) {
+bool UserDictionary::Load(
+    const user_dictionary::UserDictionaryStorage &storage) {
   size_t size = 0;
   {
     scoped_reader_lock l(mutex_.get());
@@ -513,12 +562,13 @@ bool UserDictionary::Load(const UserDictionaryStorage &storage) {
 #endif
 
   if (size >= kVeryBigUserDictionarySize) {
-    TokensIndex *dummy_empty_tokens = new TokensIndex(user_pos_,
+    TokensIndex *dummy_empty_tokens = new TokensIndex(user_pos_.get(),
                                                       suppression_dictionary_);
     Swap(dummy_empty_tokens);
   }
 
-  TokensIndex *tokens = new TokensIndex(user_pos_, suppression_dictionary_);
+  TokensIndex *tokens = new TokensIndex(user_pos_.get(),
+                                        suppression_dictionary_);
   tokens->Load(storage);
   Swap(tokens);
   return true;
@@ -527,4 +577,5 @@ bool UserDictionary::Load(const UserDictionaryStorage &storage) {
 void UserDictionary::SetUserDictionaryName(const string &filename) {
   Singleton<UserDictionaryFileManager>::get()->SetFileName(filename);
 }
+
 }  // namespace mozc
