@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,11 @@
 
 #include "dictionary/user_dictionary_session_handler.h"
 
-#include "base/base.h"
+#include "base/file_util.h"
 #include "base/logging.h"
+#include "base/port.h"
+#include "base/protobuf/protobuf.h"
+#include "base/protobuf/repeated_field.h"
 #include "base/scoped_ptr.h"
 #include "base/util.h"
 #include "dictionary/user_dictionary_session.h"
@@ -88,8 +91,8 @@ bool UserDictionarySessionHandler::Evaluate(
     case UserDictionaryCommand::GET_ENTRY_SIZE:
       GetEntrySize(command, status);
       break;
-    case UserDictionaryCommand::GET_ENTRY:
-      GetEntry(command, status);
+    case UserDictionaryCommand::GET_ENTRIES:
+      GetEntries(command, status);
       break;
     case UserDictionaryCommand::CHECK_NEW_DICTIONARY_AVAILABILITY:
       CheckNewDictionaryAvailability(command, status);
@@ -118,6 +121,9 @@ bool UserDictionarySessionHandler::Evaluate(
     case UserDictionaryCommand::IMPORT_DATA:
       ImportData(command, status);
       break;
+    case UserDictionaryCommand::GET_STORAGE:
+      GetStorage(command, status);
+      break;
     default:
       status->set_status(UserDictionaryCommandStatus::UNKNOWN_COMMAND);
       break;
@@ -145,7 +151,7 @@ void UserDictionarySessionHandler::ClearStorage(
   // File operation is not supported on NaCl.
   status->set_status(UserDictionaryCommandStatus::UNKNOWN_ERROR);
 #else
-  Util::Unlink(dictionary_path_);
+  FileUtil::Unlink(dictionary_path_);
   status->set_status(
       UserDictionaryCommandStatus::USER_DICTIONARY_COMMAND_SUCCESS);
 #endif
@@ -294,7 +300,7 @@ void UserDictionarySessionHandler::GetEntrySize(
       UserDictionaryCommandStatus::USER_DICTIONARY_COMMAND_SUCCESS);
 }
 
-void UserDictionarySessionHandler::GetEntry(
+void UserDictionarySessionHandler::GetEntries(
     const UserDictionaryCommand &command,
     UserDictionaryCommandStatus *status) {
   UserDictionarySession *session = GetSession(command, status);
@@ -302,7 +308,7 @@ void UserDictionarySessionHandler::GetEntry(
     return;
   }
 
-  if (!command.has_dictionary_id() || command.entry_index_size() != 1) {
+  if (!command.has_dictionary_id() || command.entry_index_size() <= 0) {
     status->set_status(UserDictionaryCommandStatus::INVALID_ARGUMENT);
     return;
   }
@@ -313,13 +319,24 @@ void UserDictionarySessionHandler::GetEntry(
     status->set_status(UserDictionaryCommandStatus::UNKNOWN_DICTIONARY_ID);
     return;
   }
-  int index = command.entry_index(0);
-  if (index < 0 || dictionary->entries_size() <= index) {
-    status->set_status(UserDictionaryCommandStatus::ENTRY_INDEX_OUT_OF_RANGE);
-    return;
+
+  const int entry_index_size = command.entry_index_size();
+  for (int i = 0; i < entry_index_size; ++i) {
+    const int entry_index = command.entry_index(i);
+    if (entry_index < 0 || dictionary->entries_size() <= entry_index) {
+      status->set_status(UserDictionaryCommandStatus::ENTRY_INDEX_OUT_OF_RANGE);
+      return;
+    }
   }
 
-  status->mutable_entry()->CopyFrom(dictionary->entries(index));
+  mozc::protobuf::RepeatedPtrField<UserDictionary::Entry> *mutable_entries =
+      status->mutable_entries();
+  mutable_entries->Clear();
+  mutable_entries->Reserve(entry_index_size);
+  for (int i = 0; i < entry_index_size; ++i) {
+    const int entry_index = command.entry_index(i);
+    mutable_entries->Add()->CopyFrom(dictionary->entries(entry_index));
+  }
   status->set_status(
       UserDictionaryCommandStatus::USER_DICTIONARY_COMMAND_SUCCESS);
 }
@@ -522,6 +539,18 @@ void UserDictionarySessionHandler::ImportData(
   status->set_status(result_status);
 }
 
+void UserDictionarySessionHandler::GetStorage(
+    const UserDictionaryCommand &command,
+    UserDictionaryCommandStatus *status) {
+  UserDictionarySession *session = GetSession(command, status);
+  if (session == NULL) {
+    return;
+  }
+  status->mutable_storage()->CopyFrom(session->storage());
+  status->set_status(
+      UserDictionaryCommandStatus::USER_DICTIONARY_COMMAND_SUCCESS);
+}
+
 UserDictionarySession *UserDictionarySessionHandler::GetSession(
     const UserDictionaryCommand &command,
     UserDictionaryCommandStatus *status) {
@@ -541,11 +570,7 @@ UserDictionarySession *UserDictionarySessionHandler::GetSession(
 uint64 UserDictionarySessionHandler::CreateNewSessionId() const {
   uint64 id = kInvalidSessionId;
   while (true) {
-    if (!Util::GetSecureRandomSequence(
-            reinterpret_cast<char *>(&id), sizeof(id))) {
-      LOG(ERROR) << "GetSecureRandomSequence is failed. Use random instead.";
-      id = static_cast<uint64>(Util::Random(RAND_MAX));
-    }
+    Util::GetRandomSequence(reinterpret_cast<char *>(&id), sizeof(id));
 
     if (id != kInvalidSessionId &&
         (session_.get() == NULL || session_id_ != id)) {
