@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,26 +29,22 @@
 
 #include "dictionary/user_dictionary_storage.h"
 
-#ifndef OS_WINDOWS
-#include <sys/stat.h>
-#endif
-
-#include <algorithm>
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/file_stream.h"
+#include "base/file_util.h"
 #include "base/logging.h"
 #include "base/mmap.h"
 #include "base/number_util.h"
 #include "base/protobuf/unknown_field_set.h"
-#include "base/testing_util.h"
+#include "base/system_util.h"
 #include "base/util.h"
 #include "dictionary/user_dictionary_importer.h"
 #include "dictionary/user_dictionary_util.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
+#include "testing/base/public/testing_util.h"
 
 DECLARE_string(test_tmpdir);
 
@@ -65,9 +61,9 @@ string GenRandomString(int size) {
   string result;
   const size_t len = Util::Random(size) + 1;
   for (int i = 0; i < len; ++i) {
-    const uint16 l =
-        static_cast<uint16>(Util::Random(static_cast<int>('~' - ' ')) + ' ');
-    Util::UCS2ToUTF8Append(l, &result);
+    const char32 l =
+        static_cast<char32>(Util::Random(static_cast<int>('~' - ' ')) + ' ');
+    Util::UCS4ToUTF8Append(l, &result);
   }
   return result;
 }
@@ -77,22 +73,18 @@ string GenRandomString(int size) {
 class UserDictionaryStorageTest : public ::testing::Test {
  protected:
   virtual void SetUp() {
-    backup_user_profile_directory_ = Util::GetUserProfileDirectory();
-    Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-#ifndef OS_WINDOWS
-    // TODO(hidehiko): Do we really need this?
-    chmod(FLAGS_test_tmpdir.c_str(), 0777);
-#endif
-    Util::Unlink(GetUserDictionaryFile());
+    backup_user_profile_directory_ = SystemUtil::GetUserProfileDirectory();
+    SystemUtil::SetUserProfileDirectory(FLAGS_test_tmpdir);
+    FileUtil::Unlink(GetUserDictionaryFile());
   }
 
   virtual void TearDown() {
-    Util::Unlink(GetUserDictionaryFile());
-    Util::SetUserProfileDirectory(backup_user_profile_directory_);
+    FileUtil::Unlink(GetUserDictionaryFile());
+    SystemUtil::SetUserProfileDirectory(backup_user_profile_directory_);
   }
 
   static string GetUserDictionaryFile() {
-    return Util::JoinPath(FLAGS_test_tmpdir, "test.db");
+    return FileUtil::JoinPath(FLAGS_test_tmpdir, "test.db");
   }
 
  private:
@@ -108,102 +100,24 @@ TEST_F(UserDictionaryStorageTest, FileTest) {
 TEST_F(UserDictionaryStorageTest, LockTest) {
   UserDictionaryStorage storage1(GetUserDictionaryFile());
   UserDictionaryStorage storage2(GetUserDictionaryFile());
+
+  EXPECT_FALSE(storage1.Save());
+  EXPECT_FALSE(storage2.Save());
+
   EXPECT_TRUE(storage1.Lock());
   EXPECT_FALSE(storage2.Lock());
+  EXPECT_TRUE(storage1.Save());
   EXPECT_FALSE(storage2.Save());
+
   EXPECT_TRUE(storage1.UnLock());
+  EXPECT_FALSE(storage1.Save());
+  EXPECT_FALSE(storage2.Save());
+
   EXPECT_TRUE(storage2.Lock());
+  EXPECT_FALSE(storage1.Save());
   EXPECT_TRUE(storage2.Save());
 }
 
-TEST_F(UserDictionaryStorageTest, SyncDictionaryBinarySizeTest) {
-  UserDictionaryStorage storage(GetUserDictionaryFile());
-  EXPECT_FALSE(storage.Load());
-#ifndef ENABLE_CLOUD_SYNC
-  // In binaries built without sync feature, we should make sync dictionary
-  // intentionally.
-  storage.EnsureSyncDictionaryExists();
-#endif  // ENABLE_CLOUD_SYNC
-
-  EXPECT_TRUE(storage.Lock());
-
-  EXPECT_TRUE(storage.Save());
-  {
-    // Creates an entry which exceeds the sync dictionary limit
-    // outside of the sync dictionary.
-    UserDictionaryStorage::UserDictionary *dic = storage.add_dictionaries();
-    dic->set_name("foo");
-    dic->set_syncable(false);
-    UserDictionaryStorage::UserDictionaryEntry *entry =
-        dic->add_entries();
-    entry->set_key(string(UserDictionaryStorage::max_sync_binary_size(), 'a'));
-    entry->set_value("value");
-    entry->set_pos(UserDictionary::NOUN);
-  }
-  // Okay to store such entry in the normal dictionary.
-  EXPECT_TRUE(storage.Save());
-
-  {
-    // Same on the sync dictionary.
-    UserDictionaryStorage::UserDictionary *dic =
-        storage.mutable_dictionaries(0);
-    EXPECT_TRUE(dic->syncable());  // just in case.
-    UserDictionaryStorage::UserDictionaryEntry *entry =
-        dic->add_entries();
-    entry->set_key(string(UserDictionaryStorage::max_sync_binary_size(), 'a'));
-    entry->set_value("value");
-    entry->set_pos(UserDictionary::NOUN);
-  }
-  // Such size is unacceptable for sync.
-  EXPECT_FALSE(storage.Save());
-
-  EXPECT_TRUE(storage.UnLock());
-}
-
-TEST_F(UserDictionaryStorageTest, SyncDictionaryEntrySizeTest) {
-  UserDictionaryStorage storage(GetUserDictionaryFile());
-  EXPECT_FALSE(storage.Load());
-#ifndef ENABLE_CLOUD_SYNC
-  // In binaries built without sync feature, we should make sync dictionary
-  // intentionally.
-  storage.EnsureSyncDictionaryExists();
-#endif  // ENABLE_CLOUD_SYNC
-
-  EXPECT_TRUE(storage.Lock());
-
-  EXPECT_TRUE(storage.Save());
-  {
-    // Creates an entry which exceeds the sync dictionary limit
-    // outside of the sync dictionary.
-    UserDictionaryStorage::UserDictionary *dic = storage.add_dictionaries();
-    dic->set_name("foo");
-    dic->set_syncable(false);
-    for (size_t i = 0; i < UserDictionaryStorage::max_sync_entry_size() + 1;
-         ++i) {
-      // add empty entries.
-      dic->add_entries();
-    }
-  }
-  // Okay to store such entry in the normal dictionary.
-  EXPECT_TRUE(storage.Save());
-
-  {
-    // Same on the sync dictionary.
-    UserDictionaryStorage::UserDictionary *dic =
-        storage.mutable_dictionaries(0);
-    EXPECT_TRUE(dic->syncable());  // just in case.
-    for (size_t i = 0; i < UserDictionaryStorage::max_sync_entry_size() + 1;
-         ++i) {
-      // add empty entries.
-      dic->add_entries();
-    }
-  }
-  // Such entry size is unacceptable for sync.
-  EXPECT_FALSE(storage.Save());
-
-  EXPECT_TRUE(storage.UnLock());
-}
-
 TEST_F(UserDictionaryStorageTest, BasicOperationsTest) {
   UserDictionaryStorage storage(GetUserDictionaryFile());
   EXPECT_FALSE(storage.Load());
@@ -214,8 +128,9 @@ TEST_F(UserDictionaryStorageTest, BasicOperationsTest) {
   const size_t dict_size = storage.dictionaries_size();
 
   for (size_t i = 0; i < kDictionariesSize; ++i) {
-    EXPECT_TRUE(storage.CreateDictionary("test" + NumberUtil::SimpleItoa(i),
-                                         &id[i]));
+    EXPECT_TRUE(storage.CreateDictionary(
+        "test" + NumberUtil::SimpleItoa(static_cast<uint32>(i)),
+        &id[i]));
     EXPECT_EQ(i + 1 + dict_size, storage.dictionaries_size());
   }
 
@@ -269,8 +184,9 @@ TEST_F(UserDictionaryStorageTest, DeleteTest) {
     storage.Clear();
     vector<uint64> ids(100);
     for (size_t i = 0; i < ids.size(); ++i) {
-      EXPECT_TRUE(storage.CreateDictionary("test" + NumberUtil::SimpleItoa(i),
-                                           &ids[i]));
+      EXPECT_TRUE(storage.CreateDictionary(
+          "test" + NumberUtil::SimpleItoa(static_cast<uint32>(i)),
+          &ids[i]));
     }
 
     vector<uint64> alive;
@@ -301,7 +217,7 @@ TEST_F(UserDictionaryStorageTest, ExportTest) {
 
   for (size_t i = 0; i < 1000; ++i) {
     UserDictionaryStorage::UserDictionaryEntry *entry = dic->add_entries();
-    const string prefix = NumberUtil::SimpleItoa(i);
+    const string prefix = NumberUtil::SimpleItoa(static_cast<uint32>(i));
     // set empty fields randomly
     entry->set_key(prefix + "key");
     entry->set_value(prefix + "value");
@@ -310,17 +226,26 @@ TEST_F(UserDictionaryStorageTest, ExportTest) {
     entry->set_comment(prefix + "comment");
   }
 
-  const string export_file = Util::JoinPath(FLAGS_test_tmpdir,
-                                            "export.txt");
+  const string export_file = FileUtil::JoinPath(FLAGS_test_tmpdir,
+                                                "export.txt");
 
   EXPECT_FALSE(storage.ExportDictionary(id + 1, export_file));
   EXPECT_TRUE(storage.ExportDictionary(id, export_file));
 
-  UserDictionaryStorage::UserDictionary dic2;
-  InputFileStream ifs(export_file.c_str());
-  CHECK(ifs);
-  UserDictionaryImporter::IStreamTextLineIterator iter(&ifs);
+  string file_string;
+  // Copy whole contents of the file into |file_string|.
+  {
+    InputFileStream ifs(export_file.c_str());
+    CHECK(ifs);
+    ifs.seekg(0, std::ios::end);
+    file_string.resize(ifs.tellg());
+    ifs.seekg(0, std::ios::beg);
+    ifs.read(&file_string[0], file_string.size());
+    ifs.close();
+  }
+  UserDictionaryImporter::StringTextLineIterator iter(file_string);
 
+  UserDictionaryStorage::UserDictionary dic2;
   EXPECT_EQ(UserDictionaryImporter::IMPORT_NO_ERROR,
             UserDictionaryImporter::ImportFromTextLineIterator(
                 UserDictionaryImporter::MOZC,
@@ -335,7 +260,7 @@ TEST_F(UserDictionaryStorageTest, ExportTest) {
 TEST_F(UserDictionaryStorageTest, SerializeTest) {
   // repeat 20 times
   for (int i = 0; i < 20; ++i) {
-    Util::Unlink(GetUserDictionaryFile());
+    FileUtil::Unlink(GetUserDictionaryFile());
     UserDictionaryStorage storage1(GetUserDictionaryFile());
 
     {
@@ -345,8 +270,8 @@ TEST_F(UserDictionaryStorageTest, SerializeTest) {
       for (size_t i = 0; i < dic_size; ++i) {
         uint64 id = 0;
         EXPECT_TRUE(
-            storage1.CreateDictionary("test" + NumberUtil::SimpleItoa(i),
-                                      &id));
+            storage1.CreateDictionary(
+                "test" + NumberUtil::SimpleItoa(static_cast<uint32>(i)), &id));
         const size_t entry_size = Util::Random(100) + 1;
         for (size_t j = 0; j < entry_size; ++j) {
           UserDictionaryStorage::UserDictionary *dic =
@@ -392,203 +317,126 @@ TEST_F(UserDictionaryStorageTest, GetUserDictionaryIdTest) {
   EXPECT_EQ(ret_id[1], id[1]);
 }
 
-// Following test is valid only when sync feature is enabled.
-#ifdef ENABLE_CLOUD_SYNC
-TEST_F(UserDictionaryStorageTest, SyncDictionaryTests) {
-  UserDictionaryStorage storage(GetUserDictionaryFile());
-  EXPECT_FALSE(storage.Load());
-
-  // Always the storage has the sync dictionary.
-  EXPECT_EQ(1, storage.dictionaries_size());
-  const UserDictionaryStorage::UserDictionary &sync_dict =
-      storage.dictionaries(0);
-  uint64 sync_dict_id = sync_dict.id();
-  EXPECT_NE(0, sync_dict_id);
-  EXPECT_TRUE(sync_dict.syncable());
-
-  uint64 normal_dict_id = 0;
-  EXPECT_TRUE(storage.CreateDictionary("test", &normal_dict_id));
-  EXPECT_NE(sync_dict_id, normal_dict_id);
-
-  // Storing the file.
-  EXPECT_TRUE(storage.Lock());
-  EXPECT_TRUE(storage.Save());
-  EXPECT_TRUE(storage.UnLock());
-
-  // Clear and reload from the file again.
-  storage.Clear();
-  EXPECT_TRUE(storage.Load());
-  // Regardless of the existence of the user dictionary file,
-  // UserDictionaryStorage::Load() does not append a new sync dictionary if it
-  // already contains at least one sync dictionary.
-  EXPECT_EQ(2, storage.dictionaries_size());
-  bool has_sync_dict = false;
-  bool has_normal_dict = false;
-  for (size_t i = 0; i < storage.dictionaries_size(); ++i) {
-    const UserDictionaryStorage::UserDictionary &dict =
-        storage.dictionaries(i);
-    if (dict.syncable()) {
-      // sync dictionary
-      EXPECT_EQ(sync_dict_id, dict.id());
-      has_sync_dict = true;
-    } else {
-      EXPECT_EQ(normal_dict_id, dict.id());
-      has_normal_dict = true;
-    }
-  }
-  EXPECT_TRUE(has_sync_dict);
-  EXPECT_TRUE(has_normal_dict);
-}
-#endif  // ENABLE_CLOUD_SYNC
-
-TEST_F(UserDictionaryStorageTest, SyncDictionaryOperations) {
-  UserDictionaryStorage storage(GetUserDictionaryFile());
-  EXPECT_FALSE(storage.Load());
-#ifndef ENABLE_CLOUD_SYNC
-  // In binaries built without sync feature, we should make sync dictionary
-  // intentionally.
-  storage.EnsureSyncDictionaryExists();
-#endif  // ENABLE_CLOUD_SYNC
-
-  uint64 sync_dic_id = 0;
-  EXPECT_TRUE(storage.GetUserDictionaryId(
-      UserDictionaryStorage::default_sync_dictionary_name(), &sync_dic_id));
-
-  // Cannot delete.
-  EXPECT_FALSE(storage.DeleteDictionary(sync_dic_id));
-  // Cannot rename.
-  EXPECT_FALSE(storage.RenameDictionary(sync_dic_id, "foo"));
-  // Cannot copy.
-  uint64 dummy_id = 0;
-  EXPECT_FALSE(storage.CopyDictionary(sync_dic_id, "bar", &dummy_id));
-  EXPECT_EQ(0, dummy_id);
-}
-
-TEST_F(UserDictionaryStorageTest, CreateDefaultSyncDictionaryIfNeccesary) {
-  UserDictionaryStorage storage(GetUserDictionaryFile());
-  EXPECT_FALSE(storage.Load());
-#ifdef ENABLE_CLOUD_SYNC
-  EXPECT_GT(UserDictionaryStorage::CountSyncableDictionaries(&storage), 0) <<
-      "When ENABLE_CLOUD_SYNC is defined, at least one sync dictionary should "
-      "exist after |storage.Load() is finished. Currently this is by design.";
-#else
-  EXPECT_EQ(0, UserDictionaryStorage::CountSyncableDictionaries(&storage)) <<
-      "When ENABLE_CLOUD_SYNC is not defined, there should be no sync "
-      "dictionary by default.";
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-#ifndef ENABLE_CLOUD_SYNC
-TEST_F(UserDictionaryStorageTest, Issue6004671) {
-  { // Emulate the situation in b/6004671.
-    UserDictionaryStorage storage(GetUserDictionaryFile());
-    EXPECT_FALSE(storage.Load());
-    ASSERT_TRUE(storage.EnsureSyncDictionaryExists());
-    ASSERT_GT(UserDictionaryStorage::CountSyncableDictionaries(&storage), 0);
-    ASSERT_TRUE(storage.Lock());
-    ASSERT_TRUE(storage.Save());
-    ASSERT_TRUE(storage.UnLock());
-  }
-
-  {
-    UserDictionaryStorage storage(GetUserDictionaryFile());
-    ASSERT_TRUE(storage.Load());
-    EXPECT_EQ(0, UserDictionaryStorage::CountSyncableDictionaries(&storage))
-        << "To work around Issue 6004671, we silently remove an empty sync "
-           "dictionary in UserDictionaryStorage::Load() when "
-           "ENABLE_CLOUD_SYNC is not defined.";
-  }
-}
-#endif  // ENABLE_CLOUD_SYNC
-
-TEST_F(UserDictionaryStorageTest, RemoveUnusedSyncDictionariesIfExist) {
+TEST_F(UserDictionaryStorageTest, ConvertSyncDictionariesToNormalDictionaries) {
   // "名詞"
-  static const UserDictionary::PosType kPos = UserDictionary::NOUN;
+  const UserDictionary::PosType kPos = UserDictionary::NOUN;
 
-  {
-    UserDictionaryStorage storage(GetUserDictionaryFile());
-    EXPECT_FALSE(storage.LoadWithoutChangingSyncDictionary()) <<
-        "At first, we expect there is not user dictionary file.";
+  const struct TestData {
+    bool is_sync_dictionary;
+    bool is_removed_dictionary;
+    bool has_normal_entry;
+    bool has_removed_entry;
+    string dictionary_name;
+  } test_data[] = {
+    { false, false, false, false, "non-sync dictionary (empty)" },
+    { false, false, true, false, "non-sync dictionary (normal entry)" },
+    { true, false, false, false, "sync dictionary (empty)" },
+    { true, false, false, true, "sync dictionary (removed entry)" },
+    { true, false, true, false, "sync dictionary (normal entry)" },
+    { true, false, true, true, "sync dictionary (normal & removed entries)" },
+    { true, true, false, false, "removed sync dictionary (empty)" },
+    { true, true, false, true, "removed sync dictionary (removed entry)" },
+    { true, true, true, false, "removed sync dictionary (normal entry)" },
+    { true, true, true, true,
+      "removed sync dictionary (normal & removed entries)" },
+    { true, false, true, false,
+      UserDictionaryStorage::default_sync_dictionary_name() },
+  };
 
-    { // Add an non-empty non-sync dictionary.
-      uint64 dict_id = 0;
-      ASSERT_TRUE(storage.CreateDictionary(
-          "Non-empty non-sync Dictionary", &dict_id));
-      UserDictionaryStorage::UserDictionary *dict =
-          storage.mutable_dictionaries(
-              storage.GetUserDictionaryIndex(dict_id));
-      dict->set_syncable(false);
+  UserDictionaryStorage storage(GetUserDictionaryFile());
+  EXPECT_FALSE(storage.Load())
+      << "At first, we expect there is not user dictionary file.";
+  EXPECT_FALSE(storage.ConvertSyncDictionariesToNormalDictionaries())
+      << "No sync dictionary available.";
+
+  for (size_t i = 0; i < arraysize(test_data); ++i) {
+    SCOPED_TRACE(Util::StringPrintf("add %d", static_cast<int>(i)));
+    const TestData &data = test_data[i];
+    CHECK(data.is_sync_dictionary ||
+          !(data.is_removed_dictionary || data.has_removed_entry))
+        << "Non-sync dictionary should NOT have removed dictionary / entry.";
+
+    uint64 dict_id = 0;
+    ASSERT_TRUE(storage.CreateDictionary(data.dictionary_name, &dict_id));
+    UserDictionaryStorage::UserDictionary *dict =
+        storage.mutable_dictionaries(storage.GetUserDictionaryIndex(dict_id));
+    dict->set_syncable(data.is_sync_dictionary);
+    dict->set_removed(data.is_removed_dictionary);
+    if (data.has_normal_entry) {
       UserDictionaryStorage::UserDictionaryEntry *entry = dict->add_entries();
-      entry->set_key("A");
-      entry->set_value("AA");
+      entry->set_key("normal");
+      entry->set_value("normal entry");
       entry->set_pos(kPos);
     }
-
-    { // Add an empty non-sync dictionary.
-      uint64 dict_id = 0;
-      ASSERT_TRUE(storage.CreateDictionary(
-          "Empty non-sync Dictionary", &dict_id));
-      UserDictionaryStorage::UserDictionary *dict =
-          storage.mutable_dictionaries(
-              storage.GetUserDictionaryIndex(dict_id));
-      dict->set_syncable(false);
-    }
-
-    { // Add an non-empty sync dictionary.
-      uint64 dict_id = 0;
-      ASSERT_TRUE(storage.CreateDictionary(
-          "Non-empty sync Dictionary", &dict_id));
-      UserDictionaryStorage::UserDictionary *dict =
-          storage.mutable_dictionaries(
-              storage.GetUserDictionaryIndex(dict_id));
-      dict->set_syncable(true);
+    if (data.has_removed_entry) {
       UserDictionaryStorage::UserDictionaryEntry *entry = dict->add_entries();
-      entry->set_key("A");
-      entry->set_value("AA");
+      entry->set_key("removed");
+      entry->set_value("removed entry");
       entry->set_pos(kPos);
+      entry->set_removed(true);
     }
+  }
+  EXPECT_EQ(9, UserDictionaryStorage::CountSyncableDictionaries(storage));
 
-    { // Add an empty sync dictionary.
-      uint64 dict_id = 0;
-      ASSERT_TRUE(storage.CreateDictionary("Empty sync Dictionary", &dict_id));
-      UserDictionaryStorage::UserDictionary *dict =
-          storage.mutable_dictionaries(
-              storage.GetUserDictionaryIndex(dict_id));
-      dict->set_syncable(true);
+  ASSERT_TRUE(storage.ConvertSyncDictionariesToNormalDictionaries());
+
+  // "同期用辞書"
+  const char *kDictionaryNameConvertedFromSyncableDictionary =
+      "\xE5\x90\x8C\xE6\x9C\x9F\xE7\x94\xA8\xE8\xBE\x9E\xE6\x9B\xB8";
+  const struct ExpectedData {
+    bool has_normal_entry;
+    string dictionary_name;
+  } expected_data[] = {
+    { false, "non-sync dictionary (empty)" },
+    { true, "non-sync dictionary (normal entry)" },
+    { true, "sync dictionary (normal entry)" },
+    { true, "sync dictionary (normal & removed entries)" },
+    { true, kDictionaryNameConvertedFromSyncableDictionary },
+  };
+
+  EXPECT_EQ(0, UserDictionaryStorage::CountSyncableDictionaries(storage));
+  ASSERT_EQ(arraysize(expected_data), storage.dictionaries_size());
+  for (size_t i = 0; i < arraysize(expected_data); ++i) {
+    SCOPED_TRACE(Util::StringPrintf("verify %d", static_cast<int>(i)));
+    const ExpectedData &expected = expected_data[i];
+    const UserDictionaryStorage::UserDictionary &dict = storage.dictionaries(i);
+
+    EXPECT_EQ(expected.dictionary_name, dict.name());
+    EXPECT_FALSE(dict.syncable());
+    EXPECT_FALSE(dict.removed());
+    if (expected.has_normal_entry) {
+      ASSERT_EQ(1, dict.entries_size());
+      EXPECT_EQ("normal", dict.entries(0).key());
+    } else {
+      EXPECT_EQ(0, dict.entries_size());
     }
-
-    EXPECT_GT(UserDictionaryStorage::CountSyncableDictionaries(&storage), 0);
-    EXPECT_TRUE(storage.Lock());
-    EXPECT_TRUE(storage.Save());
-    EXPECT_TRUE(storage.UnLock());
   }
 
-  ASSERT_TRUE(Util::FileExists(GetUserDictionaryFile()))
-      << "A temporary user dictionary file shoudl exist.";
-
-  // Remove unused sync dictionaries.
+  // Test duplicated dictionary name.
+  storage.Clear();
   {
-    UserDictionaryStorage storage(GetUserDictionaryFile());
-    ASSERT_TRUE(storage.LoadWithoutChangingSyncDictionary()) <<
-        "Failed to load test user dictionary file.";
-    EXPECT_EQ(2, UserDictionaryStorage::CountSyncableDictionaries(&storage));
-    storage.RemoveUnusedSyncDictionariesIfExist();
-    EXPECT_EQ(1, UserDictionaryStorage::CountSyncableDictionaries(&storage))
-        << "One sync dictionary should be removed because if is empty.";
-    EXPECT_TRUE(storage.Lock());
-    EXPECT_TRUE(storage.Save());
-    EXPECT_TRUE(storage.UnLock());
-  }
-
-  // Reload and test it again.
-  {
-    UserDictionaryStorage storage(GetUserDictionaryFile());
-    ASSERT_TRUE(storage.LoadWithoutChangingSyncDictionary()) <<
-        "Failed to load test user dictionary file.";
-    EXPECT_EQ(1, UserDictionaryStorage::CountSyncableDictionaries(&storage));
-    storage.RemoveUnusedSyncDictionariesIfExist();
-    EXPECT_EQ(1, UserDictionaryStorage::CountSyncableDictionaries(&storage));
+    uint64 dict_id = 0;
+    storage.CreateDictionary(
+        UserDictionaryStorage::default_sync_dictionary_name(), &dict_id);
+    storage.CreateDictionary(
+        kDictionaryNameConvertedFromSyncableDictionary, &dict_id);
+    ASSERT_EQ(2, storage.dictionaries_size());
+    UserDictionaryStorage::UserDictionary *dict;
+    dict = storage.mutable_dictionaries(0);
+    dict->set_syncable(true);
+    dict->add_entries()->set_key("0");
+    dict = storage.mutable_dictionaries(1);
+    dict->set_syncable(false);
+    dict->add_entries()->set_key("1");
   }
+  ASSERT_TRUE(storage.ConvertSyncDictionariesToNormalDictionaries());
+  EXPECT_EQ(0, UserDictionaryStorage::CountSyncableDictionaries(storage));
+  EXPECT_EQ(2, storage.dictionaries_size());
+  EXPECT_EQ(Util::StringPrintf("%s_1",
+                               kDictionaryNameConvertedFromSyncableDictionary),
+            storage.dictionaries(0).name());
+  EXPECT_EQ(kDictionaryNameConvertedFromSyncableDictionary,
+            storage.dictionaries(1).name());
 }
 
 TEST_F(UserDictionaryStorageTest, AddToAutoRegisteredDictionary) {
@@ -619,7 +467,7 @@ TEST_F(UserDictionaryStorageTest, AddToAutoRegisteredDictionary) {
   }
 
   {
-    Util::Unlink(GetUserDictionaryFile());
+    FileUtil::Unlink(GetUserDictionaryFile());
     UserDictionaryStorage storage(GetUserDictionaryFile());
     storage.Lock();
     // Already locked.
@@ -655,13 +503,7 @@ TEST_F(UserDictionaryStorageTest, BackwardCompatibilityTest) {
   UserDictionaryStorage storage(GetUserDictionaryFile());
   ASSERT_TRUE(storage.LoadWithoutMigration());
 
-#ifndef ENABLE_CLOUD_SYNC
-  // In binaries built without sync feature, we should make sync dictionary
-  // intentionally.
-  storage.EnsureSyncDictionaryExists();
-#endif  // ENABLE_CLOUD_SYNC
-
-  ASSERT_EQ(2, storage.dictionaries_size());
+  ASSERT_EQ(1, storage.dictionaries_size());
   const UserDictionary &dictionary = storage.dictionaries(0);
   ASSERT_EQ(1, dictionary.entries_size());
   const UserDictionary::Entry &entry = dictionary.entries(0);
@@ -676,7 +518,7 @@ TEST_F(UserDictionaryStorageTest, BackwardCompatibilityTest) {
 
 TEST_F(UserDictionaryStorageTest, Export) {
   const int kDummyDictionaryId = 10;
-  const string kPath = Util::JoinPath(FLAGS_test_tmpdir, "exported_file");
+  const string kPath = FileUtil::JoinPath(FLAGS_test_tmpdir, "exported_file");
 
   {
     UserDictionaryStorage storage(GetUserDictionaryFile());
@@ -698,13 +540,13 @@ TEST_F(UserDictionaryStorageTest, Export) {
   // Make sure the exported format, especially that the pos is exported in
   // Japanese.
   // "key value 名詞 comment" separted by a tab character.
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   EXPECT_EQ("key\tvalue\t\xE5\x90\x8D\xE8\xA9\x9E\tcomment\r\n",
             string(mapped_data.begin(), mapped_data.size()));
 #else
   EXPECT_EQ("key\tvalue\t\xE5\x90\x8D\xE8\xA9\x9E\tcomment\n",
             string(mapped_data.begin(), mapped_data.size()));
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 namespace {
@@ -771,13 +613,7 @@ TEST_P(UserDictionaryStorageMigrationTest, Load) {
   UserDictionaryStorage storage(GetUserDictionaryFile());
   ASSERT_TRUE(storage.Load());
 
-#ifndef ENABLE_CLOUD_SYNC
-  // In binaries built without sync feature, we should make sync dictionary
-  // intentionally.
-  storage.EnsureSyncDictionaryExists();
-#endif  // ENABLE_CLOUD_SYNC
-
-  ASSERT_EQ(2, storage.dictionaries_size()) << storage.Utf8DebugString();
+  ASSERT_EQ(1, storage.dictionaries_size()) << storage.Utf8DebugString();
   const UserDictionary &dictionary = storage.dictionaries(0);
   ASSERT_EQ(UserDictionary::PosType_MAX - UserDictionary::PosType_MIN + 1,
             dictionary.entries_size())
@@ -807,13 +643,7 @@ TEST_P(UserDictionaryStorageMigrationTest, LoadWithoutMigration) {
   UserDictionaryStorage storage(GetUserDictionaryFile());
   ASSERT_TRUE(storage.LoadWithoutMigration());
 
-#ifndef ENABLE_CLOUD_SYNC
-  // In binaries built without sync feature, we should make sync dictionary
-  // intentionally.
-  storage.EnsureSyncDictionaryExists();
-#endif  // ENABLE_CLOUD_SYNC
-
-  ASSERT_EQ(2, storage.dictionaries_size()) << storage.Utf8DebugString();
+  ASSERT_EQ(1, storage.dictionaries_size()) << storage.Utf8DebugString();
   const UserDictionary &dictionary = storage.dictionaries(0);
   ASSERT_EQ(UserDictionary::PosType_MAX - UserDictionary::PosType_MIN + 1,
             dictionary.entries_size())
