@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,36 +32,38 @@
 #include "dictionary/user_dictionary.h"
 
 #include <algorithm>
-#include <set>
+#include <cstring>
 #include <sstream>
 #include <string>
 #include <vector>
 
-#include "base/base.h"
-#include "base/file_stream.h"
+#include "base/file_util.h"
 #include "base/logging.h"
 #include "base/number_util.h"
+#include "base/port.h"
 #include "base/singleton.h"
+#include "base/system_util.h"
 #include "base/trie.h"
 #include "base/util.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
-#include "converter/node.h"
-#include "converter/node_allocator.h"
 #include "data_manager/testing/mock_user_pos_manager.h"
+#include "dictionary/dictionary_test_util.h"
+#include "dictionary/dictionary_token.h"
 #include "dictionary/pos_matcher.h"
 #include "dictionary/suppression_dictionary.h"
 #include "dictionary/user_dictionary_storage.h"
-#include "dictionary/user_dictionary_util.h"
 #include "dictionary/user_pos.h"
 #include "dictionary/user_pos_interface.h"
-#include "storage/registry.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
-#include "usage_stats/usage_stats.pb.h"
+#include "usage_stats/usage_stats.h"
+#include "usage_stats/usage_stats_testing_util.h"
 
 DECLARE_string(test_tmpdir);
 
+using mozc::dictionary::CollectTokenCallback;
+
 namespace mozc {
 namespace {
 
@@ -85,7 +87,17 @@ const char kUserDictionary0[] =
     // Empty POS
     "star\tvalue\t\n"
     // Duplicate entry
-    "start\tstart\tverb\n";
+    "start\tstart\tverb\n"
+    // The following are for tests for LookupComment
+    // No comment
+    "comment_key1\tcomment_value1\tnoun\n"
+    // Has comment
+    "comment_key2\tcomment_value2\tnoun\tcomment\n"
+    // Different POS
+    "comment_key3\tcomment_value3\tnoun\tcomment1\n"
+    "comment_key3\tcomment_value3\tverb\tcomment2\n"
+    // White spaces comment
+    "comment_key4\tcomment_value4\tverb\t     \n";
 
 const char kUserDictionary1[] = "end\tend\tverb\n";
 
@@ -174,8 +186,8 @@ string GenRandomAlphabet(int size) {
   string result;
   const size_t len = Util::Random(size) + 1;
   for (int i = 0; i < len; ++i) {
-    const uint16 l = Util::Random(static_cast<int>('z' - 'a')) + 'a';
-    Util::UCS2ToUTF8Append(l, &result);
+    const char32 l = Util::Random(static_cast<int>('z' - 'a')) + 'a';
+    Util::UCS4ToUTF8Append(l, &result);
   }
   return result;
 }
@@ -183,16 +195,14 @@ string GenRandomAlphabet(int size) {
 class UserDictionaryTest : public ::testing::Test {
  protected:
   virtual void SetUp() {
-    pos_mock_.reset(new UserPOSMock);
-    CHECK(pos_mock_.get() != NULL);
-    Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-    CHECK(storage::Registry::Clear());
+    SystemUtil::SetUserProfileDirectory(FLAGS_test_tmpdir);
     suppression_dictionary_.reset(new SuppressionDictionary);
+
+    mozc::usage_stats::UsageStats::ClearAllStatsForTest();
   }
 
   virtual void TearDown() {
-    pos_mock_.reset();
-    CHECK(storage::Registry::Clear());
+    mozc::usage_stats::UsageStats::ClearAllStatsForTest();
   }
 
   // Workaround for the constructor of UserDictionary being protected.
@@ -200,7 +210,7 @@ class UserDictionaryTest : public ::testing::Test {
   UserDictionary *CreateDictionaryWithMockPos() {
     const testing::MockUserPosManager user_pos_manager;
     return new UserDictionary(
-        pos_mock_.get(),
+        new UserPOSMock(),
         user_pos_manager.GetPOSMatcher(),
         suppression_dictionary_.get());
   }
@@ -208,7 +218,7 @@ class UserDictionaryTest : public ::testing::Test {
   // Creates a user dictionary with actual pos data.
   UserDictionary *CreateDictionary() {
     const testing::MockUserPosManager user_pos_manager;
-    return new UserDictionary(user_pos_manager.GetUserPOS(),
+    return new UserDictionary(new UserPOS(user_pos_manager.GetUserPOSData()),
                               user_pos_manager.GetPOSMatcher(),
                               Singleton<SuppressionDictionary>::get());
   }
@@ -220,19 +230,40 @@ class UserDictionaryTest : public ::testing::Test {
     uint16 rid;
   };
 
+  class EntryCollector : public DictionaryInterface::Callback {
+   public:
+    virtual ResultType OnToken(StringPiece,  // key
+                               StringPiece,  // actual_key
+                               const Token &token) {
+      // Collect only user dictionary entries.
+      if (token.attributes & Token::USER_DICTIONARY) {
+        entries_.push_back(Entry());
+        entries_.back().key = token.key;
+        entries_.back().value = token.value;
+        entries_.back().lid = token.lid;
+        entries_.back().rid = token.rid;
+      }
+      return TRAVERSE_CONTINUE;
+    }
+
+    const vector<Entry> &entries() const { return entries_; }
+
+   private:
+    vector<Entry> entries_;
+  };
+
   static void TestLookupPredictiveHelper(const Entry *expected,
                                          size_t expected_size,
-                                         const char *key,
-                                         size_t key_size,
+                                         StringPiece key,
                                          const UserDictionary &dic) {
-    NodeAllocator allocator;
-    Node *node = dic.LookupPredictive(key, key_size, &allocator);
+    EntryCollector collector;
+    dic.LookupPredictive(key, false, &collector);
 
     if (expected == NULL || expected_size == 0) {
-      EXPECT_TRUE(NULL == node);
+      EXPECT_TRUE(collector.entries().empty());
     } else {
-      ASSERT_TRUE(NULL != node);
-      CompareEntries(expected, expected_size, node);
+      ASSERT_FALSE(collector.entries().empty());
+      CompareEntries(expected, expected_size, collector.entries());
     }
   }
 
@@ -241,14 +272,14 @@ class UserDictionaryTest : public ::testing::Test {
                                      const char *key,
                                      size_t key_size,
                                      const UserDictionary &dic) {
-    NodeAllocator allocator;
-    Node *node = dic.LookupPrefix(key, key_size, &allocator);
+    EntryCollector collector;
+    dic.LookupPrefix(StringPiece(key, key_size), false, &collector);
 
     if (expected == NULL || expected_size == 0) {
-      EXPECT_TRUE(NULL == node);
+      EXPECT_TRUE(collector.entries().empty());
     } else {
-      ASSERT_TRUE(NULL != node);
-      CompareEntries(expected, expected_size, node);
+      ASSERT_FALSE(collector.entries().empty());
+      CompareEntries(expected, expected_size, collector.entries());
     }
   }
 
@@ -257,49 +288,46 @@ class UserDictionaryTest : public ::testing::Test {
                                     const char *key,
                                     size_t key_size,
                                     const UserDictionary &dic) {
-    NodeAllocator allocator;
-    Node *node = dic.LookupExact(key, key_size, &allocator);
+    EntryCollector collector;
+    dic.LookupExact(StringPiece(key, key_size), &collector);
 
     if (expected == NULL || expected_size == 0) {
-      EXPECT_TRUE(NULL == node);
+      EXPECT_TRUE(collector.entries().empty());
     } else {
-      ASSERT_TRUE(NULL != node);
-      CompareEntries(expected, expected_size, node);
+      ASSERT_FALSE(collector.entries().empty());
+      CompareEntries(expected, expected_size, collector.entries());
     }
   }
 
+  static string EncodeEntry(const Entry &entry) {
+    return entry.key + "\t" +
+           entry.value + "\t" +
+           NumberUtil::SimpleItoa(entry.lid) + "\t" +
+           NumberUtil::SimpleItoa(entry.rid) + "\n";
+  }
+
+  static string EncodeEntries(const Entry *array, size_t size) {
+    vector<string> encoded_items;
+    for (size_t i = 0; i < size; ++i) {
+      encoded_items.push_back(EncodeEntry(array[i]));
+    }
+    sort(encoded_items.begin(), encoded_items.end());
+    string result;
+    Util::JoinStrings(encoded_items, "", &result);
+    return result;
+  }
+
   static void CompareEntries(const Entry *expected, size_t expected_size,
-                             const Node *node) {
-    vector<string> expected_encode_items;
-    for (size_t i = 0; i < expected_size; ++i) {
-      const Entry &entry = expected[i];
-      expected_encode_items.push_back(entry.key + "\t" +
-                                      entry.value + "\t" +
-                                      NumberUtil::SimpleItoa(entry.lid) + "\t" +
-                                      NumberUtil::SimpleItoa(entry.rid) + "\n");
-    }
-    sort(expected_encode_items.begin(), expected_encode_items.end());
-    string expected_encode;
-    Util::JoinStrings(expected_encode_items, "", &expected_encode);
-
-    vector<string> actual_encode_items;
-    for ( ; node != NULL; node = node->bnext) {
-      actual_encode_items.push_back(node->key + "\t" +
-                                    node->value + "\t" +
-                                    NumberUtil::SimpleItoa(node->lid) + "\t" +
-                                    NumberUtil::SimpleItoa(node->rid) + "\n");
-    }
-    sort(actual_encode_items.begin(), actual_encode_items.end());
-    string actual_encode;
-    Util::JoinStrings(actual_encode_items, "", &actual_encode);
-
-    EXPECT_EQ(expected_encode, actual_encode);
+                             const vector<Entry> &actual) {
+    const string expected_encoded = EncodeEntries(expected, expected_size);
+    const string actual_encoded = EncodeEntries(&actual[0], actual.size());
+    EXPECT_EQ(expected_encoded, actual_encoded);
   }
 
   static void LoadFromString(const string &contents,
                              UserDictionaryStorage *storage) {
     istringstream is(contents);
-    CHECK(is);
+    CHECK(is.good());
 
     storage->Clear();
     UserDictionaryStorage::UserDictionary *dic
@@ -307,7 +335,7 @@ class UserDictionaryTest : public ::testing::Test {
     CHECK(dic);
 
     string line;
-    while (getline(is, line)) {
+    while (!getline(is, line).fail()) {
       if (line.empty() || line[0] == '#') {
         continue;
       }
@@ -324,11 +352,24 @@ class UserDictionaryTest : public ::testing::Test {
       } else if (fields[2] == "noun") {
         entry->set_pos(user_dictionary::UserDictionary::NOUN);
       }
+      if (fields.size() >= 4 && !fields[3].empty()) {
+        entry->set_comment(fields[3]);
+      }
     }
   }
 
-  scoped_ptr<const UserPOSMock> pos_mock_;
+  // Helper function to lookup comment string from |dic|.
+  static string LookupComment(const UserDictionary& dic,
+                              StringPiece key, StringPiece value) {
+    string comment;
+    dic.LookupComment(key, value, &comment);
+    return comment;
+  }
+
   scoped_ptr<SuppressionDictionary> suppression_dictionary_;
+
+ private:
+  mozc::usage_stats::scoped_usage_stats_enabler usage_stats_enabler_;
 };
 
 TEST_F(UserDictionaryTest, TestLookupPredictive) {
@@ -350,7 +391,7 @@ TEST_F(UserDictionaryTest, TestLookupPredictive) {
     { "starting", "starting", 220, 220 },
   };
   TestLookupPredictiveHelper(kExpected0, arraysize(kExpected0),
-                             "start", 5, *dic.get());
+                             "start", *dic.get());
 
   // Another normal lookup operation.
   const Entry kExpected1[] = {
@@ -365,13 +406,13 @@ TEST_F(UserDictionaryTest, TestLookupPredictive) {
     { "starting", "starting", 220, 220 },
   };
   TestLookupPredictiveHelper(kExpected1, arraysize(kExpected1),
-                             "st", 2, *dic.get());
+                             "st", *dic.get());
 
   // Invalid input values should be just ignored.
-  TestLookupPredictiveHelper(NULL, 0, "", 0, *dic.get());
-  TestLookupPredictiveHelper(NULL, 0, "\xE6\xB0\xB4\xE9\x9B\xB2",  // "水雲"
-                             strlen("\xE6\xB0\xB4\xE9\x9B\xB2"), *dic.get());
-
+  TestLookupPredictiveHelper(NULL, 0, "", *dic.get());
+  TestLookupPredictiveHelper(NULL, 0,
+                             "\xE6\xB0\xB4\xE9\x9B\xB2",  // "水雲"
+                             *dic.get());
 
   // Make a change to the dictionary file and load it again.
   {
@@ -380,64 +421,18 @@ TEST_F(UserDictionaryTest, TestLookupPredictive) {
     dic->Load(storage);
   }
 
-  // A normal lookup.
+  // A normal lookup again.
   const Entry kExpected2[] = {
     { "end", "end", 200, 200 },
     { "ended", "ended", 210, 210 },
     { "ending", "ending", 220, 220 },
   };
   TestLookupPredictiveHelper(kExpected2, arraysize(kExpected2),
-                         "end", 3, *dic.get());
+                             "end", *dic.get());
 
-  // Lookup for entries which are gone should returns empty result.
-  TestLookupPredictiveHelper(NULL, 0, "start", 5, *dic.get());
-  TestLookupPredictiveHelper(NULL, 0, "st", 2, *dic.get());
-}
-
-TEST_F(UserDictionaryTest, TestLookupPredictiveWithLimit) {
-  scoped_ptr<UserDictionary> dic(CreateDictionaryWithMockPos());
-  // Wait for async reload called from the constructor.
-  dic->WaitForReloader();
-
-  {
-    UserDictionaryStorage storage("");
-    UserDictionaryTest::LoadFromString(kUserDictionary0, &storage);
-    dic->Load(storage);
-  }
-
-  {
-    NodeAllocator allocator;
-    DictionaryInterface::Limit limit;
-    Trie<string> trie;
-    trie.AddEntry("m", "");
-    trie.AddEntry("n", "");
-    limit.begin_with_trie = &trie;
-
-    const string key = "sta";
-    Node *node = dic->LookupPredictiveWithLimit(
-        key.data(), key.size(), limit, &allocator);
-    const Entry kExpected[] = {
-      { "stamp", "stamp", 100, 100 },
-      { "stand", "stand", 200, 200 },
-      { "standed", "standed", 210, 210 },
-      { "standing", "standing", 220, 220 },
-    };
-    CompareEntries(kExpected, arraysize(kExpected), node);
-  }
-
-  {
-    NodeAllocator allocator;
-    DictionaryInterface::Limit limit;
-    const string key = "stan";
-    Node *node = dic->LookupPredictiveWithLimit(
-        key.data(), key.size(), limit, &allocator);
-    const Entry kExpected[] = {
-      { "stand", "stand", 200, 200 },
-      { "standed", "standed", 210, 210 },
-      { "standing", "standing", 220, 220 },
-    };
-    CompareEntries(kExpected, arraysize(kExpected), node);
-  }
+  // Entries in the dictionary before reloading cannot be looked up.
+  TestLookupPredictiveHelper(NULL, 0, "start", *dic.get());
+  TestLookupPredictiveHelper(NULL, 0, "st", *dic.get());
 }
 
 TEST_F(UserDictionaryTest, TestLookupPrefix) {
@@ -472,8 +467,9 @@ TEST_F(UserDictionaryTest, TestLookupPrefix) {
 
   // Invalid input values should be just ignored.
   TestLookupPrefixHelper(NULL, 0, "", 0, *dic.get());
-  TestLookupPrefixHelper(NULL, 0, "\xE6\xB0\xB4\xE9\x9B\xB2",  // "水雲"
-                         strlen("\xE6\xB0\xB4\xE9\x9B\xB2"), *dic.get());
+  TestLookupPrefixHelper(
+      NULL, 0, "\xE6\xB0\xB4\xE9\x9B\xB2",  // "水雲"
+      strlen("\xE6\xB0\xB4\xE9\x9B\xB2"), *dic.get());
 
   // Make a change to the dictionary file and load it again.
   {
@@ -488,7 +484,7 @@ TEST_F(UserDictionaryTest, TestLookupPrefix) {
     { "ending", "ending", 220, 220 },
   };
   TestLookupPrefixHelper(kExpected2, arraysize(kExpected2),
-                         "ending", 6, *dic.get());
+                                     "ending", 6, *dic.get());
 
   // Lookup for entries which are gone should returns empty result.
   TestLookupPrefixHelper(NULL, 0, "started", 7, *dic.get());
@@ -527,6 +523,45 @@ TEST_F(UserDictionaryTest, TestLookupExact) {
                          strlen("\xE6\xB0\xB4\xE9\x9B\xB2"), *dic.get());
 }
 
+TEST_F(UserDictionaryTest, TestLookupExactWithSuggestionOnlyWords) {
+  scoped_ptr<UserDictionary> user_dic(CreateDictionary());
+  user_dic->WaitForReloader();
+
+  // Create dictionary
+  const string filename = FileUtil::JoinPath(FLAGS_test_tmpdir,
+                                             "suggestion_only_test.db");
+  FileUtil::Unlink(filename);
+  UserDictionaryStorage storage(filename);
+  {
+    uint64 id = 0;
+    EXPECT_TRUE(storage.CreateDictionary("test", &id));
+    UserDictionaryStorage::UserDictionary *dic =
+        storage.mutable_dictionaries(0);
+
+    // "名詞"
+    UserDictionaryStorage::UserDictionaryEntry *entry =
+        dic->add_entries();
+    entry->set_key("key");
+    entry->set_value("noun");
+    entry->set_pos(user_dictionary::UserDictionary::NOUN);
+
+    // "サジェストのみ"
+    entry = dic->add_entries();
+    entry->set_key("key");
+    entry->set_value("suggest_only");
+    entry->set_pos(user_dictionary::UserDictionary::SUGGESTION_ONLY);
+
+    user_dic->Load(storage);
+  }
+
+  // "suggestion_only" should not be looked up.
+  const testing::MockUserPosManager user_pos_manager;
+  const uint16 kNounId = user_pos_manager.GetPOSMatcher()->GetGeneralNounId();
+  const Entry kExpected1[] = {{"key", "noun", kNounId, kNounId}};
+  TestLookupExactHelper(kExpected1, arraysize(kExpected1),
+                        "key", 3, *user_dic.get());
+}
+
 TEST_F(UserDictionaryTest, IncognitoModeTest) {
   config::Config config;
   config::ConfigHandler::GetConfig(&config);
@@ -543,22 +578,28 @@ TEST_F(UserDictionaryTest, IncognitoModeTest) {
     dic->Load(storage);
   }
 
-  NodeAllocator allocator;
-
-  EXPECT_EQ(NULL, dic->LookupPrefix("start", 4, &allocator));
-  EXPECT_EQ(NULL, dic->LookupPredictive("s", 1, &allocator));
+  TestLookupPrefixHelper(NULL, 0, "start", 4, *dic);
+  TestLookupPredictiveHelper(NULL, 0, "s", *dic);
 
   config.set_incognito_mode(false);
   config::ConfigHandler::SetConfig(config);
 
-  EXPECT_FALSE(NULL == dic->LookupPrefix("start", 4, &allocator));
-  EXPECT_FALSE(NULL == dic->LookupPredictive("s", 1, &allocator));
+  {
+    EntryCollector collector;
+    dic->LookupPrefix("start", false, &collector);
+    EXPECT_FALSE(collector.entries().empty());
+  }
+  {
+    EntryCollector collector;
+    dic->LookupPredictive("s", false, &collector);
+    EXPECT_FALSE(collector.entries().empty());
+  }
 }
 
 TEST_F(UserDictionaryTest, AsyncLoadTest) {
-  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
-                                         "async_load_test.db");
-  Util::Unlink(filename);
+  const string filename = FileUtil::JoinPath(FLAGS_test_tmpdir,
+                                             "async_load_test.db");
+  FileUtil::Unlink(filename);
 
   // Create dictionary
   vector<string> keys;
@@ -591,24 +632,23 @@ TEST_F(UserDictionaryTest, AsyncLoadTest) {
     dic->WaitForReloader();
     dic->SetUserDictionaryName(filename);
 
-    NodeAllocator allocator;
     for (int i = 0; i < 32; ++i) {
       random_shuffle(keys.begin(), keys.end());
       dic->Reload();
       for (int i = 0; i < 1000; ++i) {
-        dic->LookupPrefix(keys[i].c_str(),
-                          keys[i].size(), &allocator);
+        CollectTokenCallback callback;
+        dic->LookupPrefix(keys[i], false, &callback);
       }
     }
     dic->WaitForReloader();
   }
-  Util::Unlink(filename);
+  FileUtil::Unlink(filename);
 }
 
 TEST_F(UserDictionaryTest, AddToAutoRegisteredDictionary) {
-  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
-                                         "add_to_auto_registered.db");
-  Util::Unlink(filename);
+  const string filename = FileUtil::JoinPath(FLAGS_test_tmpdir,
+                                             "add_to_auto_registered.db");
+  FileUtil::Unlink(filename);
 
   // Create dictionary
   {
@@ -637,13 +677,8 @@ TEST_F(UserDictionaryTest, AddToAutoRegisteredDictionary) {
   {
     UserDictionaryStorage storage(filename);
     EXPECT_TRUE(storage.Load());
-#ifdef ENABLE_CLOUD_SYNC
-    int index = 1;
-    EXPECT_EQ(2, storage.dictionaries_size());
-#else
     int index = 0;
     EXPECT_EQ(1, storage.dictionaries_size());
-#endif
     EXPECT_EQ(100, storage.dictionaries(index).entries_size());
     for (int i = 0; i < 100; ++i) {
       EXPECT_EQ("key" + NumberUtil::SimpleItoa(i),
@@ -655,7 +690,7 @@ TEST_F(UserDictionaryTest, AddToAutoRegisteredDictionary) {
     }
   }
 
-  Util::Unlink(filename);
+  FileUtil::Unlink(filename);
 
   // Create dictionary
   {
@@ -684,21 +719,12 @@ TEST_F(UserDictionaryTest, AddToAutoRegisteredDictionary) {
   {
     UserDictionaryStorage storage(filename);
     EXPECT_TRUE(storage.Load());
-#ifdef ENABLE_CLOUD_SYNC
-    EXPECT_EQ(2, storage.dictionaries_size());
-    EXPECT_EQ(1, storage.dictionaries(1).entries_size());
-    EXPECT_EQ("key", storage.dictionaries(1).entries(0).key());
-    EXPECT_EQ("value", storage.dictionaries(1).entries(0).value());
-    EXPECT_EQ(user_dictionary::UserDictionary::NOUN,
-              storage.dictionaries(1).entries(0).pos());
-#else
     EXPECT_EQ(1, storage.dictionaries_size());
     EXPECT_EQ(1, storage.dictionaries(0).entries_size());
     EXPECT_EQ("key", storage.dictionaries(0).entries(0).key());
     EXPECT_EQ("value", storage.dictionaries(0).entries(0).value());
     EXPECT_EQ(user_dictionary::UserDictionary::NOUN,
               storage.dictionaries(0).entries(0).pos());
-#endif
   }
 }
 
@@ -706,9 +732,9 @@ TEST_F(UserDictionaryTest, TestSuppressionDictionary) {
   scoped_ptr<UserDictionary> user_dic(CreateDictionaryWithMockPos());
   user_dic->WaitForReloader();
 
-  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
-                                         "suppression_test.db");
-  Util::Unlink(filename);
+  const string filename = FileUtil::JoinPath(FLAGS_test_tmpdir,
+                                             "suppression_test.db");
+  FileUtil::Unlink(filename);
 
   UserDictionaryStorage storage(filename);
 
@@ -721,16 +747,20 @@ TEST_F(UserDictionaryTest, TestSuppressionDictionary) {
     for (size_t j = 0; j < 10000; ++j) {
       UserDictionaryStorage::UserDictionaryEntry *entry =
           dic->add_entries();
-      entry->set_key("no_suppress_key" + NumberUtil::SimpleItoa(j));
-      entry->set_value("no_suppress_value" + NumberUtil::SimpleItoa(j));
+      entry->set_key("no_suppress_key" +
+                     NumberUtil::SimpleItoa(static_cast<uint32>(j)));
+      entry->set_value("no_suppress_value" +
+                       NumberUtil::SimpleItoa(static_cast<uint32>(j)));
       entry->set_pos(user_dictionary::UserDictionary::NOUN);
     }
 
     for (size_t j = 0; j < 10; ++j) {
       UserDictionaryStorage::UserDictionaryEntry *entry =
           dic->add_entries();
-      entry->set_key("suppress_key" + NumberUtil::SimpleItoa(j));
-      entry->set_value("suppress_value" + NumberUtil::SimpleItoa(j));
+      entry->set_key(
+          "suppress_key" + NumberUtil::SimpleItoa(static_cast<uint32>(j)));
+      entry->set_value(
+          "suppress_value" + NumberUtil::SimpleItoa(static_cast<uint32>(j)));
       // entry->set_pos("抑制単語");
       entry->set_pos(user_dictionary::UserDictionary::SUPPRESSION_WORD);
     }
@@ -742,8 +772,8 @@ TEST_F(UserDictionaryTest, TestSuppressionDictionary) {
 
     for (size_t j = 0; j < 10; ++j) {
       EXPECT_TRUE(suppression_dictionary_->SuppressEntry(
-          "suppress_key" + NumberUtil::SimpleItoa(j),
-          "suppress_value" + NumberUtil::SimpleItoa(j)));
+          "suppress_key" + NumberUtil::SimpleItoa(static_cast<uint32>(j)),
+          "suppress_value" + NumberUtil::SimpleItoa(static_cast<uint32>(j))));
     }
   }
 
@@ -757,8 +787,10 @@ TEST_F(UserDictionaryTest, TestSuppressionDictionary) {
     for (size_t j = 0; j < 10000; ++j) {
       UserDictionaryStorage::UserDictionaryEntry *entry =
           dic->add_entries();
-      entry->set_key("no_suppress_key" + NumberUtil::SimpleItoa(j));
-      entry->set_value("no_suppress_value" + NumberUtil::SimpleItoa(j));
+      entry->set_key(
+          "no_suppress_key" + NumberUtil::SimpleItoa(static_cast<uint32>(j)));
+      entry->set_value(
+          "no_suppress_value" + NumberUtil::SimpleItoa(static_cast<uint32>(j)));
       entry->set_pos(user_dictionary::UserDictionary::NOUN);
     }
 
@@ -768,20 +800,20 @@ TEST_F(UserDictionaryTest, TestSuppressionDictionary) {
 
     for (size_t j = 0; j < 10; ++j) {
       EXPECT_FALSE(suppression_dictionary_->SuppressEntry(
-          "suppress_key" + NumberUtil::SimpleItoa(j),
-          "suppress_value" + NumberUtil::SimpleItoa(j)));
+          "suppress_key" + NumberUtil::SimpleItoa(static_cast<uint32>(j)),
+          "suppress_value" + NumberUtil::SimpleItoa(static_cast<uint32>(j))));
     }
   }
-  Util::Unlink(filename);
+  FileUtil::Unlink(filename);
 }
 
 TEST_F(UserDictionaryTest, TestSuggestionOnlyWord) {
   scoped_ptr<UserDictionary> user_dic(CreateDictionary());
   user_dic->WaitForReloader();
 
-  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
-                                         "suggestion_only_test.db");
-  Util::Unlink(filename);
+  const string filename = FileUtil::JoinPath(FLAGS_test_tmpdir,
+                                             "suggestion_only_test.db");
+  FileUtil::Unlink(filename);
 
   UserDictionaryStorage storage(filename);
 
@@ -795,7 +827,7 @@ TEST_F(UserDictionaryTest, TestSuggestionOnlyWord) {
     for (size_t j = 0; j < 10; ++j) {
       UserDictionaryStorage::UserDictionaryEntry *entry =
           dic->add_entries();
-      entry->set_key("key" + NumberUtil::SimpleItoa(j));
+      entry->set_key("key" + NumberUtil::SimpleItoa(static_cast<uint32>(j)));
       entry->set_value("default");
       // "名詞"
       entry->set_pos(user_dictionary::UserDictionary::NOUN);
@@ -804,7 +836,7 @@ TEST_F(UserDictionaryTest, TestSuggestionOnlyWord) {
     for (size_t j = 0; j < 10; ++j) {
       UserDictionaryStorage::UserDictionaryEntry *entry =
           dic->add_entries();
-      entry->set_key("key" + NumberUtil::SimpleItoa(j));
+      entry->set_key("key" + NumberUtil::SimpleItoa(static_cast<uint32>(j)));
       entry->set_value("suggest_only");
       // "サジェストのみ"
       entry->set_pos(user_dictionary::UserDictionary::SUGGESTION_ONLY);
@@ -813,29 +845,27 @@ TEST_F(UserDictionaryTest, TestSuggestionOnlyWord) {
     user_dic->Load(storage);
   }
 
-  NodeAllocator allocator;
-
   {
     const char kKey[] = "key0123";
-    const Node *node = user_dic->LookupPrefix(kKey, strlen(kKey),
-                                              &allocator);
-    CHECK(node);
-    for (; node != NULL; node = node->bnext) {
-      EXPECT_TRUE("suggest_only" != node->value && "default" == node->value);
+    CollectTokenCallback callback;
+    user_dic->LookupPrefix(kKey, false, &callback);
+    const vector<Token> &tokens = callback.tokens();
+    for (size_t i = 0; i < tokens.size(); ++i) {
+      EXPECT_EQ("default", tokens[i].value);
     }
   }
-
   {
     const char kKey[] = "key";
-    const Node *node = user_dic->LookupPredictive(kKey, strlen(kKey),
-                                                  &allocator);
-    CHECK(node);
-    for (; node != NULL; node = node->bnext) {
-      EXPECT_TRUE("suggest_only" == node->value || "default" == node->value);
+    CollectTokenCallback callback;
+    user_dic->LookupPredictive(kKey, false, &callback);
+    const vector<Token> &tokens = callback.tokens();
+    for (size_t i = 0; i < tokens.size(); ++i) {
+      EXPECT_TRUE(tokens[i].value == "suggest_only" ||
+                  tokens[i].value == "default");
     }
   }
 
-  Util::Unlink(filename);
+  FileUtil::Unlink(filename);
 }
 
 TEST_F(UserDictionaryTest, TestUsageStats) {
@@ -861,7 +891,6 @@ TEST_F(UserDictionaryTest, TestUsageStats) {
   }
   {
     UserDictionaryStorage::UserDictionary *dic2 = storage.add_dictionaries();
-    dic2->set_syncable(true);
     CHECK(dic2);
     UserDictionaryStorage::UserDictionaryEntry *entry;
     entry = dic2->add_entries();
@@ -882,22 +911,52 @@ TEST_F(UserDictionaryTest, TestUsageStats) {
   }
   dic->Load(storage);
 
-  string reg_str;
-  usage_stats::Stats stats;
-
-  EXPECT_TRUE(storage::Registry::Lookup("usage_stats.UserRegisteredWord",
-                                         &reg_str));
-  EXPECT_TRUE(stats.ParseFromString(reg_str));
-  EXPECT_EQ("UserRegisteredWord", stats.name());
-  EXPECT_EQ(usage_stats::Stats::INTEGER, stats.type());
-  EXPECT_EQ(5, stats.int_value());
-
-  EXPECT_TRUE(storage::Registry::Lookup("usage_stats.UserRegisteredSyncWord",
-                                         &reg_str));
-  EXPECT_TRUE(stats.ParseFromString(reg_str));
-  EXPECT_EQ("UserRegisteredSyncWord", stats.name());
-  EXPECT_EQ(usage_stats::Stats::INTEGER, stats.type());
-  EXPECT_EQ(3, stats.int_value());
+  EXPECT_INTEGER_STATS("UserRegisteredWord", 5);
 }
+
+TEST_F(UserDictionaryTest, LookupComment) {
+  scoped_ptr<UserDictionary> dic(CreateDictionaryWithMockPos());
+  // Wait for async reload called from the constructor.
+  dic->WaitForReloader();
+
+  {
+    UserDictionaryStorage storage("");
+    UserDictionaryTest::LoadFromString(kUserDictionary0, &storage);
+    dic->Load(storage);
+  }
+
+  // Entry is in user dictionary but has no comment.
+  string comment;
+  comment = "prev comment";
+  EXPECT_FALSE(dic->LookupComment("comment_key1", "comment_value2", &comment));
+  EXPECT_EQ("prev comment", comment);
+
+  // Usual case: single key-value pair with comment.
+  EXPECT_TRUE(dic->LookupComment("comment_key2", "comment_value2", &comment));
+  EXPECT_EQ("comment", comment);
+
+  // There exist two entries having the same key, value and POS.  Since POS is
+  // irrelevant to comment lookup, the first nonempty comment should be found.
+  EXPECT_TRUE(dic->LookupComment("comment_key3", "comment_value3", &comment));
+  EXPECT_EQ("comment1", comment);
+
+  // White-space only comments should be cleared.
+  EXPECT_FALSE(dic->LookupComment("comment_key4", "comment_value4", &comment));
+  // The previous comment should remain.
+  EXPECT_EQ("comment1", comment);
+
+  // Comment should be found iff key and value match.
+  EXPECT_TRUE(LookupComment(*dic, "comment_key", "mismatching_value").empty());
+  EXPECT_TRUE(LookupComment(*dic, "comment_key1", "mismatching_value").empty());
+  EXPECT_TRUE(LookupComment(*dic, "comment_key2", "mismatching_value").empty());
+  EXPECT_TRUE(LookupComment(*dic, "comment_key3", "mismatching_value").empty());
+  EXPECT_TRUE(LookupComment(*dic, "comment_key4", "mismatching_value").empty());
+  EXPECT_TRUE(LookupComment(*dic, "mismatching_key", "comment_value").empty());
+  EXPECT_TRUE(LookupComment(*dic, "mismatching_key", "comment_value1").empty());
+  EXPECT_TRUE(LookupComment(*dic, "mismatching_key", "comment_value2").empty());
+  EXPECT_TRUE(LookupComment(*dic, "mismatching_key", "comment_value3").empty());
+  EXPECT_TRUE(LookupComment(*dic, "mismatching_key", "comment_value4").empty());
+}
+
 }  // namespace
 }  // namespace mozc
