@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,57 +30,68 @@
 // Qt component of configure dialog for Mozc
 #include "gui/config_dialog/config_dialog.h"
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <QtGui/QWindowsStyle>
 #include <windows.h>
 #endif
 
-#include <algorithm>
-#include <sstream>
-#include <stdlib.h>
 #include <QtGui/QMessageBox>
+#include <algorithm>
+#include <cstdlib>
+#include <sstream>
 #include "base/config_file_stream.h"
 #include "base/const.h"
-#ifdef OS_MACOSX
+#include "base/logging.h"
 #include "base/mac_util.h"
-#endif  // OS_MACOSX
-#include "base/util.h"
 #include "base/run_level.h"
-#include "config/config_handler.h"
+#include "base/system_util.h"
+#include "base/util.h"
+#include "client/client.h"
 #include "config/config.pb.h"
+#include "config/config_handler.h"
 #include "config/stats_config_util.h"
-#ifdef ENABLE_CLOUD_SYNC
-#include "gui/config_dialog/auth_dialog.h"
-#endif  // ENABLE_CLOUD_SYNC
+#include "gui/base/win_util.h"
 #include "gui/config_dialog/keymap_editor.h"
 #include "gui/config_dialog/roman_table_editor.h"
-#include "gui/base/win_util.h"
 #include "ipc/ipc.h"
 #include "session/commands.pb.h"
 #include "session/internal/keymap.h"
-#include "client/client.h"
+
+namespace {
+template<typename T>
+void Connect(const QList<T *> &objects,
+             const char *signal,
+             const QObject *receiver,
+             const char *slot) {
+  for (typename QList<T *>::const_iterator itr = objects.begin();
+       itr != objects.end(); ++itr) {
+    QObject::connect(*itr, signal, receiver, slot);
+  }
+}
+}  // namespace
 
 namespace mozc {
 
 namespace gui {
-
 using mozc::config::StatsConfigUtil;
 
 ConfigDialog::ConfigDialog()
     : client_(client::ClientFactory::NewClient()),
       initial_preedit_method_(0),
-      initial_use_keyboard_to_change_preedit_method_(false) {
+      initial_use_keyboard_to_change_preedit_method_(false),
+      initial_use_mode_indicator_(true) {
   setupUi(this);
   setWindowFlags(Qt::WindowSystemMenuHint);
   setWindowModality(Qt::NonModal);
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   miscStartupWidget->setVisible(false);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #ifdef OS_MACOSX
   miscDefaultIMEWidget->setVisible(false);
   miscAdministrationWidget->setVisible(false);
+  setWindowTitle(tr("Mozc Preferences"));
 #endif  // OS_MACOSX
 
 #if defined(OS_LINUX)
@@ -100,25 +111,13 @@ ConfigDialog::ConfigDialog()
 #endif  // OS_LINUX
 #endif  // NO_LOGGING
 
-#ifndef ENABLE_CLOUD_SYNC
-  syncDescriptionLabel->setVisible(false);
-  clearSyncButton->setVisible(false);
-  removeAllDescriptionLabel->setVisible(false);
-  syncStatusLabel->setVisible(false);
-  syncButtonsLayoutWidget->setVisible(false);
-  syncHeaderLayoutWidget->setVisible(false);
-#endif  // !ENABLE_CLOUD_SYNC
-
 #ifndef ENABLE_CLOUD_HANDWRITING
   cloudHandwritingCheckBox->setVisible(false);
   cloudServersLayoutWidget->setVisible(false);
-#endif  // !ENABLE_CLOUD_HANDWRITING
-
-#if !defined(ENABLE_CLOUD_SYNC) && !defined(ENABLE_CLOUD_HANDWRITING)
   // Hide "Cloud" tab when all the cloud features are not available.
   const int kCloudTabIndex = 5;
   configDialogTabWidget->removeTab(kCloudTabIndex);
-#endif  // !ENABLE_CLOUD_SYNC && !ENABLE_CLOUD_HANDWRITING
+#endif  // !ENABLE_CLOUD_HANDWRITING
 
   suggestionsSizeSpinBox->setRange(1, 9);
 
@@ -159,12 +158,12 @@ ConfigDialog::ConfigDialog()
 
   inputModeComboBox->addItem(tr("Romaji"));
   inputModeComboBox->addItem(tr("Kana"));
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   // These options changing the preedit method by a hot key are only
   // supported by Windows.
   inputModeComboBox->addItem(tr("Romaji (switchable)"));
   inputModeComboBox->addItem(tr("Kana (switchable)"));
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
   spaceCharacterFormComboBox->addItem(tr("Follow input mode"));
   spaceCharacterFormComboBox->addItem(tr("Fullwidth"));
@@ -202,6 +201,11 @@ ConfigDialog::ConfigDialog()
   useJapaneseLayout->hide();
 #endif  // !OS_MACOSX
 
+#ifndef MOZC_ENABLE_MODE_INDICATOR
+  // If not enabled, useModeIndicator checkbox should be invisible.
+  useModeIndicator->hide();
+#endif  // !MOZC_ENABLE_MODE_INDICATOR
+
   // signal/slot
   QObject::connect(configDialogButtonBox,
                    SIGNAL(clicked(QAbstractButton *)),
@@ -264,20 +268,17 @@ ConfigDialog::ConfigDialog()
                    this,
                    SLOT(LaunchAdministrationDialog()));
 
-#ifdef ENABLE_CLOUD_SYNC
-  QObject::connect(syncToggleButton,
-                   SIGNAL(clicked()),
-                   this,
-                   SLOT(SyncToggleButtonClicked()));
-  QObject::connect(syncCustomizationButton,
-                   SIGNAL(clicked()),
-                   this,
-                   SLOT(LaunchSyncCustomizationDialog()));
-  QObject::connect(clearSyncButton,
-                   SIGNAL(clicked()),
-                   this,
-                   SLOT(ClearSyncClicked()));
-#endif  // ENABLE_CLOUD_SYNC
+  // Event handlers to enable 'Apply' button.
+  Connect(findChildren<QPushButton *>(), SIGNAL(clicked()), this,
+          SLOT(EnableApplyButton()));
+  Connect(findChildren<QCheckBox *>(), SIGNAL(clicked()), this,
+          SLOT(EnableApplyButton()));
+  Connect(findChildren<QComboBox *>(), SIGNAL(activated(int)), this,
+          SLOT(EnableApplyButton()));
+  Connect(findChildren<QSpinBox *>(), SIGNAL(editingFinished()), this,
+          SLOT(EnableApplyButton()));
+  // 'Apply' button is disabled on launching.
+  configDialogButtonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 
   // When clicking these messages, CheckBoxs corresponding
   // to them should be toggled.
@@ -286,17 +287,17 @@ ConfigDialog::ConfigDialog()
   usageStatsMessage->installEventFilter(this);
   incognitoModeMessage->installEventFilter(this);
 
-#ifndef OS_WINDOWS
+#ifndef OS_WIN
   checkDefaultCheckBox->setVisible(false);
   checkDefaultLine->setVisible(false);
   checkDefaultLabel->setVisible(false);
-#endif   // !OS_WINDOWS
+#endif   // !OS_WIN
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   launchAdministrationDialogButton->setEnabled(true);
   // if the current application is not elevated by UAC,
   // add a shield icon
-  if (mozc::Util::IsVistaOrLater()) {
+  if (mozc::SystemUtil::IsVistaOrLater()) {
     if (!mozc::RunLevel::IsElevatedByUAC()) {
       QWindowsStyle style;
       QIcon vista_icon(style.standardIcon(QStyle::SP_VistaShield));
@@ -320,7 +321,7 @@ ConfigDialog::ConfigDialog()
   administrationLine->setVisible(false);
   administrationLabel->setVisible(false);
   dictionaryPreloadingAndUACLabel->setVisible(false);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #ifdef OS_LINUX
   // On Linux, disable all fields for UsageStats
@@ -332,36 +333,11 @@ ConfigDialog::ConfigDialog()
   usageStatsMessage->setVisible(false);
   usageStatsCheckBox->setEnabled(false);
   usageStatsCheckBox->setVisible(false);
-#endif // OS_LINUX
-
-  webUsageDictionaryCheckBox->setVisible(false);
-  editWebServiceEntryButton->setVisible(false);
-
-#ifdef ENABLE_CLOUD_SYNC
-  sync_customize_dialog_.reset(new SyncCustomizeDialog(this));
-  sync_running_ = false;
-  // We do not wait with NamedEventListener to update the sync status,
-  // but just do polling every second with QTimer.  If we introduce
-  // NamedEventListener in a watcher thread, the main thread has to
-  // cancel the listener when the window is closed.  That case would
-  // cause another tricky threading issue.
-  timer_.reset(new QTimer(this));
-  QObject::connect(
-      timer_.get(), SIGNAL(timeout()), this, SLOT(UpdateSyncStatus()));
-  timer_->start(1000);
-  last_synced_timestamp_ = 0;
-  // all sync buttons are not enabled for the first time.
-  clearSyncButton->setEnabled(false);
-  syncCustomizationButton->setEnabled(false);
-  syncToggleButton->setEnabled(false);
-#endif  // ENABLE_CLOUD_SYNC
+#endif  // OS_LINUX
 
   Reload();
 
-  editWebServiceEntryButton->setEnabled(
-      webUsageDictionaryCheckBox->isChecked());
-
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   IMEHotKeyDisabledCheckBox->setChecked(WinUtil::GetIMEHotKeyDisabled());
 #else
   IMEHotKeyDisabledCheckBox->setVisible(false);
@@ -373,9 +349,6 @@ ConfigDialog::ConfigDialog()
 }
 
 ConfigDialog::~ConfigDialog() {
-#ifdef ENABLE_CLOUD_SYNC
-  timer_->stop();
-#endif  // ENABLE_CLOUD_SYNC
 }
 
 bool ConfigDialog::SetConfig(const config::Config &config) {
@@ -409,9 +382,8 @@ bool ConfigDialog::GetConfig(config::Config *config) {
 void ConfigDialog::Reload() {
   config::Config config;
   if (!GetConfig(&config)) {
-    QMessageBox::critical(this,
-                          tr("Mozc settings"),
-                          tr("Failed to get current config values"));
+    QMessageBox::critical(this, windowTitle(),
+                          tr("Failed to get current config values."));
   }
   ConvertFromProto(config);
 
@@ -420,6 +392,7 @@ void ConfigDialog::Reload() {
   initial_preedit_method_ = static_cast<int>(config.preedit_method());
   initial_use_keyboard_to_change_preedit_method_ =
       config.use_keyboard_to_change_preedit_method();
+  initial_use_mode_indicator_ = config.use_mode_indicator();
 }
 
 bool ConfigDialog::Update() {
@@ -428,9 +401,7 @@ bool ConfigDialog::Update() {
 
   if (config.session_keymap() == config::Config::CUSTOM &&
       config.custom_keymap_table().empty()) {
-    QMessageBox::warning(
-        this,
-        tr("Mozc settings"),
+    QMessageBox::warning(this, windowTitle(),
         tr("The current custom keymap table is empty. "
            "When custom keymap is selected, "
            "you must customize it."));
@@ -438,32 +409,42 @@ bool ConfigDialog::Update() {
   }
 
 
-#if defined(OS_WINDOWS) || defined(OS_LINUX)
-  if (initial_preedit_method_ !=
-      static_cast<int>(config.preedit_method()) ||
-      initial_use_keyboard_to_change_preedit_method_ !=
-      static_cast<int>(config.use_keyboard_to_change_preedit_method())) {
-    QMessageBox::information(this,
-                             tr("Mozc settings"),
+#if defined(OS_WIN)
+  if ((initial_preedit_method_ !=
+       static_cast<int>(config.preedit_method())) ||
+      (initial_use_keyboard_to_change_preedit_method_ !=
+       config.use_keyboard_to_change_preedit_method())) {
+    QMessageBox::information(this, windowTitle(),
                              tr("Romaji/Kana setting is enabled from"
                                 " new applications."));
+    initial_preedit_method_ = static_cast<int>(config.preedit_method());
+    initial_use_keyboard_to_change_preedit_method_ =
+        config.use_keyboard_to_change_preedit_method();
   }
-#endif
+#endif  // OS_WIN
+
+#ifdef OS_WIN
+  if (initial_use_mode_indicator_ != config.use_mode_indicator()) {
+    QMessageBox::information(this, windowTitle(),
+                             tr("Input mode indicator setting is enabled from"
+                                " new applications."));
+    initial_use_mode_indicator_ = config.use_mode_indicator();
+  }
+#endif  // OS_WIN
 
   if (!SetConfig(config)) {
-    QMessageBox::critical(this,
-                          tr("Mozc settings"),
+    QMessageBox::critical(this, windowTitle(),
                           tr("Failed to update config"));
   }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (!WinUtil::SetIMEHotKeyDisabled(IMEHotKeyDisabledCheckBox->isChecked())) {
     // Do not show any dialog here, since this operation will not fail
     // in almost all cases.
     // TODO(taku): better to show dialog?
     LOG(ERROR) << "Failed to update IME HotKey status";
   }
-#endif
+#endif  // OS_WIN
 
 #ifdef OS_MACOSX
   if (startupCheckBox->isChecked()) {
@@ -483,19 +464,19 @@ bool ConfigDialog::Update() {
 void ConfigDialog::SetSendStatsCheckBox() {
   // On windows, usage_stats flag is managed by
   // administration_dialog. http://b/2889759
-#ifndef OS_WINDOWS
+#ifndef OS_WIN
   const bool val = StatsConfigUtil::IsEnabled();
   usageStatsCheckBox->setChecked(val);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 void ConfigDialog::GetSendStatsCheckBox() const {
   // On windows, usage_stats flag is managed by
   // administration_dialog. http://b/2889759
-#ifndef OS_WINDOWS
+#ifndef OS_WIN
   const bool val = usageStatsCheckBox->isChecked();
   StatsConfigUtil::SetEnabled(val);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 #define SET_COMBOBOX(combobox, enumname, field) \
@@ -522,11 +503,11 @@ static const int kPreeditMethodSize = 2;
 void SetComboboxForPreeditMethod(const config::Config &config,
                                  QComboBox *combobox) {
   int index = static_cast<int>(config.preedit_method());
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (config.use_keyboard_to_change_preedit_method()) {
     index += kPreeditMethodSize;
   }
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
   combobox->setCurrentIndex(index);
 }
 
@@ -571,14 +552,15 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
   // tab2
   SET_COMBOBOX(historyLearningLevelComboBox, HistoryLearningLevel,
                history_learning_level);
-  SET_CHECKBOX(singleKanjiDictionaryCheckBox, use_single_kanji_conversion);
-  SET_CHECKBOX(symbolDictionaryCheckBox, use_symbol_conversion);
-  SET_CHECKBOX(emoticonDictionaryCheckBox, use_emoticon_conversion);
+  SET_CHECKBOX(singleKanjiConversionCheckBox, use_single_kanji_conversion);
+  SET_CHECKBOX(symbolConversionCheckBox, use_symbol_conversion);
+  SET_CHECKBOX(emoticonConversionCheckBox, use_emoticon_conversion);
   SET_CHECKBOX(dateConversionCheckBox, use_date_conversion);
+  SET_CHECKBOX(emojiConversionCheckBox, use_emoji_conversion);
   SET_CHECKBOX(numberConversionCheckBox, use_number_conversion);
   SET_CHECKBOX(calculatorCheckBox, use_calculator);
-  SET_CHECKBOX(t13nDictionaryCheckBox, use_t13n_conversion);
-  SET_CHECKBOX(zipcodeDictionaryCheckBox, use_zip_code_conversion);
+  SET_CHECKBOX(t13nConversionCheckBox, use_t13n_conversion);
+  SET_CHECKBOX(zipcodeConversionCheckBox, use_zip_code_conversion);
   SET_CHECKBOX(spellingCorrectionCheckBox, use_spelling_correction);
 
   // InfoListConfig
@@ -609,6 +591,8 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
 
   SET_CHECKBOX(useJapaneseLayout, use_japanese_layout);
 
+  SET_CHECKBOX(useModeIndicator, use_mode_indicator);
+
   // tab4
   SET_CHECKBOX(historySuggestCheckBox, use_history_suggest);
   SET_CHECKBOX(dictionarySuggestCheckBox, use_dictionary_suggest);
@@ -628,23 +612,12 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
   SET_COMBOBOX(yenSignComboBox, YenSignCharacter, yen_sign_character);
 
   characterFormEditor->Load(config);
-#ifdef ENABLE_CLOUD_SYNC
-  sync_customize_dialog_->Load(config);
-  if (config.has_sync_config()) {
-    sync_running_ = true;
-    syncStatusLabel->setText(tr("Checking sync status"));
-  } else {
-    syncStatusLabel->setText(tr("Sync is not enabled"));
-  }
-  UpdateSyncToggleButtonText();
-#endif  // ENABLE_CLOUD_SYNC
   SET_CHECKBOX(cloudHandwritingCheckBox, allow_cloud_handwriting);
 
 #ifdef OS_MACOSX
   startupCheckBox->setChecked(
       MacUtil::CheckPrelauncherLoginItemStatus());
 #endif  // OS_MACOSX
-
 }
 
 void ConfigDialog::ConvertToProto(config::Config *config) const {
@@ -671,14 +644,15 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
   // tab2
   GET_COMBOBOX(historyLearningLevelComboBox, HistoryLearningLevel,
                history_learning_level);
-  GET_CHECKBOX(singleKanjiDictionaryCheckBox, use_single_kanji_conversion);
-  GET_CHECKBOX(symbolDictionaryCheckBox, use_symbol_conversion);
-  GET_CHECKBOX(emoticonDictionaryCheckBox, use_emoticon_conversion);
+  GET_CHECKBOX(singleKanjiConversionCheckBox, use_single_kanji_conversion);
+  GET_CHECKBOX(symbolConversionCheckBox, use_symbol_conversion);
+  GET_CHECKBOX(emoticonConversionCheckBox, use_emoticon_conversion);
   GET_CHECKBOX(dateConversionCheckBox, use_date_conversion);
+  GET_CHECKBOX(emojiConversionCheckBox, use_emoji_conversion);
   GET_CHECKBOX(numberConversionCheckBox, use_number_conversion);
   GET_CHECKBOX(calculatorCheckBox, use_calculator);
-  GET_CHECKBOX(t13nDictionaryCheckBox, use_t13n_conversion);
-  GET_CHECKBOX(zipcodeDictionaryCheckBox, use_zip_code_conversion);
+  GET_CHECKBOX(t13nConversionCheckBox, use_t13n_conversion);
+  GET_CHECKBOX(zipcodeConversionCheckBox, use_zip_code_conversion);
   GET_CHECKBOX(spellingCorrectionCheckBox, use_spelling_correction);
 
   // InformationListConfig
@@ -694,6 +668,8 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
 
   GET_CHECKBOX(useJapaneseLayout, use_japanese_layout);
 
+  GET_CHECKBOX(useModeIndicator, use_mode_indicator);
+
   uint32 auto_conversion_key = 0;
   if (kutenCheckBox->isChecked()) {
     auto_conversion_key |=
@@ -736,9 +712,6 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
   GET_COMBOBOX(yenSignComboBox, YenSignCharacter, yen_sign_character);
 
   characterFormEditor->Save(config);
-#ifdef ENABLE_CLOUD_SYNC
-  sync_customize_dialog_->Save(sync_running_, config);
-#endif  // ENABLE_CLOUD_SYNC
 #ifdef ENABLE_CLOUD_HANDWRITING
   GET_CHECKBOX(cloudHandwritingCheckBox, allow_cloud_handwriting);
 #endif  // ENABLE_CLOUD_HANDWRITING
@@ -771,7 +744,7 @@ void ConfigDialog::ClearUserHistory() {
   if (QMessageBox::Ok !=
       QMessageBox::question(
           this,
-          tr("Mozc settings"),
+          windowTitle(),
           tr("Do you want to clear personalization data? "
              "Input history is not reset with this operation. "
              "Please open \"suggestion\" tab to remove input history data."),
@@ -785,7 +758,7 @@ void ConfigDialog::ClearUserHistory() {
   if (!client_->ClearUserHistory()) {
     QMessageBox::critical(
         this,
-        tr("Mozc settings"),
+        windowTitle(),
         tr("Mozc Converter is not running. "
            "Settings were not saved."));
   }
@@ -795,7 +768,7 @@ void ConfigDialog::ClearUserPrediction() {
   if (QMessageBox::Ok !=
       QMessageBox::question(
           this,
-          tr("Mozc settings"),
+          windowTitle(),
           tr("Do you want to clear all history data?"),
           QMessageBox::Ok | QMessageBox::Cancel,
           QMessageBox::Cancel)) {
@@ -807,7 +780,7 @@ void ConfigDialog::ClearUserPrediction() {
   if (!client_->ClearUserPrediction()) {
     QMessageBox::critical(
         this,
-        tr("Mozc settings"),
+        windowTitle(),
         tr("Mozc Converter is not running. "
            "Settings were not saved."));
   }
@@ -817,7 +790,7 @@ void ConfigDialog::ClearUnusedUserPrediction() {
   if (QMessageBox::Ok !=
       QMessageBox::question(
           this,
-          tr("Mozc settings"),
+          windowTitle(),
           tr("Do you want to clear unused history data?"),
           QMessageBox::Ok | QMessageBox::Cancel,
           QMessageBox::Cancel)) {
@@ -829,7 +802,7 @@ void ConfigDialog::ClearUnusedUserPrediction() {
   if (!client_->ClearUnusedUserPrediction()) {
     QMessageBox::critical(
         this,
-        tr("Mozc settings"),
+        windowTitle(),
         tr("Mozc Converter is not running. "
            "Operation was not executed."));
   }
@@ -876,9 +849,6 @@ void ConfigDialog::EditRomanTable() {
   }
 }
 
-void ConfigDialog::EditInforlistConfig() {
-}
-
 void ConfigDialog::SelectInputModeSetting(int index) {
   // enable "EDIT" button if roman mode is selected
   editRomanTableButton->setEnabled((index == 0));
@@ -901,41 +871,17 @@ void ConfigDialog::SelectSuggestionSetting(int state) {
   }
 }
 
-void ConfigDialog::SelectWebUsageDictionarySetting(bool checked) {
-  if (checked) {
-    if (QMessageBox::question(
-            this,
-            tr("Mozc settings"),
-            tr("Web Service extension enables Mozc "
-               "to display third party usage dictionaries provided as an "
-               "Web service API, e.g. REST (POX/JSON over HTTP). "
-               "Note that candidate/preedit strings are "
-               "sent to the Web service when this feature is enabled. "
-               "Do you want to use Web Service extension?"),
-            QMessageBox::Yes | QMessageBox::No, QMessageBox::No)
-        == QMessageBox::Yes) {
-      editWebServiceEntryButton->setEnabled(true);
-    } else {
-      webUsageDictionaryCheckBox->setChecked(false);
-      editWebServiceEntryButton->setEnabled(false);
-    }
-  } else {
-    editWebServiceEntryButton->setEnabled(false);
-  }
-}
-
 void ConfigDialog::ResetToDefaults() {
   if (QMessageBox::Ok ==
       QMessageBox::question(
           this,
-          tr("Mozc settings"),
-          tr("When you reset Mozc settings, any changes you've made "
-             "will be reverted to the default settings. "
+          windowTitle(),
+          tr("When you reset Mozc settings, any changes "
+             "you've made will be reverted to the default settings. "
              "Do you want to reset settings? "
              "The following items are not reset with this operation.\n"
              " - Personalization data\n"
              " - Input history\n"
-             " - Sync settings\n"
              " - Usage statistics and crash reports\n"
              " - Administrator settings"),
           QMessageBox::Ok | QMessageBox::Cancel,
@@ -945,300 +891,19 @@ void ConfigDialog::ResetToDefaults() {
     config::Config config;
     config::ConfigHandler::GetDefaultConfig(&config);
     ConvertFromProto(config);
-#ifdef ENABLE_CLOUD_SYNC
-    if (sync_running_) {
-      StopSyncForcely();
-    }
-#endif  // ENABLE_CLOUD_SYNC
   }
 }
 
 void ConfigDialog::LaunchAdministrationDialog() {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   client_->LaunchTool("administration_dialog", "");
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
-void ConfigDialog::SyncToggleButtonClicked() {
-#ifdef ENABLE_CLOUD_SYNC
-  SyncToggleButtonClickedImpl();
-#endif  // ENABLE_CLOUD_SYNC
+void ConfigDialog::EnableApplyButton() {
+  configDialogButtonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
 }
 
-void ConfigDialog::LaunchSyncCustomizationDialog() {
-#ifdef ENABLE_CLOUD_SYNC
-  LaunchSyncCustomizationDialogImpl();
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-void ConfigDialog::UpdateSyncStatus() {
-#ifdef ENABLE_CLOUD_SYNC
-  UpdateSyncStatusImpl();
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-void ConfigDialog::ClearSyncClicked() {
-#ifdef ENABLE_CLOUD_SYNC
-  ClearSyncClickedImpl();
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-#ifdef ENABLE_CLOUD_SYNC
-void ConfigDialog::UpdateSyncToggleButtonText() {
-  if (sync_running_) {
-    syncToggleButton->setText(tr("Stop Sync"));
-  } else {
-    syncToggleButton->setText(tr("Start Sync"));
-  }
-}
-
-void ConfigDialog::SyncToggleButtonClickedImpl() {
-  if (sync_running_) {
-    StopSync();
-  } else {
-    LaunchAuthDialog();
-  }
-}
-
-void ConfigDialog::StopSync() {
-  if (QMessageBox::Cancel ==
-      QMessageBox::question(
-          this,
-          tr("Stop sync and reset the auth token"),
-          tr("You are trying to stop sync and reset the auth token. "
-             "To restart sync, you will need to get a new auth token.\n"
-             "Do you really want to continue?"),
-          QMessageBox::Ok | QMessageBox::Cancel,
-          QMessageBox::Cancel)) {
-    return;
-  }
-
-  StopSyncForcely();
-}
-
-void ConfigDialog::StopSyncForcely() {
-  // To stop the current synchronization, it resets the authorization code.
-  commands::Input::AuthorizationInfo auth_info;
-  auth_info.set_auth_code("");
-  if (!client_->AddAuthCode(auth_info)) {
-    LOG(WARNING) << "Sending auth code failed";
-    return;
-  }
-  sync_running_ = false;
-  UpdateSyncToggleButtonText();
-}
-
-void ConfigDialog::LaunchAuthDialog() {
-  string auth_code;
-  if (!gui::AuthDialog::Show(this, &auth_code)) {
-    LOG(WARNING) << "Canceled.  Do nothing";
-    return;
-  }
-
-  // The default configuration at the starting timing will be "all".
-  config::Config mock_config;
-  config::SyncConfig *sync_config = mock_config.mutable_sync_config();
-  sync_config->set_use_config_sync(true);
-  sync_config->set_use_user_dictionary_sync(true);
-  sync_config->set_use_user_history_sync(true);
-  sync_config->set_use_learning_preference_sync(true);
-  sync_config->set_use_contact_list_sync(true);
-
-  // Store the current config for the rollback when failed.
-  config::Config current_config;
-  sync_customize_dialog_->Save(false, &current_config);
-  sync_customize_dialog_->Load(mock_config);
-
-  // Then ask users to customize the sync configuration.
-  if (QDialog::Accepted != sync_customize_dialog_->exec()) {
-    LOG(WARNING) << "Canceled during sync customization. "
-                 << "It means do not activate sync.";
-    // Rollback the config.
-    sync_customize_dialog_->Load(current_config);
-    return;
-  }
-
-  sync_running_ = true;
-  // Before starting sync, the sync customization should be applied.
-  // WARNING: the whole configuration (items other than sync) should
-  // be applied at the same time.
-  Update();
-
-  commands::Input::AuthorizationInfo auth_info;
-  auth_info.set_auth_code(auth_code);
-  if (!client_->AddAuthCode(auth_info)) {
-    LOG(WARNING) << "Sending auth code failed";
-    return;
-  }
-  if (!client_->StartCloudSync()) {
-    LOG(WARNING) << "Cloud sync is failed to start";
-  }
-  UpdateSyncToggleButtonText();
-  // It does not wait for the end of cloud sync.
-}
-
-void ConfigDialog::LaunchSyncCustomizationDialogImpl() {
-  if (QDialog::Accepted == sync_customize_dialog_->exec()) {
-    // Before starting sync, the sync customization should be applied.
-    // WARNING: the whole configuration (items other than sync) should
-    // be applied at the same time.
-    Update();
-    if (!client_->StartCloudSync()) {
-      LOG(WARNING) << "Cloud sync is failed to start";
-    }
-  }
-}
-
-void ConfigDialog::AddLastSyncedDateTime(
-    uint64 timestamp, QString *output) const {
-  // 0 is default value, and means to have no sync before.
-  if (timestamp == 0) {
-    output->append(tr("Not synced yet"));
-    return;
-  }
-
-  output->append(tr("Last synced time: "));
-  const int time_offset = Util::GetTime() - timestamp;
-  QDateTime datetime = QDateTime::currentDateTime().addSecs(-time_offset);
-  output->append(datetime.toString("yyyy/MM/dd hh:mm:ss"));
-}
-
-void ConfigDialog::UpdateSyncStatusImpl() {
-  // This method always updates the sync status even when sync is not
-  // running.  That's because we should change the message to "not
-  // enabled" when the user explicitly turns off sync feature.
-  QString sync_message;
-  commands::CloudSyncStatus cloud_sync_status;
-  client_->GetCloudSyncStatus(&cloud_sync_status);
-  if (sync_running_) {
-    DLOG(INFO) << cloud_sync_status.DebugString();
-  } else {
-    // The default status is 'NOSYNC'.  Only changed via IPC when sync
-    // is running.
-    cloud_sync_status.set_global_status(commands::CloudSyncStatus::NOSYNC);
-  }
-
-  commands::CloudSyncStatus::SyncGlobalStatus sync_global_status =
-      cloud_sync_status.global_status();
-
-  // clearing sync is only available when sync is succeeding ==
-  // the current authorization is valid.
-  clearSyncButton->setEnabled(
-      sync_global_status == commands::CloudSyncStatus::SYNC_SUCCESS);
-  syncCustomizationButton->setEnabled(true);
-  syncToggleButton->setEnabled(true);
-
-  switch (sync_global_status) {
-    case commands::CloudSyncStatus::SYNC_SUCCESS:
-    case commands::CloudSyncStatus::SYNC_FAILURE: {
-      uint64 last_synced_timestamp = cloud_sync_status.last_synced_timestamp();
-      AddLastSyncedDateTime(last_synced_timestamp, &sync_message);
-      if (last_synced_timestamp_ != last_synced_timestamp) {
-        Reload();
-        last_synced_timestamp_ = last_synced_timestamp;
-      }
-      break;
-    }
-    case commands::CloudSyncStatus::WAITSYNC:
-      sync_message = tr("Waiting for server to be ready");
-      syncCustomizationButton->setEnabled(false);
-      syncToggleButton->setEnabled(false);
-      clearSyncButton->setEnabled(false);
-      break;
-    case commands::CloudSyncStatus::INSYNC:
-      sync_message = tr("During synchronization");
-      syncCustomizationButton->setEnabled(false);
-      syncToggleButton->setEnabled(false);
-      clearSyncButton->setEnabled(false);
-      break;
-    case commands::CloudSyncStatus::NOSYNC:
-      sync_message = tr("Sync is not enabled");
-      sync_running_ = false;
-      syncCustomizationButton->setEnabled(false);
-      clearSyncButton->setEnabled(false);
-      break;
-  }
-
-  QString tooltip;
-  // Check error messages in sync.
-  if (cloud_sync_status.sync_errors_size() > 0) {
-    for (size_t i = 0; i < cloud_sync_status.sync_errors_size(); ++i) {
-      if (i > 0) {
-        tooltip += "\n";
-      }
-
-      switch (cloud_sync_status.sync_errors(i).error_code()) {
-        case commands::CloudSyncStatus::AUTHORIZATION_FAIL:
-          tooltip += tr("Authorization failed.");
-          break;
-        case commands::CloudSyncStatus::USER_DICTIONARY_NUM_ENTRY_EXCEEDED:
-          tooltip += tr("Cannot save dictionaries because Sync Dictionary "
-                        "exceeds its entry size limit.");
-          break;
-        case commands::CloudSyncStatus::USER_DICTIONARY_BYTESIZE_EXCEEDED:
-          tooltip += tr("Cannot save dictionaries because Sync Dictionary "
-                        "exceeds its binary size limit.");
-          break;
-        case commands::CloudSyncStatus::USER_DICTIONARY_NUM_DICTIONARY_EXCEEDED:
-          tooltip += tr("Cannot save dictionaries because the number of "
-                        "dictionaries for sync exceeds its limit.");
-          break;
-        default:
-          tooltip += tr("Unknown error was found.");
-          LOG(WARNING) << "Unknown error ("
-                       << cloud_sync_status.sync_errors(i).error_code()
-                       << ") was found in cloud sync errors.";
-          break;
-      }
-    }
-
-    sync_message += "  ";
-    // Display the first error category obviously.
-    switch (cloud_sync_status.sync_errors(0).error_code()) {
-      case commands::CloudSyncStatus::AUTHORIZATION_FAIL:
-        sync_message += tr("Authorization error");
-        break;
-      case commands::CloudSyncStatus::USER_DICTIONARY_NUM_ENTRY_EXCEEDED:
-      case commands::CloudSyncStatus::USER_DICTIONARY_BYTESIZE_EXCEEDED:
-      case commands::CloudSyncStatus::USER_DICTIONARY_NUM_DICTIONARY_EXCEEDED:
-        sync_message += tr("Dictionary sync error");
-        break;
-      default:
-        sync_message += tr("Unknown sync error");
-        break;
-    }
-  }
-
-  UpdateSyncToggleButtonText();
-  syncStatusLabel->setText(sync_message);
-  syncStatusLabel->setToolTip(tooltip);
-}
-
-void ConfigDialog::ClearSyncClickedImpl() {
-  if (QMessageBox::Cancel ==
-      QMessageBox::question(
-          this,
-          tr("Clear all sync data in Google servers"),
-          tr("You are trying to clear all the data in the sync server.\n"
-             "Do you really want to continue?"),
-          QMessageBox::Ok | QMessageBox::Cancel,
-          QMessageBox::Cancel)) {
-    return;
-  }
-
-  // Stop doing sync configurations.
-  config::Config config;
-  GetConfig(&config);
-  config.clear_sync_config();
-  sync_running_ = false;
-  ConvertFromProto(config);
-  Update();
-
-  // Do the actual clear.
-  client_->ClearCloudSync();
-}
-#endif  // ENABLE_CLOUD_SYNC
-
 // Catch MouseButtonRelease event to toggle the CheckBoxes
 bool ConfigDialog::eventFilter(QObject *obj, QEvent *event) {
   if (event->type() == QEvent::MouseButtonRelease) {
