@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,20 +33,20 @@
 #include <QtGui/QtGui>
 #include <QtGui/QProgressDialog>
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <Windows.h>
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
 #include <algorithm>
 #include <functional>
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/file_stream.h"
+#include "base/logging.h"
 #include "base/run_level.h"
+#include "base/system_util.h"
 #include "base/util.h"
-#include "gui/base/win_util.h"
 #include "client/client.h"
 #include "data_manager/user_pos_manager.h"
 #include "dictionary/user_dictionary_importer.h"
@@ -55,6 +55,7 @@
 #include "dictionary/user_dictionary_storage.pb.h"
 #include "dictionary/user_dictionary_util.h"
 #include "dictionary/user_pos.h"
+#include "gui/base/win_util.h"
 #include "gui/config_dialog/combobox_delegate.h"
 #include "gui/dictionary_tool/find_dialog.h"
 #include "gui/dictionary_tool/import_dialog.h"
@@ -103,15 +104,7 @@ QProgressDialog *CreateProgressDialog(
   return progress;
 }
 
-size_t GetMaxDictionaryEntrySize(bool is_syncable) {
-  if (is_syncable) {
-    return mozc::UserDictionaryStorage::max_sync_entry_size();
-  } else {
-    return mozc::UserDictionaryStorage::max_entry_size();
-  }
-}
-
-#if defined(OS_WINDOWS) || defined(OS_MACOSX)
+#if defined(OS_WIN) || defined(OS_MACOSX)
 void InstallStyleSheet(const QString &filename) {
   QFile file(filename);
   file.open(QFile::ReadOnly);
@@ -201,18 +194,22 @@ class MultiByteTextLineIterator
   }
 
   bool IsAvailable() const {
-    return *(ifs_.get());
+    // This means that neither failbit nor badbit is set.
+    // TODO(yukawa): Consider to remove |ifs_->eof()|. Furthermore, we should
+    // replace IsAvailable() with something easier to understand, e.g.,
+    // Done() or HasNext().
+    return ifs_->good() || ifs_->eof();
   }
 
   bool Next(string *line)  {
-    if (!*(ifs_.get())) {
+    if (!ifs_->good()) {
       return false;
     }
 
     // Can't use getline as getline doesn't support CR only text.
     char ch;
     string output_line;
-    while (*(ifs_.get())) {
+    while (ifs_->good()) {
       ifs_->get(ch);
       if (output_line.empty() && ch == '\n') {
         continue;
@@ -248,6 +245,8 @@ class MultiByteTextLineIterator
   }
 
   void Reset() {
+    // Clear state bit (eofbit, failbit, badbit).
+    ifs_->clear();
     ifs_->seekg(0, ios_base::beg);
     first_line_ = true;
   }
@@ -270,7 +269,7 @@ UserDictionaryImporter::TextLineIteratorInterface *CreateTextLineIterator(
   if (encoding_type == UserDictionaryImporter::NUM_ENCODINGS) {
     LOG(ERROR) << "GuessFileEncodingType() returns UNKNOWN encoding.";
     // set default encoding
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
     encoding_type = UserDictionaryImporter::SHIFT_JIS;
 #else
     encoding_type = UserDictionaryImporter::UTF16;
@@ -314,7 +313,8 @@ DictionaryTool::DictionaryTool(QWidget *parent)
       client_(client::ClientFactory::NewClient()),
       is_available_(true),
       max_entry_size_(mozc::UserDictionaryStorage::max_entry_size()),
-      user_pos_(UserPosManager::GetUserPosManager()->GetUserPOS()) {
+      user_pos_(new UserPOS(
+          UserPosManager::GetUserPosManager()->GetUserPOSData())) {
   setupUi(this);
 
   // Create and set up ImportDialog object.
@@ -338,6 +338,14 @@ DictionaryTool::DictionaryTool(QWidget *parent)
     return;
   }
 
+#ifndef ENABLE_CLOUD_SYNC
+  if (session_->mutable_storage()
+      ->ConvertSyncDictionariesToNormalDictionaries()) {
+    LOG(INFO) << "Syncable dictionaries are converted to normal dictionaries";
+    session_->mutable_storage()->Save();
+  }
+#endif  // !ENABLE_CLOUD_SYNC
+
   // main window
 #ifndef OS_LINUX
   // For some reason setCentralWidget crashes the dictionary_tool on Linux
@@ -443,12 +451,12 @@ DictionaryTool::DictionaryTool(QWidget *parent)
   export_action_ = dic_menu_->addAction(tr("Export current dictionary..."));
 
   // add Import from MS-IME's dictionary
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   dic_menu_->addSeparator();
   import_default_ime_action_ =
       dic_menu_->addAction(
           tr("Import from %1's user dictionary...").arg("Microsoft IME"));
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
   dic_menu_button_->setMenu(dic_menu_);
   connect(new_action_, SIGNAL(triggered()),
@@ -466,10 +474,10 @@ DictionaryTool::DictionaryTool(QWidget *parent)
   connect(export_action_, SIGNAL(triggered()),
           this, SLOT(ExportDictionary()));
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   connect(import_default_ime_action_, SIGNAL(triggered()),
           this, SLOT(ImportFromDefaultIME()));
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
   // Signals and slots to connect buttons to actions.
   connect(new_word_button_, SIGNAL(clicked()),
@@ -522,8 +530,8 @@ DictionaryTool::DictionaryTool(QWidget *parent)
 #endif
 
   // for Window Aero Glass support
-#ifdef OS_WINDOWS
-  if (Util::IsVistaOrLater()) {
+#ifdef OS_WIN
+  if (SystemUtil::IsVistaOrLater()) {
     setContentsMargins(0, 0, 0, 0);
     WinUtil::InstallStyleSheetsFiles(":win_aero_style.qss",
                                      ":win_style.qss");
@@ -608,10 +616,8 @@ void DictionaryTool::OnDictionarySelectionChanged() {
     export_action_->setEnabled(false);
   } else {
     current_dic_id_ = dic_info.id;
+    max_entry_size_ = mozc::UserDictionaryStorage::max_entry_size();
     SetupDicContentEditor(dic_info);
-    const UserDictionary *dic =
-        session_->mutable_storage()->GetUserDictionary(current_dic_id_);
-    max_entry_size_ = GetMaxDictionaryEntrySize(dic->syncable());
   }
 }
 
@@ -629,9 +635,8 @@ void DictionaryTool::SetupDicContentEditor(
   // Update the main table widget for dictionary contents.
   StopMonitoringUserEdit();
 
-  // Disable "rename" and "delete" for sync dictionary
-  rename_action_->setEnabled(!dic->syncable());
-  delete_action_->setEnabled(!dic->syncable());
+  rename_action_->setEnabled(true);
+  delete_action_->setEnabled(true);
   import_append_action_->setEnabled(true);
   export_action_->setEnabled(true);
 
@@ -913,14 +918,14 @@ void DictionaryTool::ImportHelper(
 }
 
 void DictionaryTool::ImportFromDefaultIME() {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (RunLevel::IsElevatedByUAC()) {
     // MS-IME's dictionary importer doesn't work if the current
     // process is already elevated by UAC. If user disables UAC<
     // it's not the case. Here we simply remove the MS-IME import function
     // if dictionary tool is UAC-elevated
     QMessageBox::warning(this, window_title_,
-                         tr("Microsoft IME dictioanry import function doesn't "
+                         tr("Microsoft IME dictionary import function doesn't "
                             "work on UAC-elevated process."));
     return;
   }
@@ -1151,7 +1156,7 @@ void DictionaryTool::MoveTo(int dictionary_row) {
   }
 
   const size_t target_max_entry_size =
-      GetMaxDictionaryEntrySize(target_dict->syncable());
+      mozc::UserDictionaryStorage::max_entry_size();
 
   if (target_dict->entries_size() + rows.size() > target_max_entry_size) {
     QMessageBox::critical(this, window_title_,
@@ -1366,14 +1371,8 @@ void DictionaryTool::OnContextMenuRequestedForList(const QPoint &pos) {
 
   QMenu *menu = new QMenu(this);
 
-  // Disable "rename" and "delete" for sync dictionary.
-  QAction *rename_action = NULL;
-  QAction *delete_action = NULL;
-  if (!IsCurrentDictionaryForSync()) {
-    // "rename" and "delete" are not available for sync dictionary.
-    rename_action = menu->addAction(tr("Rename..."));
-    delete_action = menu->addAction(tr("Delete"));
-  }
+  QAction *rename_action = menu->addAction(tr("Rename..."));
+  QAction *delete_action = menu->addAction(tr("Delete"));
   QAction *import_action = menu->addAction(tr("Import to this dictionary..."));
   QAction *export_action = menu->addAction(tr("Export this dictionary..."));
   QAction *selected_action = menu->exec(QCursor::pos());
@@ -1403,17 +1402,6 @@ DictionaryTool::DictionaryInfo DictionaryTool::current_dictionary() const {
   return retval;
 }
 
-bool DictionaryTool::IsCurrentDictionaryForSync() const {
-  const DictionaryInfo &dic_info = current_dictionary();
-  if (dic_info.item == NULL) {
-    return false;
-  }
-
-  UserDictionary *dic =
-      session_->mutable_storage()->GetUserDictionary(dic_info.id);
-  return dic->syncable();
-}
-
 void DictionaryTool::SyncToStorage() {
   if (current_dic_id_ == 0 || !modified_) {
     return;
@@ -1446,7 +1434,8 @@ void DictionaryTool::SyncToStorage() {
 void DictionaryTool::CreateDictionaryHelper(const QString &dic_name) {
   uint64 new_dic_id = 0;
   if (!session_->mutable_storage()->CreateDictionary(
-          dic_name.toStdString(), &new_dic_id)) {
+          dic_name.toStdString(),
+          &new_dic_id)) {
     LOG(ERROR) << "Failed to create a new dictionary.";
     ReportError();
     return;
@@ -1466,6 +1455,7 @@ void DictionaryTool::CreateDictionaryHelper(const QString &dic_name) {
 bool DictionaryTool::InitDictionaryList() {
   dic_list_->clear();
   const UserDictionaryStorage &storage = session_->storage();
+
   for (size_t i = 0; i < storage.dictionaries_size(); ++i) {
     QListWidgetItem *item = new QListWidgetItem(dic_list_);
     DCHECK(item);
@@ -1589,7 +1579,7 @@ bool DictionaryTool::IsWritableToExport(const string &file_name) {
     QFileInfo dir_info(file_info.dir().absolutePath());
     // This preprocessor conditional is a workaround for a problem
     // that export fails on Windows.
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
     return dir_info.isExecutable();
 #else
     return dir_info.isExecutable() && dir_info.isWritable();
@@ -1598,7 +1588,7 @@ bool DictionaryTool::IsWritableToExport(const string &file_name) {
 }
 
 void DictionaryTool::paintEvent(QPaintEvent *event) {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (!gui::WinUtil::IsCompositionEnabled()) {
     return;
   }
@@ -1623,14 +1613,9 @@ void DictionaryTool::UpdateUIStatus() {
   new_action_->setEnabled(is_enable_new_dic);
   import_create_action_->setEnabled(is_enable_new_dic);
 
-  // "delete" is not available for sync dictionary.
-  if (IsCurrentDictionaryForSync()) {
-    delete_action_->setEnabled(false);
-  } else {
-    delete_action_->setEnabled(dic_list_->count() > 0);
-  }
+  delete_action_->setEnabled(dic_list_->count() > 0);
   import_append_action_->setEnabled(dic_list_->count() > 0);
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   import_default_ime_action_->setEnabled(dic_list_->count() > 0);
 #endif
 
@@ -1650,7 +1635,7 @@ void DictionaryTool::UpdateUIStatus() {
     statusbar_message_.clear();
   }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (!gui::WinUtil::IsCompositionEnabled()) {
     statusbar_->showMessage(statusbar_message_);
   } else {
@@ -1661,7 +1646,7 @@ void DictionaryTool::UpdateUIStatus() {
 #endif
 }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 bool DictionaryTool::winEvent(MSG *message, long *result) {
   if (message != NULL &&
       message->message == WM_LBUTTONDOWN &&
