@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,14 +32,19 @@
 #ifndef MOZC_HANDWRITING_ZINNIA_HANDWRITING_H_
 #define MOZC_HANDWRITING_ZINNIA_HANDWRITING_H_
 
+#include <string>
+
+#include "base/port.h"
+#include "base/scoped_ptr.h"
+#include "base/string_piece.h"
 #include "handwriting/handwriting_manager.h"
 
 #ifdef USE_LIBZINNIA
 // Use default zinnia installed in /usr/include
 #include <zinnia.h>
-#else
+#else  // USE_LIBZINNIA
 #include "third_party/zinnia/v0_04/zinnia.h"
-#endif
+#endif  // USE_LIBZINNIA
 
 namespace mozc {
 class Mmap;
@@ -48,8 +53,10 @@ namespace handwriting {
 
 class ZinniaHandwriting : public HandwritingInterface {
  public:
-  ZinniaHandwriting();
-  virtual ~ZinniaHandwriting() {}
+  explicit ZinniaHandwriting(StringPiece model_file);
+  virtual ~ZinniaHandwriting();
+
+  static string GetModelFileName();
 
   HandwritingStatus Recognize(const Strokes &strokes,
                               vector<string> *candidates) const;
@@ -60,8 +67,6 @@ class ZinniaHandwriting : public HandwritingInterface {
   scoped_ptr<zinnia::Recognizer> recognizer_;
   scoped_ptr<zinnia::Character> character_;
   scoped_ptr<Mmap> mmap_;
-  size_t width_;
-  size_t height_;
   bool zinnia_model_error_;
 
   DISALLOW_COPY_AND_ASSIGN(ZinniaHandwriting);
