@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,7 @@
 
 #include "ipc/ipc.h"
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <windows.h>
 #else
 #include <sys/types.h>
@@ -37,8 +37,10 @@
 #include <errno.h>
 #endif
 
-#include <stdlib.h>
-#include "base/base.h"
+#include <cstdlib>
+
+#include "base/logging.h"
+#include "base/port.h"
 #include "base/singleton.h"
 #include "base/thread.h"
 #include "ipc/ipc_path_manager.h"
@@ -51,13 +53,17 @@ class IPCServerThread : public Thread {
  public:
   explicit IPCServerThread(IPCServer *server)
       : server_(server) {}
+  virtual ~IPCServerThread() {}
   virtual void Run() {
     if (server_ != NULL) {
       server_->Loop();
     }
   }
-  private:
+
+ private:
   IPCServer *server_;
+
+  DISALLOW_COPY_AND_ASSIGN(IPCServerThread);
 };
 
 }  // namespace
@@ -132,7 +138,7 @@ bool IPCClient::TerminateServer(const string &name) {
     return false;
   }
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   HANDLE handle = ::OpenProcess(PROCESS_TERMINATE,
                                 false, static_cast<DWORD>(pid));
   if (NULL == handle) {
