@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,12 +27,16 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#include "ipc/ipc.h"
+
 #include <algorithm>
 #include <vector>
-#include "base/base.h"
+
+#include "base/flags.h"
+#include "base/port.h"
+#include "base/system_util.h"
 #include "base/thread.h"
 #include "base/util.h"
-#include "ipc/ipc.h"
 #include "ipc/ipc_test_util.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
@@ -45,7 +49,7 @@ namespace {
 // testing tool rut.py misunderstood that the file named
 // kServerAddress is a binary to be tested.
 static const char kServerAddress[] = "test_echo_server";
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 // On windows, multiple-connections failed.
 static const int kNumThreads = 1;
 #else
@@ -65,7 +69,7 @@ class MultiConnections: public mozc::Thread {
  public:
 #ifdef OS_MACOSX
   MultiConnections()
-      : mach_port_manager_(NULL) { }
+      : mach_port_manager_(NULL) {}
 
   void SetMachPortManager(mozc::MachPortManagerInterface *manager) {
     mach_port_manager_ = manager;
@@ -92,8 +96,8 @@ class MultiConnections: public mozc::Thread {
     }
   }
 
-#ifdef OS_MACOSX
  private:
+#ifdef OS_MACOSX
   mozc::MachPortManagerInterface *mach_port_manager_;
 #endif
 };
@@ -117,10 +121,10 @@ class EchoServer: public mozc::IPCServer {
     return true;
   }
 };
-}
+}  // namespace
 
 TEST(IPCTest, IPCTest) {
-  mozc::Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+  mozc::SystemUtil::SetUserProfileDirectory(FLAGS_test_tmpdir);
 #ifdef OS_MACOSX
   mozc::TestMachPortManager manager;
 #endif
