@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,14 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <string>
-#include "base/base.h"
 #include "ipc/named_event.h"
 
+#include <string>
+
+#include "base/flags.h"
+#include "base/logging.h"
+#include "base/port.h"
+
 DEFINE_bool(listener, true, "listener mode");
 DEFINE_bool(notifier, false, "notifier mode");
 DEFINE_int32(timeout, -1, "timeout (msec)");
