@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,11 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include "base/base.h"
-#include "base/process.h"
-#include "base/util.h"
 #include "ipc/process_watch_dog.h"
+
+#include "base/logging.h"
+#include "base/port.h"
+#include "base/util.h"
 #include "testing/base/public/gunit.h"
 
 namespace mozc {
@@ -42,7 +43,7 @@ class TestProcessWatchDog : public ProcessWatchDog {
  public:
   void Signaled(ProcessWatchDog::SignalType type) {
     EXPECT_EQ(type, ProcessWatchDog::PROCESS_SIGNALED);
-    uint64 diff = Util::GetTime() - g_current_time;
+    const uint64 diff = Util::GetTime() - g_current_time;
     EXPECT_EQ(2, diff);   // allow 1-sec error
   }
 };
@@ -50,7 +51,7 @@ class TestProcessWatchDog : public ProcessWatchDog {
 TEST(ProcessWatchDog, ProcessWatchDogTest) {
   g_current_time = Util::GetTime();
 
-#ifndef OS_WINDOWS
+#ifndef OS_WIN
   // revoke myself with different parameter
   pid_t pid = fork();
   if (pid == 0) {
@@ -68,4 +69,4 @@ TEST(ProcessWatchDog, ProcessWatchDogTest) {
   }
 #endif
 }
-}  // mozc
+}  // namespace mozc
