@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,14 +29,14 @@
 
 #import "mac/GoogleJapaneseInputController.h"
 
-#import <Cocoa/Cocoa.h>
 #import <Carbon/Carbon.h>
-#import <InputMethodKit/IMKServer.h>
+#import <Cocoa/Cocoa.h>
 #import <InputMethodKit/IMKInputController.h>
+#import <InputMethodKit/IMKServer.h>
 
 #include <unistd.h>
-#include <stdlib.h>
 
+#include <cstdlib>
 #include <set>
 
 #import "mac/GoogleJapaneseInputControllerInterface.h"
@@ -81,11 +81,17 @@ const set<string> *gNoOpenLinkApps = NULL;
 const map<CompositionMode, NSString *> *gModeIdMap = NULL;
 const set<string> *gNoSelectedRangeApps = NULL;
 const set<string> *gNoDisplayModeSwitchApps = NULL;
+const set<string> *gNoSurroundingTextApps = NULL;
 
 // TODO(horo): This value should be get from system configuration.
 //  DoubleClickInterval can be get from NSEvent (MacOSX ver >= 10.6)
 const NSTimeInterval kDoubleTapInterval = 0.5;
 
+const int kMaxSurroundingLength = 20;
+// In some apllications when the client's text length is large, getting the
+// surrounding text takes too much time. So we set this limitation.
+const int kGetSurroundingTextClientLengthLimit = 1000;
+
 NSString *GetLabelForSuffix(const string &suffix) {
   string label = mozc::MacUtil::GetLabelForSuffix(suffix);
   return [[NSString stringWithUTF8String:label.c_str()] retain];
@@ -153,6 +159,7 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
 @synthesize mode = mode_;
 @synthesize rendererCommand = rendererCommand_;
 @synthesize replacementRange = replacementRange_;
+@synthesize imkClientForTest = imkClientForTest_;
 - (mozc::client::ClientInterface *)mozcClient {
   return mozcClient_;
 }
@@ -188,12 +195,15 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   cursorPosition_ = NSNotFound;
   mode_ = mozc::commands::DIRECT;
   checkInputMode_ = YES;
+  suppressSuggestion_ = NO;
   yenSignCharacter_ = mozc::config::Config::YEN_SIGN;
   candidateController_ = new(nothrow) mozc::renderer::RendererClient;
   rendererCommand_ = new(nothrow)RendererCommand;
   mozcClient_ = mozc::client::ClientFactory::NewClient();
   imkServer_ = reinterpret_cast<id<ServerCallback> >(server);
-  lastKanaKeyTime_ = 0;
+  imkClientForTest_ = nil;
+  lastKeyDownTime_ = 0;
+  lastKeyCode_ = 0;
 
   // We don't check the return value of NSBundle because it fails during tests.
   [NSBundle loadNibNamed:@"Config" owner:self];
@@ -227,6 +237,7 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   [keyCodeMap_ release];
   [originalString_ release];
   [composedString_ release];
+  [imkClientForTest_ release];
   delete clientBundle_;
   delete candidateController_;
   delete mozcClient_;
@@ -235,6 +246,13 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   [super dealloc];
 }
 
+- (id)client {
+  if (imkClientForTest_) {
+    return imkClientForTest_;
+  }
+  return [super client];
+}
+
 - (NSMenu*)menu {
   return menu_;
 }
@@ -282,6 +300,14 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
     noDisplayModeSwitchApps->insert("com.microsoft.Word");
     gNoDisplayModeSwitchApps = noDisplayModeSwitchApps;
   }
+
+  set<string> *noSurroundingTextApps = new(nothrow) set<string>;
+  if (noSurroundingTextApps) {
+    // Disables the surrounding text feature for the following application
+    // because calling attributedSubstringFromRange to it is very heavy.
+    noSurroundingTextApps->insert("com.evernote.Evernote");
+    gNoSurroundingTextApps = noSurroundingTextApps;
+  }
 }
 
 #pragma mark IMKStateSetting Protocol
@@ -298,6 +324,19 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   }
   [self handleConfig];
   [imkServer_ setCurrentController:self];
+
+  string window_name, window_owner;
+  if (mozc::MacUtil::GetFrontmostWindowNameAndOwner(&window_name,
+                                                    &window_owner)) {
+    DLOG(INFO) << "frontmost window name: \"" << window_name << "\" "
+               << "owner: \"" << window_owner << "\"";
+    if (mozc::MacUtil::IsSuppressSuggestionWindow(window_name, window_owner)) {
+      suppressSuggestion_ = YES;
+    } else {
+      suppressSuggestion_ = NO;
+    }
+  }
+
   DLOG(INFO) << kProductNameInEnglish << " client (" << self
              << "): activated for " << sender;
   DLOG(INFO) << "sender bundleID: " << *clientBundle_;
@@ -501,10 +540,12 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
     return;
   }
 
-  NSAttributedString *text =
-      [sender attributedSubstringFromRange:selectedRange];
   if (!sending_command.has_text()) {
-    sending_command.set_text([[text string] UTF8String]);
+    NSString *text = [[sender attributedSubstringFromRange:selectedRange] string];
+    if (!text) {
+      return;
+    }
+    sending_command.set_text([text UTF8String]);
   }
 
   if (mozcClient_->SendCommand(sending_command, &output)) {
@@ -620,6 +661,16 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
 
   // Handle callbacks.
   if (output->has_callback() && output->callback().has_session_command()) {
+    if (output->callback().has_delay_millisec()) {
+      callback_command_.CopyFrom(output->callback());
+      // In the current implementation, if the subsequent key event also makes
+      // callback, the second callback will be called in the timimg of the first
+      // callback.
+      [self performSelector:@selector(sendCallbackCommand)
+                 withObject:nil
+                 afterDelay:output->callback().has_delay_millisec() / 1000.0];
+      return;
+    }
     const SessionCommand &callback_command =
         output->callback().session_command();
     if (callback_command.type() == SessionCommand::CONVERT_REVERSE) {
@@ -814,6 +865,33 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   [[NSWorkspace sharedWorkspace] openURL:url];
 }
 
+- (BOOL)fillSurroundingContext:(mozc::commands::Context *)context
+                        client:(id<IMKTextInput>)client {
+  NSInteger totalLength = [client length];
+  if (totalLength == 0 || totalLength == NSNotFound ||
+      totalLength > kGetSurroundingTextClientLengthLimit) {
+    return false;
+  }
+  NSRange selectedRange = [client selectedRange];
+  if (selectedRange.location == NSNotFound ||
+      selectedRange.length == NSNotFound) {
+    return false;
+  }
+  NSRange precedingRange = NSMakeRange(0, selectedRange.location);
+  if (selectedRange.location > kMaxSurroundingLength) {
+    precedingRange =
+        NSMakeRange(selectedRange.location - kMaxSurroundingLength,
+                    kMaxSurroundingLength);
+  }
+  NSString *precedingString =
+    [[client attributedSubstringFromRange:precedingRange] string];
+  if (precedingString) {
+    context->set_preceding_text([precedingString UTF8String]);
+    DLOG(INFO) << "preceding_text: \"" << context->preceding_text() << "\"";
+  }
+  return true;
+}
+
 - (BOOL)handleEvent:(NSEvent *)event client:(id)sender {
   if ([event type] == NSCursorUpdate) {
     [self updateComposition];
@@ -822,12 +900,21 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   if ([event type] != NSKeyDown && [event type] != NSFlagsChanged) {
     return NO;
   }
+  // Cancels the callback.
+  callback_command_.Clear();
 
   // Handle KANA key and EISU key.  We explicitly handles this here
   // for mode switch because some text area such like iPhoto person
   // name editor does not call setValue:forTag:client: method.
   // see: http://www.google.com/support/forum/p/ime/thread?tid=3aafb74ff71a1a69&hl=ja&fid=3aafb74ff71a1a690004aa3383bc9f5d
   if ([event type] == NSKeyDown) {
+    NSTimeInterval currentTime = [[NSDate date] timeIntervalSince1970];
+    const NSTimeInterval elapsedTime = currentTime - lastKeyDownTime_;
+    const bool isDoubleTap = ([event keyCode] == lastKeyCode_) &&
+                             (elapsedTime < kDoubleTapInterval);
+    lastKeyDownTime_ = currentTime;
+    lastKeyCode_ = [event keyCode];
+
     // these calling of switchMode: can be duplicated if the
     // application sends the setValue:forTag:client: and handleEvent:
     // at the same key event, but that's okay because switchMode:
@@ -836,7 +923,17 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
     if ([event keyCode] == kVK_JIS_Kana) {
       [self switchMode:mozc::commands::HIRAGANA client:sender];
       [self switchDisplayMode];
+      if (isDoubleTap) {
+        SessionCommand command;
+        command.set_type(SessionCommand::CONVERT_REVERSE);
+        [self invokeReconvert:&command client:sender];
+      }
     } else if ([event keyCode] == kVK_JIS_Eisu) {
+      if (isDoubleTap) {
+        SessionCommand command;
+        command.set_type(SessionCommand::COMMIT_RAW_TEXT);
+        [self sendCommand:command];
+      }
       CompositionMode new_mode = ([composedString_ length] == 0) ?
           mozc::commands::DIRECT : mozc::commands::HALF_ASCII;
       [self switchMode:new_mode client:sender];
@@ -844,26 +941,6 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
     }
   }
 
-  // UNDO by double tapping Kana-key
-  if ([event type] == NSKeyDown) {
-    if ([event keyCode] == kVK_JIS_Kana) {
-      NSTimeInterval currentTime = [[NSDate date] timeIntervalSince1970];
-      if (lastKanaKeyTime_ == 0){
-        lastKanaKeyTime_ = currentTime;
-      } else {
-        NSTimeInterval elapsedTime = currentTime - lastKanaKeyTime_;
-        lastKanaKeyTime_ = currentTime;
-        if (elapsedTime < kDoubleTapInterval) {
-          DLOG(INFO) << "UNDO";
-          [self invokeUndo:sender];
-          return YES;
-        }
-      }
-    } else {
-      lastKanaKeyTime_ = 0;
-    }
-  }
- 
   if ([keyCodeMap_ isModeSwitchingKey:event]) {
     // Special hack for Eisu/Kana keys.  Sometimes those key events
     // come to this method but we should ignore them because some
@@ -903,8 +980,19 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
     [originalString_ appendFormat:@"%c", keyEvent.key_code()];
   }
 
+  mozc::commands::Context context;
+  if (suppressSuggestion_) {
+    // TODO(komatsu, horo): Support Google Omnibox too.
+    context.add_experimental_features("google_search_box");
+  }
   keyEvent.set_mode(mode_);
-  if (!mozcClient_->SendKey(keyEvent, &output)) {
+
+  if ([composedString_ length] == 0 &&
+      !IsBannedApplication(gNoSelectedRangeApps, *clientBundle_) &&
+      !IsBannedApplication(gNoSurroundingTextApps, *clientBundle_)) {
+    [self fillSurroundingContext:&context client:sender];
+  }
+  if (!mozcClient_->SendKeyWithContext(keyEvent, context, &output)) {
     return NO;
   }
 
@@ -912,8 +1000,16 @@ bool IsBannedApplication(const set<string>* bundleIdSet,
   return output.consumed();
 }
 
+- (void)sendCallbackCommand {
+  if (callback_command_.has_session_command()) {
+    const SessionCommand command = callback_command_.session_command();
+    callback_command_.Clear();
+    [self sendCommand:command];
+  }
+}
+
 #pragma mark callbacks
-- (void)sendCommand:(SessionCommand &)command {
+- (void)sendCommand:(const SessionCommand &)command {
   Output output;
   if (!mozcClient_->SendCommand(command, &output)) {
     return;
