@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,10 +30,12 @@
 #ifndef MOZC_PREDICTION_DICTIONARY_PREDICTOR_H_
 #define MOZC_PREDICTION_DICTIONARY_PREDICTOR_H_
 
+#include <functional>
 #include <string>
 #include <vector>
+
 #include "base/util.h"
-#include "data_manager/data_manager_interface.h"
+#include "dictionary/dictionary_token.h"
 #include "prediction/predictor_interface.h"
 // for FRIEND_TEST()
 #include "testing/base/public/gunit_prod.h"
@@ -42,27 +44,29 @@ namespace mozc {
 
 class ConnectorInterface;
 class ConversionRequest;
+class ConverterInterface;
 class DictionaryInterface;
 class ImmutableConverterInterface;
-class NodeAllocatorInterface;
 class POSMatcher;
 class SegmenterInterface;
 class Segments;
 class SuggestionFilter;
-struct Node;
 
-// Dictioanry-based predictor
+// Dictionary-based predictor
 class DictionaryPredictor : public PredictorInterface {
  public:
   // Initializes a predictor with given references to submodules. Note that
   // pointers are not owned by the class and to be deleted by the caller.
-  DictionaryPredictor(const ImmutableConverterInterface *immutable_converter,
+  DictionaryPredictor(const ConverterInterface *converter,
+                      const ImmutableConverterInterface *immutable_converter,
                       const DictionaryInterface *dictionary,
                       const DictionaryInterface *suffix_dictionary,
-                      const DataManagerInterface *data_manager);
+                      const ConnectorInterface *connector,
+                      const SegmenterInterface *segmenter,
+                      const POSMatcher *pos_matcher,
+                      const SuggestionFilter *suggestion_filter);
   virtual ~DictionaryPredictor();
 
-  virtual bool Predict(Segments *segments) const;
   virtual bool PredictForRequest(const ConversionRequest &request,
                                  Segments *segments) const;
 
@@ -71,6 +75,7 @@ class DictionaryPredictor : public PredictorInterface {
  protected:
   // Protected members for unittesting
   // For use util method accessing private members, made them protected.
+  // http://code.google.com/p/googletest/wiki/FAQ
   enum PredictionType {
     // don't need to show any suggestions.
     NO_PREDICTION = 0,
@@ -84,137 +89,192 @@ class DictionaryPredictor : public PredictorInterface {
     SUFFIX =  8,
     // add English words.
     ENGLISH = 16,
+    // add prediciton to type corrected keys
+    TYPING_CORRECTION = 32,
+
+    // Suggests from |converter_|. The difference from REALTIME is that it uses
+    // the full converter with rewriter, history, etc.
+    // TODO(noriyukit): This label should be integrated with REALTIME. This is
+    // why 65536 is used to indicate that it is a temporary assignment.
+    REALTIME_TOP = 65536,
   };
   // Bitfield to store a set of PredictionType.
   typedef int32 PredictionTypes;
 
   struct Result {
-    Result() : node(NULL), types(NO_PREDICTION), cost(0) {}
-    Result(const Node *node_, PredictionTypes types_)
-        : node(node_), types(types_), cost(0) {}
-    const Node *node;
+    Result() : types(NO_PREDICTION), wcost(0), cost(0), lid(0), rid(0),
+               candidate_attributes(0), consumed_key_size(0) {}
+
+    void InitializeByTokenAndTypes(const Token &token, PredictionTypes types);
+    void SetTypesAndTokenAttributes(PredictionTypes prediction_types,
+                                    Token::AttributesBitfield token_attr);
+
+    string key;
+    string value;
+    // Indicating which PredictionType creates this instance.
+    // UNIGRAM, BIGRAM, REALTIME, SUFFIX, ENGLISH or TYPING_CORRECTION
+    // is set exclusively.
+    // TODO(matsuzakit): Using PredictionTypes both as input and output
+    //                   makes the code complex. Let's split them.
     PredictionTypes types;
+    // Context "insensitive" candidate cost.
+    int wcost;
+    // Context "sensitive" candidate cost.
     int cost;
+    int lid;
+    int rid;
+    // Boundary information for realtime conversion.
+    // This will be set only for realtime conversion result candidates.
+    // This contains inner segment size for key and value.
+    // If the candidate key and value are
+    // "わたしの|なまえは|なかのです", " 私の|名前は|中野です",
+    // |inner_segment_boundary| have [(4,2), (4, 3), (5, 4)].
+    vector<pair<int, int> > inner_segment_boundary;
+    uint32 candidate_attributes;
+    size_t consumed_key_size;
   };
 
-  // On MSVS2008/2010, TestableDictionaryPredictor::Result(node, types) causes a
-  // compile error even if you change the access right of it to public.  You can
-  // use TestableDictionaryPredictor::MakeResult(node, types) instead.
-  static Result MakeResult(const Node *node, PredictionTypes types) {
-    return Result(node, types);
+  // On MSVS2008/2010, Constructors of TestableDictionaryPredictor::Result
+  // causes a compile error even if you change the access right of it to public.
+  // You can use TestableDictionaryPredictor::MakeEmptyResult() instead.
+  static Result MakeEmptyResult() {
+    return Result();
   }
 
+  class PredictiveLookupCallback;
+  class PredictiveBigramLookupCallback;
+  class ResultWCostLess;
+  class ResultCostLess;
+
   void AggregateRealtimeConversion(PredictionTypes types,
+                                   const ConversionRequest &request,
                                    Segments *segments,
-                                   NodeAllocatorInterface *allocator,
                                    vector<Result> *results) const;
 
   void AggregateUnigramPrediction(PredictionTypes types,
                                   const ConversionRequest &request,
-                                  Segments *segments,
-                                  NodeAllocatorInterface *allocator,
+                                  const Segments &segments,
                                   vector<Result> *results) const;
 
   void AggregateBigramPrediction(PredictionTypes types,
                                  const ConversionRequest &request,
-                                 Segments *segments,
-                                 NodeAllocatorInterface *allocator,
+                                 const Segments &segments,
                                  vector<Result> *results) const;
 
   void AggregateSuffixPrediction(PredictionTypes types,
                                  const ConversionRequest &request,
-                                 Segments *segments,
-                                 NodeAllocatorInterface *allocator,
+                                 const Segments &segments,
                                  vector<Result> *results) const;
 
   void AggregateEnglishPrediction(PredictionTypes types,
                                   const ConversionRequest &request,
-                                  Segments *segments,
-                                  NodeAllocatorInterface *allocator,
+                                  const Segments &segments,
                                   vector<Result> *results) const;
 
+  void AggregateTypeCorrectingPrediction(PredictionTypes types,
+                                         const ConversionRequest &request,
+                                         const Segments &segments,
+                                         vector<Result> *results) const;
+
   void ApplyPenaltyForKeyExpansion(const Segments &segments,
                                    vector<Result> *results) const;
 
+  bool AddPredictionToCandidates(const ConversionRequest &request,
+                                 Segments *segments,
+                                 vector<Result> *results) const;
+
  private:
   FRIEND_TEST(DictionaryPredictorTest, GetPredictionTypes);
+  FRIEND_TEST(DictionaryPredictorTest,
+              GetPredictionTypesTestWithTypingCorrection);
   FRIEND_TEST(DictionaryPredictorTest,
               GetPredictionTypesTestWithZeroQuerySuggestion);
   FRIEND_TEST(DictionaryPredictorTest, IsZipCodeRequest);
   FRIEND_TEST(DictionaryPredictorTest, GetRealtimeCandidateMaxSize);
   FRIEND_TEST(DictionaryPredictorTest, GetRealtimeCandidateMaxSizeForMixed);
-  FRIEND_TEST(DictionaryPredictorTest, AggregateRealtimeConversion);
-  FRIEND_TEST(DictionaryPredictorTest, GetUnigramCandidateCutoffThreshold);
+  FRIEND_TEST(DictionaryPredictorTest,
+              GetRealtimeCandidateMaxSizeWithActualConverter);
+  FRIEND_TEST(DictionaryPredictorTest, GetCandidateCutoffThreshold);
   FRIEND_TEST(DictionaryPredictorTest, AggregateUnigramPrediction);
   FRIEND_TEST(DictionaryPredictorTest, AggregateBigramPrediction);
   FRIEND_TEST(DictionaryPredictorTest, AggregateSuffixPrediction);
   FRIEND_TEST(DictionaryPredictorTest, ZeroQuerySuggestionAfterNumbers);
+  FRIEND_TEST(DictionaryPredictorTest, TriggerNumberZeroQuerySuggestion);
   FRIEND_TEST(DictionaryPredictorTest, GetHistoryKeyAndValue);
   FRIEND_TEST(DictionaryPredictorTest, RealtimeConversionStartingWithAlphabets);
   FRIEND_TEST(DictionaryPredictorTest, IsAggressiveSuggestion);
-  FRIEND_TEST(DictionaryPredictorTest, LookupKeyValueFromDictionary);
   FRIEND_TEST(DictionaryPredictorTest,
               RealtimeConversionWithSpellingCorrection);
   FRIEND_TEST(DictionaryPredictorTest, GetMissSpelledPosition);
   FRIEND_TEST(DictionaryPredictorTest, RemoveMissSpelledCandidates);
   FRIEND_TEST(DictionaryPredictorTest, ConformCharacterWidthToPreference);
   FRIEND_TEST(DictionaryPredictorTest, SetLMCost);
-#ifdef DEBUG
+  FRIEND_TEST(DictionaryPredictorTest, SetDescription);
   FRIEND_TEST(DictionaryPredictorTest, SetDebugDescription);
-#endif  // DEBUG
-
-  class ResultCompare {
-   public:
-    bool operator() (const Result &a, const Result &b) const {
-      return a.cost > b.cost;
-    }
-  };
 
   // Returns false if no results were aggregated.
   bool AggregatePrediction(const ConversionRequest &request,
                            Segments *segments,
-                           NodeAllocatorInterface *allocator,
                            vector<Result> *results) const;
 
-  void SetCost(const Segments &segments, vector<Result> *results) const;
+  void SetCost(const ConversionRequest &request,
+               const Segments &segments, vector<Result> *results) const;
 
   // Removes prediciton by setting NO_PREDICTION to result type if necessary.
-  void RemovePrediction(const Segments &segments,
+  void RemovePrediction(const ConversionRequest &request,
+                        const Segments &segments,
                         vector<Result> *results) const;
 
-  bool AddPredictionToCandidates(const ConversionRequest &request,
-                                 Segments *segments,
-                                 vector<Result> *results) const;
-
   // Adds prediction results from history key and value.
   void AddBigramResultsFromHistory(const string &history_key,
                                    const string &history_value,
                                    const ConversionRequest &request,
-                                   Segments *segments,
-                                   NodeAllocatorInterface *allocator,
+                                   const Segments &segments,
                                    vector<Result> *results) const;
 
   // Changes the prediction type for irrelevant bigram candidate.
-  void CheckBigramResult(const Node *history_node,
+  void CheckBigramResult(const Token &history_token,
                          const Util::ScriptType history_ctype,
                          const Util::ScriptType last_history_ctype,
-                         NodeAllocatorInterface *allocator,
                          Result *result) const;
 
-  const Node *GetPredictiveNodes(const DictionaryInterface *dictionary,
-                                 const string &history_key,
-                                 const ConversionRequest &request,
-                                 const Segments &segments,
-                                 NodeAllocatorInterface *allocator) const;
+  void GetPredictiveResults(const DictionaryInterface &dictionary,
+                            const string &history_key,
+                            const ConversionRequest &request,
+                            const Segments &segments,
+                            PredictionTypes types,
+                            size_t lookup_limit,
+                            vector<Result> *results) const;
+
+  void GetPredictiveResultsForBigram(const DictionaryInterface &dictionary,
+                                     const string &history_key,
+                                     const string &history_value,
+                                     const ConversionRequest &request,
+                                     const Segments &segments,
+                                     PredictionTypes types,
+                                     size_t lookup_limit,
+                                     vector<Result> *results) const;
 
   // Performs a custom look up for English words where case-conversion might be
   // applied to lookup key and/or output results.
-  const Node *GetPredictiveNodesForEnglish(
-      const DictionaryInterface *dictionary,
+  void GetPredictiveResultsForEnglish(const DictionaryInterface &dictionary,
+                                      const string &history_key,
+                                      const ConversionRequest &request,
+                                      const Segments &segments,
+                                      PredictionTypes types,
+                                      size_t lookup_limit,
+                                      vector<Result> *results) const;
+
+  // Performs look-ups using type-corrected queries from composer. Usually
+  // involves multiple look-ups from dictionary.
+  void GetPredictiveResultsUsingTypingCorrection(
+      const DictionaryInterface &dictionary,
       const string &history_key,
       const ConversionRequest &request,
       const Segments &segments,
-      NodeAllocatorInterface *allocator) const;
+      PredictionTypes types,
+      size_t lookup_limit,
+      vector<Result> *results) const;
 
   // Returns the position of misspelled character position.
   //
@@ -230,16 +290,10 @@ class DictionaryPredictor : public PredictorInterface {
   size_t GetMissSpelledPosition(const string &key,
                                 const string &value) const;
 
-  // Returns true if key/value is in dictionary.
-  const Node *LookupKeyValueFromDictionary(
-      const string &key,
-      const string &value,
-      NodeAllocatorInterface *allocator) const;
-
-  // Returns language model cost of |node| given prediciton type |type|.
-  // |rid| is the right id of previous word (node).
+  // Returns language model cost of |token| given prediciton type |type|.
+  // |rid| is the right id of previous word (token).
   // If |rid| is uknown, set 0 as a default value.
-  int GetLMCost(PredictionTypes type, const Node &node, int rid) const;
+  int GetLMCost(const Result &result, int rid) const;
 
   // Given the results aggregated by aggregates, remove
   // miss-spelled results from the |results|.
@@ -295,9 +349,6 @@ class DictionaryPredictor : public PredictorInterface {
   void SetLMCost(const Segments &segments,
                  vector<Result> *results) const;
 
-  // Returns true if key consistes of '0'-'9' or '-'
-  bool IsZipCodeRequest(const string &key) const;
-
   // Returns true if the suggestion is classified
   // as "aggressive".
   bool IsAggressiveSuggestion(
@@ -311,55 +362,67 @@ class DictionaryPredictor : public PredictorInterface {
                              string *key, string *value) const;
 
   // Returns a bitfield of PredictionType.
-  PredictionTypes GetPredictionTypes(const ConversionRequest &request,
-                                     const Segments &segments) const;
+  static PredictionTypes GetPredictionTypes(const ConversionRequest &request,
+                                            const Segments &segments);
+
+  // Returns true if the realtime conversion should be used.
+  // TODO(hidehiko): add Config and Request instances into the arguments
+  //   to represent the dependency explicitly.
+  static bool ShouldRealTimeConversionEnabled(const ConversionRequest &request,
+                                              const Segments &segments);
+
+  // Returns true if key consistes of '0'-'9' or '-'
+  static bool IsZipCodeRequest(const string &key);
 
   // Returns max size of realtime candidates.
   size_t GetRealtimeCandidateMaxSize(const Segments &segments,
                                      bool mixed_conversion,
                                      size_t max_size) const;
 
+  // Aggregates unigram candidate for non mixed conversion.
+  void AggregateUnigramCandidate(const ConversionRequest &request,
+                                 const Segments &segments,
+                                 vector<Result> *results) const;
+
   // Aggregates unigram candidate for mixed conversion.
   // This reduces redundant candidates.
   void AggregateUnigramCandidateForMixedConversion(
       const ConversionRequest &request,
-      Segments *segments,
-      NodeAllocatorInterface *allocator,
+      const Segments &segments,
       vector<Result> *results) const;
 
-  // Aggregates unigram candidate for non mixed conversion.
-  void AggregateUnigramCandidate(const ConversionRequest &request,
-                                 Segments *segments,
-                                 NodeAllocatorInterface *allocator,
-                                 vector<Result> *results) const;
-
-  // Aggregates English candidates, basically for Latin input modes.
-  void AggregateEnglishCandidate(const ConversionRequest &request,
-                                 Segments *segments,
-                                 NodeAllocatorInterface *allocator,
-                                 vector<Result> *results) const;
-
   // Returns cutoff threshold of unigram candidates.
   // AggregateUnigramPrediction method does not return any candidates
   // if there are too many (>= cutoff threshold) eligible candidates.
   // This behavior prevents a user from seeing too many prefix-match
   // candidates.
-  size_t GetUnigramCandidateCutoffThreshold(const Segments &segments) const;
+  size_t GetCandidateCutoffThreshold(const Segments &segments) const;
 
-#ifdef DEBUG
-  // Provides useful information of suggestion source in debug build.
+  // Generates a top conversion result from |converter_| and adds its result to
+  // |results|.
+  bool PushBackTopConversionResult(const ConversionRequest &request,
+                                   const Segments &segments,
+                                   vector<Result> *results) const;
+
+  // Sets candidate description.
+  static void SetDescription(PredictionTypes types,
+                             uint32 attributes,
+                             string *description);
+  // Description for DEBUG mode.
   static void SetDebugDescription(PredictionTypes types,
                                   string *description);
-#endif  // DEBUG
 
+  const ConverterInterface *converter_;
   const ImmutableConverterInterface *immutable_converter_;
   const DictionaryInterface *dictionary_;
   const DictionaryInterface *suffix_dictionary_;
   const ConnectorInterface *connector_;
   const SegmenterInterface *segmenter_;
-  scoped_ptr<SuggestionFilter> suggestion_filter_;
+  const SuggestionFilter *suggestion_filter_;
   const uint16 counter_suffix_word_id_;
   const string predictor_name_;
+
+  DISALLOW_COPY_AND_ASSIGN(DictionaryPredictor);
 };
 }  // namespace mozc
 
