@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,9 +30,7 @@
 #ifndef MOZC_RENDERER_UNIX_CONST_H_
 #define MOZC_RENDERER_UNIX_CONST_H_
 
-#include <string>
-
-#include "base/base.h"
+#include "base/port.h"
 
 namespace mozc {
 namespace renderer {
