@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -126,12 +126,14 @@ void GtkWrapper::GdkThreadsLeave() {
   gdk_threads_leave();
 }
 
-guint GtkWrapper::GdkScreenGetWidth(GdkScreen *screen) {
-  return gdk_screen_get_width(screen);
+gint GtkWrapper::GdkScreenGetMonitorAtPoint(
+    GdkScreen *screen, gint x, gint y) {
+  return gdk_screen_get_monitor_at_point(screen, x, y);
 }
 
-guint GtkWrapper::GdkScreenGetHeight(GdkScreen *screen) {
-  return gdk_screen_get_height(screen);
+void GtkWrapper::GdkScreenGetMonitorGeometry(
+    GdkScreen *screen, gint monitor, GdkRectangle *rectangle) {
+  gdk_screen_get_monitor_geometry(screen, monitor, rectangle);
 }
 
 bool GtkWrapper::GtkWindowIsActive(GtkWidget *window) {
@@ -142,6 +144,15 @@ void GtkWrapper::GtkWidgetAddEvents(GtkWidget *widget, gint events) {
   gtk_widget_add_events(widget, events);
 }
 
+void GtkWrapper::GtkWidgetRealize(GtkWidget *widget) {
+  gtk_widget_realize(widget);
+}
+
+void GtkWrapper::GdkWindowSetTypeHint(GtkWidget *widget,
+                                      GdkWindowTypeHint hint) {
+  gdk_window_set_type_hint(widget->window, hint);
+}
+
 }  // namespace gtk
 }  // namespace renderer
 }  // namespace mozc
