@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,8 +33,9 @@
 
 #include <sstream>
 
-#include "base/util.h"
 #include "base/coordinates.h"
+#include "base/logging.h"
+#include "base/util.h"
 #include "client/client_interface.h"
 #include "renderer/renderer_command.pb.h"
 #include "renderer/renderer_style.pb.h"
@@ -71,7 +72,7 @@ const UINT_PTR kIdDelayShowHideTimer = 100;
 
 bool SendUsageStatsEvent(client::SendCommandInterface *command_sender,
                          const SessionCommand::UsageStatsEvent &event) {
-  if (command_sender == NULL) {
+  if (command_sender == nullptr) {
     return false;
   }
   SessionCommand command;
@@ -81,7 +82,7 @@ bool SendUsageStatsEvent(client::SendCommandInterface *command_sender,
   Output dummy_output;
   return command_sender->SendCommand(command, &dummy_output);
 }
-}
+}  // namespace
 
 
 // ------------------------------------------------------------------------
@@ -91,11 +92,10 @@ bool SendUsageStatsEvent(client::SendCommandInterface *command_sender,
 InfolistWindow::InfolistWindow()
     : candidates_(new commands::Candidates),
       metrics_changed_(false),
-      text_renderer_(new TextRenderer),
+      text_renderer_(TextRenderer::Create()),
       style_(new RendererStyle),
       visible_(false),
-      send_command_interface_(NULL) {
-  text_renderer_->Init();
+      send_command_interface_(nullptr) {
   mozc::renderer::RendererStyleHandler::GetRendererStyle(style_.get());
 }
 
@@ -126,7 +126,7 @@ void InfolistWindow::OnPaint(CDCHandle dc) {
   CRect client_rect;
   this->GetClientRect(&client_rect);
 
-  if (dc != NULL) {
+  if (dc != nullptr) {
     CMemoryDC memdc(dc, client_rect);
     DoPaint(memdc.m_hDC);
   } else  {
@@ -147,7 +147,7 @@ void InfolistWindow::OnPrintClient(CDCHandle dc, UINT uFlags) {
 }
 
 Size InfolistWindow::DoPaint(CDCHandle dc) {
-  if (dc.m_hDC != NULL) {
+  if (dc.m_hDC != nullptr) {
     dc.SetBkMode(TRANSPARENT);
   }
   const RendererStyle::InfolistStyle &infostyle = style_->infolist_style();
@@ -155,7 +155,7 @@ Size InfolistWindow::DoPaint(CDCHandle dc) {
 
   int ypos = infostyle.window_border();
 
-  if ((dc.m_hDC != NULL) && infostyle.has_caption_string()) {
+  if ((dc.m_hDC != nullptr) && infostyle.has_caption_string()) {
     const RendererStyle::TextStyle &caption_style =
       infostyle.caption_style();
     const int caption_height = infostyle.caption_height();
@@ -194,7 +194,7 @@ Size InfolistWindow::DoPaint(CDCHandle dc) {
   }
   ypos += infostyle.window_border();
 
-  if (dc.m_hDC != NULL) {
+  if (dc.m_hDC != nullptr) {
     const CRect rect(0, 0, infostyle.window_width(), ypos);
     dc.SetDCBrushColor(
         RGB(infostyle.border_color().r(),
@@ -236,7 +236,7 @@ Size InfolistWindow::DoPaintRow(CDCHandle dc, int row, int ypos) {
   int row_height = title_size.height + desc_size.height +
                    infostyle.row_rect_padding() * 2;
 
-  if (dc.m_hDC == NULL) {
+  if (dc.m_hDC == nullptr) {
     return Size(0, row_height);
   }
   const Rect title_rect(
@@ -336,7 +336,7 @@ void InfolistWindow::DelayShow(UINT mseconds) {
   visible_ = true;
   KillTimer(kIdDelayShowHideTimer);
   if (mseconds <= 0) {
-    const bool current_visible = IsWindowVisible();
+    const bool current_visible = (IsWindowVisible() != FALSE);
     SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0,
         SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE | SWP_SHOWWINDOW);
     SendMessageW(WM_NCACTIVATE, FALSE);
@@ -345,7 +345,7 @@ void InfolistWindow::DelayShow(UINT mseconds) {
           SessionCommand::INFOLIST_WINDOW_SHOW);
     }
   } else {
-    SetTimer(kIdDelayShowHideTimer, mseconds, NULL);
+    SetTimer(kIdDelayShowHideTimer, mseconds, nullptr);
   }
 }
 
@@ -353,14 +353,14 @@ void InfolistWindow::DelayHide(UINT mseconds) {
   visible_ = false;
   KillTimer(kIdDelayShowHideTimer);
   if (mseconds <= 0) {
-    const bool current_visible = IsWindowVisible();
+    const bool current_visible = (IsWindowVisible() != FALSE);
     ShowWindow(SW_HIDE);
     if (current_visible) {
       SendUsageStatsEvent(send_command_interface_,
           SessionCommand::INFOLIST_WINDOW_HIDE);
     }
   } else {
-    SetTimer(kIdDelayShowHideTimer, mseconds, NULL);
+    SetTimer(kIdDelayShowHideTimer, mseconds, nullptr);
   }
 }
 
@@ -369,7 +369,7 @@ void InfolistWindow::UpdateLayout(const commands::Candidates &candidates) {
 
   // If we detect any change of font parameters, update text renderer
   if (metrics_changed_) {
-    text_renderer_->Init();
+    text_renderer_->OnThemeChanged();
     metrics_changed_ = false;
   }
 }
@@ -380,7 +380,7 @@ void InfolistWindow::SetSendCommandInterface(
 }
 
 Size InfolistWindow::GetLayoutSize() {
-  CDCHandle dmyDc(NULL);
+  CDCHandle dmyDc(nullptr);
   return DoPaint(dmyDc);
 }
 }  // namespace win32
