@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,6 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/logging.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
@@ -53,11 +52,12 @@ void SetCandidate(const ReadingCorrectionItem *item,
       // "もしかして"
       "<\xE3\x82\x82\xE3\x81\x97\xE3\x81\x8B\xE3\x81\x97\xE3\x81\xA6: " +
       string(item->correction) + ">";
+  DCHECK(candidate->IsValid());
 }
 
 struct ReadingCorrectionItemCompare {
   bool operator()(const ReadingCorrectionItem &s1,
-                  const ReadingCorrectionItem &s2) {
+                  const ReadingCorrectionItem &s2) const {
     return (strcmp(s1.error, s2.error) < 0);
   }
 };
@@ -161,6 +161,7 @@ bool CorrectionRewriter::Rewrite(const ConversionRequest &request,
           top_candidate.functional_key();
       mutable_candidate->value = results[k]->value +
           top_candidate.functional_value();
+      mutable_candidate->inner_segment_boundary.clear();
       SetCandidate(results[k], mutable_candidate);
       modified = true;
     }
