@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,6 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/file_stream.h"
 #include "base/logging.h"
 #include "base/util.h"
@@ -55,13 +54,13 @@ struct CompilerToken {
 
 struct TokenCompare {
   bool operator()(const EmbeddedDictionary::Token &s1,
-                  const EmbeddedDictionary::Token &s2) {
+                  const EmbeddedDictionary::Token &s2) const {
     return (strcmp(s1.key, s2.key) < 0);
   }
 };
 
 struct CompareByCost {
-  bool operator()(const CompilerToken &t1, const CompilerToken &t2) {
+  bool operator()(const CompilerToken &t1, const CompilerToken &t2) const {
     return (t1.cost < t2.cost);
   }
 };
@@ -101,15 +100,15 @@ void EmbeddedDictionary::Compile(const string &name,
                                  const string &input,
                                  const string &output) {
   InputFileStream ifs(input.c_str());
-  CHECK(ifs);
+  CHECK(ifs.good());
 
   OutputFileStream ofs(output.c_str());
-  CHECK(ofs);
+  CHECK(ofs.good());
 
   string line;
   vector<string> fields;
   map<string, vector<CompilerToken> > dic;
-  while (getline(ifs, line)) {
+  while (!getline(ifs, line).fail()) {
     fields.clear();
     Util::SplitStringUsing(line, "\t", &fields);
     CHECK_GE(fields.size(), 4);
