@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,15 +27,16 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#include "rewriter/english_variants_rewriter.h"
+
+#include <cstddef>
 #include <string>
 
 #include "base/number_util.h"
-#include "base/util.h"
+#include "base/system_util.h"
 #include "converter/conversion_request.h"
 #include "converter/segments.h"
-#include "rewriter/english_variants_rewriter.h"
 #include "session/commands.pb.h"
-#include "session/request_handler.h"
 #include "testing/base/public/gunit.h"
 
 DECLARE_string(test_tmpdir);
@@ -45,7 +46,7 @@ namespace mozc {
 class EnglishVariantsRewriterTest : public testing::Test {
  protected:
   virtual void SetUp() {
-    Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+    SystemUtil::SetUserProfileDirectory(FLAGS_test_tmpdir);
   }
 };
 
@@ -202,8 +203,8 @@ TEST_F(EnglishVariantsRewriterTest, Regression3242753) {
     EXPECT_EQ(1, seg->candidates_size());
     EXPECT_EQ("Michael Jackson", seg->candidate(0).value);
     EXPECT_EQ("Michael Jackson", seg->candidate(0).content_value);
-    EXPECT_TRUE(seg->candidate(0).attributes &
-                Segment::Candidate::NO_VARIANTS_EXPANSION);
+    EXPECT_NE(0, (seg->candidate(0).attributes &
+                  Segment::Candidate::NO_VARIANTS_EXPANSION));
   }
 }
 
@@ -282,8 +283,8 @@ TEST_F(EnglishVariantsRewriterTest, ExpandEnglishEntry) {
     EXPECT_EQ(1, seg->candidates_size());
     EXPECT_EQ("Google", seg->candidate(0).value);
     EXPECT_EQ("Google", seg->candidate(0).content_value);
-    EXPECT_TRUE(seg->candidate(0).attributes &
-                Segment::Candidate::NO_VARIANTS_EXPANSION);
+    EXPECT_NE(0, (seg->candidate(0).attributes &
+                  Segment::Candidate::NO_VARIANTS_EXPANSION));
   }
 }
 
@@ -291,13 +292,17 @@ TEST_F(EnglishVariantsRewriterTest, MobileEnvironmentTest) {
   commands::Request input;
   EnglishVariantsRewriter rewriter;
 
-  input.set_mixed_conversion(true);
-  commands::RequestHandler::SetRequest(input);
-  EXPECT_EQ(RewriterInterface::ALL, rewriter.capability());
+  {
+    input.set_mixed_conversion(true);
+    const ConversionRequest request(NULL, &input);
+    EXPECT_EQ(RewriterInterface::ALL, rewriter.capability(request));
+  }
 
-  input.set_mixed_conversion(false);
-  commands::RequestHandler::SetRequest(input);
-  EXPECT_EQ(RewriterInterface::CONVERSION, rewriter.capability());
+  {
+    input.set_mixed_conversion(false);
+    const ConversionRequest request(NULL, &input);
+    EXPECT_EQ(RewriterInterface::CONVERSION, rewriter.capability(request));
+  }
 }
 
 }  // namespace mozc
