@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -46,12 +46,12 @@
 #include "base/file_stream.h"
 #include "base/logging.h"
 #include "base/util.h"
-#include "rewriter/gen_existence_header.h"
+#include "rewriter/gen_existence_data.h"
 
 DEFINE_string(suppression_data, "", "suppression data text");
 DEFINE_string(output, "", "output file name (default: stdout)");
 DEFINE_double(error_rate, 0.00001, "error rate");
-DECLARE_bool(logtostderr);
+DEFINE_bool(binary_mode, false, "outputs binary file");
 
 namespace mozc {
 namespace {
@@ -67,7 +67,7 @@ void Convert() {
     InputFileStream ifs(FLAGS_suppression_data.c_str());
     string line;
 
-    while (getline(ifs, line)) {
+    while (!getline(ifs, line).fail()) {
       if (line.empty()) {
         continue;
       }
@@ -80,13 +80,19 @@ void Convert() {
 
   ostream *ofs = &cout;
   if (!FLAGS_output.empty()) {
-    ofs = new OutputFileStream(FLAGS_output.c_str());
+    if (FLAGS_binary_mode) {
+      ofs = new OutputFileStream(FLAGS_output.c_str(), ios::out | ios::binary);
+    } else {
+      ofs = new OutputFileStream(FLAGS_output.c_str());
+    }
   }
 
-  const string kNameSpace = "CollocationSuppressionData";
-
-  GenExistenceHeader::GenExistenceHeader(entries, kNameSpace, ofs,
-                                               FLAGS_error_rate);
+  if (FLAGS_binary_mode) {
+    OutputExistenceBinary(entries, ofs, FLAGS_error_rate);
+  } else {
+    const string kNameSpace = "CollocationSuppressionData";
+    OutputExistenceHeader(entries, kNameSpace, ofs, FLAGS_error_rate);
+  }
 
   if (ofs != &cout) {
     delete ofs;
@@ -96,7 +102,6 @@ void Convert() {
 }  // namespace mozc
 
 int main(int argc, char *argv[]) {
-  FLAGS_logtostderr = true;
   InitGoogle(argv[0], &argc, &argv, true);
 
   LOG(INFO) << FLAGS_suppression_data;
