@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,11 +32,13 @@
 
 #include <vector>
 
-#include "base/base.h"
 #include "base/stl_util.h"
+#include "config/config.pb.h"
+#include "config/config_handler.h"
 #include "converter/conversion_request.h"
 #include "converter/segments.h"
 #include "rewriter/rewriter_interface.h"
+#include "session/commands.pb.h"
 
 namespace mozc {
 
@@ -48,22 +50,25 @@ class MergerRewriter : public RewriterInterface {
   }
 
   // return true if rewriter can be called with the segments.
-  bool CheckCapablity(Segments *segments,
+  bool CheckCapablity(const ConversionRequest &request, Segments *segments,
                       RewriterInterface *rewriter) const {
     if (segments == NULL) {
       return false;
     }
     switch (segments->request_type()) {
       case Segments::CONVERSION:
-        return (rewriter->capability() & RewriterInterface::CONVERSION);
+        return ((rewriter->capability(request) & RewriterInterface::CONVERSION)
+                != 0);
 
       case Segments::PREDICTION:
       case Segments::PARTIAL_PREDICTION:
-        return (rewriter->capability() & RewriterInterface::PREDICTION);
+        return ((rewriter->capability(request) & RewriterInterface::PREDICTION)
+                != 0);
 
       case Segments::SUGGESTION:
       case Segments::PARTIAL_SUGGESTION:
-        return (rewriter->capability() & RewriterInterface::SUGGESTION);
+        return ((rewriter->capability(request) & RewriterInterface::SUGGESTION)
+                != 0);
 
       case Segments::REVERSE_CONVERSION:
       default:
@@ -80,10 +85,22 @@ class MergerRewriter : public RewriterInterface {
                        Segments *segments) const {
     bool result = false;
     for (size_t i = 0; i < rewriters_.size(); ++i) {
-      if (CheckCapablity(segments, rewriters_[i])) {
+      if (CheckCapablity(request, segments, rewriters_[i])) {
         result |= rewriters_[i]->Rewrite(request, segments);
       }
     }
+
+    if (segments->request_type() == Segments::SUGGESTION &&
+        segments->conversion_segments_size() == 1 &&
+        !request.request().mixed_conversion()) {
+      const size_t max_suggestions = GET_CONFIG(suggestions_size);
+      Segment *segment = segments->mutable_conversion_segment(0);
+      const size_t candidate_size = segment->candidates_size();
+      if (candidate_size > max_suggestions) {
+        segment->erase_candidates(max_suggestions,
+                                  candidate_size - max_suggestions);
+      }
+    }
     return result;
   }
 
@@ -105,9 +122,9 @@ class MergerRewriter : public RewriterInterface {
   }
 
   // Hook(s) for all mutable operations
-  virtual void Finish(Segments *segments) {
+  virtual void Finish(const ConversionRequest &request, Segments *segments) {
     for (size_t i = 0; i < rewriters_.size(); ++i) {
-      rewriters_[i]->Finish(segments);
+      rewriters_[i]->Finish(request, segments);
     }
   }
 
