@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,6 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/logging.h"
 #include "base/text_normalizer.h"
 #include "base/util.h"
@@ -84,7 +83,7 @@ NormalizationRewriter::NormalizationRewriter() {}
 
 NormalizationRewriter::~NormalizationRewriter() {}
 
-int NormalizationRewriter::capability() const {
+int NormalizationRewriter::capability(const ConversionRequest &request) const {
   return RewriterInterface::ALL;
 }
 
