@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,26 +30,35 @@
 #ifndef MOZC_REWRITER_NUMBER_REWRITER_H_
 #define MOZC_REWRITER_NUMBER_REWRITER_H_
 
+#include "base/port.h"
 #include "rewriter/rewriter_interface.h"
 
 namespace mozc {
+
 class ConversionRequest;
+class DataManagerInterface;
 class POSMatcher;
 class Segments;
+struct CounterSuffixEntry;
 
 class NumberRewriter : public RewriterInterface  {
  public:
-  explicit NumberRewriter(const POSMatcher *pos_matcher);
+  explicit NumberRewriter(const DataManagerInterface *data_manager);
   virtual ~NumberRewriter();
 
-  virtual int capability() const;
+  virtual int capability(const ConversionRequest &request) const;
 
   virtual bool Rewrite(const ConversionRequest &request,
                        Segments *segments) const;
 
  private:
+  const CounterSuffixEntry *suffix_array_;
+  size_t suffix_array_size_;
   const POSMatcher *pos_matcher_;
+
+  DISALLOW_COPY_AND_ASSIGN(NumberRewriter);
 };
 
 }  // namespace mozc
+
 #endif  // MOZC_REWRITER_NUMBER_REWRITER_H_
