@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,23 +29,36 @@
 
 #include "rewriter/number_rewriter.h"
 
+#include <cstddef>
 #include <string>
 
-#include "base/base.h"
 #include "base/logging.h"
+#include "base/port.h"
+#include "base/system_util.h"
 #include "base/util.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
 #include "converter/conversion_request.h"
 #include "converter/segments.h"
-#include "data_manager/user_pos_manager.h"
+#ifdef MOZC_USE_PACKED_DICTIONARY
+#include "data_manager/packed/packed_data_manager.h"
+#include "data_manager/packed/packed_data_mock.h"
+#endif  // MOZC_USE_PACKED_DICTIONARY
+#include "data_manager/testing/mock_data_manager.h"
 #include "dictionary/pos_matcher.h"
 #include "session/commands.pb.h"
-#include "session/request_handler.h"
 #include "testing/base/public/gunit.h"
 
 DECLARE_string(test_tmpdir);
 
+// To show the value of size_t, 'z' speficier should be used.
+// But MSVC doesn't support it yet so use 'l' instead.
+#ifdef _MSC_VER
+#define SIZE_T_PRINTF_FORMAT "%lu"
+#else  // _MSC_VER
+#define SIZE_T_PRINTF_FORMAT "%zu"
+#endif  // _MSC_VER
+
 namespace mozc {
 
 namespace {
@@ -76,13 +89,12 @@ bool FindValue(const Segment &segment, const string &value) {
   return false;
 }
 
-Segment *SetupSegments(const string &candidate_value, Segments *segments) {
+Segment *SetupSegments(const POSMatcher* pos_matcher,
+                       const string &candidate_value, Segments *segments) {
   segments->Clear();
   Segment *segment = segments->push_back_segment();
   Segment::Candidate *candidate = segment->add_candidate();
   candidate->Init();
-  const POSMatcher *pos_matcher =
-      UserPosManager::GetUserPosManager()->GetPOSMatcher();
   candidate->lid = pos_matcher->GetNumberId();
   candidate->rid = pos_matcher->GetNumberId();
   candidate->value = candidate_value;
@@ -111,25 +123,45 @@ bool FindCandidateId(const Segment &segment, const string &value, int *id) {
 }
 }  // namespace
 
-class NumberRewriterTest : public testing::Test {
+class NumberRewriterTest : public ::testing::Test {
  protected:
   // Explicitly define constructor to prevent Visual C++ from
   // considering this class as POD.
   NumberRewriterTest() {}
 
   virtual void SetUp() {
-    Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+#ifdef MOZC_USE_PACKED_DICTIONARY
+    // TODO(noriyukit): Currently this test uses mock data manager.  Check if we
+    // can remove this registration of packed data manager.
+    // Registers mocked PackedDataManager.
+    scoped_ptr<packed::PackedDataManager>
+        data_manager(new packed::PackedDataManager());
+    CHECK(data_manager->Init(string(kPackedSystemDictionary_data,
+                                    kPackedSystemDictionary_size)));
+    packed::RegisterPackedDataManager(data_manager.release());
+#endif  // MOZC_USE_PACKED_DICTIONARY
+
+    SystemUtil::SetUserProfileDirectory(FLAGS_test_tmpdir);
     config::Config default_config;
     config::ConfigHandler::GetDefaultConfig(&default_config);
     config::ConfigHandler::SetConfig(default_config);
-    pos_matcher_ = UserPosManager::GetUserPosManager()->GetPOSMatcher();
+    pos_matcher_ = mock_data_manager_.GetPOSMatcher();
+  }
+
+  virtual void TearDown() {
+#ifdef MOZC_USE_PACKED_DICTIONARY
+    // Unregisters mocked PackedDataManager.
+    packed::RegisterPackedDataManager(NULL);
+#endif  // MOZC_USE_PACKED_DICTIONARY
   }
 
   NumberRewriter *CreateNumberRewriter() {
-    return new NumberRewriter(pos_matcher_);
+    return new NumberRewriter(&mock_data_manager_);
   }
 
+  const testing::MockDataManager mock_data_manager_;
   const POSMatcher *pos_matcher_;
+  const ConversionRequest default_request_;
 };
 
 namespace {
@@ -152,7 +184,7 @@ TEST_F(NumberRewriterTest, BasicTest) {
   candidate->value = "012";
   candidate->content_value = "012";
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   const ExpectResult kExpectResults[] = {
     {"012", "012", ""},
@@ -186,7 +218,7 @@ TEST_F(NumberRewriterTest, BasicTest) {
   EXPECT_EQ(kExpectResultSize, seg->candidates_size());
 
   for (size_t i = 0; i < kExpectResultSize; ++i) {
-    SCOPED_TRACE(Util::StringPrintf("i = %lu", i));
+    SCOPED_TRACE(Util::StringPrintf("i = " SIZE_T_PRINTF_FORMAT, i));
     EXPECT_EQ(kExpectResults[i].value, seg->candidate(i).value);
     EXPECT_EQ(kExpectResults[i].content_value,
               seg->candidate(i).content_value);
@@ -215,7 +247,7 @@ TEST_F(NumberRewriterTest, RequestType) {
 
   scoped_ptr<NumberRewriter> number_rewriter(CreateNumberRewriter());
 
-  for (size_t i = 0; i < ARRAYSIZE_UNSAFE(test_data_list); ++i) {
+  for (size_t i = 0; i < arraysize(test_data_list); ++i) {
     TestData& test_data = test_data_list[i];
     Segments segments;
     segments.set_request_type(test_data.request_type_);
@@ -226,7 +258,7 @@ TEST_F(NumberRewriterTest, RequestType) {
     candidate->rid = pos_matcher_->GetNumberId();
     candidate->value = "012";
     candidate->content_value = "012";
-    EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+    EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
     EXPECT_EQ(test_data.expected_candidate_number_, seg->candidates_size());
   }
 }
@@ -243,7 +275,7 @@ TEST_F(NumberRewriterTest, BasicTestWithSuffix) {
   candidate->value = "012""\xE3\x81\x8C";   // "012が"
   candidate->content_value = "012";
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   const ExpectResult kExpectResults[] = {
     // "012"
@@ -278,7 +310,7 @@ TEST_F(NumberRewriterTest, BasicTestWithSuffix) {
   EXPECT_EQ(kExpectResultSize, seg->candidates_size());
 
   for (size_t i = 0; i < kExpectResultSize; ++i) {
-    SCOPED_TRACE(Util::StringPrintf("i = %lu", i));
+    SCOPED_TRACE(Util::StringPrintf("i = " SIZE_T_PRINTF_FORMAT, i));
     EXPECT_EQ(kExpectResults[i].value, seg->candidate(i).value);
     EXPECT_EQ(kExpectResults[i].content_value,
               seg->candidate(i).content_value);
@@ -301,7 +333,7 @@ TEST_F(NumberRewriterTest, BasicTestWithNumberSuffix) {
   candidate->value = "\xE5\x8D\x81\xE4\xBA\x94\xE5\x80\x8B";  // "十五個"
   candidate->content_value = "\xE5\x8D\x81\xE4\xBA\x94\xE5\x80\x8B";  // ditto
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   EXPECT_EQ(2, seg->candidates_size());
 
@@ -336,7 +368,7 @@ TEST_F(NumberRewriterTest, TestWithMultipleNumberSuffix) {
   candidate->value = "\xE5\x8D\x81\xE4\xBA\x94\xE9\x9A\x8E";  // "十五階"
   candidate->content_value = "\xE5\x8D\x81\xE4\xBA\x94\xE9\x9A\x8E";  // ditto
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   EXPECT_EQ(4, seg->candidates_size());
 
@@ -368,46 +400,45 @@ TEST_F(NumberRewriterTest, TestWithMultipleNumberSuffix) {
 TEST_F(NumberRewriterTest, SpecialFormBoundaries) {
   scoped_ptr<NumberRewriter> number_rewriter(CreateNumberRewriter());
   Segments segments;
-  const ConversionRequest request;
 
   // Special forms doesn't have zeros.
-  Segment *seg = SetupSegments("0", &segments);
-  EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+  Segment *seg = SetupSegments(pos_matcher_, "0", &segments);
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_FALSE(HasDescription(*seg, kMaruNumberDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanCapitalDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanNoCapitalDescription));
 
   // "1" has special forms.
-  seg = SetupSegments("1", &segments);
-  EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+  seg = SetupSegments(pos_matcher_, "1", &segments);
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_TRUE(HasDescription(*seg, kMaruNumberDescription));
   EXPECT_TRUE(HasDescription(*seg, kRomanCapitalDescription));
   EXPECT_TRUE(HasDescription(*seg, kRomanNoCapitalDescription));
 
   // "12" has every special forms.
-  seg = SetupSegments("12", &segments);
-  EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+  seg = SetupSegments(pos_matcher_, "12", &segments);
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_TRUE(HasDescription(*seg, kMaruNumberDescription));
   EXPECT_TRUE(HasDescription(*seg, kRomanCapitalDescription));
   EXPECT_TRUE(HasDescription(*seg, kRomanNoCapitalDescription));
 
   // "13" doesn't have roman forms.
-  seg = SetupSegments("13", &segments);
-  EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+  seg = SetupSegments(pos_matcher_, "13", &segments);
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_TRUE(HasDescription(*seg, kMaruNumberDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanCapitalDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanNoCapitalDescription));
 
   // "50" has circled numerics.
-  seg = SetupSegments("50", &segments);
-  EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+  seg = SetupSegments(pos_matcher_, "50", &segments);
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_TRUE(HasDescription(*seg, kMaruNumberDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanCapitalDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanNoCapitalDescription));
 
   // "51" doesn't have special forms.
-  seg = SetupSegments("51", &segments);
-  EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+  seg = SetupSegments(pos_matcher_, "51", &segments);
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_FALSE(HasDescription(*seg, kMaruNumberDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanCapitalDescription));
   EXPECT_FALSE(HasDescription(*seg, kRomanNoCapitalDescription));
@@ -433,7 +464,7 @@ TEST_F(NumberRewriterTest, OneOfCandidatesIsEmpty) {
   second_candidate->rid = pos_matcher_->GetNumberId();
   second_candidate->content_value = second_candidate->value;
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   EXPECT_EQ("", seg->candidate(0).value);
   EXPECT_EQ("", seg->candidate(0).content_value);
@@ -478,7 +509,7 @@ TEST_F(NumberRewriterTest, RewriteDoesNotHappen) {
   candidate->content_value = candidate->value;
 
   // Number rewrite should not occur
-  EXPECT_FALSE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_FALSE(number_rewriter->Rewrite(default_request_, &segments));
 
   // Number of cahdidates should be maintained
   EXPECT_EQ(1, seg->candidates_size());
@@ -498,7 +529,7 @@ TEST_F(NumberRewriterTest, NumberIsZero) {
   candidate->value = "0";
   candidate->content_value = "0";
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   EXPECT_EQ(4, seg->candidates_size());
 
@@ -540,7 +571,7 @@ TEST_F(NumberRewriterTest, NumberIsZeroZero) {
   candidate->value = "00";
   candidate->content_value = "00";
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   EXPECT_EQ(4, seg->candidates_size());
 
@@ -580,7 +611,7 @@ TEST_F(NumberRewriterTest, NumberIs19Digit) {
   candidate->value = "1000000000000000000";
   candidate->content_value = "1000000000000000000";
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   const ExpectResult kExpectResults[] = {
     {"1000000000000000000", "1000000000000000000", ""},
@@ -641,7 +672,7 @@ TEST_F(NumberRewriterTest, NumberIs19Digit) {
   EXPECT_EQ(kExpectResultSize, seg->candidates_size());
 
   for (size_t i = 0; i < kExpectResultSize; ++i) {
-    SCOPED_TRACE(Util::StringPrintf("i = %lu", i));
+    SCOPED_TRACE(Util::StringPrintf("i = " SIZE_T_PRINTF_FORMAT, i));
     EXPECT_EQ(kExpectResults[i].value, seg->candidate(i).value);
     EXPECT_EQ(kExpectResults[i].content_value,
               seg->candidate(i).content_value);
@@ -664,7 +695,7 @@ TEST_F(NumberRewriterTest, NumberIsGreaterThanUInt64Max) {
   candidate->value = "18446744073709551616";  // 2^64
   candidate->content_value = "18446744073709551616";
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   const ExpectResult kExpectResults[] = {
     {"18446744073709551616", "18446744073709551616", ""},
@@ -746,7 +777,7 @@ TEST_F(NumberRewriterTest, NumberIsGreaterThanUInt64Max) {
   EXPECT_EQ(kExpectResultSize, seg->candidates_size());
 
   for (size_t i = 0; i < kExpectResultSize; ++i) {
-    SCOPED_TRACE(Util::StringPrintf("i = %lu", i));
+    SCOPED_TRACE(Util::StringPrintf("i = " SIZE_T_PRINTF_FORMAT, i));
     EXPECT_EQ(kExpectResults[i].value, seg->candidate(i).value);
     EXPECT_EQ(kExpectResults[i].content_value,
               seg->candidate(i).content_value);
@@ -776,7 +807,7 @@ TEST_F(NumberRewriterTest, NumberIsGoogol) {
   candidate->value = input;
   candidate->content_value = input;
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   EXPECT_EQ(6, seg->candidates_size());
 
@@ -855,7 +886,7 @@ TEST_F(NumberRewriterTest, RankingForKanjiCandidate) {
     candidate->content_value = "\xe4\xb8\x89\xe7\x99\xbe";
   }
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   EXPECT_NE(0, segments.segments_size());
   int kanji_pos = 0, arabic_pos = 0;
   // "三百"
@@ -902,7 +933,7 @@ TEST_F(NumberRewriterTest, ModifyExsistingRanking) {
     candidate->content_value = "\xe4\xb8\x89\xe7\x99\xbe";
   }
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
   int kanji_pos = 0, old_kanji_pos = 0;
   EXPECT_NE(0, segments.segments_size());
   // "三百"
@@ -950,7 +981,7 @@ TEST_F(NumberRewriterTest, EraseExistingCandidates) {
     candidate->content_value = "\xe4\xb8\x80";
   }
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 
   // "一" become the base candidate, instead of "壱"
   int base_pos = 0;
@@ -971,7 +1002,6 @@ TEST_F(NumberRewriterTest, EraseExistingCandidates) {
 
 TEST_F(NumberRewriterTest, SeparatedArabicsTest) {
   scoped_ptr<NumberRewriter> number_rewriter(CreateNumberRewriter());
-  const ConversionRequest request;
 
   // Expected data to succeed tests.
   const char* kSuccess[][3] = {
@@ -996,7 +1026,7 @@ TEST_F(NumberRewriterTest, SeparatedArabicsTest) {
     candidate->rid = pos_matcher_->GetNumberId();
     candidate->value = kSuccess[i][0];
     candidate->content_value = kSuccess[i][0];
-    EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+    EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
     EXPECT_TRUE(FindValue(segments.segment(0), kSuccess[i][1]))
         << "Input : " << kSuccess[i][0];
     EXPECT_TRUE(FindValue(segments.segment(0), kSuccess[i][2]))
@@ -1023,7 +1053,7 @@ TEST_F(NumberRewriterTest, SeparatedArabicsTest) {
     candidate->rid = pos_matcher_->GetNumberId();
     candidate->value = kFail[i][0];
     candidate->content_value = kFail[i][0];
-    EXPECT_TRUE(number_rewriter->Rewrite(request, &segments));
+    EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
     EXPECT_FALSE(FindValue(segments.segment(0), kFail[i][1]))
         << "Input : " << kFail[i][0];
     EXPECT_FALSE(FindValue(segments.segment(0), kFail[i][2]))
@@ -1059,7 +1089,7 @@ TEST_F(NumberRewriterTest, PreserveUserDictionaryAttibute) {
           Segment::Candidate::NO_VARIANTS_EXPANSION;
     }
 
-    EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+    EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
     bool base_candidate_found = false;
     {
       const Segment &segment = segments.segment(0);
@@ -1112,20 +1142,41 @@ TEST_F(NumberRewriterTest, DuplicateCandidateTest) {
     candidate->content_value = "\xe4\xb8\x80";
   }
 
-  EXPECT_TRUE(number_rewriter->Rewrite(ConversionRequest(), &segments));
+  EXPECT_TRUE(number_rewriter->Rewrite(default_request_, &segments));
 }
 
 TEST_F(NumberRewriterTest, MobileEnvironmentTest) {
   commands::Request input;
   scoped_ptr<NumberRewriter> rewriter(CreateNumberRewriter());
 
-  input.set_mixed_conversion(true);
-  commands::RequestHandler::SetRequest(input);
-  EXPECT_EQ(RewriterInterface::ALL, rewriter->capability());
+  {
+    input.set_mixed_conversion(true);
+    const ConversionRequest request(NULL, &input);
+    EXPECT_EQ(RewriterInterface::ALL, rewriter->capability(request));
+  }
 
-  input.set_mixed_conversion(false);
-  commands::RequestHandler::SetRequest(input);
-  EXPECT_EQ(RewriterInterface::CONVERSION, rewriter->capability());
+  {
+    input.set_mixed_conversion(false);
+    const ConversionRequest request(NULL, &input);
+    EXPECT_EQ(RewriterInterface::CONVERSION, rewriter->capability(request));
+  }
+}
+
+TEST_F(NumberRewriterTest, NonNumberNounTest) {
+  // Test if "百舌鳥" is not rewritten to "100舌鳥", etc.
+  scoped_ptr<NumberRewriter> number_rewriter(CreateNumberRewriter());
+  Segments segments;
+  Segment *segment = segments.push_back_segment();
+  segment->set_key("\xE3\x82\x82\xE3\x81\x9A");  // "もず"
+  Segment::Candidate *cand = segment->add_candidate();
+  cand->Init();
+  cand->key = "\xE3\x82\x82\xE3\x81\x9A";  // "もず"
+  cand->content_key = cand->key;
+  cand->value = "\xE7\x99\xBE\xE8\x88\x8C\xE9\xB3\xA5";  // "百舌鳥"
+  cand->content_value = cand->value;
+  cand->lid = pos_matcher_->GetGeneralNounId();
+  cand->rid = pos_matcher_->GetGeneralNounId();
+  EXPECT_FALSE(number_rewriter->Rewrite(default_request_, &segments));
 }
 
 }  // namespace mozc
