@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -36,18 +36,16 @@ namespace mozc {
 
 class ConverterInterface;
 class DataManagerInterface;
+class DictionaryInterface;
 class POSMatcher;
 class PosGroup;
 
 class RewriterImpl : public MergerRewriter {
  public:
   RewriterImpl(const ConverterInterface *parent_converter,
-               const DataManagerInterface *data_manager);
-
- private:
-  const ConverterInterface *parent_converter_;
-  const POSMatcher *pos_matcher_;
-  const PosGroup *pos_group_;
+               const DataManagerInterface *data_manager,
+               const PosGroup *pos_group,
+               const DictionaryInterface *dictionary);
 };
 
 }  // namespace mozc
