@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,6 @@
 #include <vector>
 #include <set>
 
-#include "base/base.h"
 #include "base/logging.h"
 #include "base/singleton.h"
 #include "base/util.h"
@@ -46,7 +45,6 @@
 #include "rewriter/embedded_dictionary.h"
 #include "rewriter/rewriter_interface.h"
 #include "session/commands.pb.h"
-#include "session/request_handler.h"
 
 namespace mozc {
 
@@ -91,7 +89,8 @@ class NounPrefixDictionary {
 };
 
 struct SingleKanjiListCompare {
-  bool operator() (const SingleKanjiList &lhs, const SingleKanjiList &rhs) {
+  bool operator()(const SingleKanjiList &lhs,
+                  const SingleKanjiList &rhs) const {
     return (strcmp(lhs.key, rhs.key) < 0);
   }
 };
@@ -114,7 +113,8 @@ bool LookupKanjiList(const string &key, vector<string> *kanji_list) {
 }
 
 struct KanjiVariantItemCompare {
-  bool operator() (const KanjiVariantItem &lhs, const KanjiVariantItem &rhs) {
+  bool operator()(const KanjiVariantItem &lhs,
+                  const KanjiVariantItem &rhs) const {
     return (strcmp(lhs.target, rhs.target) < 0);
   }
 };
@@ -241,8 +241,8 @@ SingleKanjiRewriter::SingleKanjiRewriter(const POSMatcher &pos_matcher)
 
 SingleKanjiRewriter::~SingleKanjiRewriter() {}
 
-int SingleKanjiRewriter::capability() const {
-  if (GET_REQUEST(mixed_conversion)) {
+int SingleKanjiRewriter::capability(const ConversionRequest &request) const {
+  if (request.request().mixed_conversion()) {
     return RewriterInterface::ALL;
   }
   return RewriterInterface::CONVERSION;
