@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,8 +33,8 @@
 #include <utility>
 #include <vector>
 
-#include "base/base.h"
 #include "base/config_file_stream.h"
+#include "base/file_util.h"
 #include "base/logging.h"
 #include "base/util.h"
 #include "config/config.pb.h"
@@ -49,7 +49,7 @@
 
 namespace mozc {
 
-using mozc::storage::LRUStorage;
+using storage::LRUStorage;
 
 namespace {
 const int kValueSize  = 4;
@@ -57,7 +57,6 @@ const uint32 kLRUSize = 5000;
 const uint32 kSeedValue = 0x761fea81;
 
 const char kFileName[] = "user://boundary.db";
-LRUStorage *g_lru_storage = NULL;
 
 enum { INSERT, RESIZE };
 
@@ -113,13 +112,17 @@ UserBoundaryHistoryRewriter::UserBoundaryHistoryRewriter(
     : parent_converter_(parent_converter),
       storage_(new LRUStorage) {
   DCHECK(parent_converter_);
-  g_lru_storage = storage_.get();
   Reload();
 }
 
 UserBoundaryHistoryRewriter::~UserBoundaryHistoryRewriter() {}
 
-void UserBoundaryHistoryRewriter::Finish(Segments *segments) {
+void UserBoundaryHistoryRewriter::Finish(const ConversionRequest &request,
+                                         Segments *segments) {
+  if (segments->request_type() != Segments::CONVERSION) {
+    return;
+  }
+
   if (GET_CONFIG(incognito_mode)) {
     VLOG(2) << "incognito mode";
     return;
@@ -142,9 +145,7 @@ void UserBoundaryHistoryRewriter::Finish(Segments *segments) {
   }
 
   if (segments->resized()) {
-    // |ResizeOrInsert| does NOT call Converter::ResizeSegment since we pass
-    // INSERT as an argument, so we can use dummy ConversionRequest.
-    ResizeOrInsert(segments, ConversionRequest(), INSERT);
+    ResizeOrInsert(segments, request, INSERT);
 #ifdef OS_ANDROID
     // TODO(hidehiko): UsageStats requires some functionalities, e.g. network,
     // which are not needed for mozc's main features.
@@ -183,6 +184,10 @@ bool UserBoundaryHistoryRewriter::Rewrite(
     return false;
   }
 
+  if (request.skip_slow_rewriters()) {
+    return false;
+  }
+
   if (!segments->resized()) {
     return ResizeOrInsert(segments, request, RESIZE);
   }
@@ -196,7 +201,6 @@ bool UserBoundaryHistoryRewriter::Reload() {
                               kValueSize, kLRUSize, kSeedValue)) {
     LOG(WARNING) << "cannot initialize UserBoundaryHistoryRewriter";
     storage_.reset(NULL);
-    g_lru_storage = NULL;
     return false;
   }
 
@@ -204,9 +208,9 @@ bool UserBoundaryHistoryRewriter::Reload() {
   const string merge_pending_file = filename + kFileSuffix;
 
   // merge pending file does not always exist.
-  if (Util::FileExists(merge_pending_file)) {
+  if (FileUtil::FileExists(merge_pending_file)) {
     storage_->Merge(merge_pending_file.c_str());
-    Util::Unlink(merge_pending_file);
+    FileUtil::Unlink(merge_pending_file);
   }
 
   return true;
@@ -310,7 +314,7 @@ bool UserBoundaryHistoryRewriter::ResizeOrInsert(
       }
 
       length_array[j] = 0;
-      key = key.substr(0, key.size() - keys[j].first.size());
+      key.erase(key.size() - keys[j].first.size());
     }
 
     keys.pop_front();  // delete first item
@@ -326,8 +330,4 @@ void UserBoundaryHistoryRewriter::Clear() {
   }
 }
 
-// static
-LRUStorage *UserBoundaryHistoryRewriter::GetStorage() {
-  return g_lru_storage;
-}
 }  // namespace mozc
