@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,10 @@
 
 #include <vector>
 #include <string>
+
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "rewriter/rewriter_interface.h"
-#include "base/base.h"
 
 namespace mozc {
 
@@ -55,25 +57,16 @@ class UserBoundaryHistoryRewriter : public RewriterInterface {
   virtual bool Rewrite(const ConversionRequest &request,
                        Segments *segments) const;
 
-  virtual void Finish(Segments *segments);
+  virtual void Finish(const ConversionRequest &request, Segments *segments);
 
   virtual bool Reload();
 
   virtual void Clear();
 
-  // Return the current active storage.
-  // UserBoundaryHistoryRewriter is basically singleton in converter thread.
-  // GetStorage() returns the current active storage managed by converter.
-  // TODO(taku): This design is ad-hoc. Needs refactoring.
-  static mozc::storage::LRUStorage *GetStorage();
-
  private:
   bool ResizeOrInsert(Segments *segments, const ConversionRequest &request,
                       int type) const;
 
-  void Insert(const vector<const Segment *> &segs1,
-              const vector<const Segment *> &segs2);
-
   const ConverterInterface *parent_converter_;
   scoped_ptr<mozc::storage::LRUStorage> storage_;
 };
