@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -35,11 +35,12 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/compiler_specific.h"
 #include "base/config_file_stream.h"
+#include "base/file_util.h"
 #include "base/logging.h"
 #include "base/number_util.h"
+#include "base/string_piece.h"
 #include "base/util.h"
 #include "config/character_form_manager.h"
 #include "config/config.pb.h"
@@ -65,15 +66,21 @@ const uint32 kValueSize = 4;
 const uint32 kLRUSize   = 20000;
 const uint32 kSeedValue = 0xf28defe3;
 const uint32 kMaxCandidatesSize = 255;
+// Size of candidates to be reranked to the top at one sorting operation.
+// Note, if sorting operation is called twice, up to 10 (= 5 * 2) candidates
+// could be reranked in total.
+const size_t kMaxRerankSize = 5;
 
 const char kFileName[] = "user://segment.db";
 
-LRUStorage *g_lru_storage = NULL;
-
 // Temporarily disable unused private field warning against
 // FeatureValue::reserved_ from Clang.
+// We use MOZC_CLANG_HAS_WARNING to check whether "-Wunused-private-field" is
+// available, because XCode 4.4 clang (based on LLVM 3.1svn) doesn't have it.
 MOZC_CLANG_PUSH_WARNING();
+#if MOZC_CLANG_HAS_WARNING(unused-private-field)
 MOZC_CLANG_DISABLE_WARNING(unused-private-field);
+#endif
 class FeatureValue {
  public:
   FeatureValue() : feature_type_(1), reserved_(0) {}
@@ -100,8 +107,12 @@ bool IsPunctuationInternal(const string &str) {
 
 // Temporarily disable unused private field warning against
 // KeyTriggerValue::reserved_ from Clang.
+// We use MOZC_CLANG_HAS_WARNING to check whether "-Wunused-private-field" is
+// available, because XCode 4.4 clang (based on LLVM 3.1svn) doesn't have it.
 MOZC_CLANG_PUSH_WARNING();
+#if MOZC_CLANG_HAS_WARNING(unused-private-field)
 MOZC_CLANG_DISABLE_WARNING(unused-private-field);
+#endif
 class KeyTriggerValue {
  public:
   KeyTriggerValue()
@@ -140,7 +151,10 @@ class ScoreTypeCompare {
 
 // return the first candiadte which has "BEST_CANDIDATE" attribute
 inline int GetDefaultCandidateIndex(const Segment &segment) {
-  const int size = min(static_cast<int>(segment.candidates_size()), 5);
+  // Check up to kMaxRerankSize + 1 candidates because candidate with
+  // BEST_CANDIATE is highly possibly in that range (http://b/9992330).
+  const int size = static_cast<int>(min(segment.candidates_size(),
+                                        kMaxRerankSize + 1));
   for (int i = 0; i < size; ++i) {
     if (segment.candidate(i).attributes &
         Segment::Candidate::BEST_CANDIDATE) {
@@ -154,6 +168,53 @@ inline int GetDefaultCandidateIndex(const Segment &segment) {
   return 0;
 }
 
+// JoinStringWithTabN joins N strings with TAB delimiters ('\t') in a way
+// similar to Util::JoinStrings() and/or Util::AppendStringWithDelimiter() but
+// in a more efficient way. Since this module is called every key stroke and
+// performs many string concatenation, we use these functions instead of ones
+// from Util.
+inline void JoinStringsWithTab2(
+    const StringPiece s1, const StringPiece s2, string *output) {
+  // Pre-allocate the buffer, including 1 TAB delimiter.
+  output->reserve(s1.size() + s2.size() + 1);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size());
+}
+
+inline void JoinStringsWithTab3(
+    const StringPiece s1, const StringPiece s2, const StringPiece s3,
+    string *output) {
+  // Pre-allocate the buffer, including 2 TAB delimiters.
+  output->reserve(s1.size() + s2.size() + s3.size() + 2);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size()).append("\t")
+      .append(s3.data(), s3.size());
+}
+
+inline void JoinStringsWithTab4(
+    const StringPiece s1, const StringPiece s2, const StringPiece s3,
+    const StringPiece s4, string *output) {
+  // Pre-allocate the buffer, including 3 TAB delimiters.
+  output->reserve(s1.size() + s2.size() + s3.size() + s4.size() + 3);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size()).append("\t")
+      .append(s3.data(), s3.size()).append("\t")
+      .append(s4.data(), s4.size());
+}
+
+inline void JoinStringsWithTab5(
+    const StringPiece s1, const StringPiece s2, const StringPiece s3,
+    const StringPiece s4, const StringPiece s5, string *output) {
+  // Pre-allocate the buffer, including 4 TAB delimiters.
+  output->reserve(
+      s1.size() + s2.size() + s3.size() + s4.size() + s5.size() + 4);
+  output->assign(s1.data(), s1.size()).append("\t")
+      .append(s2.data(), s2.size()).append("\t")
+      .append(s3.data(), s3.size()).append("\t")
+      .append(s4.data(), s4.size()).append("\t")
+      .append(s5.data(), s5.size());
+}
+
 // Feature "Left Right"
 inline bool GetFeatureLR(const Segments &segments, size_t i,
                          const string &base_key,
@@ -164,10 +225,12 @@ inline bool GetFeatureLR(const Segments &segments, size_t i,
   }
   const int j1 = GetDefaultCandidateIndex(segments.segment(i - 1));
   const int j2 = GetDefaultCandidateIndex(segments.segment(i + 1));
-  *value = string("LR") + '\t' + base_key + '\t' +
-      segments.segment(i - 1).candidate(j1).value + '\t' +
-      base_value + '\t' +
-      segments.segment(i + 1).candidate(j2).value;
+  JoinStringsWithTab5(StringPiece("LR", 2),
+                      base_key,
+                      segments.segment(i - 1).candidate(j1).value,
+                      base_value,
+                      segments.segment(i + 1).candidate(j2).value,
+                      value);
   return true;
 }
 
@@ -181,10 +244,12 @@ inline bool GetFeatureLL(const Segments &segments, size_t i,
   }
   const int j1 = GetDefaultCandidateIndex(segments.segment(i - 2));
   const int j2 = GetDefaultCandidateIndex(segments.segment(i - 1));
-  *value = string("LL") + '\t' + base_key + '\t' +
-      segments.segment(i - 2).candidate(j1).value + '\t' +
-      segments.segment(i - 1).candidate(j2).value + '\t' +
-      base_value;
+  JoinStringsWithTab5(StringPiece("LL", 2),
+                      base_key,
+                      segments.segment(i - 2).candidate(j1).value,
+                      segments.segment(i - 1).candidate(j2).value,
+                      base_value,
+                      value);
   return true;
 }
 
@@ -198,10 +263,12 @@ inline bool GetFeatureRR(const Segments &segments, size_t i,
   }
   const int j1 = GetDefaultCandidateIndex(segments.segment(i + 1));
   const int j2 = GetDefaultCandidateIndex(segments.segment(i + 2));
-  *value = string("RR") + '\t' + base_key + '\t' +
-      base_value + '\t' +
-      segments.segment(i + 1).candidate(j1).value + '\t' +
-      segments.segment(i + 2).candidate(j2).value;
+  JoinStringsWithTab5(StringPiece("RR", 2),
+                      base_key,
+                      base_value,
+                      segments.segment(i + 1).candidate(j1).value,
+                      segments.segment(i + 2).candidate(j2).value,
+                      value);
   return true;
 }
 
@@ -214,9 +281,11 @@ inline bool GetFeatureL(const Segments &segments, size_t i,
     return false;
   }
   const int j = GetDefaultCandidateIndex(segments.segment(i - 1));
-  *value = string("L") + '\t' + base_key + '\t' +
-      segments.segment(i - 1).candidate(j).value + '\t' +
-      base_value;
+  JoinStringsWithTab4(StringPiece("L", 1),
+                      base_key,
+                      segments.segment(i - 1).candidate(j).value,
+                      base_value,
+                      value);
   return true;
 }
 
@@ -229,9 +298,11 @@ inline bool GetFeatureR(const Segments &segments, size_t i,
     return false;
   }
   const int j = GetDefaultCandidateIndex(segments.segment(i + 1));
-  *value = string("R") + '\t' + base_key + '\t' +
-      base_value + '\t' +
-      segments.segment(i + 1).candidate(j).value;
+  JoinStringsWithTab4(StringPiece("R", 1),
+                      base_key,
+                      base_value,
+                      segments.segment(i + 1).candidate(j).value,
+                      value);
   return true;
 }
 
@@ -240,7 +311,7 @@ inline bool GetFeatureC(const Segments &segments, size_t i,
                         const string &base_key,
                         const string &base_value, string *value) {
   DCHECK(value);
-  *value = string("C") + '\t' + base_key + '\t' + base_value;
+  JoinStringsWithTab3(StringPiece("C", 1), base_key, base_value, value);
   return true;
 }
 
@@ -252,7 +323,7 @@ inline bool GetFeatureS(const Segments &segments, size_t i,
   if (segments.segments_size() - segments.history_segments_size() != 1) {
     return false;
   }
-  *value = string("S") + '\t' + base_key + '\t' + base_value;
+  JoinStringsWithTab3(StringPiece("S", 1), base_key, base_value, value);
   return true;
 }
 
@@ -260,7 +331,7 @@ inline bool GetFeatureS(const Segments &segments, size_t i,
 // used for number rewrite
 inline bool GetFeatureN(uint16 type, string *value) {
   DCHECK(value);
-  *value = string("N") + '\t' + NumberUtil::SimpleItoa(type);
+  JoinStringsWithTab2(StringPiece("N", 1), NumberUtil::SimpleItoa(type), value);
   return true;
 }
 
@@ -370,13 +441,12 @@ bool IsT13NCandidate(const Segment::Candidate &cand) {
 bool UserSegmentHistoryRewriter::SortCandidates(
     const vector<ScoreType> &sorted_scores, Segment *segment) const {
   const uint32 top_score = sorted_scores[0].score;
-  const size_t kMaxRerankSize = min(sorted_scores.size(),
-                                    static_cast<size_t>(5));
+  const size_t size = min(sorted_scores.size(), kMaxRerankSize);
   const uint32 kScoreGap = 20;   // TODO(taku): no justification
   set<string> seen;
 
   size_t next_pos = 0;
-  for (size_t n = 0; n < kMaxRerankSize; ++n) {
+  for (size_t n = 0; n < size; ++n) {
     // Move candidates when the score is close to the top score.
     if (kScoreGap < (top_score - sorted_scores[n].score)) {
       break;
@@ -453,7 +523,6 @@ UserSegmentHistoryRewriter::UserSegmentHistoryRewriter(
       pos_matcher_(pos_matcher),
       pos_group_(pos_group) {
   Reload();
-  g_lru_storage = storage_.get();
 
   CHECK_EQ(sizeof(uint32), sizeof(FeatureValue));
   CHECK_EQ(sizeof(uint32), sizeof(KeyTriggerValue));
@@ -633,7 +702,7 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   // even if the candiate was the top (default) candidate,
   // ERANKED will be set when user changes the ranking
   const bool force_insert =
-      (candidate.attributes & Segment::Candidate::RERANKED);
+      ((candidate.attributes & Segment::Candidate::RERANKED) != 0);
 
   // Compare the POS group and Functional value.
   // if "is_replaceable" is true, it means that  the target candidate can
@@ -715,7 +784,12 @@ bool UserSegmentHistoryRewriter::IsAvailable(const Segments &segments) const {
   return true;
 }
 
-void UserSegmentHistoryRewriter::Finish(Segments *segments) {
+void UserSegmentHistoryRewriter::Finish(const ConversionRequest &request,
+                                        Segments *segments) {
+  if (segments->request_type() != Segments::CONVERSION) {
+    return;
+  }
+
   if (!IsAvailable(*segments)) {
     return;
   }
@@ -752,7 +826,6 @@ bool UserSegmentHistoryRewriter::Reload() {
                               kValueSize, kLRUSize, kSeedValue)) {
     LOG(WARNING) << "cannot initialize UserSegmentHistoryRewriter";
     storage_.reset(NULL);
-    g_lru_storage = NULL;
     return false;
   }
 
@@ -760,9 +833,9 @@ bool UserSegmentHistoryRewriter::Reload() {
   const string merge_pending_file = filename + kFileSuffix;
 
   // merge pending file does not always exist.
-  if (Util::FileExists(merge_pending_file)) {
+  if (FileUtil::FileExists(merge_pending_file)) {
     storage_->Merge(merge_pending_file.c_str());
-    Util::Unlink(merge_pending_file);
+    FileUtil::Unlink(merge_pending_file);
   }
 
   return true;
@@ -806,7 +879,8 @@ void UserSegmentHistoryRewriter::InsertTriggerKey(const Segment &segment) {
   DCHECK(storage_.get());
 
   KeyTriggerValue v;
-  COMPILE_ASSERT(sizeof(uint32) == sizeof(v), check_sizeof_KeyTriggerValue);
+  static_assert(sizeof(uint32) == sizeof(v),
+                "KeyTriggerValue must be 32-bit int size.");
 
   // TODO(taku): saving segment.candidate_size() might be too heavy and
   // increases the chance of hash collisions.
@@ -881,8 +955,7 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
   }
 
   // set BEST_CANDIDATE marker in advance
-  for (size_t i = segments->history_segments_size();
-       i < segments->segments_size(); ++i) {
+  for (size_t i = 0; i < segments->segments_size(); ++i) {
     Segment *segment = segments->mutable_segment(i);
     DCHECK(segment);
     DCHECK_GT(segment->candidates_size(), 0);
@@ -915,10 +988,8 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
       continue;
     }
 
-    if (segment->candidates_size() < max_candidates_size) {
-      LOG(WARNING) << "cannot expand candidates. ignored."
-                   << "rewrite may be failed ";
-    }
+    DVLOG_IF(2, (segment->candidates_size() < max_candidates_size))
+        << "Cannot expand candidates. ignored. Rewrite may be failed";
 
     // for each all candidates expanded
     vector<ScoreType> scores;
@@ -934,7 +1005,7 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
       uint32 score = 0;
       uint32 last_access_time = 0;
       if (GetScore(*segments, i, j, &score, &last_access_time)) {
-        scores.resize(scores.size() + 1);
+        scores.push_back(ScoreType());
         scores.back().score = score;
         scores.back().last_access_time = last_access_time;
         scores.back().candidate = segment->mutable_candidate(j);
@@ -958,11 +1029,6 @@ void UserSegmentHistoryRewriter::Clear() {
   }
 }
 
-// static
-LRUStorage *UserSegmentHistoryRewriter::GetStorage() {
-  return g_lru_storage;
-}
-
 bool UserSegmentHistoryRewriter::IsPunctuation(
     const Segment &seg,
     const Segment::Candidate &candidate) const {
@@ -987,7 +1053,7 @@ bool UserSegmentHistoryRewriter::GetFeatureLN(const Segments &segments,
   if (pos_matcher_->IsNumber(candidate.rid) ||
       pos_matcher_->IsKanjiNumber(candidate.rid) ||
       Util::GetScriptType(candidate.value) == Util::NUMBER) {
-    *value = string("LN") + '\t' + base_key + '\t' + base_value;
+    JoinStringsWithTab3(StringPiece("LN", 2), base_key, base_value, value);
     return true;
   }
   return false;
@@ -1008,7 +1074,7 @@ bool UserSegmentHistoryRewriter::GetFeatureRN(const Segments &segments,
   if (pos_matcher_->IsNumber(candidate.lid) ||
       pos_matcher_->IsKanjiNumber(candidate.lid) ||
       Util::GetScriptType(candidate.value) == Util::NUMBER) {
-    *value = string("RN") + '\t' + base_key + '\t' + base_value;
+    JoinStringsWithTab3(StringPiece("RN", 2), base_key, base_value, value);
     return true;
   }
   return false;
