@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,9 +34,7 @@
 #include <vector>
 
 #include "rewriter/rewriter_interface.h"
-#include "base/base.h"
 #include "converter/segments.h"
-#include "dictionary/pos_matcher.h"
 
 namespace mozc {
 class ConversionRequest;
@@ -62,18 +60,12 @@ class UserSegmentHistoryRewriter : public RewriterInterface {
   virtual bool Rewrite(const ConversionRequest &request,
                        Segments *segments) const;
 
-  virtual void Finish(Segments *segments);
+  virtual void Finish(const ConversionRequest &request, Segments *segments);
 
   virtual bool Reload();
 
   virtual void Clear();
 
-  // Return the current active storage.
-  // UserSegmentHistoryRewriter is basically singleton in converter thread.
-  // GetStorage() returns the current active storage managed by converter.
-  // TODO(taku): This design is ad-hoc. Needs refactoring.
-  static mozc::storage::LRUStorage *GetStorage();
-
  private:
   bool IsAvailable(const Segments &segments) const;
   bool GetScore(const Segments &segments,
