@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,12 +31,14 @@
 
 #include <string>
 
-#include "base/util.h"
+#include "base/system_util.h"
+#include "config/config.pb.h"
 #include "config/config_handler.h"
-#include "session/key_parser.h"
+#include "session/commands.pb.h"
 #include "session/internal/keymap.h"
-#include "testing/base/public/gunit.h"
+#include "session/key_parser.h"
 #include "testing/base/public/googletest.h"
+#include "testing/base/public/gunit.h"
 
 namespace mozc {
 namespace config {
@@ -45,7 +47,7 @@ class ImeSwitchUtilTest : public testing::Test {
  protected:
   virtual void SetUp() {
     ImeSwitchUtil::Reload();
-    Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+    SystemUtil::SetUserProfileDirectory(FLAGS_test_tmpdir);
   }
 };
 
