@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,16 +29,17 @@
 
 // CandidateList and Candidate classes to be used with Session class.
 
+#include "session/internal/candidate_list.h"
+
 #include <algorithm>
 #include <map>
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/freelist.h"
 #include "base/logging.h"
+#include "base/port.h"
 #include "base/util.h"
-#include "session/internal/candidate_list.h"
 
 namespace mozc {
 namespace session {
