@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -36,10 +36,11 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/config_file_stream.h"
 #include "base/file_stream.h"
 #include "base/logging.h"
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "base/util.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
@@ -57,10 +58,7 @@ static const char kKotoeriKeyMapFile[] = "system://kotoeri.tsv";
 static const char kCustomKeyMapFile[] = "user://keymap.tsv";
 static const char kMobileKeyMapFile[] = "system://mobile.tsv";
 
-// TODO(team): Investigate if "InputModeX" commands should be
-//     functional on ChromeOS and/or Android or not. Remove the
-//     condition if available.
-#if defined(OS_MACOSX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#if defined(OS_MACOSX)
 const bool kInputModeXCommandSupported = false;
 #else
 const bool kInputModeXCommandSupported = true;
@@ -203,7 +201,7 @@ bool KeyMapManager::LoadStreamWithErrors(istream *ifs, vector<string> *errors) {
   }
 
   commands::KeyEvent key_event;
-  KeyParser::ParseKey("ASCII", &key_event);
+  KeyParser::ParseKey("TextInput", &key_event);
   keymap_precomposition_.AddRule(key_event,
                                  PrecompositionState::INSERT_CHARACTER);
   keymap_composition_.AddRule(key_event, CompositionState::INSERT_CHARACTER);
@@ -219,22 +217,14 @@ bool KeyMapManager::AddCommand(const string &state_name,
                                const string &key_event_name,
                                const string &command_name) {
 #ifdef NO_LOGGING  // means RELEASE BUILD
-  // On the release build, we do not support the Abort and ReportBug
+  // On the release build, we do not support the ReportBug
   // commands.  Note, true is returned as the arguments are
   // interpreted properly.
-  if (command_name == "Abort" || command_name == "ReportBug") {
+  if (command_name == "ReportBug") {
     return true;
   }
 #endif  // NO_LOGGING
 
-#ifndef DEBUG
-  // Only debug build supports the Abort command.  Note, true is
-  // returned as the arguments are interpreted properly.
-  if (command_name == "Abort") {
-    return true;
-  }
-#endif  // DEBUG
-
   commands::KeyEvent key_event;
   if (!KeyParser::ParseKey(key_event_name, &key_event)) {
     return false;
@@ -381,7 +371,7 @@ void KeyMapManager::InitCommandData() {
   // Support InputMode command only on Windows for now.
   // TODO(toshiyuki): delete #ifdef when we support them on Mac, and
   // activate SessionTest.InputModeConsumedForTestSendKey.
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   RegisterDirectCommand("InputModeHiragana",
                         DirectInputState::INPUT_MODE_HIRAGANA);
   RegisterDirectCommand("InputModeFullKatakana",
@@ -403,7 +393,7 @@ void KeyMapManager::InitCommandData() {
                         DirectInputState::NONE);
   RegisterDirectCommand("InputModeHalfAlphanumeric",
                         DirectInputState::NONE);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
   RegisterDirectCommand("Reconvert",
                         DirectInputState::RECONVERT);
 
@@ -475,10 +465,6 @@ void KeyMapManager::InitCommandData() {
   RegisterPrecompositionCommand("PredictAndConvert",
                                 PrecompositionState::PREDICT_AND_CONVERT);
 
-#ifdef DEBUG  // only for debugging
-  RegisterPrecompositionCommand("Abort", PrecompositionState::ABORT);
-#endif  // DEBUG
-
   // Composition
   RegisterCompositionCommand("IMEOff", CompositionState::IME_OFF);
   RegisterCompositionCommand("IMEOn", CompositionState::IME_ON);
@@ -565,9 +551,6 @@ void KeyMapManager::InitCommandData() {
     RegisterCompositionCommand("InputModeHalfAlphanumeric",
                                CompositionState::NONE);
   }
-#ifdef DEBUG  // only for debugging
-  RegisterCompositionCommand("Abort", CompositionState::ABORT);
-#endif  // DEBUG
 
   // Conversion
   RegisterConversionCommand("IMEOff", ConversionState::IME_OFF);
@@ -637,6 +620,8 @@ void KeyMapManager::InitCommandData() {
                             ConversionState::TRANSLATE_FULL_ASCII);
   RegisterConversionCommand("DisplayAsHalfAlphanumeric",
                             ConversionState::TRANSLATE_HALF_ASCII);
+  RegisterConversionCommand("DeleteSelectedCandidate",
+                            ConversionState::DELETE_SELECTED_CANDIDATE);
   if (kInputModeXCommandSupported) {
     RegisterConversionCommand("InputModeHiragana",
                               ConversionState::INPUT_MODE_HIRAGANA);
@@ -663,13 +648,8 @@ void KeyMapManager::InitCommandData() {
 #ifndef NO_LOGGING  // means NOT RELEASE build
   RegisterConversionCommand("ReportBug", ConversionState::REPORT_BUG);
 #endif  // NO_LOGGING
-#ifdef DEBUG  // only for dubugging
-  RegisterConversionCommand("Abort", ConversionState::ABORT);
-#endif  // DEBUG
 }
 
-#undef ADD_TO_COMMAND_MAP
-
 bool KeyMapManager::GetCommandDirect(
     const commands::KeyEvent &key_event,
     DirectInputState::Commands *command) const {
