@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,13 +32,12 @@
 #ifndef MOZC_SESSION_INTERNAL_KEYMAP_INTERFACE_H_
 #define MOZC_SESSION_INTERNAL_KEYMAP_INTERFACE_H_
 
-#include <map>
-#include "base/base.h"
+#include "base/port.h"
 
 namespace mozc {
 namespace commands {
 class KeyEvent;
-}
+}  // namespace commands
 
 namespace keymap {
 
@@ -80,7 +79,6 @@ enum Commands {
   LAUNCH_WORD_REGISTER_DIALOG,
   REVERT,  // revert last operation (preedit still remains)
   UNDO,    // undo last operation (preedit is restored)
-  ABORT,   // Abort the server.  The process is killed.
   RECONVERT,
 
   // For ZeroQuerySuggestion
@@ -138,7 +136,6 @@ enum Commands {
   INPUT_MODE_HALF_KATAKANA,
   INPUT_MODE_FULL_ALPHANUMERIC,
   INPUT_MODE_HALF_ALPHANUMERIC,
-  ABORT,  // Abort the server.  The process is killed.
 };
 };
 
@@ -190,8 +187,8 @@ enum Commands {
   INPUT_MODE_HALF_KATAKANA,
   INPUT_MODE_FULL_ALPHANUMERIC,
   INPUT_MODE_HALF_ALPHANUMERIC,
+  DELETE_SELECTED_CANDIDATE,
   REPORT_BUG,
-  ABORT,  // Abort the server.  The process is killed.
 };
 };
 
@@ -208,4 +205,5 @@ class KeyMapInterface {
 
 }  // namespace keymap
 }  // namespace mozc
+
 #endif  // MOZC_SESSION_INTERNAL_KEYMAP_INTERFACE_H_
