@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,8 @@
 #define MOZC_SESSION_INTERNAL_SESSION_OUTPUT_H_
 
 #include <string>
-#include "base/base.h"
+
+#include "base/port.h"
 #include "session/commands.pb.h"
 
 namespace mozc {
