@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,14 +29,16 @@
 
 // Session Handler of Mozc server.
 // Migrated from ipc/interpreter and session/session_manager
+
 #include "session/session_handler.h"
 
 #include <algorithm>
 #include <string>
 #include <vector>
 
-#include "base/base.h"
+#include "base/init.h"
 #include "base/logging.h"
+#include "base/port.h"
 #include "base/process.h"
 #include "base/singleton.h"
 #include "base/stopwatch.h"
@@ -44,13 +46,13 @@
 #include "composer/table.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
-#include "converter/user_data_manager_interface.h"
 #include "dictionary/user_dictionary_session_handler.h"
+#include "dictionary/user_dictionary_storage.pb.h"
+#include "engine/engine_interface.h"
+#include "engine/user_data_manager_interface.h"
 #include "session/commands.pb.h"
 #include "session/generic_storage_manager.h"
-#include "session/request_handler.h"
-#include "session/session_factory_manager.h"
-#include "session/session_interface.h"
+#include "session/session.h"
 #include "session/session_observer_handler.h"
 #ifndef MOZC_DISABLE_SESSION_WATCHDOG
 #include "session/session_watch_dog.h"
@@ -59,10 +61,9 @@
 // TODO(kkojima): Remove this guard after
 // enabling session watch dog for android.
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
-#ifdef ENABLE_CLOUD_SYNC
-#include "sync/sync_handler.h"
-#include "sync/syncer_interface.h"
-#endif  // ENABLE_CLOUD_SYNC
+#include "usage_stats/usage_stats.h"
+
+using mozc::usage_stats::UsageStats;
 
 DEFINE_int32(timeout, -1,
              "server timeout. "
@@ -104,17 +105,17 @@ bool IsApplicationAlive(const session::SessionInterface *session) {
   // Here, we want to kill the session only when the target thread/process
   // are terminated with 100% probability. Otherwise, it's better to do
   // nothing to prevent any side effects.
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   if (info.has_thread_id()) {
     return Process::IsThreadAlive(
         static_cast<size_t>(info.thread_id()), true);
   }
-#else   // OS_WINDOWS
+#else   // OS_WIN
   if (info.has_process_id()) {
     return Process::IsProcessAlive(
         static_cast<size_t>(info.process_id()), true);
   }
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 #else  // MOZC_DISABLE_SESSION_WATCHDOG
   // Currently the process is not available through android mozc and nacl mozc.
   // TODO(kkojima): remove this guard after
@@ -122,23 +123,32 @@ bool IsApplicationAlive(const session::SessionInterface *session) {
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
   return true;
 }
+
+bool IsCarrierEmoji(const string &utf8_str) {
+  if (Util::CharsLen(utf8_str) != 1) {
+    return false;
+  }
+  const char *utf8_begin = utf8_str.c_str();
+  size_t mblen = 0;
+  const uint32 ucs4_val = static_cast<uint32>(
+      Util::UTF8ToUCS4(utf8_begin, utf8_begin + utf8_str.size(), &mblen));
+  const uint32 kMinEmojiPuaCodePoint = 0xFE000;
+  const uint32 kMaxEmojiPuaCodePoint = 0xFEEA0;
+  return kMinEmojiPuaCodePoint <= ucs4_val && ucs4_val <= kMaxEmojiPuaCodePoint;
+}
 }  // namespace
 
-SessionHandler::SessionHandler()
+SessionHandler::SessionHandler(EngineInterface *engine)
     : is_available_(false),
-      keyevent_counter_(0),
       max_session_size_(0),
       last_session_empty_time_(Util::GetTime()),
       last_cleanup_time_(0),
       last_create_session_time_(0),
-      session_factory_(
-          session::SessionFactoryManager::GetSessionFactory()),
+      engine_(engine),
       observer_handler_(new session::SessionObserverHandler()),
       stopwatch_(new Stopwatch),
-#ifndef __native_client__
       user_dictionary_session_handler_(
           new user_dictionary::UserDictionarySessionHandler),
-#endif  // __native_client__
       table_manager_(new composer::TableManager),
       request_(new commands::Request) {
   if (FLAGS_restricted) {
@@ -167,7 +177,7 @@ SessionHandler::SessionHandler()
   max_session_size_ = max(2, min(FLAGS_max_session_size, 128));
   session_map_.reset(new SessionMap(max_session_size_));
 
-  if (session_factory_ == NULL) {
+  if (engine_ == NULL) {
     return;
   }
 
@@ -225,7 +235,7 @@ void SessionHandler::ReloadConfig() {
        element != NULL; element = element->next) {
     if (element->value != NULL) {
       element->value->ReloadConfig();
-      element->value->SetRequest(*request_);
+      element->value->SetRequest(request_.get());
       element->value->SetTable(table);
     }
   }
@@ -233,8 +243,7 @@ void SessionHandler::ReloadConfig() {
 
 bool SessionHandler::SyncData(commands::Command *command) {
   VLOG(1) << "Syncing user data";
-  session_factory_->GetUserDataManager()->Sync();
-  command->mutable_output()->set_id(command->input().id());
+  engine_->GetUserDataManager()->Sync();
   return true;
 }
 
@@ -243,36 +252,36 @@ bool SessionHandler::Shutdown(commands::Command *command) {
   SyncData(command);
   ReloadSession();   // for saving log_commands
   is_available_ = false;
+  UsageStats::IncrementCount("ShutDown");
   return true;
 }
 
 bool SessionHandler::Reload(commands::Command *command) {
   VLOG(1) << "Reloading server";
   ReloadSession();
-  session_factory_->Reload();
+  engine_->Reload();
   RunReloaders();  // call all reloaders defined in .cc file
-  command->mutable_output()->set_id(command->input().id());
   return true;
 }
 
 bool SessionHandler::ClearUserHistory(commands::Command *command) {
   VLOG(1) << "Clearing user history";
-  session_factory_->GetUserDataManager()->ClearUserHistory();
-  command->mutable_output()->set_id(command->input().id());
+  engine_->GetUserDataManager()->ClearUserHistory();
+  UsageStats::IncrementCount("ClearUserHistory");
   return true;
 }
 
 bool SessionHandler::ClearUserPrediction(commands::Command *command) {
   VLOG(1) << "Clearing user prediction";
-  session_factory_->GetUserDataManager()->ClearUserPrediction();
-  command->mutable_output()->set_id(command->input().id());
+  engine_->GetUserDataManager()->ClearUserPrediction();
+  UsageStats::IncrementCount("ClearUserPrediction");
   return true;
 }
 
 bool SessionHandler::ClearUnusedUserPrediction(commands::Command *command) {
   VLOG(1) << "Clearing unused user prediction";
-  session_factory_->GetUserDataManager()->ClearUnusedUserPrediction();
-  command->mutable_output()->set_id(command->input().id());
+  engine_->GetUserDataManager()->ClearUnusedUserPrediction();
+  UsageStats::IncrementCount("ClearUnusedUserPrediction");
   return true;
 }
 
@@ -290,7 +299,6 @@ bool SessionHandler::GetStoredConfig(commands::Command *command) {
     LOG(WARNING) << "cannot get config";
     return false;
   }
-  command->mutable_output()->set_id(command->input().id());
   return true;
 }
 
@@ -307,8 +315,8 @@ bool SessionHandler::SetStoredConfig(commands::Command *command) {
   }
 
   command->mutable_output()->mutable_config()->CopyFrom(config);
-
   Reload(command);
+  UsageStats::IncrementCount("SetConfig");
 
   return true;
 }
@@ -336,57 +344,14 @@ bool SessionHandler::SetRequest(commands::Command *command) {
   }
 
   request_->CopyFrom(command->input().request());
-  // TODO(yoichio): After refactoring  to remove all GET_REQUEST, delete this.
-  commands::RequestHandler::SetRequest(*request_);
 
   Reload(command);
 
   return true;
 }
 
-bool SessionHandler::StartCloudSync(commands::Command *command) {
-  VLOG(1) << "Start cloud sync operation";
-  command->mutable_output()->set_id(command->input().id());
-#ifdef ENABLE_CLOUD_SYNC
-  return sync::SyncHandler::Sync();
-#else
-  return true;
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-bool SessionHandler::ClearCloudSync(commands::Command *command) {
-  VLOG(1) << "Clear cloud sync";
-  command->mutable_output()->set_id(command->input().id());
-#ifdef ENABLE_CLOUD_SYNC
-  return sync::SyncHandler::Clear();
-#else
-  return true;
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-bool SessionHandler::GetCloudSyncStatus(commands::Command *command) {
-  VLOG(1) << "GetSyncStatus";
-#ifdef ENABLE_CLOUD_SYNC
-  return sync::SyncHandler::GetCloudSyncStatus(
-      command->mutable_output()->mutable_cloud_sync_status());
-#else
-  return false;
-#endif  // ENABLE_CLOUD_SYNC
-}
-
-bool SessionHandler::AddAuthCode(commands::Command *command) {
-  VLOG(1) << "AddAuthCode";
-#ifdef ENABLE_CLOUD_SYNC
-  return sync::SyncHandler::SetAuthorization(
-      command->input().auth_code());
-#else
-  return false;
-#endif  // ENABLE_CLOUD_SYNC
-}
-
 bool SessionHandler::InsertToStorage(commands::Command *command) {
   VLOG(1) << "Insert to generic storage";
-  command->mutable_output()->set_id(command->input().id());
   if (!command->input().has_storage_entry()) {
     LOG(WARNING) << "No storage_entry";
     return false;
@@ -401,7 +366,7 @@ bool SessionHandler::InsertToStorage(commands::Command *command) {
   }
 
   GenericStorageInterface *storage =
-    GenericStorageManagerFactory::GetStorage(storage_entry.type());
+      GenericStorageManagerFactory::GetStorage(storage_entry.type());
   if (!storage) {
     LOG(WARNING) << "No storage found";
     return false;
@@ -409,15 +374,25 @@ bool SessionHandler::InsertToStorage(commands::Command *command) {
 
   for (int i = 0; i < storage_entry.value_size(); ++i) {
     const string &value = storage_entry.value(i);
-    storage->Insert(value, value.data());
+    storage->Insert(value, value.c_str());
   }
+
+  if (storage_entry.type() == commands::GenericStorageEntry::EMOJI_HISTORY) {
+    for (int i = 0; i < storage_entry.value_size(); ++i) {
+      if (IsCarrierEmoji(storage_entry.value(i))) {
+        UsageStats::IncrementCount("CommitCarrierEmoji");
+      } else {
+        UsageStats::IncrementCount("CommitUnicodeEmoji");
+      }
+    }
+  }
+
   return true;
 }
 
 bool SessionHandler::ReadAllFromStorage(commands::Command *command) {
   VLOG(1) << "Read all from storage";
   commands::Output *output = command->mutable_output();
-  output->set_id(command->input().id());
   if (!command->input().has_storage_entry()) {
     LOG(WARNING) << "No storage_entry";
     return false;
@@ -430,7 +405,7 @@ bool SessionHandler::ReadAllFromStorage(commands::Command *command) {
   commands::GenericStorageEntry::StorageType storage_type =
     command->input().storage_entry().type();
   GenericStorageInterface *storage =
-    GenericStorageManagerFactory::GetStorage(storage_type);
+      GenericStorageManagerFactory::GetStorage(storage_type);
   if (!storage) {
     LOG(WARNING) << "No storage found";
     return false;
@@ -448,7 +423,6 @@ bool SessionHandler::ReadAllFromStorage(commands::Command *command) {
 bool SessionHandler::ClearStorage(commands::Command *command) {
   VLOG(1) << "Clear storage";
   commands::Output *output = command->mutable_output();
-  output->set_id(command->input().id());
   if (!command->input().has_storage_entry()) {
     LOG(WARNING) << "No storage_entry";
     return false;
@@ -461,7 +435,7 @@ bool SessionHandler::ClearStorage(commands::Command *command) {
   commands::GenericStorageEntry::StorageType storage_type =
     command->input().storage_entry().type();
   GenericStorageInterface *storage =
-    GenericStorageManagerFactory::GetStorage(storage_type);
+      GenericStorageManagerFactory::GetStorage(storage_type);
   if (!storage) {
     LOG(WARNING) << "No storage found";
     return false;
@@ -472,6 +446,7 @@ bool SessionHandler::ClearStorage(commands::Command *command) {
 
 bool SessionHandler::EvalCommand(commands::Command *command) {
   if (!is_available_) {
+    LOG(ERROR) << "SessionHandler is not available.";
     return false;
   }
 
@@ -528,18 +503,6 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
     case commands::Input::CLEANUP:
       eval_succeeded = Cleanup(command);
       break;
-    case commands::Input::START_CLOUD_SYNC:
-      eval_succeeded = StartCloudSync(command);
-      break;
-    case commands::Input::CLEAR_CLOUD_SYNC:
-      eval_succeeded = ClearCloudSync(command);
-      break;
-    case commands::Input::GET_CLOUD_SYNC_STATUS:
-      eval_succeeded = GetCloudSyncStatus(command);
-      break;
-    case commands::Input::ADD_AUTH_CODE:
-      eval_succeeded = AddAuthCode(command);
-      break;
     case commands::Input::INSERT_TO_STORAGE:
       eval_succeeded = InsertToStorage(command);
       break;
@@ -559,36 +522,33 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
       eval_succeeded = false;
   }
 
-  if (!eval_succeeded) {
+  if (eval_succeeded) {
+    UsageStats::IncrementCount("SessionAllEvent");
+    if (command->input().type() != commands::Input::CREATE_SESSION) {
+      // Fill a session ID even if command->input() doesn't have a id to ensure
+      // that response size should not be 0, which causes disconnection of IPC.
+      command->mutable_output()->set_id(command->input().id());
+    }
+  } else {
     command->mutable_output()->set_id(0);
     command->mutable_output()->set_error_code(
         commands::Output::SESSION_FAILURE);
   }
 
-
-  // Since ElappsedTime is processed by UsageStats,
-  // Stop the timer before usage stats aggregation.
-  // This won't be good if UsageStats aggregator consumes a lot of
-  // CPU time
-  stopwatch_->Stop();
-  command->mutable_output()->set_elapsed_time(
-      static_cast<int32>(stopwatch_->GetElapsedMicroseconds()));
-
   if (eval_succeeded) {
     // TODO(komatsu): Make sre if checking eval_succeeded is necessary or not.
     observer_handler_->EvalCommandHandler(*command);
   }
 
+  stopwatch_->Stop();
+  UsageStats::UpdateTiming("ElapsedTimeUSec",
+                           stopwatch_->GetElapsedMicroseconds());
+
   return is_available_;
 }
 
 session::SessionInterface *SessionHandler::NewSession() {
-  return session_factory_->NewSession();
-}
-
-void SessionHandler::SetSessionFactory(
-    session::SessionFactoryInterface *new_factory) {
-  session_factory_ = new_factory;
+  return new session::Session(engine_);
 }
 
 void SessionHandler::AddObserver(session::SessionObserverInterface *observer) {
@@ -597,8 +557,6 @@ void SessionHandler::AddObserver(session::SessionObserverInterface *observer) {
 
 bool SessionHandler::SendKey(commands::Command *command) {
   const SessionID id = command->input().id();
-  command->mutable_output()->set_id(id);
-
   session::SessionInterface **session = session_map_->MutableLookup(id);
   if (session == NULL || *session == NULL) {
     LOG(WARNING) << "SessionID " << id << " is not available";
@@ -610,7 +568,6 @@ bool SessionHandler::SendKey(commands::Command *command) {
 
 bool SessionHandler::TestSendKey(commands::Command *command) {
   const SessionID id = command->input().id();
-  command->mutable_output()->set_id(id);
   session::SessionInterface **session = session_map_->MutableLookup(id);
   if (session == NULL || *session == NULL) {
     LOG(WARNING) << "SessionID " << id << " is not available";
@@ -622,7 +579,6 @@ bool SessionHandler::TestSendKey(commands::Command *command) {
 
 bool SessionHandler::SendCommand(commands::Command *command) {
   const SessionID id = command->input().id();
-  command->mutable_output()->set_id(id);
   session::SessionInterface **session =
     const_cast<session::SessionInterface **>(session_map_->Lookup(id));
   if (session == NULL || *session == NULL) {
@@ -636,14 +592,13 @@ bool SessionHandler::SendCommand(commands::Command *command) {
 bool SessionHandler::CreateSession(commands::Command *command) {
   // prevent DOS attack
   // don't allow CreateSession in very short period.
-  const int kCreateSessionMinimumInterval =
+  const int create_session_minimum_interval =
       max(0, min(FLAGS_create_session_min_interval, 10));
 
   uint64 current_time = Util::GetTime();
   if (last_create_session_time_ != 0 &&
       (current_time - last_create_session_time_) <
-      kCreateSessionMinimumInterval) {
-    last_create_session_time_ = current_time;
+      create_session_minimum_interval) {
     return false;
   }
 
@@ -662,7 +617,7 @@ bool SessionHandler::CreateSession(commands::Command *command) {
     session_map_->Erase(oldest_element->key);
     VLOG(1) << "Session is FULL, oldest SessionID "
             << oldest_element->key << " is removed";
-  };
+  }
 
   session::SessionInterface *session = NewSession();
   if (session == NULL) {
@@ -684,6 +639,12 @@ bool SessionHandler::CreateSession(commands::Command *command) {
 
   if (command->input().has_application_info()) {
     session->set_application_info(command->input().application_info());
+#ifdef __native_client__
+    if (command->input().application_info().has_timezone_offset()) {
+      Util::SetTimezoneOffset(
+          command->input().application_info().timezone_offset());
+    }
+#endif  // __native_client__
   }
 
   // Ensure the onmemory config is same as the locally stored one
@@ -693,14 +654,14 @@ bool SessionHandler::CreateSession(commands::Command *command) {
   // session is not empty.
   last_session_empty_time_ = 0;
 
+  UsageStats::IncrementCount("SessionCreated");
+
   return true;
 }
 
 bool SessionHandler::DeleteSession(commands::Command *command) {
-  const SessionID id = command->input().id();
-  command->mutable_output()->set_id(id);
-  DeleteSessionID(id);
-  session_factory_->GetUserDataManager()->Sync();
+  DeleteSessionID(command->input().id());
+  engine_->GetUserDataManager()->Sync();
   return true;
 }
 
@@ -712,9 +673,6 @@ bool SessionHandler::DeleteSession(commands::Command *command) {
 // no active session and client doesn't send any conversion
 // request to the server for FLAGS_timeout sec.
 bool SessionHandler::Cleanup(commands::Command *command) {
-  const SessionID id = command->input().id();
-  command->mutable_output()->set_id(id);
-
   const uint64 current_time = Util::GetTime();
 
   // suspend/hibernation may happen
@@ -736,12 +694,12 @@ bool SessionHandler::Cleanup(commands::Command *command) {
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
 
   // allow [1..600] sec. default: 300
-  const uint64 kCreateSessionTimeout =
+  const uint64 create_session_timeout =
       suspend_time +
       max(1, min(FLAGS_last_create_session_timeout, 600));
 
   // allow [10..7200] sec. default 3600
-  const uint64 kLastCommandTimeout =
+  const uint64 last_command_timeout =
       suspend_time +
       max(10, min(FLAGS_last_command_timeout, 7200));
 
@@ -756,12 +714,12 @@ bool SessionHandler::Cleanup(commands::Command *command) {
     } else if (session->last_command_time() == 0) {
       // no command is exectuted
       if ((current_time - session->create_session_time()) >=
-          kCreateSessionTimeout) {
+          create_session_timeout) {
         remove_ids.push_back(element->key);
       }
     } else {  // some commands are executed already
       if ((current_time - session->last_command_time()) >=
-          kLastCommandTimeout) {
+          last_command_timeout) {
         remove_ids.push_back(element->key);
       }
     }
@@ -772,9 +730,8 @@ bool SessionHandler::Cleanup(commands::Command *command) {
     VLOG(1) << "Session ID " << remove_ids[i] << " is removed by server";
   }
 
-  // Sync all data.
-  // This is a regression bug fix http://b/issue?id=3033708
-  session_factory_->GetUserDataManager()->Sync();
+  // Sync all data. This is a regression bug fix http://b/3033708
+  engine_->GetUserDataManager()->Sync();
 
   // timeout is enabled.
   if (FLAGS_timeout > 0 &&
@@ -793,7 +750,6 @@ bool SessionHandler::SendUserDictionaryCommand(commands::Command *command) {
   if (!command->input().has_user_dictionary_command()) {
     return false;
   }
-#ifndef __native_client__
   user_dictionary::UserDictionaryCommandStatus status;
   const bool result = user_dictionary_session_handler_->Evaluate(
       command->input().user_dictionary_command(), &status);
@@ -802,15 +758,9 @@ bool SessionHandler::SendUserDictionaryCommand(commands::Command *command) {
         command->mutable_output()->mutable_user_dictionary_command_status());
   }
   return result;
-#else  // __native_client__
-  // TODO(horo): UserDictionaryCommand is not aviable from nacl mozc now.
-  return false;
-#endif  // __native_client__
 }
 
 bool SessionHandler::NoOperation(commands::Command *command) {
-  const SessionID id = command->input().id();
-  command->mutable_output()->set_id(id);
   return true;
 }
 
@@ -818,16 +768,7 @@ bool SessionHandler::NoOperation(commands::Command *command) {
 SessionID SessionHandler::CreateNewSessionID() {
   SessionID id = 0;
   while (true) {
-    if (!Util::GetSecureRandomSequence(
-            reinterpret_cast<char *>(&id), sizeof(id))) {
-      LOG(ERROR) << "GetSecureRandomSequence() failed. use random value";
-      id = static_cast<uint64>(Util::Random(RAND_MAX));
-    }
-#ifdef  __native_client__
-    // Because JavaScript does not support uint64.
-    // So we downsize the session id range from uint64 to uint32 in NaCl.
-    id = static_cast<uint32>(id);
-#endif  // __native_client__
+    Util::GetRandomSequence(reinterpret_cast<char *>(&id), sizeof(id));
     // don't allow id == 0, as it is reserved for
     // "invalid id"
     if (id != 0 && !session_map_->HasKey(id)) {
