@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,9 +34,8 @@
 
 #include <map>
 #include <string>
-#include <utility>
 
-#include "base/base.h"
+#include "base/port.h"
 #include "base/scoped_ptr.h"
 #include "composer/table.h"
 #include "session/common.h"
@@ -52,6 +51,7 @@
 #endif  // OS_ANDROID || __native_client__
 
 namespace mozc {
+class EngineInterface;
 #ifndef MOZC_DISABLE_SESSION_WATCHDOG
 class SessionWatchDog;
 #else  // MOZC_DISABLE_SESSION_WATCHDOG
@@ -63,10 +63,10 @@ class Stopwatch;
 
 namespace commands {
 class Command;
+class Request;
 }  // namespace commands
 
 namespace session {
-class SessionFactoryInterface;
 class SessionInterface;
 class SessionObserverHandler;
 class SessionObserverInterface;
@@ -78,7 +78,8 @@ class UserDictionarySessionHandler;
 
 class SessionHandler : public SessionHandlerInterface {
  public:
-  SessionHandler();
+  // This class doesn't take an ownership of |engine|.
+  explicit SessionHandler(EngineInterface *engine);
   virtual ~SessionHandler();
 
   // Returns true if SessionHandle is available.
@@ -95,12 +96,10 @@ class SessionHandler : public SessionHandlerInterface {
 
   virtual void AddObserver(session::SessionObserverInterface *observer);
 
-  void SetSessionFactory(session::SessionFactoryInterface *new_factory);
-
  private:
   FRIEND_TEST(SessionHandlerTest, StorageTest);
 
-  typedef mozc::storage::LRUCache<SessionID, session::SessionInterface*>
+  typedef mozc::storage::LRUCache<SessionID, session::SessionInterface *>
       SessionMap;
   typedef SessionMap::Element SessionElement;
 
@@ -124,10 +123,6 @@ class SessionHandler : public SessionHandlerInterface {
   bool SetStoredConfig(commands::Command *command);
   bool SetImposedConfig(commands::Command *command);
   bool SetRequest(commands::Command *command);
-  bool StartCloudSync(commands::Command *command);
-  bool ClearCloudSync(commands::Command *command);
-  bool GetCloudSyncStatus(commands::Command *command);
-  bool AddAuthCode(commands::Command *command);
 
   bool InsertToStorage(commands::Command *command);
   bool ReadAllFromStorage(commands::Command *command);
@@ -148,19 +143,16 @@ class SessionHandler : public SessionHandlerInterface {
   // enabling session watch dog for android.
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
   bool is_available_;
-  int keyevent_counter_;
   uint32 max_session_size_;
   uint64 last_session_empty_time_;
   uint64 last_cleanup_time_;
   uint64 last_create_session_time_;
 
-  session::SessionFactoryInterface *session_factory_;
+  EngineInterface *engine_;
   scoped_ptr<session::SessionObserverHandler> observer_handler_;
   scoped_ptr<Stopwatch> stopwatch_;
-#ifndef __native_client__
   scoped_ptr<user_dictionary::UserDictionarySessionHandler>
       user_dictionary_session_handler_;
-#endif  // __native_client__
   scoped_ptr<composer::TableManager> table_manager_;
   scoped_ptr<commands::Request> request_;
 
