@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,18 +32,16 @@
 #ifndef MOZC_SESSION_SESSION_SERVER_H_
 #define MOZC_SESSION_SESSION_SERVER_H_
 
-#include <string>
-#include "base/base.h"
+#include "base/port.h"
 #include "ipc/ipc.h"
-#include "session/session_handler.h"
-
-class SessionHandlerInterface;
 
 namespace mozc {
+class EngineInterface;
+class SessionHandlerInterface;
 
 namespace session {
 class SessionUsageObserver;
-}
+}  // namespace session
 
 // Session IPC Server
 // Usage:
@@ -68,8 +66,11 @@ class SessionServer: public IPCServer {
                        size_t *response_size);
 
  private:
-  scoped_ptr<SessionHandlerInterface> handler_;
+  // Must be defined earlier than session_handler_, which depends on this.
+  scoped_ptr<EngineInterface> engine_;
   scoped_ptr<session::SessionUsageObserver> usage_observer_;
+  scoped_ptr<SessionHandlerInterface> session_handler_;
+
   DISALLOW_COPY_AND_ASSIGN(SessionServer);
 };
 }  // namespace mozc
