@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,14 +27,15 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#include "storage/existence_filter.h"
+
 #include <string>
 #include <vector>
-#include <utility>
 
-#include "base/base.h"
 #include "base/logging.h"
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "base/util.h"
-#include "storage/existence_filter.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
 
@@ -134,7 +135,7 @@ TEST(ExistenceFilterTest, ReadWriteTest) {
   }
 
   delete [] buf;
-};
+}
 
 TEST(ExistenceFilterTest, InsertAndExistsTest) {
   vector<string> words;
@@ -163,7 +164,7 @@ TEST(ExistenceFilterTest, InsertAndExistsTest) {
   for (int i = 0; i < words.size(); ++i) {
     EXPECT_TRUE(filter->Exists(Util::Fingerprint(words[i])));
   }
-};
+}
 
 }  // namespace storage
 }  // namespace mozc
