@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,12 +27,14 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <string>
-#include <iostream>
-#include "base/base.h"
-#include "base/util.h"
 #include "storage/lru_cache.h"
 
+#include <string>
+#include <iostream>  // NOLINT
+
+#include "base/flags.h"
+#include "base/util.h"
+
 int main(int argc, char **argv) {
   InitGoogle(argv[0], &argc, &argv, false);
   mozc::storage::LRUCache<string, string> cache(5);
