@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -36,5 +36,11 @@
 #error "ibus-mozc now requires IBus>=1.4.1"
 #endif  // libibus (<1.4.1)
 
+#if IBUS_CHECK_VERSION(1, 5, 4)
+#if !defined(MOZC_ENABLE_IBUS_INPUT_PURPOSE)
+#define MOZC_ENABLE_IBUS_INPUT_PURPOSE
+#endif  // !MOZC_ENABLE_IBUS_INPUT_PURPOSE
+#endif  // libibus (>=1.5.4)
+
 
 #endif  // MOZC_UNIX_IBUS_IBUS_HEADER_H_
