@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -37,16 +37,18 @@
 #include <sstream>
 #include <string>
 
-#include "base/base.h"
 #include "base/const.h"
+#include "base/file_util.h"
+#include "base/logging.h"
 #include "base/protobuf/descriptor.h"
 #include "base/protobuf/message.h"
 #include "base/singleton.h"
+#include "base/system_util.h"
 #include "base/util.h"
+#include "client/client.h"
 #include "config/config.pb.h"
 #include "session/commands.pb.h"
 #include "session/ime_switch_util.h"
-#include "unix/ibus/config_util.h"
 #include "unix/ibus/engine_registrar.h"
 #include "unix/ibus/ibus_candidate_window_handler.h"
 #include "unix/ibus/key_event_handler.h"
@@ -57,14 +59,6 @@
 #include "unix/ibus/property_handler.h"
 #include "unix/ibus/surrounding_text_util.h"
 
-#ifdef OS_CHROMEOS
-// use standalone (in-process) session
-#include "unix/ibus/client.h"
-#else
-// use server/client session
-#include "client/client.h"
-#endif
-
 #ifdef ENABLE_GTK_RENDERER
 #include "renderer/renderer_client.h"
 #include "unix/ibus/gtk_candidate_window_handler.h"
@@ -98,35 +92,6 @@ const char kMozcDefaultUILocale[] = "en_US.UTF-8";
 // for every 5 minutes, call SyncData
 const uint64 kSyncDataInterval = 5 * 60;
 
-#ifdef OS_CHROMEOS
-// IBus config names for Mozc.
-// This list must match the preference names of ibus-mozc for Chrome OS.
-// Check the following file in the chrome repository when you modify this list.
-// chrome/browser/chromeos/language_preferences.cc
-const gchar* kMozcConfigNames[] = {
-  "incognito_mode",
-  "use_auto_ime_turn_off",
-  "use_date_conversion",
-  "use_single_kanji_conversion",
-  "use_symbol_conversion",
-  "use_number_conversion",
-  "use_history_suggest",
-  "use_dictionary_suggest",
-  "use_realtime_conversion",
-  "preedit_method",
-  "session_keymap",
-  "punctuation_method",
-  "symbol_method",
-  "space_character_form",
-  "history_learning_level",
-  //  "selection_shortcut",  // currently not supported
-  "shift_key_mode_switch",
-  "numpad_character_form",
-  "suggestions_size",
-};
-#endif  // OS_CHROMEOS
-
-#ifndef OS_CHROMEOS
 const char *kUILocaleEnvNames[] = {
   "LC_ALL",
   "LC_MESSAGES",
@@ -143,7 +108,22 @@ string GetMessageLocale() {
   }
   return kMozcDefaultUILocale;
 }
-#endif  // !OS_CHROMEOS
+
+bool GetString(GVariant *value, string *out_string) {
+  if (g_variant_classify(value) != G_VARIANT_CLASS_STRING) {
+    return false;
+  }
+  *out_string = static_cast<const char *>(g_variant_get_string(value, NULL));
+  return true;
+}
+
+bool GetBoolean(GVariant *value, bool *out_boolean) {
+  if (g_variant_classify(value) != G_VARIANT_CLASS_BOOLEAN) {
+    return false;
+  }
+  *out_boolean = (g_variant_get_boolean(value) != FALSE);
+  return true;
+}
 
 struct IBusMozcEngineClass {
   IBusEngineClass parent;
@@ -196,22 +176,72 @@ namespace mozc {
 namespace ibus {
 
 namespace {
-client::ClientInterface *CreateClient() {
-#ifdef OS_CHROMEOS
-  return new ibus::Client();
-#else
-  return client::ClientFactory::NewClient();
-#endif
+struct SurroundingTextInfo {
+  SurroundingTextInfo()
+      : relative_selected_length(0) {}
+  int32 relative_selected_length;
+  string preceding_text;
+  string selection_text;
+  string following_text;
+};
+
+bool GetSurroundingText(IBusEngine *engine,
+#ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
+                        SelectionMonitorInterface *selection_monitor,
+#endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
+                        SurroundingTextInfo *info) {
+  if (!(engine->client_capabilities & IBUS_CAP_SURROUNDING_TEXT)) {
+    VLOG(1) << "Give up CONVERT_REVERSE due to client_capabilities: "
+            << engine->client_capabilities;
+    return false;
+  }
+  guint cursor_pos = 0;
+  guint anchor_pos = 0;
+  // DO NOT call g_object_unref against this.
+  // http://ibus.googlecode.com/svn/docs/ibus-1.4/IBusText.html
+  // http://developer.gnome.org/gobject/stable/gobject-The-Base-Object-Type.html#gobject-The-Base-Object-Type.description
+  IBusText *text = NULL;
+  ibus_engine_get_surrounding_text(engine, &text, &cursor_pos,
+                                   &anchor_pos);
+  const string surrounding_text(ibus_text_get_text(text));
+
+#ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
+  if (cursor_pos == anchor_pos && selection_monitor != NULL) {
+    const SelectionInfo &info = selection_monitor->GetSelectionInfo();
+    guint new_anchor_pos = 0;
+    if (SurroundingTextUtil::GetAnchorPosFromSelection(
+            surrounding_text, info.selected_text,
+            cursor_pos, &new_anchor_pos)) {
+      anchor_pos = new_anchor_pos;
+    }
+  }
+#endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
+
+  if (!SurroundingTextUtil::GetSafeDelta(cursor_pos, anchor_pos,
+                                         &info->relative_selected_length)) {
+    LOG(ERROR) << "Too long text selection.";
+    return false;
+  }
+
+  const size_t selection_start = min(cursor_pos, anchor_pos);
+  const size_t selection_length = abs(info->relative_selected_length);
+  Util::SubStringPiece(surrounding_text, 0, selection_start)
+      .CopyToString(&info->preceding_text);
+  Util::SubStringPiece(surrounding_text, selection_start, selection_length)
+      .CopyToString(&info->selection_text);
+  Util::SubStringPiece(surrounding_text, selection_start + selection_length)
+      .CopyToString(&info->following_text);
+  return true;
 }
 
-MessageTranslatorInterface *CreateTranslator() {
-// It is OK not to translate messages in our side on ChromeOS, where built-in
-// translation mechanism exists.
-#ifdef OS_CHROMEOS
-  return new NullMessageTranslator();
-#else
-  return new LocaleBasedMessageTranslator(GetMessageLocale());
-#endif
+std::unique_ptr<client::ClientInterface> CreateAndConfigureClient() {
+  std::unique_ptr<client::ClientInterface> client(
+      client::ClientFactory::NewClient());
+  // Currently client capability is fixed.
+  commands::Capability capability;
+  capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
+  client->set_client_capability(capability);
+  return client;
 }
 
 }  // namespace
@@ -219,27 +249,20 @@ MessageTranslatorInterface *CreateTranslator() {
 MozcEngine::MozcEngine()
     : last_sync_time_(Util::GetTime()),
       key_event_handler_(new KeyEventHandler),
-      client_(CreateClient()),
+      client_(CreateAndConfigureClient()),
 #ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
       selection_monitor_(SelectionMonitorFactory::Create(1024)),
 #endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
-      property_handler_(new PropertyHandler(CreateTranslator(),
-                                            client_.get())),
+      property_handler_(new PropertyHandler(
+          new LocaleBasedMessageTranslator(GetMessageLocale()), client_.get())),
       preedit_handler_(new PreeditHandler()),
 #ifdef ENABLE_GTK_RENDERER
       gtk_candidate_window_handler_(new GtkCandidateWindowHandler(
           new renderer::RendererClient())),
-#else
-      gtk_candidate_window_handler_(NULL),
 #endif  // ENABLE_GTK_RENDERER
       ibus_candidate_window_handler_(new IBusCandidateWindowHandler()),
       preedit_method_(config::Config::ROMAN) {
 
-  // Currently client capability is fixed.
-  commands::Capability capability;
-  capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
-  client_->set_client_capability(capability);
-
 #ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
   if (selection_monitor_.get() != NULL) {
     selection_monitor_->StartMonitoring();
@@ -285,6 +308,7 @@ void MozcEngine::CursorUp(IBusEngine *engine) {
 void MozcEngine::Disable(IBusEngine *engine) {
   RevertSession(engine);
   GetCandidateWindowHandler(engine)->Hide(engine);
+  key_event_handler_->Clear();
 }
 
 void MozcEngine::Enable(IBusEngine *engine) {
@@ -310,6 +334,7 @@ void MozcEngine::FocusIn(IBusEngine *engine) {
 
 void MozcEngine::FocusOut(IBusEngine *engine) {
   GetCandidateWindowHandler(engine)->Hide(engine);
+  property_handler_->ResetContentType(engine);
 
   // Do not call SubmitSession or RevertSession. Preedit string will commit on
   // Focus Out event automatically by ibus_engine_update_preedit_text_with_mode
@@ -335,6 +360,9 @@ gboolean MozcEngine::ProcessKeyEvent(
   VLOG(2) << "keyval: " << keyval
           << ", keycode: " << keycode
           << ", modifiers: " << modifiers;
+  if (property_handler_->IsDisabled()) {
+    return FALSE;
+  }
 
   // Send current caret location to mozc_server to manage suggest window
   // position.
@@ -345,19 +373,6 @@ gboolean MozcEngine::ProcessKeyEvent(
                     engine->cursor_area.width,
                     engine->cursor_area.height);
 
-  // Since IBus for ChromeOS is based on in-process conversion,
-  // it is basically ok to call GetConfig() at every keyevent.
-  // On the other hands, IBus for Linux is based on out-process (IPC)
-  // conversion and user may install large keybinding/roman-kana tables.
-  // To reduce IPC overheads, we don't call UpdatePreeditMethod()
-  // at every keyevent. When user changes the preedit method via
-  // config dialog, the dialog shows a message saying that
-  // "preedit method is enabled after new applications.".
-  // This behavior is the same as Google Japanese Input for Windows.
-#ifdef OS_CHROMEOS
-  UpdatePreeditMethod();
-#endif
-
   // TODO(yusukes): use |layout| in IBusEngineDesc if possible.
   const bool layout_is_jp =
       !g_strcmp0(ibus_engine_get_name(engine), "mozc-jp");
@@ -372,15 +387,24 @@ gboolean MozcEngine::ProcessKeyEvent(
   VLOG(2) << key.DebugString();
   if (!property_handler_->IsActivated() &&
       !config::ImeSwitchUtil::IsDirectModeCommand(key)) {
-    // We DO consume keys that should be handled even when in the DIRECT
-    // mode such as (default) Henkan key to enable Mozc.
     return FALSE;
   }
 
+  key.set_activated(property_handler_->IsActivated());
   key.set_mode(property_handler_->GetOriginalCompositionMode());
 
+  commands::Context context;
+  SurroundingTextInfo surrounding_text_info;
+  if (GetSurroundingText(engine,
+#ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
+                         selection_monitor_.get(),
+#endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
+                         &surrounding_text_info)) {
+    context.set_preceding_text(surrounding_text_info.preceding_text);
+    context.set_following_text(surrounding_text_info.following_text);
+  }
   commands::Output output;
-  if (!client_->SendKey(key, &output)) {
+  if (!client_->SendKeyWithContext(key, context, &output)) {
     LOG(ERROR) << "SendKey failed";
     return FALSE;
   }
@@ -427,6 +451,18 @@ void MozcEngine::SetCursorLocation(IBusEngine *engine,
   // Do nothing
 }
 
+void MozcEngine::SetContentType(IBusEngine *engine,
+                                guint purpose,
+                                guint hints) {
+  const bool prev_disabled =
+      property_handler_->IsDisabled();
+  property_handler_->UpdateContentType(engine);
+  if (!prev_disabled && property_handler_->IsDisabled()) {
+    // Make sure on-going composition is reverted.
+    RevertSession(engine);
+  }
+}
+
 GType MozcEngine::GetType() {
   static GType type = 0;
 
@@ -478,16 +514,16 @@ void MozcEngine::InitRendererConfig(IBusConfig *config) {
   GVariant *use_custom_font_value = ibus_config_get_value(config,
                                                           kMozcPanelSectionName,
                                                           "use_custom_font");
-  gboolean use_custom_font;
-  if (ConfigUtil::GetBoolean(use_custom_font_value, &use_custom_font)) {
+  bool use_custom_font;
+  if (GetBoolean(use_custom_font_value, &use_custom_font)) {
     gtk_candidate_window_handler_->OnIBusUseCustomFontDescriptionChanged(
-        static_cast<bool>(use_custom_font != FALSE));
+        use_custom_font);
   } else {
     LOG(ERROR) << "Initialize Failed: "
                << "Cannot get panel:use_custom_font configuration.";
   }
-  const gchar *font_description;
-  if (ConfigUtil::GetString(custom_font_value, &font_description)) {
+  string font_description;
+  if (GetString(custom_font_value, &font_description)) {
     gtk_candidate_window_handler_->OnIBusCustomFontDescriptionChanged(
        font_description);
   } else {
@@ -503,20 +539,8 @@ void MozcEngine::InitConfig(IBusConfig *config) {
   MozcEngine *engine = mozc::Singleton<MozcEngine>::get();
   engine->InitRendererConfig(config);
 #endif  // ENABLE_GTK_RENDERER
-#ifdef OS_CHROMEOS
-  map<string, const char*> name_to_field;
-  for (size_t i = 0; i < arraysize(kMozcConfigNames); ++i) {
-    // Mozc uses identical names for ibus config names and protobuf config
-    // field names.
-    name_to_field.insert(make_pair(kMozcConfigNames[i], kMozcConfigNames[i]));
-  }
-  // Initialize the mozc config with the config loaded from ibus-memconf, which
-  // is the primary config storage on Chrome OS.
-  ConfigUtil::InitConfig(config, kMozcSectionName, name_to_field);
-#endif  // OS_CHROMEOS
 }
 
-
 bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
   UpdateDeletionRange(engine, output);
   UpdateResult(engine, output);
@@ -590,16 +614,16 @@ void MozcEngine::UpdateConfig(const gchar *section,
   // TODO(nona): Introduce ConfigHandler
   if (g_strcmp0(section, kMozcPanelSectionName) == 0) {
     if (g_strcmp0(name, "use_custom_font") == 0) {
-      gboolean use_custom_font;
-      if (!ConfigUtil::GetBoolean(value, &use_custom_font)) {
+      bool use_custom_font;
+      if (!GetBoolean(value, &use_custom_font)) {
         LOG(ERROR) << "Cannot get " << section << ":" << name << " value.";
         return;
       }
       gtk_candidate_window_handler_->OnIBusUseCustomFontDescriptionChanged(
-          static_cast<bool>(use_custom_font != FALSE));
+          use_custom_font);
     } else if (g_strcmp0(name, "custom_font") == 0) {
-      const gchar *font_description;
-      if (!ConfigUtil::GetString(value, &font_description)) {
+      string font_description;
+      if (!GetString(value, &font_description)) {
         LOG(ERROR) << "Cannot get " << section << ":" << name << " value.";
         return;
       }
@@ -608,18 +632,6 @@ void MozcEngine::UpdateConfig(const gchar *section,
     }
   }
 #endif  // ENABLE_GTK_RENDERER
-#ifdef OS_CHROMEOS
-  if (g_strcmp0(section, kMozcSectionName) == 0) {
-    config::Config mozc_config;
-    client_->GetConfig(&mozc_config);
-    ConfigUtil::SetFieldForName(name, value, &mozc_config);
-
-    // Update config1.db.
-    client_->SetConfig(mozc_config);
-    client_->SyncData();  // TODO(yusukes): remove this call?
-    VLOG(2) << "Client::SetConfig() is called: " << name;
-  }
-#endif  // OS_CHROMEOS
 }
 
 void MozcEngine::UpdatePreeditMethod() {
@@ -698,7 +710,7 @@ bool MozcEngine::ExecuteCallback(IBusEngine *engine,
   // Note that you should not allow 0x80000000 for |relative_selected_length|
   // because you cannot safely use |-relative_selected_length| nor
   // |abs(relative_selected_length)| in this case due to integer overflow.
-  int32 relative_selected_length = 0;
+  SurroundingTextInfo surrounding_text_info;
 
   switch (callback_command.type()) {
     case commands::SessionCommand::UNDO:
@@ -712,57 +724,14 @@ bool MozcEngine::ExecuteCallback(IBusEngine *engine,
       // }
       break;
     case commands::SessionCommand::CONVERT_REVERSE: {
-      if (!(engine->client_capabilities & IBUS_CAP_SURROUNDING_TEXT)) {
-        VLOG(1) << "Give up CONVERT_REVERSE due to client_capabilities: "
-                << engine->client_capabilities;
-        return false;
-      }
-      guint cursor_pos = 0;
-      guint anchor_pos = 0;
-      // DO NOT call g_object_unref against this.
-      // http://ibus.googlecode.com/svn/docs/ibus-1.4/IBusText.html
-      // http://developer.gnome.org/gobject/stable/gobject-The-Base-Object-Type.html#gobject-The-Base-Object-Type.description
-      IBusText *text = NULL;
-      ibus_engine_get_surrounding_text(engine, &text, &cursor_pos,
-                                       &anchor_pos);
-      const string surrounding_text(ibus_text_get_text(text));
-
+      if (!GetSurroundingText(engine,
 #ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
-      if (cursor_pos == anchor_pos && selection_monitor_.get() != NULL) {
-        const SelectionInfo &info = selection_monitor_->GetSelectionInfo();
-        guint new_anchor_pos = 0;
-        if (SurroundingTextUtil::GetAnchorPosFromSelection(
-                surrounding_text, info.selected_text,
-                cursor_pos, &new_anchor_pos)) {
-          anchor_pos = new_anchor_pos;
-        }
-      }
+                              selection_monitor_.get(),
 #endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
-
-      if (cursor_pos == anchor_pos) {
-        // There is no selection text.
-        VLOG(1) << "Failed to retrieve non-empty text selection.";
+                              &surrounding_text_info)) {
         return false;
       }
-
-      if (!SurroundingTextUtil::GetSafeDelta(cursor_pos, anchor_pos,
-                                             &relative_selected_length)) {
-        LOG(ERROR) << "Too long text selection.";
-        return false;
-      }
-
-      // TODO(nona): Write a test for this logic (especially selection_length).
-      // TODO(nona): Check integer range because Util::SubString works
-      //     on size_t, not uint32.
-      string selection_text;
-      const uint32 selection_start = min(cursor_pos, anchor_pos);
-      const uint32 selection_length = abs(relative_selected_length);
-      Util::SubString(surrounding_text,
-                      selection_start,
-                      selection_length,
-                      &selection_text);
-
-      session_command.set_text(selection_text);
+      session_command.set_text(surrounding_text_info.selection_text);
       break;
     }
     default:
@@ -783,11 +752,11 @@ bool MozcEngine::ExecuteCallback(IBusEngine *engine,
     // offset should be a negative value to delete preceding text.
     // For backward selection (that is, |relative_selected_length < 0|),
     // IBus and/or some applications seem to expect |offset == 0| somehow.
-    const int32 offset = relative_selected_length > 0
-        ? -relative_selected_length  // forward selection
-        : 0;                         // backward selection
+    const int32 offset = surrounding_text_info.relative_selected_length > 0
+        ? -surrounding_text_info.relative_selected_length  // forward selection
+        : 0;                                               // backward selection
     range->set_offset(offset);
-    range->set_length(abs(relative_selected_length));
+    range->set_length(abs(surrounding_text_info.relative_selected_length));
   }
 
   // Here uses recursion of UpdateAll but it's okay because the converter
@@ -810,9 +779,9 @@ CandidateWindowHandlerInterface *MozcEngine::GetCandidateWindowHandler(
   }
 
   // TODO(nona): integrate with renderer/renderer_client.cc
-  const string renderer_path = mozc::Util::JoinPath(
-      mozc::Util::GetServerDirectory(), "mozc_renderer");
-  if (!Util::FileExists(renderer_path)) {
+  const string renderer_path = FileUtil::JoinPath(
+      SystemUtil::GetServerDirectory(), "mozc_renderer");
+  if (!FileUtil::FileExists(renderer_path)) {
     return ibus_candidate_window_handler_.get();
   }
 
