@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ class PreeditHandlerInterface {
   virtual ~PreeditHandlerInterface() {}
 
   virtual bool Update(IBusEngine *engine,
-                      const commands::Output &output) ABSTRACT;
+                      const commands::Output &output) = 0;
 };
 
 }  // namespace ibus
