@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,17 +29,29 @@
 
 #include "unix/ibus/property_handler.h"
 
+#include <string>
+
 #include "base/const.h"
-#include "base/util.h"
+#include "base/file_util.h"
+#include "base/logging.h"
+#include "base/system_util.h"
 #include "client/client.h"  // For client interface
+#include "unix/ibus/message_translator.h"
 #include "unix/ibus/mozc_engine_property.h"
 #include "unix/ibus/path_util.h"
-#include "unix/ibus/message_translator.h"
+
+// On Gnome Shell with IBus 1.5, new property named "symbol" is used to
+// represent the mode indicator on the system panel. Note that "symbol" does
+// not exist in IBus 1.4.x.
+#if IBUS_CHECK_VERSION(1, 5, 0)
+#define MOZC_IBUS_HAS_SYMBOL
+#endif  // IBus >= 1.5
 
 namespace mozc {
 namespace ibus {
 
 namespace {
+
 // A key which associates an IBusProperty object with MozcEngineProperty.
 const char kGObjectDataKey[] = "ibus-mozc-aux-data";
 
@@ -48,8 +60,31 @@ const char kMozcToolIconPath[] = "tool.png";
 
 // Returns true if mozc_tool is installed.
 bool IsMozcToolAvailable() {
-  return mozc::Util::FileExists(mozc::Util::GetToolPath());
+  return FileUtil::FileExists(SystemUtil::GetToolPath());
 }
+
+bool GetDisabled(IBusEngine *engine) {
+  bool disabled = false;
+#if defined(MOZC_ENABLE_IBUS_INPUT_PURPOSE)
+  guint purpose = IBUS_INPUT_PURPOSE_FREE_FORM;
+  guint hints = IBUS_INPUT_HINT_NONE;
+  ibus_engine_get_content_type(engine, &purpose, &hints);
+  disabled = (purpose == IBUS_INPUT_PURPOSE_PASSWORD ||
+              purpose == IBUS_INPUT_PURPOSE_PIN);
+#endif  // MOZC_ENABLE_IBUS_INPUT_PURPOSE
+  return disabled;
+}
+
+// Some users expect that Mozc is turned off by default on IBus 1.5.0 and later.
+// https://code.google.com/p/mozc/issues/detail?id=201
+// On IBus 1.4.x, IBus expects that an IME should always be turned on and
+// IME on/off keys are handled by IBus itself rather than each IME.
+#if IBUS_CHECK_VERSION(1, 5, 0)
+const bool kActivatedOnLaunch = false;
+#else
+const bool kActivatedOnLaunch = true;
+#endif  // IBus>=1.5.0
+
 }  // namespace
 
 PropertyHandler::PropertyHandler(MessageTranslatorInterface *translator,
@@ -60,13 +95,22 @@ PropertyHandler::PropertyHandler(MessageTranslatorInterface *translator,
       client_(client),
       translator_(translator),
       original_composition_mode_(kMozcEngineInitialCompositionMode),
-      is_activated_(true) {
+      is_activated_(kActivatedOnLaunch),
+      is_disabled_(false) {
+  commands::SessionCommand command;
+  if (is_activated_) {
+    command.set_type(commands::SessionCommand::TURN_ON_IME);
+  } else {
+    command.set_type(commands::SessionCommand::TURN_OFF_IME);
+  }
+  command.set_composition_mode(original_composition_mode_);
+  commands::Output output;
+  if (!client->SendCommand(command, &output)) {
+    LOG(ERROR) << "SendCommand failed";
+  }
 
   AppendCompositionPropertyToPanel();
-  AppendSwitchPropertyToPanel();
-#ifndef OS_CHROMEOS
   AppendToolPropertyToPanel();
-#endif
 
   // We have to sink |prop_root_| as well so ibus_engine_register_properties()
   // in FocusIn() does not destruct it.
@@ -86,12 +130,6 @@ PropertyHandler::~PropertyHandler() {
     prop_mozc_tool_ = NULL;
   }
 
-  for (size_t i = 0; i < prop_switch_properties_.size(); ++i) {
-    // The ref counter will drop to one.
-    g_object_unref(prop_switch_properties_[i]);
-  }
-  prop_switch_properties_.clear();
-
   if (prop_root_) {
     // Destroy all objects under the root.
     g_object_unref(prop_root_);
@@ -101,6 +139,7 @@ PropertyHandler::~PropertyHandler() {
 
 void PropertyHandler::Register(IBusEngine *engine) {
   ibus_engine_register_properties(engine, prop_root_);
+  UpdateContentType(engine);
 }
 
 // TODO(nona): do not use kMozcEngine*** directory.
@@ -114,15 +153,21 @@ void PropertyHandler::AppendCompositionPropertyToPanel() {
   IBusPropList *sub_prop_list = ibus_prop_list_new();
 
   // Create items for the radio menu.
+  const commands::CompositionMode initial_mode = is_activated_ ?
+      original_composition_mode_ :
+      kMozcEnginePropertyIMEOffState->composition_mode;
+
   string icon_path_for_panel;
+  const char *mode_symbol = NULL;
   for (size_t i = 0; i < kMozcEnginePropertiesSize; ++i) {
     const MozcEngineProperty &entry = kMozcEngineProperties[i];
     IBusText *label = ibus_text_new_from_string(
         translator_->MaybeTranslate(entry.label).c_str());
     IBusPropState state = PROP_STATE_UNCHECKED;
-    if (entry.composition_mode == kMozcEngineInitialCompositionMode) {
+    if (entry.composition_mode == initial_mode) {
       state = PROP_STATE_CHECKED;
       icon_path_for_panel = GetIconPath(entry.icon);
+      mode_symbol = entry.label_for_panel;
     }
     IBusProperty *item = ibus_property_new(entry.key,
                                            PROP_TYPE_RADIO,
@@ -136,15 +181,22 @@ void PropertyHandler::AppendCompositionPropertyToPanel() {
     g_object_set_data(G_OBJECT(item), kGObjectDataKey, (gpointer)&entry);
     ibus_prop_list_append(sub_prop_list, item);
     // |sub_prop_list| owns |item| by calling g_object_ref_sink for the |item|.
-    // Due to api behavior changes, the label object is leak on ibus-1.2.x
-    // environment, but we decide to give up.
   }
   DCHECK(!icon_path_for_panel.empty());
+  DCHECK(mode_symbol != NULL);
+
+  const string &mode_label =
+      translator_->MaybeTranslate("Input Mode") + " (" + mode_symbol + ")";
+  IBusText *label = ibus_text_new_from_string(mode_label.c_str());
 
   // The label of |prop_composition_mode_| is shown in the language panel.
-  prop_composition_mode_ = ibus_property_new("CompositionMode",
+  // Note that the property name "InputMode" is hard-coded in the Gnome shell.
+  // Do not change the name. Othewise the Gnome shell fails to recognize that
+  // this property indicates Mozc's input mode.
+  // See /usr/share/gnome-shell/js/ui/status/keyboard.js for details.
+  prop_composition_mode_ = ibus_property_new("InputMode",
                                              PROP_TYPE_MENU,
-                                             NULL /* label */,
+                                             label,
                                              icon_path_for_panel.c_str(),
                                              NULL /* tooltip */,
                                              TRUE /* sensitive */,
@@ -152,6 +204,13 @@ void PropertyHandler::AppendCompositionPropertyToPanel() {
                                              PROP_STATE_UNCHECKED,
                                              sub_prop_list);
 
+  // Gnome shell uses symbol property for the mode indicator text icon iff the
+  // property name is "InputMode".
+#ifdef MOZC_IBUS_HAS_SYMBOL
+  IBusText *symbol = ibus_text_new_from_static_string(mode_symbol);
+  ibus_property_set_symbol(prop_composition_mode_, symbol);
+#endif  // MOZC_IBUS_HAS_SYMBOL
+
   // Likewise, |prop_composition_mode_| owns |sub_prop_list|. We have to sink
   // |prop_composition_mode_| here so ibus_engine_update_property() call in
   // PropertyActivate() does not destruct the object.
@@ -160,6 +219,27 @@ void PropertyHandler::AppendCompositionPropertyToPanel() {
   ibus_prop_list_append(prop_root_, prop_composition_mode_);
 }
 
+void PropertyHandler::UpdateContentTypeImpl(IBusEngine *engine,
+                                            bool disabled) {
+  const bool prev_is_disabled = is_disabled_;
+  is_disabled_ = disabled;
+  if (prev_is_disabled == is_disabled_) {
+    return;
+  }
+  const auto visible_mode = (prev_is_disabled && !is_disabled_ && IsActivated())
+      ? original_composition_mode_ :
+        kMozcEnginePropertyIMEOffState->composition_mode;
+  UpdateCompositionModeIcon(engine, visible_mode);
+}
+
+void PropertyHandler::ResetContentType(IBusEngine *engine) {
+  UpdateContentTypeImpl(engine, false);
+}
+
+void PropertyHandler::UpdateContentType(IBusEngine *engine) {
+  UpdateContentTypeImpl(engine, GetDisabled(engine));
+}
+
 // TODO(nona): do not use kMozcEngine*** directory.
 void PropertyHandler::AppendToolPropertyToPanel() {
   if (kMozcEngineToolProperties == NULL || kMozcEngineToolPropertiesSize == 0 ||
@@ -177,7 +257,7 @@ void PropertyHandler::AppendToolPropertyToPanel() {
         translator_->MaybeTranslate(entry.label).c_str());
     // TODO(yusukes): It would be better to use entry.icon here?
     IBusProperty *item = ibus_property_new(entry.mode,
-                                             PROP_TYPE_NORMAL,
+                                           PROP_TYPE_NORMAL,
                                            label,
                                            NULL /* icon */,
                                            NULL /* tooltip */,
@@ -187,14 +267,14 @@ void PropertyHandler::AppendToolPropertyToPanel() {
                                            NULL);
     g_object_set_data(G_OBJECT(item), kGObjectDataKey, (gpointer)&entry);
     ibus_prop_list_append(sub_prop_list, item);
-    // Due to api behavior changes, the label object is leak on ibus-1.2.x
-    // environment, but we decide to give up.
   }
 
+  IBusText *tool_label = ibus_text_new_from_string(
+      translator_->MaybeTranslate("Tools").c_str());
   const string icon_path = GetIconPath(kMozcToolIconPath);
   prop_mozc_tool_ = ibus_property_new("MozcTool",
                                       PROP_TYPE_MENU,
-                                      NULL /* label */,
+                                      tool_label,
                                       icon_path.c_str(),
                                       NULL /* tooltip */,
                                       TRUE /* sensitive */,
@@ -210,45 +290,12 @@ void PropertyHandler::AppendToolPropertyToPanel() {
   ibus_prop_list_append(prop_root_, prop_mozc_tool_);
 }
 
-// TODO(nona): do not use kMozcEngine*** directory.
-void PropertyHandler::AppendSwitchPropertyToPanel() {
-  if (kMozcEngineSwitchProperties == NULL ||
-      kMozcEngineSwitchPropertiesSize == 0) {
+void PropertyHandler::Update(IBusEngine *engine,
+                             const commands::Output &output) {
+  if (IsDisabled()) {
     return;
   }
 
-  for (size_t i = 0; i < kMozcEngineSwitchPropertiesSize; ++i) {
-    const MozcEngineSwitchProperty &entry = kMozcEngineSwitchProperties[i];
-    IBusText *label = ibus_text_new_from_string(
-        translator_->MaybeTranslate(entry.label).c_str());
-    IBusText *tooltip = ibus_text_new_from_string(
-        translator_->MaybeTranslate(entry.tooltip).c_str());
-    IBusProperty *item = ibus_property_new(entry.key,
-                                           PROP_TYPE_NORMAL,
-                                           label,
-                                           GetIconPath(entry.icon).c_str(),
-                                           tooltip,
-                                           TRUE,
-                                           TRUE,
-                                           PROP_STATE_UNCHECKED,
-                                           NULL);
-    g_object_set_data(G_OBJECT(item), kGObjectDataKey, (gpointer)&entry);
-    prop_switch_properties_.push_back(item);
-
-    // We have to sink |*item| here so ibus_engine_update_property() call in
-    // PropertyActivate() does not destruct the object.
-    g_object_ref_sink(item);
-    // Due to api behavior changes, the label object is leak on ibus-1.2.x
-    // environment, but we decide to give up.
-  }
-
-  for (size_t i = 0; i < prop_switch_properties_.size(); ++i) {
-    ibus_prop_list_append(prop_root_, prop_switch_properties_[i]);
-  }
-}
-
-void PropertyHandler::Update(IBusEngine *engine,
-                             const commands::Output &output) {
   if (output.has_status() &&
       (output.status().activated() != is_activated_ ||
        output.status().mode() != original_composition_mode_)) {
@@ -297,6 +344,19 @@ void PropertyHandler::UpdateCompositionModeIcon(
     }
     // No need to call unref since ibus_prop_list_get does not add ref.
   }
+
+  const char *mode_symbol = entry->label_for_panel;
+  // Update the text icon for Gnome shell.
+#ifdef MOZC_IBUS_HAS_SYMBOL
+  IBusText *symbol = ibus_text_new_from_static_string(mode_symbol);
+  ibus_property_set_symbol(prop_composition_mode_, symbol);
+#endif  // MOZC_IBUS_HAS_SYMBOL
+
+  const string &mode_label =
+      translator_->MaybeTranslate("Input Mode") + " (" + mode_symbol + ")";
+  IBusText *label = ibus_text_new_from_string(mode_label.c_str());
+  ibus_property_set_label(prop_composition_mode_, label);
+
   ibus_engine_update_property(engine, prop_composition_mode_);
 }
 
@@ -314,9 +374,9 @@ void PropertyHandler::SetCompositionMode(
   if (kMozcEnginePropertyIMEOffState
       && is_activated_
       && composition_mode == kMozcEnginePropertyIMEOffState->composition_mode) {
-    commands::KeyEvent key;
-    key.set_special_key(mozc::commands::KeyEvent::OFF);
-    client_->SendKey(key, &output);
+    command.set_type(commands::SessionCommand::TURN_OFF_IME);
+    command.set_composition_mode(original_composition_mode_);
+    client_->SendCommand(command, &output);
   } else {
     command.set_type(commands::SessionCommand::SWITCH_INPUT_MODE);
     command.set_composition_mode(composition_mode);
@@ -330,7 +390,10 @@ void PropertyHandler::SetCompositionMode(
 void PropertyHandler::ProcessPropertyActivate(IBusEngine *engine,
                                               const gchar *property_name,
                                               guint property_state) {
-#ifndef OS_CHROMEOS
+  if (IsDisabled()) {
+    return;
+  }
+
   if (prop_mozc_tool_) {
     for (guint prop_index = 0; ; ++prop_index) {
       IBusProperty *prop = ibus_prop_list_get(
@@ -350,26 +413,6 @@ void PropertyHandler::ProcessPropertyActivate(IBusEngine *engine,
       }
     }
   }
-#endif
-
-  for (size_t i = 0; i < prop_switch_properties_.size(); ++i) {
-    IBusProperty *prop = prop_switch_properties_[i];
-    if (!g_strcmp0(property_name, ibus_property_get_key(prop))) {
-      const MozcEngineSwitchProperty *entry =
-          reinterpret_cast<const MozcEngineSwitchProperty *>(
-              g_object_get_data(G_OBJECT(prop), kGObjectDataKey));
-      DCHECK(entry->id);
-      commands::Output output;
-      commands::SessionCommand command;
-      command.set_language_bar_command_id(entry->id);
-      command.set_type(commands::SessionCommand::SEND_LANGUAGE_BAR_COMMAND);
-      if (!client_->SendCommand(command, &output)) {
-        LOG(ERROR) << "cannot send command to update session config: "
-                   << entry->id;
-      }
-      return;
-    }
-  }
 
   if (property_state != PROP_STATE_CHECKED) {
     return;
@@ -399,6 +442,10 @@ bool PropertyHandler::IsActivated() const {
   return is_activated_;
 }
 
+bool PropertyHandler::IsDisabled() const {
+  return is_disabled_;
+}
+
 commands::CompositionMode PropertyHandler::GetOriginalCompositionMode() const {
   return original_composition_mode_;
 }
