@@ -1,4 +1,4 @@
-// Copyright 2010-2012, Google Inc.
+// Copyright 2010-2014, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,22 +32,23 @@
 #include <ime.h>
 #include <windows.h>
 
+#include <memory>
+
+#include "base/logging.h"
 #include "base/util.h"
 #include "client/client_interface.h"
 #include "config/config_handler.h"
 #include "session/commands.pb.h"
-#include "session/ime_switch_util.h"
 #include "session/output_util.h"
 #include "win32/base/conversion_mode_util.h"
+#include "win32/base/deleter.h"
+#include "win32/base/input_state.h"
+#include "win32/base/keyboard.h"
+#include "win32/base/imm_reconvert_string.h"
 #include "win32/ime/ime_candidate_info.h"
 #include "win32/ime/ime_composition_string.h"
-#include "win32/ime/ime_deleter.h"
-#include "win32/ime/ime_keyboard.h"
-#include "win32/ime/ime_mouse_tracker.h"
 #include "win32/ime/ime_private_context.h"
-#include "win32/ime/ime_reconvert_string.h"
 #include "win32/ime/ime_scoped_context.h"
-#include "win32/ime/ime_state.h"
 #include "win32/ime/ime_ui_context.h"
 #include "win32/ime/ime_ui_visibility_tracker.h"
 
@@ -57,58 +58,63 @@ namespace win32 {
 using commands::KeyEvent;
 using commands::Output;
 using commands::SessionCommand;
+using std::unique_ptr;
 
 namespace {
+
 const size_t kReconvertStringSizeLimit = 1024*64;
 // An embedded object in the RichEdit will be replaced with this character.
 // See b/3406434 for details.
 const wchar_t kObjectReplacementCharacter = L'\uFFFC';
 
 bool GetNextState(HIMC himc, const commands::Output &output,
-                  mozc::win32::ImeState *next_state) {
+                  mozc::win32::InputState *next_state) {
   DCHECK(next_state);
 
   UIContext context(himc);
   bool next_open_status = false;
-  DWORD next_mode = 0;
+  DWORD next_logical_mode = 0;
+  DWORD next_visible_mode = 0;
   if (!output.has_status()) {
     // |output| does not have |status|.  Preserve the current status.
     next_open_status = context.GetOpenStatus();
-    if (!context.GetConversionMode(&next_mode)) {
+    if (!context.GetConversionMode(&next_logical_mode)) {
       return false;
     }
+    next_visible_mode = next_logical_mode;
   } else if (!mozc::win32::ConversionModeUtil::ConvertStatusFromMozcToNative(
       output.status(), context.IsKanaInputPreferred(),
-      &next_open_status, &next_mode)) {
+      &next_open_status, &next_logical_mode, &next_visible_mode)) {
     return false;
   }
 
   next_state->open = next_open_status;
-  next_state->conversion_status = next_mode;
+  next_state->logical_conversion_mode = next_logical_mode;
+  next_state->visible_conversion_mode = next_visible_mode;
   return true;
 }
 
-BOOL UpdateInputContext(
+bool UpdateInputContext(
     HIMC himc, const commands::Output &output, bool generate_message) {
-  ImeState next_state;
+  InputState next_state;
   if (!GetNextState(himc, output, &next_state)) {
-    return FALSE;
+    return false;
   }
   if (!generate_message) {
-    if (!ImeCore::UpdateContext(himc, next_state, output, NULL)) {
-      return FALSE;
+    if (!ImeCore::UpdateContext(himc, next_state, output, nullptr)) {
+      return false;
     }
-    return TRUE;
+    return true;
   }
   MessageQueue message_queue(himc);
   if (!ImeCore::UpdateContext(himc, next_state, output, &message_queue)) {
     return FALSE;
   }
-  return (message_queue.Send() ? TRUE : FALSE);
+  return message_queue.Send();
 }
 
 HIMCC EnsureHIMCCSize(HIMCC himcc, DWORD size) {
-  if (himcc == NULL) {
+  if (himcc == nullptr) {
     return ::ImmCreateIMCC(size);
   }
   const DWORD current_size = ::ImmGetIMCCSize(himcc);
@@ -129,7 +135,7 @@ bool UpdateCompositionString(HIMC himc,
   // TODO(yukawa): Move this logic into more appropriate place.
   const HIMCC composition_string_handle = EnsureHIMCCSize(
       context->hCompStr, sizeof(CompositionString));
-  if (composition_string_handle == NULL) {
+  if (composition_string_handle == nullptr) {
     return false;
   }
   ScopedHIMCC<CompositionString> compstr(composition_string_handle);
@@ -151,7 +157,7 @@ bool UpdateCompositionStringAndPushMessages(
     return false;
   }
 
-  const bool generate_message = (message_queue != NULL);
+  const bool generate_message = (message_queue != nullptr);
   if (!generate_message) {
     return true;
   }
@@ -182,8 +188,8 @@ bool GetReconvertString(const RECONVERTSTRING *reconvert_string,
   wstring target_text;
   wstring following_composition;
   if (!ReconvertString::Decompose(
-          reconvert_string, NULL, &preceding_composition,
-          &target_text, &following_composition, NULL)) {
+          reconvert_string, nullptr, &preceding_composition,
+          &target_text, &following_composition, nullptr)) {
     DLOG(INFO) << "ReconvertString::Decompose failed.";
     return false;
   }
@@ -207,22 +213,74 @@ bool GetReconvertString(const RECONVERTSTRING *reconvert_string,
 
   return true;
 }
-}  // anonymous namespace
+
+bool QueryDocumentFeed(HIMC himc,
+                       wstring *preceding_text,
+                       wstring *following_text) {
+  LRESULT result = ::ImmRequestMessageW(himc, IMR_DOCUMENTFEED, 0);
+  if (result == 0) {
+    // IMR_DOCUMENTFEED is not supported.
+    return false;
+  }
+  const size_t buffer_size = static_cast<size_t>(result);
+  if (buffer_size > kReconvertStringSizeLimit) {
+    LOG(ERROR) << "Too large RECONVERTSTRING.";
+    return false;
+  }
+
+  unique_ptr<BYTE[]> buffer(new BYTE[buffer_size]);
+
+  RECONVERTSTRING *reconvert_string =
+      reinterpret_cast<RECONVERTSTRING *>(buffer.get());
+  reconvert_string->dwSize = buffer_size;
+  reconvert_string->dwVersion = 0;
+
+  result = ::ImmRequestMessageW(himc, IMR_DOCUMENTFEED,
+                                reinterpret_cast<LPARAM>(reconvert_string));
+  if (result == 0) {
+    DLOG(ERROR) << "RECONVERTSTRING is nullptr.";
+    return false;
+  }
+
+  return ReconvertString::Decompose(
+      reconvert_string, preceding_text, nullptr, nullptr, nullptr,
+      following_text);
+}
+
+}  // namespace
+
+void ImeCore::UpdateContextWithSurroundingText(HIMC himc,
+                                               commands::Context *context) {
+  if (context == nullptr) {
+    return;
+  }
+  context->clear_preceding_text();
+  context->clear_following_text();
+  wstring preceding_text;
+  wstring following_text;
+  if (!QueryDocumentFeed(himc, &preceding_text, &following_text)) {
+    return;
+  }
+  Util::WideToUTF8(preceding_text, context->mutable_preceding_text());
+  Util::WideToUTF8(following_text, context->mutable_following_text());
+}
 
 KeyEventHandlerResult ImeCore::ImeProcessKey(
     mozc::client::ClientInterface *client,
     const VirtualKey &virtual_key,
     const LParamKeyInfo &lparam,
     const KeyboardStatus &keyboard_status,
-    const ImeBehavior &behavior,
-    const ImeState &initial_state,
-    ImeState *next_state,
+    const InputBehavior &behavior,
+    const InputState &initial_state,
+    const mozc::commands::Context &context,
+    InputState *next_state,
     commands::Output *output) {
-  scoped_ptr<Win32KeyboardInterface>
+  unique_ptr<Win32KeyboardInterface>
       keyboard(Win32KeyboardInterface::CreateDefault());
   return KeyEventHandler::ImeProcessKey(
-      virtual_key, lparam, keyboard_status, behavior, initial_state,
-      client, keyboard.get(), next_state, output);
+      virtual_key, lparam.GetScanCode(), lparam.IsKeyDownInImeProcessKey(),
+      keyboard_status, behavior, initial_state, context, client, keyboard.get(),
+      next_state, output);
 }
 
 KeyEventHandlerResult ImeCore::ImeToAsciiEx(
@@ -231,31 +289,29 @@ KeyEventHandlerResult ImeCore::ImeToAsciiEx(
     BYTE scan_code,
     bool is_key_down,
     const KeyboardStatus &keyboard_status,
-    const ImeBehavior &behavior,
-    const ImeState &initial_state,
-    ImeState *next_state,
+    const InputBehavior &behavior,
+    const InputState &initial_state,
+    const commands::Context &context,
+    InputState *next_state,
     commands::Output *output) {
-  scoped_ptr<Win32KeyboardInterface>
+  unique_ptr<Win32KeyboardInterface>
       keyboard(Win32KeyboardInterface::CreateDefault());
   return KeyEventHandler::ImeToAsciiEx(
       virtual_key, scan_code, is_key_down, keyboard_status, behavior,
-      initial_state, client, keyboard.get(), next_state, output);
+      initial_state, context, client, keyboard.get(), next_state, output);
 }
 
-bool ImeCore::OpenIME(
-    mozc::client::ClientInterface *client,
-    DWORD next_mode) {
-  commands::Status status;
-  if (!ConversionModeUtil::ConvertStatusFromNativeToMozc(
-          true, next_mode, &status)) {
+bool ImeCore::OpenIME(mozc::client::ClientInterface *client, DWORD next_mode) {
+  commands::CompositionMode mode = commands::DIRECT;
+  if (!ConversionModeUtil::GetMozcModeFromNativeMode(next_mode, &mode)) {
     return false;
   }
 
-  mozc::commands::KeyEvent key;
-  key.set_special_key(mozc::commands::KeyEvent::ON);
-  key.set_mode(status.mode());
+  SessionCommand command;
+  command.set_type(commands::SessionCommand::TURN_ON_IME);
+  command.set_composition_mode(mode);
   commands::Output output;
-  if (!client->SendKey(key, &output)) {
+  if (!client->SendCommand(command, &output)) {
     return false;
   }
   if (!output.consumed()) {
@@ -264,12 +320,18 @@ bool ImeCore::OpenIME(
   return true;
 }
 
-bool ImeCore::CloseIME(
-    mozc::client::ClientInterface *client,
-    commands::Output *output) {
-  mozc::commands::KeyEvent key;
-  key.set_special_key(mozc::commands::KeyEvent::OFF);
-  if (!client->SendKey(key, output)) {
+bool ImeCore::CloseIME(mozc::client::ClientInterface *client,
+                       DWORD next_mode, commands::Output *output) {
+  commands::CompositionMode mode = commands::DIRECT;
+  if (!ConversionModeUtil::GetMozcModeFromNativeMode(next_mode, &mode)) {
+    return false;
+  }
+
+  SessionCommand command;
+  command.set_type(commands::SessionCommand::TURN_OFF_IME);
+  command.set_composition_mode(mode);
+
+  if (!client->SendCommand(command, output)) {
     return false;
   }
   return true;
@@ -297,6 +359,47 @@ bool ImeCore::CancelComposition(HIMC himc, bool generate_message) {
   return UpdateInputContext(himc, output, generate_message);
 }
 
+bool ImeCore::SwitchInputMode(
+    HIMC himc, DWORD native_mode, bool generate_message) {
+  UIContext context(himc);
+
+  const bool open = context.GetOpenStatus();
+  if (!open) {
+    return true;
+  }
+  mozc::commands::Output output;
+  mozc::commands::SessionCommand command;
+  command.set_type(mozc::commands::SessionCommand::SWITCH_INPUT_MODE);
+
+  commands::CompositionMode mozc_mode = commands::HIRAGANA;
+  if (!ConversionModeUtil::ToMozcMode(native_mode, &mozc_mode)) {
+    return false;
+  }
+
+  command.set_composition_mode(mozc_mode);
+  if (!context.client()->SendCommand(command, &output)) {
+    return false;
+  }
+  return UpdateInputContext(himc, output, generate_message);
+}
+
+bool ImeCore::SendCallbackCommand(HIMC himc, bool generate_message) {
+  UIContext context(himc);
+  commands::Output last_output;
+  context.GetLastOutput(&last_output);
+
+  if (!last_output.has_callback()) {
+    return false;
+  }
+
+  mozc::commands::Output output;
+  if (!context.client()->SendCommand(last_output.callback().session_command(),
+                                     &output)) {
+    return false;
+  }
+  return UpdateInputContext(himc, output, generate_message);
+}
+
 DWORD ImeCore::GetSupportableConversionMode(DWORD raw_conversion_mode) {
   // If the initial |fdwConversion| is not a supported combination of flags,
   // we have to update it and then send the IMN_SETCONVERSIONMODE message.
@@ -348,13 +451,13 @@ DWORD ImeCore::GetSupportableSentenceMode(DWORD raw_sentence_mode) {
 }
 
 bool ImeCore::IsInputContextInitialized(HIMC himc) {
-  if (himc == NULL) {
+  if (himc == nullptr) {
     return false;
   }
   ScopedHIMC<InputContext> context(himc);
   // For some reason, we fail to lock input context.  See b/3088049 for
   // details.
-  if (context.get() == NULL) {
+  if (context.get() == nullptr) {
     return false;
   }
   if (!PrivateContextUtil::IsValidPrivateContext(context->hPrivate)) {
@@ -445,7 +548,7 @@ void ImeCore::SortIMEMessages(
 }
 
 bool ImeCore::UpdateContext(HIMC himc,
-                            const ImeState &next_state,
+                            const InputState &next_state,
                             const commands::Output &new_output,
                             MessageQueue *message_queue) {
   if (!IsInputContextInitialized(himc)) {
@@ -458,7 +561,15 @@ bool ImeCore::UpdateContext(HIMC himc,
     return UpdateContextMain(himc, next_state, new_output, message_queue);
   }
 
-  // Callback exists.
+  // Delayed callback exists.
+  if (new_output.callback().has_delay_millisec()) {
+    message_queue->AddMessage(WM_IME_NOTIFY,
+                              IMN_PRIVATE,
+                              kNotifyDelayedCallback);
+    return UpdateContextMain(himc, next_state, new_output, message_queue);
+  }
+
+  // Immediate callback exists.
   const SessionCommand &callback_command =
       new_output.callback().session_command();
 
@@ -467,7 +578,7 @@ bool ImeCore::UpdateContext(HIMC himc,
   // The UI handler will invoke reconversion later as other Japanese IMEs do.
   if (callback_command.has_type() &&
       callback_command.type() == SessionCommand::CONVERT_REVERSE) {
-    if (message_queue != NULL) {
+    if (message_queue != nullptr) {
       message_queue->AddMessage(WM_IME_NOTIFY, IMN_PRIVATE,
                                 kNotifyReconvertFromIME);
     }
@@ -481,7 +592,7 @@ bool ImeCore::UpdateContext(HIMC himc,
   if (!context.client()->SendCommand(callback_command, &callback_output)) {
     return false;
   }
-  ImeState callback_state;
+  InputState callback_state;
   if (!GetNextState(himc, callback_output, &callback_state)) {
     return false;
   }
@@ -490,11 +601,11 @@ bool ImeCore::UpdateContext(HIMC himc,
 }
 
 bool ImeCore::UpdateContextMain(HIMC himc,
-                                const ImeState &next_state,
+                                const InputState &next_state,
                                 const commands::Output &new_output,
                                 MessageQueue *message_queue) {
   DCHECK(IsInputContextInitialized(himc));
-  const bool generate_message = (message_queue != NULL);
+  const bool generate_message = (message_queue != nullptr);
   ScopedHIMC<InputContext> context(himc);
   ScopedHIMCC<PrivateContext> private_context(context->hPrivate);
 
@@ -539,7 +650,7 @@ bool ImeCore::UpdateContextMain(HIMC himc,
 
   // Update context.
   context->fOpen = next_state.open ? TRUE : FALSE;
-  context->fdwConversion = next_state.conversion_status;
+  context->fdwConversion = next_state.logical_conversion_mode;
 
   vector<UIMessage> composition_messages;
   if (!UpdateCompositionString(himc, output, &composition_messages)) {
@@ -550,7 +661,7 @@ bool ImeCore::UpdateContextMain(HIMC himc,
   context->hCandInfo =
       mozc::win32::CandidateInfoUtil::Update(context->hCandInfo,
                                              output, &candidate_messages);
-  if (context->hCandInfo == NULL) {
+  if (context->hCandInfo == nullptr) {
     return false;
   }
 
@@ -564,7 +675,7 @@ bool ImeCore::UpdateContextMain(HIMC himc,
                     previous_open,
                     previous_conversion,
                     next_state.open,
-                    next_state.conversion_status,
+                    next_state.logical_conversion_mode,
                     &sorted_messages);
 
     // Allow visibility trackers to track if each UI message will be
@@ -593,28 +704,38 @@ BOOL ImeCore::IMEOff(HIMC himc, bool generate_message) {
   }
 
   mozc::win32::UIContext context(himc);
+
+  DWORD logical_conversion_mode = 0;
+  if (!context.GetLogicalConversionMode(&logical_conversion_mode)) {
+    return FALSE;
+  }
+
   commands::Output output;
-  if (!ImeCore::CloseIME(context.client(), &output)) {
+  if (!ImeCore::CloseIME(context.client(), logical_conversion_mode,
+                         &output)) {
     return FALSE;
   }
   bool next_open_status = false;
-  DWORD next_mode = 0;
+  DWORD next_logical_mode = 0;
+  DWORD next_visible_mode = 0;
   if (!output.has_status()) {
     mozc::win32::UIContext uicontext(himc);
-    if (!uicontext.GetConversionMode(&next_mode)) {
+    if (!uicontext.GetConversionMode(&next_logical_mode)) {
       return FALSE;
     }
+    next_visible_mode = next_logical_mode;
   } else if (!mozc::win32::ConversionModeUtil::
                   ConvertStatusFromMozcToNative(
       output.status(), context.IsKanaInputPreferred(),
-      &next_open_status, &next_mode)) {
+      &next_open_status, &next_logical_mode, &next_visible_mode)) {
     return FALSE;
   }
-  ImeState next_state;
+  InputState next_state;
   next_state.open = false;  // We ignore the returned status.
-  next_state.conversion_status = next_mode;
+  next_state.logical_conversion_mode = next_logical_mode;
+  next_state.visible_conversion_mode = next_visible_mode;
   if (!generate_message) {
-    if (!UpdateContext(himc, next_state, output, NULL)) {
+    if (!UpdateContext(himc, next_state, output, nullptr)) {
       return FALSE;
     }
     return TRUE;
@@ -725,7 +846,7 @@ BOOL ImeCore::CloseCandidate(HIMC himc, bool generate_message) {
 bool ImeCore::IsActiveContext(HIMC himc) {
   bool is_active = false;
   const HWND focus_window = ::GetFocus();
-  if (focus_window != NULL &&
+  if (focus_window != nullptr &&
       ::IsWindow(focus_window) != FALSE) {
     const HIMC active_himc = ::ImmGetContext(focus_window);
     is_active = (himc == active_himc);
@@ -739,13 +860,13 @@ bool ImeCore::TurnOnIMEAndTryToReconvertFromIME(HIMC himc) {
   if (context.IsEmpty()) {
     return false;
   }
-  if (context.input_context() == NULL) {
+  if (context.input_context() == nullptr) {
     return false;
   }
   if (!context.IsCompositionStringEmpty()) {
     // TODO(yukawa): Use Mozc server to determine the behavior when any
     //   appropriate protocol becomes available.
-    DLOG(INFO) << "Ongoing comosition exists.";
+    DLOG(INFO) << "Ongoing composition exists.";
     return false;
   }
 
@@ -771,7 +892,7 @@ bool ImeCore::TurnOnIMEAndTryToReconvertFromIME(HIMC himc) {
     }
   }
 
-  return (UpdateInputContext(himc, output, true) != FALSE);
+  return UpdateInputContext(himc, output, true);
 }
 
 string ImeCore::GetTextForReconversionFromIME(HIMC himc) {
@@ -787,8 +908,7 @@ string ImeCore::GetTextForReconversionFromIME(HIMC himc) {
   // |RECONVERTSTRING::dwTargetStrOffset| and |RECONVERTSTRING::dwTargetStrLen|
   // is to be reconverted.  Technically most of the following processes should
   // be done at the server-side.
-  LRESULT result =
-      ::ImmRequestMessageW(himc, IMR_RECONVERTSTRING, NULL);
+  LRESULT result = ::ImmRequestMessageW(himc, IMR_RECONVERTSTRING, 0);
   if (result == 0) {
     DLOG(INFO) << "IMR_RECONVERTSTRING is not supported.";
     return "";
@@ -800,26 +920,26 @@ string ImeCore::GetTextForReconversionFromIME(HIMC himc) {
     return "";
   }
 
-  scoped_array<BYTE> buffer(new BYTE[buffer_size]);
+  unique_ptr<BYTE[]> buffer(new BYTE[buffer_size]);
 
   RECONVERTSTRING *reconvert_string =
-      reinterpret_cast<RECONVERTSTRING*>(buffer.get());
+      reinterpret_cast<RECONVERTSTRING *>(buffer.get());
   reconvert_string->dwSize = buffer_size;
   reconvert_string->dwVersion = 0;
 
   result = ::ImmRequestMessageW(himc, IMR_RECONVERTSTRING,
                                 reinterpret_cast<LPARAM>(reconvert_string));
   if (result == 0) {
-    DLOG(ERROR) << "RECONVERTSTRING is NULL.";
+    DLOG(ERROR) << "RECONVERTSTRING is nullptr.";
     return "";
   }
 
-  scoped_array<BYTE> copied_buffer(new BYTE[buffer_size]);
+  unique_ptr<BYTE[]> copied_buffer(new BYTE[buffer_size]);
   for (size_t i = 0; i < buffer_size; ++i) {
     copied_buffer[i] = buffer[i];
   }
   RECONVERTSTRING *expanded_reconvert_string =
-      reinterpret_cast<RECONVERTSTRING*>(copied_buffer.get());
+      reinterpret_cast<RECONVERTSTRING *>(copied_buffer.get());
 
   // Expand the composition range if necessary.
   if (!ReconvertString::EnsureCompositionIsNotEmpty(
@@ -870,14 +990,14 @@ bool ImeCore::ReconversionFromApplication(
   if (context.IsEmpty()) {
     return false;
   }
-  if (context.input_context() == NULL) {
+  if (context.input_context() == nullptr) {
     return false;
   }
 
   if (!context.IsCompositionStringEmpty()) {
     // TODO(yukawa): Use Mozc server to determine the behavior when any
     //   appropriate protocol becomes available.
-    DLOG(INFO) << "Ongoing comosition exists.";
+    DLOG(INFO) << "Ongoing composition exists.";
     return false;
   }
 
@@ -900,37 +1020,8 @@ bool ImeCore::ReconversionFromApplication(
     return false;
   }
 
-  return (UpdateInputContext(himc, output, true) != FALSE);
+  return UpdateInputContext(himc, output, true);
 }
 
-bool ImeCore::ResetServerContextIfNeccesary(HIMC himc) {
-  UIContext context(himc);
-  if (context.IsEmpty()) {
-    return false;
-  }
-  if (context.input_context() == NULL) {
-    return false;
-  }
-  if (context.client() == NULL) {
-    return false;
-  }
-
-  // TODO(yukawa): Use IMR_DOCUMENTFEED in addition to mouse event.
-  const bool was_left_pressed = ThreadLocalMouseTracker::WasLeftButtonPressed();
-  ThreadLocalMouseTracker::ResetWasLeftButtonPressed();
-  if (!context.GetOpenStatus() || !context.IsCompositionStringEmpty()) {
-    return true;
-  }
-  if (was_left_pressed) {
-    SessionCommand command;
-    command.set_type(commands::SessionCommand::RESET_CONTEXT);
-    Output output;
-    if (!context.client()->SendCommand(command, &output)) {
-      LOG(ERROR) << "TestSendKey failed.";
-      return false;
-    }
-  }
-  return true;
-}
 }  // namespace win32
 }  // namespace mozc
